package musr.client.bestfit;

import musr.client.datacolumn.*;
import musr.client.function.*;
import java.util.StringTokenizer;
import java.util.Date;

public class BezierSpline extends AbstractFit
{
    //used to hold the input data
    private double[] xData;
    private double[] yData;
    
    //used to hold the functions which generate the splines
    private String xFunction[];
    private String yFunction[];
    
    //used to hold the values in the splines
    private ConcreteDataColumn xSpline;
    private ConcreteDataColumn ySpline;

    public BezierSpline(DataColumn xColumn, DataColumn yColumn)
    {
        System.out.println("constructor called");
        xData = dataToArray(xColumn);
        yData = dataToArray(yColumn);
        
        xFunction = new String[xData.length-1];
        yFunction = new String[yData.length-1];
        
        xSpline = new ConcreteDataColumn(xColumn.getName()+"_s", xColumn.getLabel(), xColumn.getUnits(), new Date());
        ySpline = new ConcreteDataColumn(yColumn.getName()+"_s", yColumn.getLabel(), yColumn.getUnits(), new Date());
        System.out.println("call to constructor ended"); 
    }    
    
    public void fit()
    {
        System.out.println("fit() called");
        //initialize the array of functions
        for(int i=0; i<xFunction.length; i++)
        {
            xFunction[i] = Double.toString(xData[i]);
            yFunction[i] = Double.toString(yData[i]);
        }
        //use recursive relationships to fill out functions
        int j=xFunction.length-1;
        for(int i=0; i<j; i++)
        {
            xFunction[i] = "u*" + xFunction[i] + "+t*" + xFunction[i+1];
            yFunction[i] = "u*" + yFunction[i] + "+t*" + yFunction[i+1];
        }
        xFunction[xFunction.length-1] = "u*" + xFunction[j-1] + "+t*" + xData[j];
        yFunction[yFunction.length-1] = "u*" + yFunction[j-1] + "+t*" + xData[j];
        
        for(j=xFunction.length-2; j>0; j--)
            for(int i=0; i<j; i++)
            {
                xFunction[i] = "u*" + xFunction[i] + "+t*" + xFunction[i+1];
                yFunction[i] = "u*" + yFunction[i] + "+t*" + yFunction[i+1];
            }
        System.out.println("call to fit() ended");
        /*double[] tempX = new double[xData.length];
        double[] tempY = new double[yData.length];
        double t;
        //double u;
        for(int k=1; k<INTERPOLATION; k++)
        {            
            System.arraycopy(xData, 0, tempX, 0, xData.length);
            System.arraycopy(yData, 0, tempY, 0, yData.length);
            
            t = (double)k/INTERPOLATION;
            //u = (double)(INTERPOLATION-k)/INTERPOLATION;
            System.out.println("t="+t);
            
            for (int j=tempX.length-1; j>0; j--)
                for (int i=0; i< j; i++)
                {
                    tempX[i] = (1.0-t)*tempX[i] + t*tempX[i+1];
                    tempY[i] = (1.0-t)*tempY[i] + t*tempY[i+1];
                }
            
            for(int z=0; z<tempX.length; z++)
            {
                xSpline.addElement(new Double(tempX[z]));
                ySpline.addElement(new Double(tempY[z]));
            }
        }*/
    }
    
    private void makeSplineColumns(int factor)
    {
        System.out.println("makeSplineColumns() called");
        for(int i=0; i<xFunction.length-1; i++)
            for(int j=0; j<factor; j++)
            {
                xSpline.addElement(new Double(evaluate(xFunction[i], j*(xData[i]-xData[i+1])/(factor*(xData[i]-xData[i+1])))));
                ySpline.addElement(new Double(evaluate(yFunction[i], j*(xData[i]-xData[i+1])/(factor*(xData[i]-xData[i+1])))));
            }
        System.out.println("call to makeSplineColumns() ended");
    }
    
    public String toString()
    {
        return "Splines for " + ySpline.getLabel() + " vs " + xSpline.getLabel();
    }
    
    public DataColumn[] getSplineColumns(int factor)
    {
        System.out.println("getSplineColumns() called");
        if((factor<0)||(xSpline.getSize()==0))
            makeSplineColumns(factor);
        ConcreteDataColumn zero = new ConcreteDataColumn();
        for(int i=0; i<ySpline.getSize(); i++)
            zero.addElement(new Double(0.0));
        System.out.println("call to getSplineColumns() ended");
        return new DataColumn[]{xSpline, new MultiColumn(ySpline, zero, zero)};
    }
    
    public DataColumn[] getSplineColumns()
    {
        return getSplineColumns(-1);
    }
    
    public static void main(String[] args)
    {
        ConcreteDataColumn testX = new ConcreteDataColumn();
        ConcreteDataColumn testY = new ConcreteDataColumn();
        for(double i=0.0; i<10.0; i++)
        {
            testX.addElement(new Double(i));
            testY.addElement(new Double(f(i)));
        }
        BezierSpline test = new BezierSpline(testX, testY);
        test.fit();
        test.makeSplineColumns(5);
        DataColumn[] result = test.getSplineColumns();
        for(int i=0; i<result[0].getSize(); i++)
            System.out.println(result[0].getValue(i) + "\t" + result[1].getValue(i) + "\t" + 
                    f(Double.parseDouble(result[0].getValue(i).toString())));
    }
    
    
    //test function
    private static final double f(double x)
    {
        return 5*x*x*x-18*x*x+2*x+3;
    }
    
    //evaluates the function represented by the String f at the value t
    private static double evaluate(String f, double t)
    {
        System.out.println("evaluate() called with t="+t);
        //System.out.println("f="+f+", t="+t);
        //first, replace every u with the numerical value (1.0-t), and every t with the numerical value t:
        int uIndex = f.indexOf("u");
        while(uIndex>=0)
        {
            f = f.substring(0, uIndex) + (1.0-t) + f.substring(uIndex+1);
            uIndex = f.indexOf("u");
        }
        int tIndex = f.indexOf("t");
        while(tIndex>=0)
        {
            f = f.substring(0, tIndex) + t + f.substring(tIndex+1);
            tIndex = f.indexOf("t");
        }
        //now, all that is left is numbers, +s, and *s... 
        String numBefore;
        String numAfter;
        //do the *s:
        int timesIndex = f.indexOf("*");
        while(timesIndex>=0)
        {
            numBefore = findNumber(f, timesIndex, -1);
            numAfter = findNumber(f, timesIndex, 1);
            f = f.substring(0, timesIndex-numBefore.length()) + (Double.parseDouble(numBefore)*Double.parseDouble(numAfter)) + 
                    f.substring(timesIndex+numAfter.length());
            timesIndex = f.indexOf("*");
        }
        //now the +s:
        int plusIndex = f.indexOf("+");
        while(plusIndex>=0)
        {
            numBefore = findNumber(f, plusIndex, -1);
            numAfter = findNumber(f, plusIndex, 1);
            f = f.substring(0, plusIndex-numBefore.length()) + (Double.parseDouble(numBefore)+Double.parseDouble(numAfter)) + 
                    f.substring(plusIndex+numAfter.length());
            plusIndex = f.indexOf("+");
        }
        System.out.println("call to evaluate() ended");
        if(f.length()==0)
            System.out.println("ALERT: f.length() = 0");
        System.out.println("evaluated:"+f);
        return Double.parseDouble(f.trim());
    }
    
    private static String findNumber(String expr, int index, int direction)
    {
        String result = "";
        char temp = ' ';
        if(direction>0)
        {
            index++;
            temp = expr.charAt(index);
            while((index<expr.length())&&((expr.charAt(index)=='.')||(expr.charAt(index)=='-')||(Character.isDigit(expr.charAt(index)))||
                    (expr.charAt(index)=='E')||(expr.charAt(index)=='e')))
            {
                //System.out.println(expr.charAt(index));
                result += expr.charAt(index);
                index++;
            }
            //if(index<expr.length())
                //System.out.println("next char: " + expr.charAt(index));
        }
        else if(direction<0)
        {
            index--;
            while((index>=0)&&((expr.charAt(index)=='.')||(expr.charAt(index)=='-')||(Character.isDigit(expr.charAt(index)))||
                    (expr.charAt(index)=='E')||(expr.charAt(index)=='e')))
            {
                result = expr.charAt(index) + result;
                index--;
            }
            //if(index>0)
               // System.out.println("next char: " + expr.charAt(index));
        }
        //System.out.println("result="+result);
        if(result.length()==0)
        {
            System.out.println("Error: the following non-numerical character was encountered: "+temp);
            System.out.println("index="+index+"\tdirection="+direction+"\texpr="+expr);
        }
        return result;
    }
}