package musr.client.bestfit;

import musr.client.datacolumn.*;
import java.util.Date;

public class SplineFit extends AbstractFit
{
    /*//holds x values from data column
    private double[] xData;
    
    //holds y values from data column
    private double[] yData;*/
    
    //holds x and y values
    private Point[] data;
    
    //holds spline coefficients
    private double[][] coefficient;
    
    //used for the toString() method
    private String xDataName;
    private String yDataName;
    
    //resulting spline columns
    private ConcreteDataColumn xSplines;
    private ConcreteDataColumn ySplines;
    
    //used in sorting
    private int n;
    
    private static final int NUMBER_OF_POINTS = 255;        //work on this: must be greater than length of intitial data set!

    public SplineFit(DataColumn xColumn, DataColumn yColumn)
    {
        double[] xData = dataToArray(xColumn);
        xDataName = xColumn.getName();
        xSplines = new ConcreteDataColumn(xDataName, xColumn.getLabel(), xColumn.getUnits(), new Date());
        
        double[] yData = dataToArray(yColumn);
        yDataName = yColumn.getName();
        ySplines = new ConcreteDataColumn(xDataName, yColumn.getLabel(), yColumn.getUnits(), new Date());
        
        data = new Point[xData.length];
        for(int i=0; i<data.length; i++)
            data[i] = new Point(xData[i], yData[i]);
        
        n = data.length;
        heapSort();
            
        
        /*System.out.println("x values before sorting:");
        printArray(xData);*/
        
        /*System.out.println("x values after sorting:");
        printArray(xData);*/
        
        coefficient = new double[xData.length-1][4];
    }
    
    /**
     *This method sorts data using a heap sort algorithm.
     *This was used instead of faster algorithms such as quick sort because quick sort is slowest with already-sorted data, and there is a
     *nontrivial chance that the x data is already sorted.
     */
    private void heapSort()
    {
        buildHeap();
        while(n>1)
        {
            n--;
            swap(0, n);
            downHeap(0);
        } 
    }


    private void buildHeap()
    {
        for(int j = n/2-1; j>=0; j--)
            downHeap(j);
    }


    private void downHeap(int topIndex)
    {
        int childIndex=2*topIndex+1;
        while (childIndex<n)
        {
            if((childIndex+1<n)&&(data[childIndex+1].x>data[childIndex].x))
                childIndex++;

            if(data[topIndex].x>=data[childIndex].x) return;

            swap(topIndex, childIndex);
            topIndex = childIndex;  
            childIndex = 2*topIndex+1;
        }
    }


    private void swap(int i, int j)
    {
        /*double tempX = xData[i];
        double tempY = yData[i];
        
        xData[i] = xData[j];
        yData[i] = yData[j];
        
        xData[j] = tempX;
        yData[j] = tempY;*/
        Point temp = data[i];
        data[i] = data[j];
        data[j] = temp;
    }
    
    public static void main(String[] args)
    {
        ConcreteDataColumn xColumn = new ConcreteDataColumn();
        ConcreteDataColumn yColumn = new ConcreteDataColumn();
        for(int i=0; i<10; i++)
        {
            xColumn.addElement(new Double(i));
            yColumn.addElement(new Double(f(i)));
        }
        
        /*xColumn.addElement(new Double(-343.0));
        yColumn.addElement(new Double(f(-343.0)));*/
        
        SplineFit test = new SplineFit(xColumn, yColumn);
        long time = System.currentTimeMillis();
        test.fit();
        time = System.currentTimeMillis() - time;
        System.out.println("Fit succesfully completed in " + time + " ms.");
        for(int i=0; i<test.xSplines.getSize(); i++)
            System.out.println(test.xSplines.getValue(i)+"\t"+test.ySplines.getValue(i));
        //for(int i=0; i<4; i++)
            //System.out.println("Evaluated with function: " + f(i+.5) + " Evaluated with spline: " + test.evaluateSplineAt(i+.5));
            //new SplineFit(xColumn, yColumn).findDArray();*/
    }
    
    /*public double evaluateSplineAt(double value)
    {
        //System.out.println("evaluateSplineAt("+value+")");
        //if the value is not in the spline range, return zero:
        if((value<xData[0])||(value>xData[xData.length-1]))
            return 0.0;
        
        //find which x values the given value lies between:
        int index = findXIndex(value, 0, xData.length-1);
        
        double x = (value-xData[index])/(xData[index+1]-xData[index]);
        //System.out.println("index="+index+" x="+x);
        return coefficient[index][0] + coefficient[index][1]*x + coefficient[index][2]*x*x + coefficient[index][3]*x*x*x;
    }
    
    private int findXIndex(double value, int startIndex, int endIndex)
    {
       // System.out.println("findXIndex("+value+", "+startIndex+", "+endIndex+")");
        if((value>xData[endIndex])||(value<xData[startIndex]))
        {
            System.out.println("Something has gone terribly wrong with the findXIndex() method in the SplineFit class.");
            return 0;
        }
        else if((startIndex==endIndex)||(endIndex==startIndex+1))
            return startIndex;
        else
        {
            int midIndex = (endIndex-startIndex)/2+startIndex;
            if(value==xData[midIndex])
                return midIndex;
            else if(value>xData[midIndex])
                return findXIndex(value, midIndex, endIndex);
            else if(value<xData[midIndex])
                return findXIndex(value, startIndex, midIndex);
        }
        
        //keep the compiler happy
        return 0;
    }*/
    
    public String toString()
    {
        return "Cubic splines for " + yDataName + " vs " + xDataName;
    }
    
    public void fit()
    {   
        while(xSplines.getSize()<NUMBER_OF_POINTS)
        {
            Point[] point = (Point[])(data.clone());
            double t;

            for(int j=point.length-1; j>0; j--)
            {
                for(int i=0; i<j; i++)
                {
                    t = Math.random();
                    point[i].x = (1.0-t)*point[i].x + t*point[i+1].x;
                    point[i].y = (1.0-t)*point[i].y + t*point[i+1].y;
                }
                //copy result[] into the DataColumns
                for(int l=0; l<point.length; l++)
                {
                    xSplines.addElement(new Double(point[l].x));
                    ySplines.addElement(new Double(point[l].y));
                }
            }
        }
    }
    
    //used in testing only
    private static double f(double x)
    {
        return x*x+8;
    }
    
    /*
     *used in the math for the Bzier splines
     **/
    private class Point implements Cloneable
    {
        double x;
        double y;
        
        public Point(double newX, double newY)
        {
            x = newX;
            y = newY;
        }
        
        public void multiply(double n)
        {
            x *= n;
            y *= n;
        }
        
        public Object clone(Point p) throws CloneNotSupportedException
        {
            return new Point(x, y);
        }
    }
}