package musr.client.csv;
import musr.client.datacolumn.*;
import musr.client.function.*;
import musr.client.gui.*;
import musr.client.plot2d.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class CSVReader
{
    public Frame parentFrame;
    BufferedReader inputStream = null;
    
    /**
     Creates a new instance of CSVReader.
     @param parent The Frame (ie, GUI) which is contains the DataTable that this method will be filling.
     */
    public CSVReader(Frame parent)
    {
        parentFrame = parent;
    }
    
    public void setCaseSensitive(boolean sensitive)
    {
        ((GUI)parentFrame).caseMenuItem.setState(sensitive);
    }
    /**
     Reads in a file from the given location, and displays it as a DataTable.
     @param filepath The String representation of the file's location.
     */
    public void loadStream(String filepath)
    {
        File inFile = new File(filepath);
        String currentLine; //currently read line
        CSVTokenizer tokenizer; //strips off commas, quotation marks, et cetera put in by ExportCSV
        Vector dataColumnHolder = new Vector(); //temporarily holds the ConcreteDataColumns containing the columns
        Vector multiColumnHolder = new Vector(); //temporarily holds MultiColumns
        ConcreteDataColumn temp; //temporary column which holds ConcreteDataColumns before they are put into dataColumnHolder or DataTables
        int count; //used when putting data into ConcreteDataColumns
        MultiColumn tempMultiColumn;
        Vector nameHolder = new Vector();      //Holds names of columns
        Vector labelHolder = new Vector();     //Holds labels of columns
        Vector unitsHolder = new Vector();     //Holds units of columns
                
        if(inFile.exists()&&inFile.canRead())
        {        
            try
            {            
                ((GUI)parentFrame).tempDataColumns = new DataTable(filepath);
                inputStream = new BufferedReader(new FileReader(inFile));
                ((GUI)parentFrame).tempDataColumns = new DataTable(filepath);

                currentLine = inputStream.readLine(); //get title
                if((currentLine==null)||(currentLine.length()<8))    
                    JOptionPane.showMessageDialog(null,"The selected file appears to be empty.");
                else
                {
                    currentLine = currentLine.substring(7); //remove "Title: " from beginning of title
                    currentLine = replaceBreaks(currentLine);
                    (((GUI)parentFrame).tempDataColumns).setTitle(currentLine);
                }

                currentLine = inputStream.readLine(); //get abstract
                if((currentLine!=null)&&(currentLine.length()>11))
                {
                    currentLine = currentLine.substring(10); //remove "Abstract: " from beginning of abstract
                    currentLine = replaceBreaks(currentLine);
                    (((GUI)parentFrame).tempDataColumns).setAbstract(currentLine);
                }

                currentLine = inputStream.readLine(); //get comments
                if((currentLine!=null)&&(currentLine.length()>11))
                {
                    currentLine = currentLine.substring(10); //remove "Comments: " from beginnging of comments
                    currentLine = replaceBreaks(currentLine);
                    (((GUI)parentFrame).tempDataColumns).setComment(currentLine);
                }

                currentLine = inputStream.readLine();
                CSVTokenizer labelTokenizer = new CSVTokenizer(currentLine);
                String currentLabel;

                currentLine = inputStream.readLine();
                CSVTokenizer nameTokenizer = new CSVTokenizer(currentLine);
                String currentName;

                currentLine = inputStream.readLine();
                CSVTokenizer unitTokenizer = new CSVTokenizer(currentLine);
                String currentUnit;

                while(nameTokenizer.hasMoreTokens()) //Takes column header information (i.e., names, labels, and units) and puts it into MultiColumns
                {
                    currentName = nameTokenizer.getNextToken();
                    currentLabel = labelTokenizer.getNextToken();
                    currentUnit = unitTokenizer.getNextToken();
                    tempMultiColumn = new MultiColumn(new ConcreteDataColumn(), new ConcreteDataColumn(), new ConcreteDataColumn());    

                    if(currentName.trim().length()>0)    
                    {   
                        nameHolder.add(currentName);
                        labelHolder.add(currentLabel);
                        unitsHolder.add(currentUnit);
                        multiColumnHolder.add(tempMultiColumn);
                    }               
                }

                //Time to initialize the ConcreteDataColumns (which contain the values, the positive errors, and the negative errors)
                currentLine = inputStream.readLine();
                tokenizer = new CSVTokenizer(currentLine);
                while(tokenizer.hasMoreTokens())
                {
                    temp = new ConcreteDataColumn();
                    temp.addElement(tokenizer.getNextToken());
                    dataColumnHolder.addElement(temp);
                }

                currentLine = inputStream.readLine();
                //Now, fill up the ConcreteDataColumns
                while(currentLine!=null)
                {                    
                    tokenizer = new CSVTokenizer(currentLine);
                    count = 0;
                    while(tokenizer.hasMoreTokens())
                    {
                        ((ConcreteDataColumn)dataColumnHolder.elementAt(count)).addElement(tokenizer.getNextToken());
                        count++;
                    }
                    currentLine = inputStream.readLine();
                }          

            System.out.println("Data has been read in from file.");
            }

            catch(IOException ioe) //Post-mortem error messages
            {
                System.out.println("Error in reading from file.\n" + ioe.getMessage());
                File checker = new File(filepath);
                if(!checker.isFile()||!checker.exists())
                    JOptionPane.showMessageDialog(null, "The file you have selected is not a valid file.");
                else if(!checker.canRead())
                    JOptionPane.showMessageDialog(null, "The file you selected can not be read by this program.");
            }

            finally     //Input stream must be closed
            {
                try
                {
                    inputStream.close();
                }
                catch(IOException e)
                {
                    System.out.println("An error occured, and the input stream has not been closed.");
                }
            }

            //Now, take the ConcreteDataColumns back out of the dataColumnHolder and put them into the MultiColumns:
            for(int i=0; i<dataColumnHolder.size(); i++)
            {       //Integer division truncates fractions (decimals) and so the arguments to multiColumnHolder.elementAt will all be integers
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).setValueColumn((ConcreteDataColumn)dataColumnHolder.elementAt(i));
                i++;
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).setErrorPlusColumn((ConcreteDataColumn)dataColumnHolder.elementAt(i));
                i++;
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).setErrorMinusColumn((ConcreteDataColumn)dataColumnHolder.elementAt(i));
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).getValueColumn().setName((String)nameHolder.elementAt(i/3));
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).getValueColumn().setLabel((String)labelHolder.elementAt(i/3));
                ((MultiColumn)(multiColumnHolder.elementAt(i/3))).getValueColumn().setUnits((String)unitsHolder.elementAt(i/3));
            }

            //Finally, put the MultiColumns into the DataTable:
            for(int i=0; i<multiColumnHolder.size(); i++)         
                (((GUI)parentFrame).tempDataColumns).addColumn((MultiColumn)multiColumnHolder.elementAt(i));

            if (((GUI)parentFrame).dataTableList.getItemCount()>0) 
            {
                ((GUI)parentFrame).dataTableList.select(((GUI)parentFrame).dataTableList.getItemCount()-1); // Select the last item (which would be the datatable just loaded)
                ((GUI)parentFrame).reloadColumnList();
            }
        }
        else
            JOptionPane.showMessageDialog(null, "The selected file does not exist or is not readable. Select another file, and try again.");
    }
    
    private String replaceBreaks(String line)       //Puts line breaks back into title, abstract, and comments
    {
        int spaceIndex = line.indexOf("     ");
        while(spaceIndex>0)
        {
            line = line.substring(0, spaceIndex) + "\n" + line.substring(spaceIndex+5);
            spaceIndex = line.indexOf("     ");
        }
        return line;
    }
}