package musr.client.datacolumn;
import java.util.*;
import java.lang.*;
import java.util.Date;

/**
 * This class contains an array of elements that are 'manually' loaded once the class is initialized.
 * It's size depends on however big the array we are loading is.
 * It is loaded into the MultiColumn when loading db files. There are three columns that are loaded: value, posErr and negErr
 * It is also loaded from MUD File Loading, for all Spectrums except the "Raw" Spectrum which generates a PoissonDataColumn
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class ConcreteDataColumn extends ColumnHeader implements Cloneable, DataColumn, java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID

    // instance variables
    private Vector values;

    // constructors
    public ConcreteDataColumn() { this("","","", new Date()); }
    public ConcreteDataColumn(String name, String label, String units, Date date) {
      super(name, label, units, date);
      values = new Vector();
    }

    // instace methods only in ConcreteDataColumn
    public void addElement(Object value) { values.addElement(value); }
    public void removElement(Object value) { values.removeElement(value); }
    public void removeElementAt(int index) { values.removeElementAt(index); }
    public void removeAllElements() { values.removeAllElements(); }
    public void setElementAt(Object value, int index) { values.setElementAt(value, index); }
    public boolean isEmpty() { return values.isEmpty(); }

    // Methods from the DataColumn interface
    public int getSize() { return values.size(); }
    public Object getValue(int index) { return values.elementAt(index); }
    public String getType() { return "real"; }

    public Object getErrorPlus(int index){return new Double(0.0);}
    public Object getErrorMinus(int index) {return new Double(0.0);}

    /**
     * this method extracts the wanted range of numbers from the array
    */
    public DataColumn getSubColumn(Range r) {
      ConcreteDataColumn temp = new ConcreteDataColumn(name, label, units, date);
      for(int i=r.min; i<=r.max; i++) temp.addElement(getValue(i));
      return temp;
    }

    public Object clone() throws CloneNotSupportedException {
      ConcreteDataColumn returnValue = (ConcreteDataColumn)super.clone();
      returnValue.values = (Vector)values.clone();
      //note: we don't need to clone the Objects (which are converted doubles) within the Vector!
      String cloneFrom[] = new String[3];
      String cloneTo[] = new String[3];
      cloneFrom[0] = super.label;
      cloneFrom[1] = super.name;
      cloneTo = (String[])cloneFrom.clone();
      returnValue.label = cloneTo[0];
      returnValue.name = cloneTo[1];
      return returnValue;
    }

    /**
     * With this overridden method the column can be printed in a nice format
     */
     public String toString() {
       String returnValue =  "Name: " + getName() + "\n" ;
       //						"Label: " + getLabel() + "\n";
       returnValue += "Column: ";
       //for(int i=0; i<getSize(); i++)
       //		returnValue += getValue(i) + " ";
       returnValue += "\n";
       return returnValue;
      }

}

