package musr.client.datacolumn;

import java.util.*;
import java.lang.*;
import java.util.Date;

/**
 * A class that represents a Column with Objects as its elements
 * It differs from regular columns because it's Error Columns are not 'real' values. They are calculated based on the data values
 *
 * So far, within the GUI, the only time this Column will be loaded is if the user loads a Raw Spectrum from a MUD file.
 * Any other spectrum type will generate calculated, different errors than what poisson column calculates. So those will be MultiColumns.
 *
*/

public class PoissonDataColumn extends ColumnHeader implements DataColumn, java.io.Serializable {

  // instance variables
  private Vector values;
		/*
			public static void main(String args[]) {
			PoissonDataColumn a = new PoissonDataColumn("Juhu");
			for(int i=0; i<17; i++) {
			a.addElement(new Long(i+3));
			}
			Range r = new Range(2, 12);
			a = (PoissonDataColumn) a.getSubColumn(r);

			System.out.println("label: "+a.getLabel());
			System.out.println("size: "+a.getSize());
			for(int i=0; i<a.getSize(); i++) {
			System.out.println("value"+i+" : "+a.getValue(i));
			}
		}
		*/
    // constructors
    public PoissonDataColumn() {
      this("","","", new Date());
    }
    public PoissonDataColumn(String name, String label, String units, Date date) {
      super(name, label, units, date);
      values = new Vector();
    }
    public PoissonDataColumn(String name, String label, String units, int size, int[] data, Date date) {
				super(name, label, units, date);
				values = new Vector();
        for (int i=0;i<size;i++)
             values.addElement(new Integer(data[i]));
    }

    // instace methods
    public void addElement(Object value){values.addElement(value);}
    public void setElementAt(Object value, int index) { values.setElementAt(value, index); }

    // Methods from the DataColumn interface
    public int getSize() { return values.size(); }
    public DataColumn getSubColumn(Range r) {
      PoissonDataColumn temp = new PoissonDataColumn(name, label, units, date);
      for(int i=r.min; i<=r.max; i++)
        temp.addElement(getValue(i));
        return temp;
      }
    public Object getValue(int index) { return values.elementAt(index); }
    public String getType() { return "poisson"; }

    public boolean isUncertain(){return true;}

    public Object getErrorPlus(int index){
        Double error = new Double(1);
        Number value=(Number) values.elementAt(index);
        if (value.doubleValue()!=0)
          error = new Double(Math.sqrt(Math.abs(value.doubleValue())));
        return error;
    }
    public Object getErrorMinus(int index){
        Double error = new Double(1);
        Number value=(Number) values.elementAt(index);
        if (value.doubleValue()!=0)
          error = new Double(Math.sqrt(Math.abs(value.doubleValue())));
        return error;
    }

    public boolean isEmpty() { return values.isEmpty(); }
    //public void removElement(Object value) { values.removeElement(value); }
    //public void removeElementAt(int index) { values.removeElementAt(index); }
    public void removeAllElements() { values.removeAllElements(); }


    public Object clone() throws CloneNotSupportedException {
      PoissonDataColumn returnValue = (PoissonDataColumn)super.clone();
      // Need to clone the instances: begsin, end, label, name, size, step and units!!
      return returnValue;
    }
}
