package musr.client.datacolumn;
import musr.client.gui.*;
import musr.client.plot2d.*;
import java.util.*;
import java.lang.*;

/**
 * This Class contains sorting algorithm for the Columns of DataTable using a reference column
 *   The way that it works is that the user creates a Sort Class which will create a helperColumn Vector with Value Objects
 *   in it. This Value object has two instances, counter and value.
 *       -The value will take column valuse and used to sort the vector.
 *       -The counter will intially be in order and will get out of order when the helperColumn Vector is sorted in reference to value values
 *    Then the GUI will call sortAll function which will use the order of 'counter' value to reorder (sort) all DataColumns in the
 *                DataTable the reference COlumn came from, create a new DataTable with all the new columns and return it.
 *
 * @author Ivan Tucakov
 */
public class Sort {
              /**
               * This Field is used to create (load) a reference Array so that the columns in the DataTable can be sorted
               * It uses the refColumn as the reference to sort all the other columns
               *
               *   int counter  - is used to keep track of the displacement of refColumn numbers
               *   int vaule - is one value from the refColumn
               */
              public class Value {
                int counter;
                double value;
              }
              /**
               * This Filed is an array of Value Objects. It is built using the provided sortDataTable and refColumn index.
               */
              public Vector helperColumn = new Vector();

              /**
               * This Field is the DataTable that is given and is used to sort Values into the sortedDataTable
               */
              public DataTable sortDataTable;
              /**
               * This Field is the intefer value of the column that is to used as the reference column for sorting all the columns
               */
              //public int refColumn;
              /**
               * This Field is the DataTable that is constructed (by sorting the given sortDataTable) and retuned by the method SortAll()
               */
              public DataTable sortedDataTable;

              /** this is the Constructor
               * Create an array of Value() Objects using the Column in sortDataTable that is refered to by integer refColumn (which is the Column index)
               */
              public Sort(DataTable sortDataTable, int refColumn) {
                this.sortDataTable = sortDataTable;
                try {
                // fill up the refCOlumn Vector with value from the refColumn in sortDataTable. (note that the 'value' fields in Value() are in accending order
                // Note: the DataColumn in sortDataTable will be MultiCOlumn since all the values in it always have error columns too
                DataColumn referenceColumn = ((DataColumn)sortDataTable.getColumn(refColumn));
                if (referenceColumn!=null) {
                  for (int i=1; i<= referenceColumn.getSize(); i++) {
                    Value val = new Value();
                    val.value = (double)Tools.objToDoub(referenceColumn.getValue(i-1));
                    val.counter = i;
                    helperColumn.add(val);
                  }
                }
//for (int i=0; i<helperColumn.size(); i++)
//System.out.println("value: "+((Value)(helperColumn.get(i))).value+"counter: "+((Value)(helperColumn.get(i))).counter);
                // now sort helperColumn (see code below for details about quicksort)
                quickSort(helperColumn);
//for (int i=0; i<helperColumn.size(); i++)
//System.out.println("value: "+((Value)(helperColumn.get(i))).value+"counter: "+((Value)(helperColumn.get(i))).counter);
              } catch (Exception e) { System.out.println("Exception in Sort Class Constructor: "+e); }
              }

              /**
               * This Method sorts all the columns in the DataTable and return a new DataTable that is sorted in reference to the refColumn
               */
              public DataTable sortAll() {
                // Create a mew reference date
                Date date = new Date();
                // initialize the sortedDataTable
                sortedDataTable = new DataTable(sortDataTable.filepath);
                try {
                // Create and sort all the Columns
                for(int i=0; i<sortDataTable.columnCount(); i++) {
//System.out.println("   i: "+i);
                  // Extract the MultiColumn from sortDataTable for easier work
                  DataColumn sortMultiColumn = ((DataColumn)sortDataTable.getColumn(i));
//System.out.println("name: "+sortMultiColumn.getName()+"label: "+sortMultiColumn.getLabel());
                  // Create Value ConcreteDataColumn that is sorted (note that I will load the label and title for this one)
                  ConcreteDataColumn sortedValueDataColumn =
                         new ConcreteDataColumn(sortMultiColumn.getName(), sortMultiColumn.getLabel(), sortMultiColumn.getUnits(), date );
                  // Create Plus Error Value ConcreteDataColumn that is sorted
                  ConcreteDataColumn sortedPlusErrorDataColumn = new ConcreteDataColumn();
                  // Create Minus Error Value ConcreteDataColumn that is sorted
                  ConcreteDataColumn sortedMinusErrorDataColumn = new ConcreteDataColumn();
                  // now put sorted values into these three ConcreteDataColumns
                  for (int j=0; j<sortMultiColumn.getSize(); j++) {
                    int counter = ((Value)helperColumn.get(j)).counter;
//System.out.println("j: "+j + "counter: "+counter);
                    sortedValueDataColumn.addElement(sortMultiColumn.getValue( counter-1 ));
                    sortedPlusErrorDataColumn.addElement(sortMultiColumn.getErrorPlus( counter-1 ));
                    sortedMinusErrorDataColumn.addElement(sortMultiColumn.getErrorMinus( counter-1 ));
                  }
                  // Now add this sorted column into the sortedDataTable
                  sortedDataTable.addColumn(new MultiColumn(sortedValueDataColumn, sortedPlusErrorDataColumn, sortedMinusErrorDataColumn));
                }
                // Add fields from the original DataTable
                sortedDataTable.setAbstract(sortDataTable.getAbstract());
                sortedDataTable.setComment(sortDataTable.getComment());
                sortedDataTable.setTitle(sortDataTable.getTitle());
              } catch (Exception e) { System.out.println("Exception in Sort Class sortAll() Method: "+e); 
              e.printStackTrace();}
              return sortedDataTable;
              }


// the code below is quicksort algorithm:

/** Sort the entire vector, if it is not empty
 */
  public void quickSort(Vector elements)
  { if (! elements.isEmpty())
    { quickSort(elements, 0, elements.size()-1);
    }
  }

/**
 * QuickSort.java by Henk Jan Nootenboom, 9 Sep 2002
 * Copyright 2002-2003 SUMit. All Rights Reserved.
 *
 * Algorithm designed by prof C. A. R. Hoare, 1962
 * See http://www.sum-it.nl/en200236.html
 * for algorithm improvement by Henk Jan Nootenboom, 2002.
 *
 * Recursive Quicksort, sorts (part of) a Vector by
 *  1.  Choose a pivot, an element used for comparison
 *  2.  dividing into two parts:
 *      - less than-equal pivot
 *      - and greater than-equal to pivot.
 *      A element that is equal to the pivot may end up in any part.
 *      See www.sum-it.nl/en200236.html for the theory behind this.
 *  3. Sort the parts recursively until there is only one element left.
 *
 * www.sum-it.nl/QuickSort.java this source code
 * www.sum-it.nl/quicksort.php3 demo of this quicksort in a java applet
 *
 * Permission to use, copy, modify, and distribute this java source code
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is hereby granted.
 * See http://www.sum-it.nl/security/index.html for copyright laws.
 */
  private void quickSort(Vector elements, int lowIndex, int highIndex) {
    try {
    int lowToHighIndex;
    int highToLowIndex;
    int pivotIndex;
    double pivotValue;
    // These values will represent the 'value'  vlue of the 'Value' Class defined above (that is stored in 'elements' Vector)
    double lowToHighValue;
    double highToLowValue;
    double parking;
    // These two are the values that will hold the 'counter' value of the 'Value' Class defined above (that is stored in 'elements' Vector)
    int lowToHighCounter;
    int highToLowCounter;
    int parkingCounter;

    int newLowIndex;
    int newHighIndex;
    int compareResult;

    lowToHighIndex = lowIndex;
    highToLowIndex = highIndex;
    /** Choose a pivot, remember it's value
     *  No special action for the pivot element itself.
     *  It will be treated just like any other element.
     */
    pivotIndex = (lowToHighIndex + highToLowIndex) / 2;
    pivotValue = ((Value)elements.elementAt(pivotIndex)).value;

    /** Split the Vector in two parts.
     *
     *  The lower part will be lowIndex - newHighIndex,
     *  containing elements <= pivot Value
     *
     *  The higher part will be newLowIndex - highIndex,
     *  containting elements >= pivot Value
     */
    newLowIndex = highIndex + 1;
    newHighIndex = lowIndex - 1;
    // loop until low meets high
    while ((newHighIndex + 1) < newLowIndex) // loop until partition complete
    { // loop from low to high to find a candidate for swapping
      lowToHighValue = ((Value)elements.elementAt(lowToHighIndex)).value;
      lowToHighCounter = ((Value)elements.elementAt(lowToHighIndex)).counter;
      while (lowToHighIndex < newLowIndex
        & lowToHighValue<pivotValue )
      { newHighIndex = lowToHighIndex; // add element to lower part
        lowToHighIndex ++;
        lowToHighValue = ((Value)elements.elementAt(lowToHighIndex)).value;
        lowToHighCounter = ((Value)elements.elementAt(lowToHighIndex)).counter;
      }

      // loop from high to low find other candidate for swapping
      highToLowValue = ((Value)elements.elementAt(highToLowIndex)).value;
      highToLowCounter = ((Value)elements.elementAt(highToLowIndex)).counter;
      while (newHighIndex <= highToLowIndex
        & (highToLowValue>pivotValue)
        )
      { newLowIndex = highToLowIndex; // add element to higher part
        highToLowIndex --;
        highToLowValue = ((Value)elements.elementAt(highToLowIndex)).value;
        highToLowCounter = ((Value)elements.elementAt(highToLowIndex)).counter;
      }

      // swap if needed
      if (lowToHighIndex == highToLowIndex) // one last element, may go in either part
      { newHighIndex = lowToHighIndex; // move element arbitrary to lower part
      }
      else if (lowToHighIndex < highToLowIndex) // not last element yet
      {
        if (lowToHighValue>=highToLowValue) // low >= high, swap, even if equal
        { parking = lowToHighValue;
          parkingCounter = lowToHighCounter;
          ((Value)elements.get(lowToHighIndex)).value = highToLowValue;
          ((Value)elements.get(lowToHighIndex)).counter = highToLowCounter;
          ((Value)elements.get(highToLowIndex)).value = parking;
          ((Value)elements.get(highToLowIndex)).counter = parkingCounter;

          //elements.setElementAt(highToLowValue, lowToHighIndex);
          //elements.setElementAt(parking, highToLowIndex);

          newLowIndex = highToLowIndex;
          newHighIndex = lowToHighIndex;

          lowToHighIndex ++;
          highToLowIndex --;
        }
      }
    }

    // Continue recursion for parts that have more than one element
    if (lowIndex < newHighIndex)
    { this.quickSort(elements, lowIndex, newHighIndex); // sort lower subpart
    }
    if (newLowIndex < highIndex)
    { this.quickSort(elements, newLowIndex, highIndex); // sort higher subpart
    }
  } catch (Exception e) { System.out.println("Exception in quicksort method of Sort Class: "+e); }
  }

}