package musr.client.dbreader;
import musr.client.datacolumn.*;
import musr.client.function.*; //Ari's Function class is used in evaluateExpr(String)
import musr.client.gui.*;
import musr.client.plot2d.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.net.*;
import java.util.*;
import java.security.*;
import java.util.Date;

/**
 * This class opens a db file or url and reads in the variables from it
 * All the variables are read into a DataTable.
 * Note: -The DATA in the db file that is loaded should end with two line breaks at the end of the file
 *
 * I am using Ari's Function package to evaluate expressions that are parsed in the db file.
 *
 * Remarks: -it is possible to have either column or constant parameters in the expressions for MOD and GEN
 *           commands. But if there is any reference where the parameter name exists both in constants and
 *           it is a column name, I will take the column name, load the appropriate row value in it and
 *           ignore the constant value from the table.
 *          -The constants in the expressions are being 'handled' and 'exchanged' with their values within this
 *           class, whereas, the functions in the expressions are 'handled' in the Function class (method evaluate()
 *           is bing passed the FunctionsTable..)
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class DBFileReader {

                /** This class contains and loads all the DatatTable variables and any other constants */
                private Constants dataTableVariables = new Constants();
                /** This class contains and loads 'custom' defined functions */
                private Functions functionsTable = new Functions();

                public JFrame parent;  // This is the GUI where the console is called from

                /**
                 * Date is used to label the columns for future reference
                 */
                private Date date = new Date();

                // This Dialog will contain one OK button and will be used for Displaying any kind of information,
                // error warnings or anything that would only have one OK button.
                Dialog infoDialog;
                Button OKButton = new Button("OK");
                // Event Listeners used
                private class ActionListenerS implements java.awt.event.ActionListener, java.io.Serializable {
                  public void actionPerformed(ActionEvent e) {}
                  };
                ActionListenerS buttonListener;

                public DBFileReader(JFrame paren, Constants dataTableVariables, Functions functionsTable) {
                  this.dataTableVariables = dataTableVariables;
                  this.functionsTable = functionsTable;
                  this.parent = paren;

                  infoDialog = new Dialog(parent,"", true);
                }



        /** Sets the Constant variables table*/
        public void setConstantsTable(Constants dataTableVariables) {this.dataTableVariables = dataTableVariables; }

        /** Sets the Functions table*/
        public void setFunctionsTable(Functions functionsTable) {this.functionsTable = functionsTable; }

        /** Set the case-INsensitivity for constant and column names ON or OFF*/
        public void setCaseSensitive(boolean set) {((GUI)parent).caseMenuItem.setState(set); }

      /**
         * Opens and load the given URL
         */
      public void loadURL(String URLname) throws IOException {

        URLLineReader inURL = null;
        try {
          URL url = new URL("http", "musr.triumf.ca/~itucakov/", URLname);                    // Create the URL
          inURL = new URLLineReader(url.openStream());   // Create stream to read
          loadStream(inURL, URLname);                             // load the data from the stream/url
        }
        catch (IOException e) {System.err.println("I/O Exception DBFileReader URL: "+ e.getMessage()); }
        finally {			// Always be sure to close the input stream!
          try { if (inURL != null) inURL.close(); }
          catch (IOException e) { System.err.println(e.getMessage()); }
        }
      }

      /**
      * Read the given stream and load the data into the class variables
      * I recommend looking up the file 'temp.db' (as an example) while going through this code
      **/
      public void loadStream(LineReader in, String filepath) {
          String line;
          String delimiters = "=,\\ ";
          StringTokenizer tokenizer;

          try {
                // Handle events on the Buttons
                // Add/Removing Columns and their information in/from x-axis/y-axis Lists
                buttonListener = new ActionListenerS() {
                  public void actionPerformed(ActionEvent e) {
                    String command = ((Button)e.getSource()).getLabel();
                    if (command.equals("OK")) infoDialog.hide();
                  }
                };
                OKButton.addActionListener(buttonListener);


            // First, add a new datatable into the GUI
            // Create the DataTable that will be added into the GUI
            //   Note! The DataTable will be added into the GUI right after it has obtained the Title, Abstract, Comment and the Data
            //            because it needs all these fields to build the fields in the DataTable and Column Lists in the GUI when the
            //            method addDataTable is called (look for it below after the Data ). After that, GEN, MOD and such are called
            //---------------------------
            ((GUI)parent).tempDataColumns = new DataTable(filepath);
            //---------------------------

            // Load the data from the file
            while ((line = in.readLine()) != null) {
              String newLine =""; // this variable is used ONLY as a helper variable in the code right below
              // The line already got a value when called in the 'while' loop
              if ( line.trim()!="") { //check if the line has chars
//System.out.println("line:"+line);
               if (line.endsWith("\\")) { // look at the last character in the string
                newLine = line.substring(0, line.length()-1);
//System.out.println("newline:"+newLine);
                // Load the rest of the columns in this 'block' of data that end with backslash character
                while (String.valueOf((line = in.readLine()).charAt(line.length()-1)).equals("\\"))
                  newLine = newLine.concat(line.substring(0, line.length()-1));
                // Finally, load the last line that doesn't have backslash at the end.
                // Note: the line has already been read in the while loop above
                newLine = newLine.concat(line);
                line = newLine; // assign this value to line since it will be used throughout the code below
               } else { // this means that the all the data and the comment are in the same line, so just leave it as it is
               }
              }

              StringTokenizer commandTokenizer = new StringTokenizer(line.trim(), " ", false);
              int numTokens = commandTokenizer.countTokens();
              String command = "";
              if (commandTokenizer.hasMoreTokens()) {command = commandTokenizer.nextToken().trim();}
//System.out.println("command: "+command);
              // if there is a ! at the begininning of the line, skip the line
              if (!command.equals("!") || !command.substring(0,1).equals("!")) { // note: there has to be space between ! and the next token (in the db file)

                if (command!="" && command.substring(0,1).equals("@")) { // first one is to make sure I can take substring()
                  String fileName = command.substring(1); // get rid of "@"
                  // Check to see if the path is Relative (doesn't start with "/") or Absolute (contains full directory path)
                  // if Absolute: leave fileName as is (note: in dbConsole this directory path is saved into dirPath (in GUI). NOT HERE though!)\
                  if (!fileName.startsWith("/")) fileName = ((GUI)parent).dirPath+fileName;
                  // Display a message in the console jta (Text Area)
                  ((GUI)parent).console.jta.append("Loading the file: "+fileName+" \n");
                  // because at this point it is uncertain whether we are using GUI dirPath or a new directory path in fileName, break up fileName
                  ((GUI)parent).loadDBFile(fileName.substring(0, fileName.lastIndexOf("/")+1),
                                                        fileName.substring(fileName.lastIndexOf("/")+1, fileName.length()));
                  // Since DATA has been most probaly loaded, the variable tempLabels has to be updated
                  ((GUI)parent).tempLabels.clear();
                  int dataTableKey = ((GUI)parent).getDataTableIndex();
                  //if DATA has been loaded
                  if ( ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount()!=0)
                    for (int i=0;i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount();i++) {
                      ((GUI)parent).tempLabels.add(i,
                        ((MultiColumn)((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i)).getLabel());
                    }
                }
                else if ((command.toUpperCase().equals("DEFINE")) || (command.toUpperCase().equals("DEFIN")) ||
                         (command.toUpperCase().equals("DEFI")) || (command.toUpperCase().equals("DEF"))) {
                  // get the name for the constant
                  String constName = commandTokenizer.nextToken().trim();
                  // skip the "=" sign
                  String equalSign = commandTokenizer.nextToken().trim();
                  // get the expression
                  String constExpr = commandTokenizer.nextToken().trim();
                  //insert the constant into the Constants Table
                  ((GUI)parent).dataTableVariables.setConstant(constName,constExpr,new String(""));
                }
              /*  else if ((command.toUpperCase().equals("ZERO")) || (command.toUpperCase().equals("ZER")) ||
                         (command.toUpperCase().equals("ZE")) || (command.toUpperCase().equals("Z"))) {
                  String zStr = in.readLine().trim();
                  // Only worry if 'yes' is entered, if not then leave everything as it is.
                  if (zStr.toUpperCase().equals("Y") || zStr.toUpperCase().equals("YE") || zStr.toUpperCase().equals("YES")) {
                   ((GUI)parent).console.jta.append("Zeroing the (default) Data Table \n");
                   ((GUI)parent) .console.zeroDataTable();
                  }
                }*/
                else if ((command.toUpperCase().equals("PLOT")) || (command.toUpperCase().equals("PLO")) ||
                         (command.toUpperCase().equals("PL")) || (command.toUpperCase().equals("P"))) {
                  if(numTokens < 3) { // PLOT has been entered without the right number of arguments
                    System.out.println("Error, the PLOT command doesn't have enough arguments ");
                    // Display a message in the console jta (Text Area)
                    ((GUI)parent).console.jta.append("Error, the PLOT command doesn't have enough arguments "+" \n");
                  } else { // 'PLOT x y' has been entered OR 'PLOT x y IF ..' has been entered
                  int dataTableKey = ((GUI)parent).getDataTableIndex();
                    // check if <x> and <y> exist, and which columns they are
                    boolean checkX = false; int columnX = -1;
                    boolean checkY = false; int columnY = -1;
                    for (int i=0; i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount(); i++) {
                      // get X and Y arguments
                      String X = commandTokenizer.nextToken().trim();
                      String Y = commandTokenizer.nextToken().trim();
                      String tempX = ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName();
                      String tempY = ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName();
                      // Set it to upper case if required
                      if (!((GUI)parent).caseMenuItem.getState()) {
                        X = X.toUpperCase();
                        Y = Y.toUpperCase();
                        tempX = tempX.toUpperCase();
                        tempY = tempY.toUpperCase();
                      }
                      if( tempX.equals(X) ) {
                        checkX = true; columnX = i; }
                      if( tempY.equals(Y) ) {
                        checkY = true; columnY = i; }
                    }
                    // If OK, put the two columns into the GUI Columns List
                    if (checkX && checkY) { // if they are both true, that means that columnX and columnY values have also changed
                      //** This is the REGULAR PLOT command *//
                      if(numTokens == 3) {
                        // Display a message in the console jta (Text Area)
                        ((GUI)parent).console.jta.append(line+" \n");
                        // Load the X Column
                        ((GUI)parent).addX(columnX, ((GUI)parent).layoutChoice.getItemCount()-1);
                        // Load the Y Column
                        ((GUI)parent).layoutChoice.select(((GUI)parent).layoutChoice.getItemCount()-1); // this fixes the problem of it not being selected. It doesent matter anyways because there is no more than one item in it anyways
                        ((GUI)parent).xAxisList.select(((GUI)parent).xAxisList.getItemCount()-1); // this fixes the problem of it not being selected. It doesent matter anyways because there is no more than one item in it anyways
                        ((GUI)parent).addY(columnY, ((GUI)parent).xAxisList.getItemCount()-1,
                                                    ((GUI)parent).layoutChoice.getItemCount()-1);

                        //* read the next line in order to get the number that represents the plot type *
                        String type = in.readLine().trim();
                            // Since the method 'plotPointType()' displays the jtf context in the jta Text Area (of the console),
                            //       I'll add the 'plot type' into it.
                            ((GUI)parent).console.jta.append(type+"\n");
                      // The command is a number which represents one of the point types
                        if (Character.isDigit(type.charAt(type.length()-1)))
                          ((GUI)parent).console.plotPointType(type);
                        else { //it is not a number it is some mistake
                          System.out.println("Spelling Error in the PLOT command: point type has to be an integer ");
                          // Display a message in the console jta (Text Area)
                          ((GUI)parent).console.jta.append("Spelling Error in the PLOT command: point type has to be an integer "+" \n");
                      }
                      //** This is the CONDITIONAL PLOT command (..IF..)*//
                      } else if(numTokens > 3) {
                        // Display a message in the console jta (Text Area)
                        ((GUI)parent).console.jta.append(line+" \n");
                        if (commandTokenizer.nextToken().trim().toUpperCase().equals("IF")){ // check if 'IF' was spelled correctly

                          if ( ((GUI)parent).findCondColumn(line, columnX, columnY) ) {
                            //* read the next line in order to get the number that represents the plot type *
                            String type = in.readLine().trim();
                                // Since the method 'plotPointType()' displays the jtf context in the jta Text Area (of the console),
                                //       I'll add the 'plot type' into it
                                ((GUI)parent).console.jta.append(type+"\n");
                            // The command is a number which represents one of the point types
                            if (type!=null && type.length()!=0 && Character.isDigit(type.charAt(0)))
                              ((GUI)parent).console.plotPointType(type);
                            //it is not a number it is some mistake
                            //else System.out.println(" ");

                          } else {
                            System.out.println("Warning: no columns were added since the conditions capture no data points.");
                            // Display a message in the console jta (Text Area)
                            ((GUI)parent).console.jta.append("Warning: no columns were added since the conditions capture no data points."+" \n");
                          }
                        } else { // 'IF' not spelled correctly
                          System.out.println("Spelling Error (if) in the PLOT command ");
                          // Display a message in the console jta (Text Area)
                          ((GUI)parent).console.jta.append("Spelling Error (if) in the PLOT command "+" \n");
                        }
                      } // no 'else' here since it has already been checked whether there is enough arguments
                    } // there is no 'else' here for unrecognized column names. The two 'if' commands below take care of that.
                    if (!checkX) {
                      System.out.println("Unrecognized X argument in the PLOT command (Check for case sensitivity)");
                      // Display a message in the console jta (Text Area)
                      ((GUI)parent).console.jta.append("Unrecognized X argument in the PLOT command (Check for case sensitivity)"+" \n");
                    }
                    if (!checkY) {
                      System.out.println("Unrecognized Y argument in the PLOT command (Check for case sensitivity)");
                      // Display a message in the console jta (Text Area)
                      ((GUI)parent).console.jta.append("Unrecognized Y argument in the PLOT command (Check for case sensitivity)"+" \n");
                    }
                  }
                } // // end: 'if "Plot"'

                else if ((command.toUpperCase().equals("TITLE")) || (command.toUpperCase().equals("TITL")) ||
                    (command.toUpperCase().equals("TIT")) || (command.toUpperCase().equals("TI")) ||
                    (command.toUpperCase().equals("T")) ) {
                  line = in.readLine(); // get the Title in the new line
                  ((GUI)parent).tempDataColumns.setTitle(line);   // Put the title into (dataTable) !!
                }
                else if ((command.toUpperCase().equals("ABSTRACT")) || (command.toUpperCase().equals("ABSTRAC")) ||
                         (command.toUpperCase().equals("ABSTRA")) || (command.toUpperCase().equals("ABSTR")) ||
                         (command.toUpperCase().equals("ABST")) || (command.toUpperCase().equals("ABS")) ||
                         (command.toUpperCase().equals("AB")) || (command.toUpperCase().equals("A"))) {
                  String tempAbstract = new String("");
                  while((line = in.readLine().trim()).length() != 0)
                    tempAbstract += new String(line.trim()+"\n"); // Read in the abstract
                  ((GUI)parent).tempDataColumns.setAbstract(tempAbstract); // Put the abstract into (dataTable) !!
                }
                else if ((command.toUpperCase().equals("COMMENT")) || (command.toUpperCase().equals("COMMEN")) ||
                         (command.toUpperCase().equals("COMME")) || (command.toUpperCase().equals("COMM")) ||
                         (command.toUpperCase().equals("COM")) || (command.toUpperCase().equals("CO")) ||
                         (command.toUpperCase().equals("C"))) {
                  ((GUI)parent).tempRowDescription.clear(); // 'initialize' it
                  while((line = in.readLine().trim()).length() != 0)
                    ((GUI)parent).tempRowDescription.add(line); // Read in the comments
                  String tempComm = "";
                  for(int i=1; i<=((GUI)parent).tempRowDescription.size(); i++)
                    tempComm = tempComm.concat(i+") "+((GUI)parent).tempRowDescription.get(i-1)+"\n"); // Put all the comments into (dataTable) !!
                  ((GUI)parent).tempDataColumns.setComment(tempComm);
                }
                else if ((command.toUpperCase().equals("LABELS")) || (command.toUpperCase().equals("LABEL")) ||
                         (command.toUpperCase().equals("LABE")) || (command.toUpperCase().equals("LAB")) ||
                         (command.toUpperCase().equals("LA")) || (command.toUpperCase().equals("L"))) {
                  // there is only one label to be changed
                  if (commandTokenizer.hasMoreTokens()) {
                    // find which label is supposed to be changed, check for case sensitivity first
                    int labIndex =  ((GUI)parent).indexOfDataColumnNames(parent, commandTokenizer.nextToken().trim());
//System.out.println("lab next token index: "+labIndex);
                    line = in.readLine(); // get the label in the new line
                    ((GUI)parent).tempLabels.setElementAt(line.trim(), labIndex); // Change the appropriate Label
                    ((DataColumn)((GUI)parent).tempDataColumns.getColumn(labIndex)).setLabel(line.trim());
                  // there is an array of new labels
                  } else {
                    int ind=0;
                    ((GUI)parent).tempLabels.removeAllElements(); // Remove all the old Labels
                    while((line = in.readLine().trim()).length() != 0) {
                      if(((GUI)parent).tempDataColumns.columnCount()==0) // If the DATA hasn't been called yet, just load the label array
                        ((GUI)parent).tempLabels.addElement(line.trim());    // Read in the lables
                      else { // DATA has been called so put the labels into the DataColumns
                        ((GUI)parent).tempLabels.setElementAt(line.trim(), ind);
                        ((DataColumn)((GUI)parent).tempDataColumns.getColumn(ind)).setLabel(line.trim());
                      }
                      ind++;
                    }
                    // Error display: If the DATA has already been called and the number of LABELS is different than
                    // the number of DATA Columns, inform the user that the .db file is inconsistent.
                    // (this is when a whole new array of labels has been added)
                    if (((GUI)parent).tempDataColumns.columnCount()!=0 && ((GUI)parent).tempDataColumns.columnCount()!=((GUI)parent).tempLabels.size()) {
                      infoDialog.removeAll();
                      infoDialog.setLayout(new GridLayout(4,1));
                      infoDialog.add(new Label("Error message: .db file inconsistency: "));
                      infoDialog.add(new Label(" Number of Labels and Number of Data Columns is different. "));
                      infoDialog.add(new Label("Your .db file might not load properly! "));
                      infoDialog.add(OKButton);
                      infoDialog.setSize(new Dimension(450, 150));
                      infoDialog.setLocation(200,200);
                      infoDialog.show();
                    }
                  }
                }
                else if ((command.toUpperCase().equals("DATA")) || (command.toUpperCase().equals("DAT")) ||
                         (command.toUpperCase().equals("DA")) || (command.toUpperCase().equals("D"))) {
                  // Read in the names of variable arrays
                  setDataVariableArray(line, delimiters);

                  //					if ((line.trim()).charAt(0) == '\\')
//                  line = in.readLine();   // skip line with '\-e'
//! still have to fix the above (see bogus.db)

                  // if the DataTable doesn't have columns, create new ones (otherwise, the 'temp' have been already created)
                  // Note to the programmer: this will work if in LABELS< the columnCount is unaltered..
                  //if ( columnCount<=0 ) {
                    ((GUI)parent).tempDataColumns.removeAllColumns();
                    int columnCount = ((GUI)parent).tempNames.size();

                    for(int i=0; i<columnCount; i++) {
                      // Create all the columns and add set their column names and labels
                      if(((GUI)parent).tempLabels.isEmpty()) // If LABELS command hasn't been called jet
                        // Note! I am setting the label as " ", not as "". THis is important since the Plot2D
                        // has problems when trying to draw the labels that are Strings of the form ""
                        ((GUI)parent).tempDataColumns.addColumn( new MultiColumn(
                                                                              new ConcreteDataColumn((String) ((GUI)parent).tempNames.elementAt(i)," ", " ", date),
                                                                              new ConcreteDataColumn(),
                                                                              new ConcreteDataColumn()) );
                      else // LABELS command has been called so load the labels in as well
                        ((GUI)parent).tempDataColumns.addColumn( new MultiColumn(
                                                                              new ConcreteDataColumn((String) ((GUI)parent).tempNames.elementAt(i),(String) ((GUI)parent).tempLabels.elementAt(i), " ", date),
                                                                              new ConcreteDataColumn(),
                                                                              new ConcreteDataColumn()) );
                    }
                  //}

                  // loop through all rows and load them up
                  while((line = in.readLine().trim()).length()!=0) { // Loop until empty line
                    try {
                      String fullLine = new String("");
                      // if the line starts with '!' and if there is a line with "!" at the beginning, skip it
                      if ((!String.valueOf(line.charAt(0)).equals("!")) && (!String.valueOf(line.charAt(0)).equals("\\"))) {
                        // Load the first line since it already got a value when called in the 'while' loop
                        if (String.valueOf(line.charAt(line.length()-1)).equals("\\")) { // look at the last character in the string
                          fullLine = fullLine.concat(line.substring(0, line.length()-1));
                          // Load the rest of the columns in this 'block' of data that end with backslash character
                          while (String.valueOf((line = in.readLine()).charAt(line.length()-1)).equals("\\"))
                            fullLine = fullLine.concat(line.substring(0, line.length()-1));
                          // Finally, load the last line that doesn't have backslash at the end.
                          // Note: the line has already been read in the while loop above
                          fullLine = fullLine.concat(line);
                        } else { // this means that the all the data and the comment are in the same line, so just read it in
                          fullLine = fullLine.concat(line);
                        }

                        // set all the column values for this row
                        ((GUI)parent).setDataVariables(fullLine);

                      }

                    }
                    catch (NoSuchElementException e) {
                      System.out.println("Error in reading data: DBFileReader.java");
                      System.err.println(e.getMessage());
                    }
                  }
                  //---------------------
                  // Now add this DataTable into the GUI
                  ((GUI)parent).addDataTable(((GUI)parent).tempDataColumns, false, "");
                  // Reading the DataTable (otherwise it is not associated with it anymore)
                  ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement( ((GUI)parent).dataTableList.getItemCount()-1 );
                  //---------------------

                } // end of DATA input 'block'

  // NOTE! From now on update tempDataColumns,

                else if ((command.toUpperCase().equals("MODIFY")) || (command.toUpperCase().equals("MODIF")) ||
                         (command.toUpperCase().equals("MODI")) || (command.toUpperCase().equals("MOD")) ||
                         (command.toUpperCase().equals("MO")) || (command.toUpperCase().equals("M"))) {
                  // Reading the DataTable (otherwise it is not associated with it anymore)

                  ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement( ((GUI)parent).dataTableList.getItemCount()-1 );
                  // find the name that references this column
                  String modName = commandTokenizer.nextToken().trim();
//System.out.println("modify this column: "+modName);
                    line = in.readLine(); // get the expression in the new line

                    ((GUI)parent).modifyColumn(line.trim(), modName);
                    // Update this dataTable's history (for the XML output later..)
                    ((GUI)parent).tempDataColumns.tableHistory.append(new String(" MOD "+modName+" "+line.trim())+"\n");
                }
                else if ((command.toUpperCase().equals("GENERATE")) || (command.toUpperCase().equals("GENERAT")) ||
                         (command.toUpperCase().equals("GENERA")) || (command.toUpperCase().equals("GENER")) ||
                         (command.toUpperCase().equals("GENE")) || (command.toUpperCase().equals("GEN")) ||
                         (command.toUpperCase().equals("GE")) || (command.toUpperCase().equals("G"))) {

                  // find the name that references this column
                  String genName = commandTokenizer.nextToken().trim();
                  // If this name already exists as a column name, don't allow the user to generate it
                  if (((GUI)parent).tempNames.indexOf(genName)!=-1 ) {// this means that there is a same name that was already loaded
                    ((GUI)parent).popAlertDialog("Error:", "Generating a column with a name that already exists: "+genName, "column will not be generated.");
                  } else {
                    // Reading the DataTable (otherwise it is not associated with it anymore)
                    ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement( ((GUI)parent).dataTableList.getItemCount()-1 );
                    // add the new name into the array, check for case sensitivity first
                    ((GUI)parent).tempNames.addElement(genName);
                    // add an new dummy label which should be changed in the db file right after
                    ((GUI)parent).tempLabels.addElement(genName);
                    // refresh the number of columns
                    // create the new column
                    ((GUI)parent).tempDataColumns.addColumn(new MultiColumn( new ConcreteDataColumn(genName, genName, " ", date),
                                                                                      new ConcreteDataColumn(),
                                                                                      new ConcreteDataColumn() ) );

                    // create dummy rows in the columns and then 'modify' them using the modifyColumn() method
                    // if this is the first column to be added, the next line will have to contain the number of rows for it
                    long rowCount = 0;
                    int columnCount = ((DataTable)((GUI)parent).dictionary.getElement(((GUI)parent).dataTableList.getSelectedIndex())).columnCount();
                    if (columnCount==1) rowCount= Long.parseLong(in.readLine().trim());
                    else rowCount = ((DataColumn) ((GUI)parent).tempDataColumns.getColumn(0)).getSize();
                    for (int i=0; i<rowCount; i++) {
                      ((ConcreteDataColumn)((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getValueColumn()).addElement("0");// just add a dummy zero string
                      ((ConcreteDataColumn)((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getErrorPlusColumn()).addElement("0");
                      ((ConcreteDataColumn)((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getErrorMinusColumn()).addElement("0");
                    }
                    // get the expression in the new line
                    line = in.readLine();
                    // load the values into the columns
                    ((GUI)parent).modifyColumn(line.trim(), genName);
                    // Last step: show dataTable's column contents in the ColumnList
                    ((GUI)parent).reloadColumnList();
                    // Update this dataTable's history (for the XML output later..)
                    ((GUI)parent).tempDataColumns.tableHistory.append(new String(" GEN "+genName+" "+line.trim())+"\n");
                  }
                }
                else if ((command.toUpperCase().equals("DELETE")) || (command.toUpperCase().equals("DELET")) ||
                         (command.toUpperCase().equals("DELE")) || (command.toUpperCase().equals("DEL")) ||
                         (command.toUpperCase().equals("DE")) || (command.toUpperCase().equals("D"))) {
                  // Reading the DataTable (otherwise it is not associated with it anymore)
                  ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement( ((GUI)parent).dataTableList.getItemCount()-1 );
                  // find the name that references this column
                  String delName = commandTokenizer.nextToken().trim();
                  // find it's index, check for case sensitivity
                  int index =  ((GUI)parent).indexOfDataColumnNames(parent, delName);
                    // remove the name from the array
                    ((GUI)parent).tempNames.removeElementAt(index);
                    // remove the label
                    ((GUI)parent).tempLabels.removeElementAt(index);
                    // delete the column
                    ((GUI)parent).tempDataColumns.removeColumnAt(index);
                }
                // this command is ignored since the GUI provides the same features
                // I am still 'looking it up' since there can be calls to other commands in the line after
                // this call that could 'confuse' the DBFileReader.
                else if ((command.toUpperCase().equals("SHOW")) || (command.toUpperCase().equals("SHO")) ||
                         (command.toUpperCase().equals("SH")) || (command.toUpperCase().equals("S"))) {
                  // Reading the DataTable (otherwise it is not associated with it anymore)
                  ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement( ((GUI)parent).dataTableList.getItemCount()-1 );
                  // If there is more tokens, then it is "safe"
                  // Note: it is relatively safe since the new line could still have words with first characters
                  //   starting the same way some of the commands start. I will not worry about that now though.
                  if (commandTokenizer.hasMoreTokens()) {}
                  // skip the next line since that one will definitely contain one of the command calls
                  else in.readLine();
                }
              } // end: if ("!")
            } // end: while loop

            // Once all the loading is done, reset the displays in the GUI DataTables Frame
            if (((GUI)parent).dataTableList.getItemCount()>0) {
              ((GUI)parent).dataTableList.select(((GUI)parent).dataTableList.getItemCount()-1); // Select the last item (which would be the datatable just loaded)
              ((GUI)parent).reloadColumnList();
            }

          } // end try

          // Display messages if something goes wrong
          catch (Exception e) {
            System.err.println("Error in DBFileReader.LoadStream(): " + e.getMessage());
          }
      }

        /** This method initializes and clears the Abstract String */
   //     private void initAbstract() { tempAbstract = new String(""); }
        /** This is a method for the "command-line" ABSTRACT */
   //     private void setAbstract(String abstr) { tempAbstract += new String(abstr+"\n"); }


        /** This is a method for the "command-line" DATA, it loads the data variable array (column names) */
        private void setDataVariableArray(String line, String delimiters) {
                // Read in the names of variable arrays
                StringTokenizer tokenizer;
                tokenizer = new StringTokenizer(line, delimiters, false);
                line = tokenizer.nextToken();  // just pop "DATA"
                int i=0; // this is to count the number of columns
                ((GUI)parent).tempNames.removeAllElements();
                while (tokenizer.hasMoreTokens())  { // Read name
                  String newName = tokenizer.nextToken().trim();
                  // If there is a Column with a same name, alert the user
                  if (((GUI)parent).tempNames.indexOf(newName)!=-1 ) // this means that there is a same name that was already loaded
                  ((GUI)parent).popAlertDialog("Warning", "You have to columns that are named the same: "+newName, "this could cause ambiguities");
                  // check for Case sensitivity before adding the Name into the Vector
                  ((GUI)parent).tempNames.addElement(newName);
                  i++;
                }
                int columnCount=i; // define the number of coulums

                    // Error display: If the LABEL has already been called and the number of LABELS is different than
                    // the number of DATA Columns, inform the user that the .db file is inconsistent.
                    // NOTE: in case LABEL hasn't been called, the column size consistency will be again checked
                    // when the LABEL is called.
                    if (((GUI)parent).tempLabels.size()!=0 && columnCount!=((GUI)parent).tempLabels.size()) {
                      infoDialog.removeAll();
                      infoDialog.setLayout(new GridLayout(4,1));
                      infoDialog.add(new Label("Error message: .db file inconsistency: "));
                      infoDialog.add(new Label(" Number of Labels and Number of Data Columns is different. "));
                      infoDialog.add(new Label("Your .db file might not load properly! "));
                      infoDialog.add(OKButton);
                      infoDialog.setSize(new Dimension(450, 150));
                      infoDialog.setLocation(200,200);
                      infoDialog.show();
                    }

//System.out.println("Data columns (column COunt0:"+columnCount+" Label Columns:"+tempLabels.size());
        }

        /** This method initializes and clears the Comment String */
//        private void initComment() { tempComment = new String(""); }
        /** This is a method for the "command-line" COMMENT */
//        private void setComment(String comm) { tempComment += new String(comm+"\n"); }



}
