package musr.client.fourier;

import javax.swing.*;
import musr.client.datacolumn.*;
import java.util.Vector;
import musr.client.gui.*;

public class RealFourierTransform
{    
    public FastFourierTransform fft;
    
    //Main class for testing purposes only
    
    /*public static void main(String[] args)
    {
        //Create fake data set
        double[] input = new double[25];
        for(int i=1; i<input.length; i++)
            input[i] = Math.cos(20*Math.PI/i);
        RealFourierTransform test = new RealFourierTransform(input);
        Complex[] result = test.realTransform();
        for(int i=0; i<result.length; i++)
            System.out.println(result[i]);
    }*/
    
    //Takes an array of values as an argument representing the real data to be FFT'd
    public RealFourierTransform(double[] input)
    {
        int dataLength = input.length;
        double[][] repacked;
        if(dataLength%2==1)
        {
            repacked = new double[dataLength/2+1][2];
            for(int i=0; i<repacked.length-1; i++)
            {
                repacked[i][0] = input[2*i];
                repacked[i][1] = input[2*i+1];
            }
            repacked[repacked.length-1][0] = input[dataLength-1];
            repacked[repacked.length-1][1] = 0;
        }
        else    //(input.length%2===0) hopefully
        {
            repacked = new double[dataLength/2][2];
            for(int i=0; i<repacked.length; i++)
            {
                repacked[i][0] = input[2*i];
                repacked[i][1] = input[2*i+1];
            }
        }
        /*for(int i=0; i<repacked.length; i++)
        {
            System.out.println("repacked[" + i + "][0] = " + repacked[i][0]);
            System.out.println("repacked[" + i + "][0] = " + repacked[i][1]);
        }*/
        fft = new FastFourierTransform(repacked);
    }
    
    //Same as previous constructor but 
    public RealFourierTransform(DataColumn input)
    {
        System.out.println("RealFourierTransform constructor called");
        double[][] repacked;
        if(input.getSize()%2==1)
        {
            repacked = new double[input.getSize()/2+1][2];
            
            for(int i=0; i<repacked.length-1; i++)
            {
                repacked[i][0] = Double.parseDouble(input.getValue(2*i).toString());
                repacked[i][1] = Double.parseDouble(input.getValue(2*i+1).toString());
                /*repacked[i][0] = ((Double)(input.getValue(2*i))).doubleValue();
                repacked[i][1] = ((Double)(input.getValue(2*i+1))).doubleValue();*/
            }
            repacked[repacked.length-1][0] = Double.parseDouble(input.getValue(input.getSize()-1).toString());
            //repacked[repacked.length-1][0] = ((Double)(input.getValue(input.getSize()-1))).doubleValue();
            repacked[repacked.length-1][1] = 0;
        }
        else    //(input.getSize()%2=0), hopefully
        {
            repacked = new double[input.getSize()/2][2];
            for(int i=0; i<repacked.length; i++)
            {
                repacked[i][0] = Double.parseDouble(input.getValue(2*i).toString());
                repacked[i][1] = Double.parseDouble(input.getValue(2*i+1).toString());
                /*repacked[i][0] = ((Double)(input.getValue(2*i))).doubleValue();
                repacked[i][1] = ((Double)(input.getValue(2*i+1))).doubleValue();*/
            }
        }
        fft = new FastFourierTransform(repacked);
    }
    
    public Complex[] realTransform()
    {
        Complex[] transformed = transformRealData();
        //The -1 is required because the transformRealData() method has already added on one element
        Complex[] result = new Complex[2*(transformed.length-1)];
        //The data that has just been transformed is real data, "masquerading" (as this text puts it) as complex data.
        //Therefore, it is necessary to "unpack" the transformed values for this array back out into the Fourier transform of the real data

        //These two values are used in every iteration of the loop - explicitly declared outside of loop to save speed
        Complex oneHalf = new Complex(0.5, 0);
        Complex oneHalfi = new Complex(0, 0.5);
        for(int i=0; i<transformed.length; i++)
        {
            result[i] = oneHalf.multiply(transformed[i].add(transformed[transformed.length-1].conjugate())).
                    subtract(oneHalfi.multiply(transformed[i].subtract(transformed[transformed.length-1].conjugate())));
            //System.out.println("result[" + i + "] = " + result[i]);
        }
        //Due to the relationship (true for real values) that the Fn = conjugate(FN-n)
        for(int i=transformed.length; i<result.length; i++)
        {
            /*if(result[i]==null)  
                System.out.println("transformed.length = " + transformed.length + "   result.length = " + result.length + "   i = " + i);*/
            result[i] = result[result.length-i].conjugate();
        }
        return result;
    }
   
    private Complex[] transformRealData()
    {
        //Temporarily holds the values from the FFT function
        Complex[] temp = fft.transform();
        //Holds the result to be returned
        Complex[] transformed = new Complex[temp.length+1];
        //Copy the transformed values into transformed
        for(int i=0; i<temp.length; i++)
            transformed[i] = (Complex)(temp[i].clone());
        //Symmetry dictates that for a real array repacked into a Complex array and Fourier-transformed,
        //term 0 of the Fourier-transformed series must be equal to term N-1
        transformed[temp.length] = (Complex)(temp[0].clone());
        /*for(int i=0; i<transformed.length; i++)
            System.out.println("transformed[" + i + "] = " + transformed[i]);*/
        return transformed;
    }
}