package musr.client.gui;
import java.util.Vector;
import musr.client.function.Function;
import java.awt.Frame;
import musr.client.dbreader.Constants;
import musr.client.dbreader.Functions;
import java.text.DecimalFormat;
import musr.client.datacolumn.DataColumn;


public class BestFit
{
    private Vector xData;
    private Vector plusXError;
    private Vector minusXError;
    private Vector yData;
    private Vector plusYError;
    private Vector minusYError;
    private Function fit;
    public DecimalFormat formatter;
    private double[] a;                 //Stores coefficient of polynomial size, and the constant term
    private double[][] sigma;             //Stores constants in terms of equations. Indices are row and column, respectively.
    private double[] value;             //Stores constant on other side of linear equations
    private int degree;                 //Stores degree of polynomial

    public BestFit()
    {
        xData = new Vector();
        plusXError = new Vector();
        minusXError = new Vector();
        yData = new Vector();
        plusYError = new Vector();
        minusYError = new Vector();
        formatter = new DecimalFormat("######.00##");
        degree = 0;
        a = new double[degree+1];
        sigma = new double[degree+1][degree+1];
        value = new double[degree+1];
    }
    
    public BestFit(Vector x, Vector plusX, Vector minusX, Vector y, Vector plusY, Vector minusY, int newDegree)
    {
        xData = x;
        plusXError = plusX;
        minusXError = minusX;
        yData = y;
        plusYError = plusY;
        minusYError = minusY;
        formatter = new DecimalFormat("######.00##");
        degree = newDegree;
        a = new double[degree+1];
        sigma = new double[degree+1][degree+1];
        value = new double[degree+1];
    }
    
    public BestFit(DataColumn x, DataColumn y, int newDegree)
    {
        xData = new Vector();
        plusXError = new Vector();
        minusXError = new Vector();
        yData = new Vector();
        plusYError = new Vector();
        minusYError = new Vector();
        for(int i=0; i<x.getSize(); i++)
        {
            xData.addElement((Double)(x.getValue(i)));
            plusXError.addElement((Double)(x.getErrorPlus(i)));
            minusYError.addElement((Double)(x.getErrorMinus(i)));
            yData.addElement((Double)(y.getValue(i)));
            plusYError.addElement((Double)(y.getErrorPlus(i)));
            minusYError.addElement((Double)(y.getErrorMinus(i)));
        }
        formatter = new DecimalFormat("######.00##");
        degree = newDegree;
        a = new double[degree+1];
        sigma = new double[degree+1][degree+1];
        value = new double[degree+1];
    }
    
    public Vector getXData()
    {
        return xData;
    }
    
    public Vector getYData()
    {
        return yData;
    }
    
    public void setXData(Vector x)
    {
       xData = x;
    }
    
    public void setYData(Vector y)
    {
        yData = y;
    }
    
    public Function getFit()
    {
        return fit;
    }
    
    public double getXValue(int index)
    {
        return ((Double)(xData.elementAt(index))).doubleValue();
    }
    
    public double getPlusXError(int index)
    {
        return((Double)(plusXError.elementAt(index))).doubleValue();
    }
    
    public double getMinusXError(int index)
    {
        return ((Double)(minusYError.elementAt(index))).doubleValue();
    }
    
    public double getYValue(int index)
    {
        return ((Double)(yData.elementAt(index))).doubleValue();
    }
    
    public double getPlusYError(int index)
    {
        return((Double)plusYError.elementAt(index)).doubleValue();
    }
    
    public double getMinusYError(int index)
    {
        return((Double)minusYError.elementAt(index)).doubleValue();
    }
    
    public double ySum()
    {
        double result = getYValue(0);
        for(int i=1; i<yData.size(); i++)
            result = result + getYValue(i);///yWeight(i);
        return result;
    }
    
    public double xySum()
    {
        double result = getYValue(0)*getXValue(0);
        for(int i=1; i<xData.size(); i++)
            result = result + (getYValue(i)*getXValue(i));///(xWeight(i)*yWeight(i));
        return result;
    }
    
    public double xToKSum(int k)        //Optimized, hopefully.
    {
        double result = pow(getXValue(0), k);
        
        switch(k)
        {
            case 0:
                return xData.size();            //IS THIS RIGHT???????
            case 1:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i);
                break;
            case 2:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i);
                break;
            case 3:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 4:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 5:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 6:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 7:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 8:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i);
                break;
            case 9:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i);
                break;
            case 10:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 11:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 12:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 13:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            case 14:
                for(int i=1; i<xData.size(); i++)
                    result = result + getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*
                            getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
                break;
            default:
                for(int i=1; i<xData.size(); i++)
                    result = result + pow(getXValue(i), k);///(pow(xWeight(i),k)*yWeight(i));
                break;
        }
        
        return result;        
    }
    
    public double xToKYSum(int k)       //Optimized, hopefully.
    {
        double result = getYValue(0)*pow(getXValue(0), k);
        
        if(k==0)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i);
        else if(k==1)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i);
        else if(k==2)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i);
        else if(k==3)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
        else if(k==4)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
        else if(k==5)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
        else if(k==6)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
        else if(k==7)
            for(int i=1; i<xData.size(); i++)
                result = result + getYValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i)*getXValue(i);
        else
            for(int i=1; i<xData.size(); i++)
                result = result + (getYValue(i)*pow(getXValue(i), k));///(pow(xWeight(i),k)*yWeight(i));
        
        return result;
    }
    
    public static double pow(double base, int exponent)
    {
        double result = base;
        for(int i=1; i<exponent; i++)
            result = result*base;
        //System.out.println(base + "^" + exponent + "=" + result);
        return result;
    }

    public Function findBestFit()
    {
        for(int i=0; i<value.length; i++)
        {
            value[i] = xToKYSum(i);
            for(int j=0; j<sigma[i].length; j++)
                sigma[i][j] = xToKSum(i+j);
        }
        solve();
        fit = new Function(this.toString());
        return fit;
    }
    
    public String toString()
    {
        String result = "";
        
        if(a.length>0)
            result = result + formatter.format(a[0]);
        
        if(a.length>1)
        {
            if(a[1]>0)
                result = result + " + " + formatter.format(a[1]) + "*x";
            else if(a[1]<0)
                result = result + " " +formatter.format(a[1]) + "*x";
        }

         for(int i=2; i<a.length; i++)
         {
            if(a[i]>0)
                result = result + " + " + formatter.format(a[i]) + "*x^" + i;
            else if(a[i]<0)
                result = result + " " + formatter.format(a[i]) + "*x^" + i;
            else        //a[i]==0, or some error value
                result = result + " ";
         }

        return result;
    }
    
    //Does the actual calculations to solve for the values in a.
    public void solve()
    {
        double detK;
        double det;
        for(int i=0; i<a.length; i++)
        {
            //detK = detK(sigma, i);
            //det = det(sigma);
            a[i] = detK(sigma, i)/det(sigma);
            //a[i] = detK/det;
        }
    }
    
    //Returns the determinant D of the matrix.
    public double det(double[][] matrix)
    {
        switch(matrix.length)
        {
            case 0:
                return 0;
            case 1:
                return matrix[0][0];
            case 2:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1));
            case 3:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1))
                    + matrix[0][2]*det(copyExcluding(matrix, 2));
            case 4:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1)) 
                    + matrix[0][2]*det(copyExcluding(matrix, 2)) - matrix[0][3]*det(copyExcluding(matrix, 3));
            case 5:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1)) 
                    + matrix[0][2]*det(copyExcluding(matrix, 2)) - matrix[0][3]*det(copyExcluding(matrix, 3))
                    + matrix[0][4]*det(copyExcluding(matrix, 4));
            case 6:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1)) 
                    + matrix[0][2]*det(copyExcluding(matrix, 2)) - matrix[0][3]*det(copyExcluding(matrix, 3))
                    + matrix[0][4]*det(copyExcluding(matrix, 4)) - matrix[0][5]*det(copyExcluding(matrix, 5));
            case 7:
                return matrix[0][0]*det(copyExcluding(matrix, 0)) - matrix[0][1]*det(copyExcluding(matrix, 1)) 
                    + matrix[0][2]*det(copyExcluding(matrix, 2)) - matrix[0][3]*det(copyExcluding(matrix, 3))
                    + matrix[0][4]*det(copyExcluding(matrix, 4)) - matrix[0][5]*det(copyExcluding(matrix, 5))
                    + matrix[0][6]*det(copyExcluding(matrix, 6));
            default:
                System.out.println("Error in matrix: matrix.length==" + matrix.length);
                return 0;
        }
    }
    
    //Returns a copy of matrix, excluding the first row and the specified column.
    public static double[][] copyExcluding(double[][] matrix, int column)
    {
        double[][] result = new double[matrix.length-1][matrix.length-1];
        for(int i=1; i<=result.length; i++)
                for(int j=0; j<=result.length; j++)
                {
                    if(j<column)
                        result[i-1][j] = matrix[i][j];
                    else if(j>column)
                        result[i-1][j-1] = matrix[i][j];
                }
        return result;
    }
    
    //Returns the determinant Dk. 
    public double detK(double[][] matrix, int k)
    {
        double[][] matrixCopy = copy(matrix);          
        for(int i=0; i<matrix.length; i++)
            matrixCopy[i][k] = value[i];
        return det(matrixCopy);
        //System.out.println("D" + k + " = "+ temp);
        //return temp;
    }
    
    public static double[][] copy(double[][] matrix)  //Returns a copy of matrix.
    {
        double[][] result = new double[matrix.length][];
        for(int i=0; i<result.length; i++)
        {
            result[i] = new double[matrix[i].length];
            for(int j=0; j<result[i].length; j++)
                result[i][j] = matrix[i][j];
        }
        return result;
    }
    
    public static int pow(int base, int exponent)     //Returns base^exponent. More precise than Math.pow()
    {
        int result = 1;
        for(int i=0; i<exponent; i++)
            result = result * base;
        return result;
    }
    
    public double evaluateAt(double x)
    {
        if(a.length<1)
            return -9999;
        double result = a[0];
        //MORE HERE
        for(int i=0; i<a.length; i++)
            result = result + a[i]*pow(x, i);
        return result;
    }
}