package musr.client.gui;
import musr.client.bestfit.*;
import javax.swing.*;
import musr.client.datacolumn.*;
import java.awt.event.*;
import musr.client.function.Function;
import java.awt.*;
import java.util.Date;
import java.util.StringTokenizer;
import musr.client.bestfit.*;
import java.util.Vector;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Arrays;

public class BestFitGUI extends JFrame implements ActionListener
{
    private DataColumn xValues;
    private DataColumn yValues;
    //private BestFit fitter = null;
    private PolynomialFit fitter = null;
    private Function bestFit = null;
    private JTextArea outputArea;
    public GUI parent;
    private JSpinner degreeSetter;
    
    private FitOptionsFrame fitOptions;
    
    private static final int WIDTH = 420;
    private static final int HEIGHT = 237;
    private static final int TEXT_AREA_ROWS = 3;
    private static final int TEXT_AREA_COLUMNS = 45;
    
    private static final int STAT_FRAME_HEIGHT = 230;
    private static final int STAT_FRAME_WIDTH = 375;
    
    private static final int FIT_OPTIONS_HEIGHT = 285;
    private static final int FIT_OPTIONS_WIDTH = 340;
    
    public BestFitGUI(GUI parentGUI)
    {
        parent = parentGUI;

        int xIndex = parent.xAxisList.getSelectedIndex();
        int yIndex = parent.yAxisList.getSelectedIndex();
        int tableIndex = parent.layoutChoice.getSelectedIndex();
         
        /*System.out.println("xAxisDataTable contains: " + parent.xAxisDataTable.elementAt(xIndex).getClass().toString());
        System.out.println("yAxisDataTable contains: " + parent.yAxisDataTable.elementAt(yIndex).getClass().toString());*/
        
        xValues = (DataColumn) ((DataTable)parent.xAxisDataTable.elementAt(tableIndex)).getColumn(xIndex);
        yValues = (DataColumn) ((DataTable)((Vector)parent.yAxisDataTable.elementAt(tableIndex)).elementAt(xIndex)).getColumn(yIndex);
      
        this.setTitle("Best Fit");
        this.setSize(WIDTH, HEIGHT);
        this.setLocationRelativeTo(null);
        this.addWindowListener(new WindowDestroyer(this));
        
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.setBackground(Color.BLACK);
        
        JPanel specsPanel = new JPanel();
        specsPanel.setLayout(new GridLayout(5,1));

        specsPanel.add(new JLabel("Order of fit:"));
        
        degreeSetter = new JSpinner(new SpinnerNumberModel(2, 1, (xValues.getSize()-2), 1));    //must have min. 1 degree of freedom
        //degreeSetter.setSize(degreeSetter.getWidth()/2, degreeSetter.getHeight());
        specsPanel.add(degreeSetter);
        
        JButton optionsButton = new JButton("Advanced Fit");
        optionsButton.setToolTipText("Select options for the fit.");
        optionsButton.addActionListener(this);
        specsPanel.add(optionsButton);
        
        JButton findFit = new JButton("Find Fit");
        findFit.setToolTipText("Fit the data to a polynomial of the selected degree.");
        findFit.setBackground(Color.ORANGE);
        findFit.addActionListener(this);
        specsPanel.add(findFit);
        
        JButton reset = new JButton("Reset");
        reset.setToolTipText("Reset all best-fit settings to default.");
        reset.addActionListener(this);
        specsPanel.add(reset);
        
        contentPane.add(specsPanel, BorderLayout.EAST);
        
        JPanel outputPanel = new JPanel();
        outputPanel.setLayout(new BorderLayout());
        
        outputPanel.add(new JLabel("Best fit for \n" + yValues.getName() + " vs. " + xValues.getName() + ":"), BorderLayout.NORTH);
        
        outputArea = new JTextArea(TEXT_AREA_ROWS, TEXT_AREA_COLUMNS);
        outputArea.setText("To find the best-fit equation, select the order\nof the fit equation, then click \"Find Fit\"." +
                "\nFor high-order polynomials or very long columns,\nroundoff error will accumulate.");
        outputPanel.add(new JScrollPane(outputArea), BorderLayout.CENTER);
        
        contentPane.add(outputPanel, BorderLayout.CENTER);
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        
        JButton stats = new JButton("View Fit Statistics");
        stats.setToolTipText("View the chi-squared value and confidence level for the fit, and the variance of each parameter.");
        stats.addActionListener(this);
        buttonPanel.add(stats);
        
        JButton newCol = new JButton("Add to Columns");
        newCol.setToolTipText("Add a line generated from this fit as a data column in a new data table.");
        newCol.addActionListener(this);
        buttonPanel.add(newCol);
        
        JButton newFunc = new JButton("Add to Functions");
        newFunc.setToolTipText("Add the function produced by this fit to the functions table.");
        newFunc.addActionListener(this);
        buttonPanel.add(newFunc);
        
        JButton close = new JButton("Close");
        close.setToolTipText("Close the best-fit window.");
        close.addActionListener(this);
        buttonPanel.add(close);
        
        contentPane.add(buttonPanel, BorderLayout.SOUTH);
        
        fitOptions = new FitOptionsFrame();
    }
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        
        if(command.equals("Close"))
            this.dispose();
        
        else if(command.equals("View Fit Statistics"))
        {
            if(fitter!=null)
            {
                FitStatisticsFrame stats = new FitStatisticsFrame();
                stats.setVisible(true);
            }
            else
                JOptionPane.showMessageDialog(null, "No best fit has been found yet! Click \"Find Fit\" to find the curve of best fit.");
        }
        
        else if(command.equals("Add to Columns"))
        {
            if(fitter!=null)
                addFitToColumns();
            else
                JOptionPane.showMessageDialog(null, "No best fit has been found yet! Click \"Find Fit\" to find the curve of best fit.");
        }
        
        else if(command.equals("Add to Functions"))
        {
            if(fitter!=null)
                parent.functionsTable.setFunction(getName(), fitter.toString());
            else
                JOptionPane.showMessageDialog(null, "No best fit has been found yet! Click \"Find Fit\" to find the curve of best fit.");
        }
                    
        else if(command.equals("Find Fit"))
            findFit();
        
        else if(command.equals("Reset"))
        {   
            if((fitter!=null)&&(JOptionPane.showConfirmDialog(this, "Are you sure you want to reset?",
                    "Confirm Reset", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION))
            {
                fitter = null;
                outputArea.setText("To find the best-fit equation, select the order\nof the fit equation, then click " +
                        "\"Find Fit\".\nFor high-order polynomials or very long columns,\nroundoff error will accumulate.");
            }
        }
        
        else if(command.equals("Advanced Fit"))
            fitOptions.setVisible(true);
        
        else
            System.out.println("Error in user interface.");
    }
    
    public String getName()
    {
        return yValues.getName() + "_" + xValues.getName();
    }
    
    private class WindowDestroyer extends WindowAdapter
    {
        private JFrame gui;
        
        public WindowDestroyer(JFrame newGUI)
        {
            super();
            gui = newGUI;
        }
        
        public void windowClosing(WindowEvent e)
        {
            gui.dispose();
        }
    }
    
    public String toString()
    {
        return "y = " + fitter.toString();
        /*String result = "y = ";
        String temp = "";
        StringTokenizer tokenizer = new StringTokenizer(fitter.toString());
        
        while(tokenizer.hasMoreTokens())
        {
            while((temp.length()<TEXT_AREA_COLUMNS)&&tokenizer.hasMoreTokens())
            {
                result += tokenizer.nextToken() + " ";
            }
            result += "\n";
        }*/
        /*for(int i=1; i<result.length()/TEXT_AREA_COLUMNS; i++)
        {
            int index = TEXT_AREA_COLUMNS*i;
            while((result.charAt(index)!='+')&&(result.charAt(index)!='-'))
                index++;
            result = result.substring(0, index+1) + "    \n" + result.substring(index+1);
        }*/
    }
    
    private void findFit()
    {
        //fitter = new BestFit(xValues, yValues, ((Integer)degreeSetter.getValue()).intValue());
        int degree = ((Integer)degreeSetter.getValue()).intValue();
        if((degree<=9)||((degree>9)&&(degree<=25)&&
                (JOptionPane.showConfirmDialog(this,"The selected degree of " + degree + " for this fit" +
                " seems rather large.\nIt may take up to several minutes to complete the fit.\n" +
                "Are you sure you want to proceed?", 
                "High degree selected", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)))
        {
            fitter = new PolynomialFit(degree, xValues, (MultiColumn)yValues);
            displayFit();
        }
        if(degree>25)
            JOptionPane.showMessageDialog(null, "The selected degree is too high.\n" +
                    "Not only will it the fit take a long time to complete, but the resulting fit will be a polynomial of " +
                    "order " + degree + ",\nwhich seems highly unlikely to be necessary for any foreseeable data analysis." +
                    "\nPlease select a lower degree for the fit, and try again.");
    }
    
    private void addFitToColumns()
    {
        ConcreteDataColumn xColumn = new ConcreteDataColumn(xValues.getName(), (xValues.getLabel()), 
                xValues.getUnits(), new Date());
        ConcreteDataColumn yColumn = new ConcreteDataColumn("values_"+yValues.getName(), yValues.getLabel(), 
                yValues.getUnits(), new Date());
        ConcreteDataColumn zeroErrorBars = new ConcreteDataColumn();
        int pointCount = -1;
        int degree = Integer.parseInt(degreeSetter.getValue().toString());
        while((pointCount<degree)||(pointCount>10000))
            pointCount = Integer.parseInt(JOptionPane.showInputDialog(this, "Please choose a reasonable number of  " +
                    "points to include in the column of fitted points:", new Integer(255)));       
        //find min and max x, and determine spacing:
        double currentValue;
        double minimumX = Double.MAX_VALUE;
        double maximumX = Double.MIN_VALUE;
        for(int i=0; i<xValues.getSize(); i++)
        {
            currentValue = Double.parseDouble(xValues.getValue(i).toString());
            if(currentValue<minimumX)
                minimumX = currentValue;
            if(currentValue>maximumX)
                maximumX = currentValue;
        }
        double step = (maximumX-minimumX)/(pointCount-1);
        //fill xColumn, yColumn, and zeroErrorBars with pointCount number of points
        //in xColumn, put evenly spaced values ranging from the minimum value in xValues to the maximum value in xValues
        //in yColumn, put the values of the fitted function evaluated at the corresponding xValue
        //in zeroErrorBars, put zero - there are no errors calculated for the fit function
        for(int i=0; i<pointCount; i++)
        {
            xColumn.addElement(new Double(minimumX + i*step));
            yColumn.addElement(new Double(fitter.evaluateFitAt(minimumX + i*step)));
            zeroErrorBars.addElement(new Double(0));
        }
        MultiColumn xMultiColumn = new MultiColumn(xColumn, zeroErrorBars, zeroErrorBars);
        MultiColumn yMultiColumn = new MultiColumn(yColumn, zeroErrorBars, zeroErrorBars);
        
        DataTable newTable;     
        if(JOptionPane.showConfirmDialog(this, "The newly created data columns must be placed in a new data table.\n" +
                "Would you like to choose a location for the data table, so that it can be saved later?", "Select location?",
                JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
            newTable = new DataTable(getFilepath());
        else
            newTable = new DataTable();
        
        newTable.addColumn(xMultiColumn);
        newTable.addColumn(yMultiColumn);
        newTable.setTitle("Fit: " + xValues.getName() + " vs " + yValues.getName());
        newTable.setComment("Polynomial best fit of " + xValues.getName() + " vs " +  yValues.getName());
        
        //add new data table to the computer's and user's lists  of data tables
        parent.dictionary.addElement(newTable);
        parent.dataTableList.add(newTable.getTitle());
        parent.dataTableList.select(parent.dataTableList.getItemCount()-1);
        parent.reloadColumnList();
        //notify the user with a reassuring message
        //JOptionPane.showMessageDialog(this, "The new data table, entitled \"" + newTable.getTitle() + "\", has successfully " +
                //"been created.");
    }
    
    //returns the filepath used as the argument to the constructor for the DataTable newTable (which holds the FFT'd data)
    private String getFilepath()
    {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
        String filepath = "";
        if(fileChooser.showDialog(this, "Select")==JFileChooser.APPROVE_OPTION)
            filepath = fileChooser.getSelectedFile().getAbsolutePath();
        return filepath;
    }

    /**
     *Given the initialized data, calculates the reuslts of the fit and displays it in the text box
     */ 
    private void displayFit()
    {
        try
        {
            fitter.fit();
        }
        catch(DataNotFittableException e)       //if the data is not fittable, say so, then stop trying to fit it
        {
            JOptionPane.showMessageDialog(null, e.getMessage());
            this.dispose();
        }
        bestFit = new Function(fitter.toString());
        String result = "y = " + fitter.toString();
        //Format result to put it on lines:
        /*StringTokenizer tokenizer = new StringTokenizer(result);
        int currentPosition=0;
        String currentToken;
        while(tokenizer.hasMoreTokens())
        {
            currentToken = tokenizer.nextToken();
            currentPosition = currentPosition + currentToken.length() +1;
            if(((currentToken.indexOf("+")>0)||(currentToken.indexOf("-"))>0)&&(currentPosition%20>2))
                result = result.substring(0, currentPosition) + "\n" + result.substring(currentPosition);
        }*/
        outputArea.setText(result);
    }
    
    ///holds the popup window used to display statistics
    private class FitStatisticsFrame extends JFrame implements ActionListener, ChangeListener
    {       
        //used to display variance in coefficients
        JSpinner varianceSpinner;
        JLabel varianceValue;
        
        public FitStatisticsFrame()
        {
            super("Fit Statistics");
            this.setSize(STAT_FRAME_WIDTH, STAT_FRAME_HEIGHT);
            this.addWindowListener(new WindowDestroyer(this));
            this.setLayout(new GridLayout(3, 1));
            
            JPanel outputPanel = new JPanel();
            outputPanel.setLayout(new GridLayout(2,2));
            outputPanel.add(new JLabel("Chi-squared value:"));
            outputPanel.add(new JLabel(Double.toString(fitter.getChiSquared())));
            outputPanel.add(new JLabel("Confidence level:"));
            outputPanel.add(new JLabel(Double.toString(fitter.getQValue())));
            this.add(outputPanel);
            
            JPanel variancePanel = new JPanel();
            variancePanel.setLayout(new GridLayout(3,2));
            variancePanel.setBorder(new LineBorder(Color.BLACK));
            variancePanel.add(new JLabel("Variance of Coefficients"));
            variancePanel.add(new JLabel(""));
            
            variancePanel.add(new JLabel("Degree of term: "));
            varianceSpinner = new JSpinner(new SpinnerNumberModel(0, 0, fitter.getCoefficients().length, 1));
            varianceSpinner.addChangeListener(this);
            variancePanel.add(varianceSpinner);
            
            variancePanel.add(new JLabel("Variance of coefficient: "));
            varianceValue = new JLabel(Double.toString(fitter.getVarianceAt(0)));
            variancePanel.add(varianceValue);
            
            this.add(variancePanel);

            JButton closeButton = new JButton("Close");
            closeButton.setToolTipText("Close the fit statistics window.");
            closeButton.addActionListener(this);
            this.add(closeButton);
        }
        
        public void actionPerformed(ActionEvent e)
        {
            if(e.getActionCommand().equals("Close"))
                this.dispose();
        }
        
        public void stateChanged(ChangeEvent e)
        {
            varianceValue.setText(Double.toString(fitter.getVarianceAt(Integer.parseInt(varianceSpinner.getValue().toString()))));
        }
    }
    
    private class FitOptionsFrame extends JFrame implements ActionListener
    {
        JSpinner minValueSpinner;
        JSpinner maxValueSpinner;
        
        JRadioButton errorBarsOn;
        JRadioButton errorBarsOff;
        
        JRadioButton keepOutliers;
        JRadioButton removeOutliers;
        
        public FitOptionsFrame()
        {
            super("Advanced Fit");
            this.setSize(FIT_OPTIONS_WIDTH, FIT_OPTIONS_HEIGHT);
            this.setLayout(new GridLayout(4, 1));
            this.addWindowListener(new WindowDestroyer(this));
            
            //a panel which selects the range of x-values on which to perform the fit
            /*JPanel fitRangePanel = new JPanel();
            fitRangePanel.setLayout(new GridLayout(3, 1));
            fitRangePanel.setBorder(new TitledBorder(new LineBorder(Color.BLACK), "Fitting range"));*/
            this.add(new JLabel("Select range of x values to perform the fit on:"));
            
            JPanel dataRangePanel = new JPanel();
            dataRangePanel.setLayout(new FlowLayout());
            
            IndexedDouble[] xValuesArray = new IndexedDouble[xValues.getSize()];  //create a new array holding the x values 
                                                                                //(for the JSpinners)
            //TO DO:
            //Fix these spinners so that their behaviour conforms with expectations
            for(int i=0; i<xValuesArray.length; i++)
                xValuesArray[i] = new IndexedDouble(Double.parseDouble(xValues.getValue(i).toString()), xValuesArray.length-i-1);
            //Arrays.sort(xValuesArray);
            
            dataRangePanel.add(new JLabel("from:"));
            minValueSpinner = new JSpinner(new SpinnerListModel(xValuesArray));
            minValueSpinner.setValue(xValuesArray[0]);
            dataRangePanel.add(minValueSpinner);
            
            dataRangePanel.add(new JLabel("to:"));
            maxValueSpinner = new JSpinner(new SpinnerListModel(xValuesArray));
            maxValueSpinner.setValue(xValuesArray[xValuesArray.length-1]);
            dataRangePanel.add(maxValueSpinner);
            
            this.add(dataRangePanel);
            
            //this.add(fitRangePanel);

            //a panel which selects whether to do the fit using error bars
            JPanel errorBarPanel = new JPanel();
            errorBarPanel.setLayout(new FlowLayout());
            errorBarPanel.setBorder(new TitledBorder(new LineBorder(Color.BLACK), "Error bars"));
            ButtonGroup errorBarButtons = new ButtonGroup();
            
            errorBarsOn = new JRadioButton("Weighted fit");
            errorBarsOn.setSelected(true);
            errorBarButtons.add(errorBarsOn);
            errorBarPanel.add(errorBarsOn);
            
            errorBarsOff = new JRadioButton("Unweighted fit");
            errorBarsOn.setSelected(false);
            errorBarButtons.add(errorBarsOff);
            errorBarPanel.add(errorBarsOff);
            
            this.add(errorBarPanel);
            
            //a panel which selects whether to keep outlying values
            /*JPanel outlierPanel = new JPanel();
            outlierPanel.setLayout(new FlowLayout());
            outlierPanel.setBorder(new TitledBorder(new LineBorder(Color.BLACK), "Outliers"));
            ButtonGroup outlierButtons = new ButtonGroup();
            
            keepOutliers = new JRadioButton("Keep outlying y values");
            keepOutliers.setSelected(true);
            outlierButtons.add(keepOutliers);
            outlierPanel.add(keepOutliers);
            
            removeOutliers = new JRadioButton("Remove outlying y values");
            removeOutliers.setSelected(false);
            outlierButtons.add(removeOutliers);
            outlierPanel.add(removeOutliers);
            
            this.add(outlierPanel);*/
            
            //a panel with the buttons clicked after changes have been made
            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new FlowLayout());
            
            JButton closeButton = new JButton("Close");
            closeButton.addActionListener(this);
            buttonPanel.add(closeButton);
            
            JButton saveButton = new JButton("Find Best Fit");
            saveButton.addActionListener(this);
            buttonPanel.add(saveButton);
            
            this.add(buttonPanel);
        }
        
        public void actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            MultiColumn newYColumn = null;
            
            if(command.equals("Close"))
                this.setVisible(false);
            
            else if(command.equals("Find Best Fit"))
            {
                //first, remove the outlying values if the option has been selected
                if(removeOutliers.isSelected())
                {
                    Vector yValueVector = new Vector();
                    for(int i=0; i<yValues.getSize(); i++)
                        yValueVector.add(yValues.getValue(i));
                    Statistics stats = new Statistics(yValueVector);
                    double[] outlier = stats.outliers();
                    
                    for(int i=0; i<outlier.length; i++)
                        for(int j=0; j<yValueVector.size(); j++)
                            if(Double.parseDouble(yValueVector.elementAt(j).toString())==outlier[i])
                                yValueVector.remove(j);
                    
                    ConcreteDataColumn newYValues = new ConcreteDataColumn();
                    for(int i=0; i<yValueVector.size(); i++)
                        newYValues.addElement(yValueVector.elementAt(i));
                    
                    newYColumn = new MultiColumn(newYValues, (ConcreteDataColumn)((MultiColumn)(yValues)).getErrorPlusColumn(), 
                            (ConcreteDataColumn)((MultiColumn)(yValues)).getErrorMinusColumn());
                }
                else if(keepOutliers.isSelected())
                    newYColumn = (MultiColumn)yValues;
                
                int minValueIndex = ((IndexedDouble)(minValueSpinner.getValue())).getIndex();
                int maxValueIndex = ((IndexedDouble)(maxValueSpinner.getValue())).getIndex();
                System.out.println("minValueIndex = " + minValueIndex);
                System.out.println("maxValueIndex = " + maxValueIndex);
                
                fitter = new PolynomialFit(((Integer)degreeSetter.getValue()).intValue(), xValues, newYColumn, 
                        maxValueIndex, minValueIndex, errorBarsOn.isSelected());
                this.setVisible(false);
                displayFit();  
            }
        }
    }
    
    //Wee little class used in the spinners to select the fit value range
    private class IndexedDouble
    {
        private double value;
        private int index;
        
        public IndexedDouble(double newValue, int newIndex)
        {
            value = newValue;
            index = newIndex;
        }
        
        public String toString()
        {
            return Double.toString(value);
        }
        
        public int getIndex()
        {
            return index;
        }
        
        public double getValue()
        {
            return value;
        }
    }
}