package musr.client.gui;
import musr.client.datacolumn.*;
import java.util.Vector;
import java.util.Date;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;

public class DataRepacker extends JFrame implements ActionListener
{
    GUI parent;
    private DataColumn xColumn;
    private DataColumn yColumn;
    
    private static final int WIDTH = 250;
    private static final int HEIGHT = 235;
        
    public DataRepacker(GUI parentGUI)
    {
           super("Data Repacker");
           parent = parentGUI;
           this.setSize(WIDTH, HEIGHT);
           this.setLocationRelativeTo(null);

           Container contentPane = this.getContentPane();
           contentPane.setLayout(new BorderLayout());

            int xIndex = parent.xAxisList.getSelectedIndex();
            int yIndex = parent.yAxisList.getSelectedIndex();
            int tableIndex = parent.layoutChoice.getSelectedIndex();

            DataColumn xColumn = (DataColumn) ((DataTable)parent.xAxisDataTable.elementAt(tableIndex)).
                    getColumn(xIndex);
            DataColumn yColumn = (DataColumn) ((DataTable)((Vector)parent.yAxisDataTable.elementAt(tableIndex)).elementAt(xIndex)).
                    getColumn(yIndex);
           
           JPanel columnIndicatorPanel = new JPanel();
           columnIndicatorPanel.setLayout(new GridLayout(2,2));
           columnIndicatorPanel.add(new JLabel("Selected x column:"));
           columnIndicatorPanel.add(new JLabel(xColumn.getName()));
           columnIndicatorPanel.add(new JLabel("Selected y column:"));
           columnIndicatorPanel.add(new JLabel(yColumn.getName()));
           
           contentPane.add(columnIndicatorPanel, BorderLayout.CENTER);
           
           JPanel buttonPanel = new JPanel();
           buttonPanel.setLayout(new FlowLayout());
           
           JButton repackButton = new JButton("Repack");
           repackButton.addActionListener(this);
           buttonPanel.add(repackButton);
           
           JButton cancelButton = new JButton("Cancel");
           cancelButton.addActionListener(this);
           buttonPanel.add(cancelButton);
           
           contentPane.add(buttonPanel, BorderLayout.SOUTH);
    }

    public void actionPerformed(ActionEvent e)
    {
           if(e.getActionCommand().equals("Repack"))
           {
               DataColumn[] repacked = repack();
               DataTable newTable;     
                if(JOptionPane.showConfirmDialog(this, "The repacked data columns must be placed in a new data table.\n" +
                        "Would you like to choose a location for the data table, so that it can be saved later?", "Select location?",
                        JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                    newTable = new DataTable(getFilepath());
                else
                    newTable = new DataTable();

                newTable.addColumn(repacked[0]);
                newTable.addColumn(repacked[1]);
                newTable.setTitle(repacked[1].getName() + " repacked");
                newTable.setComment(repacked[1].getName() + " repacked");

                parent.dictionary.addElement(newTable);
                parent.dataTableList.add(newTable.getTitle());
                parent.dataTableList.select(parent.dataTableList.getItemCount()-1);
                parent.reloadColumnList();

                JOptionPane.showMessageDialog(this, "The new data table, entitled \"" + newTable.getTitle() + "\", has successfully " +
                        "been created.");
                
                this.dispose();
           }

           else if(e.getActionCommand().equals("Cancel"))
               this.dispose();
           
           else
           {
               System.out.println("A serious error has occured in the user interface.");
               System.exit(0);
           }
    }
    
    public DataColumn[] repack()
    {
        //initialize result columns
        ConcreteDataColumn xResult = new ConcreteDataColumn(xColumn.getName()+"_repacked", xColumn.getLabel()+"_rpkd", "bins", new Date());
        ConcreteDataColumn yResultValues = new ConcreteDataColumn(yColumn.getName()+"_repacked", yColumn.getLabel()+"_rpkd", 
                yColumn.getUnits(), new Date());
        ConcreteDataColumn yResultPlusErrors = new ConcreteDataColumn();
        ConcreteDataColumn yResultMinusErrors = new ConcreteDataColumn();
        
        int index = 0;
        double binCount = 0.0;
        while(index<yColumn.getSize()-1)
        {
            if(!yColumn.getValue(index).equals(yColumn.getValue(index+1)))
            {
                xResult.addElement(new Double(binCount));
                yResultValues.addElement(yColumn.getValue(index));
                yResultPlusErrors.addElement(yColumn.getErrorPlus(index));
                yResultMinusErrors.addElement(yColumn.getErrorMinus(index));
                
                index++;
                binCount++;
            }
            else
            {
                int repeatCount = getRepeatCount(index);
                xResult.addElement(new Double(binCount));
                yResultValues.addElement(yColumn.getValue(index));
                yResultPlusErrors.addElement(new Double(Double.parseDouble(yColumn.getErrorPlus(index).toString()) /
                        Math.sqrt((double)repeatCount)));
                yResultMinusErrors.addElement(new Double(Double.parseDouble(yColumn.getErrorMinus(index).toString()) /
                        Math.sqrt((double)repeatCount)));
                
                index += repeatCount;
                binCount++;
            }
        }
        
        MultiColumn yResult = new MultiColumn(yResultValues, yResultPlusErrors, yResultMinusErrors);
        return new DataColumn[]{xResult, yResult};
    }
    
    //returns the number of repeats of the value at index after index
    private int getRepeatCount(int index)
    {
        int result = 0;
        while(((index+result)<yColumn.getSize())&&(yColumn.getValue(index+result).equals(yColumn.getValue(index))))
            result++;
        return result;
    }
        
    //returns the filepath used as the argument to the constructor for the DataTable newTable (which holds the FFT'd data)
    private String getFilepath()
    {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
        String filepath = "";
        if(fileChooser.showDialog(this, "Select")==JFileChooser.APPROVE_OPTION)
            filepath = fileChooser.getSelectedFile().getAbsolutePath();
        return filepath;
    }
}