/*VITALLY IMPORTANT: Must ensure that there are DataColumns in the columnList and DataTables in the dataTableList before instantiating
 *FFT_GUI, or horrific error messages and crashes and other unpleasant things will ensue when loops that start at zero don't find any
 *elements, and other similar cases.
 **/

package musr.client.gui;

import musr.client.bestfit.*;
import javax.swing.*;
import musr.client.datacolumn.*;
import java.awt.event.*;
import musr.client.function.Function;
import java.awt.*;
import java.util.Date;
import java.io.*;
import musr.client.fourier.*;
import java.util.Vector;

public class FFT_GUI extends JFrame implements ActionListener, ItemListener
{
    GUI parent;
    
    //Used to select real or complex FFT
    Choice typeSelector;
    
    //Used to select which data column is real and which is imaginary, respectively.
    Choice realDataSelector;
    Choice imaginaryDataSelector;
    
    //Used to select the type of data returned (real, imaginary, modulus, phase, etc.)
    //Choice resultTypeSelector;
    
    //Used to select where the output is sent (plot or text file)
    //Choice outputSelector;
    
    //Used to select column containing time data:
    Choice timeDataSelector;
    
    //Used to select what - if any - apodization will be used:
    String adopizationFunction;
    
    //Clicked when the user is satisfied with all choices
    JButton performFit;
    //Makes the mean window go away
    JButton close;
    
    //To reassure the user that things are happening
    JProgressBar progressBar;
    
    private static int WIDTH = 600;
    private static int HEIGHT = 360;
    private static int ROW_COUNT = 5;
    
    public static void main(String[] args)
    {
        JFileChooser test = new JFileChooser();
        test.showSaveDialog(null);
        System.out.println(test.getSelectedFile().getPath());
    }
    
    public FFT_GUI(GUI parentGUI)
    {
        super("Fast Fourier Transform Options");
        parent = parentGUI;
        
        setSize(WIDTH, HEIGHT);
        setLocationRelativeTo(null);
        Container contentPane = getContentPane();
        contentPane.setLayout(new GridLayout(ROW_COUNT,1));
        
        //Set choices for type selector and add to panel
        typeSelector = new Choice();
        typeSelector.addItemListener(this);
        typeSelector.add("Real");
        typeSelector.add("Complex");
        typeSelector.setVisible(true);
        contentPane.add(new JLabel("Select the type of input data:"));
        contentPane.add(typeSelector);
        
        //Read choices for real and imaginary data selectors off of columnList in parent
        realDataSelector = new Choice();
        imaginaryDataSelector = new Choice();
        for(int i=0; i<parent.columnList.getItemCount(); i++)
        {
            realDataSelector.add(((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(i).getName());
            imaginaryDataSelector.add(((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(i).getName());
        }
        //Since Real is the default Fourier transform, start with the imaginaryDataSelector disabled:
        imaginaryDataSelector.setEnabled(false);
        
        //Add real and imaginary data selectors to panel
        contentPane.add(new JLabel("Select real data column:"));
        contentPane.add(realDataSelector);
        contentPane.add(new JLabel("Select imaginary data column:"));
        contentPane.add(imaginaryDataSelector);

        /*//Set choices for type of data returned and add to panel:
        resultTypeSelector = new Choice();
        resultTypeSelector.add("Full complex data");
        resultTypeSelector.add("Real component");
        resultTypeSelector.add("Imaginary component");
        resultTypeSelector.add("Complex modulus");
        resultTypeSelector.add("Complex phase");
        contentPane.add(new JLabel("Select the type of output data:"));
        contentPane.add(resultTypeSelector);*/
        
        //Set choices for time data column:
        timeDataSelector = new Choice();
        //For the not-unlikely case that none  of the columns are actually holding time data:
        timeDataSelector.add("[none]");
        for(int i=0; i<parent.columnList.getItemCount(); i++)
            timeDataSelector.add(((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(i).getName());

        //Add to panel:
        contentPane.add(new JLabel("Select time data column:"));
        contentPane.add(timeDataSelector);
        
        //
        /*outputSelector = new Choice();
        outputSelector.add("Output to graph");
        outputSelector.add("Output to .csv file");
        contentPane.add(new JLabel("Select where thee output data should be sent:"));
        contentPane.add(outputSelector);*/
        
        //Add the controls for the adopization function
        adopizationFunction = "";
        
        JPanel adopizationPanel = new JPanel();
        adopizationPanel.setLayout(new FlowLayout());
        
        JButton adopizationButton = new JButton("Set adopization function");
        adopizationButton.addActionListener(this);
        adopizationPanel.add(adopizationButton);
        adopizationPanel.add(new JLabel(""));
        adopizationPanel.add(new JLabel(""));
        adopizationPanel.add(new JLabel(""));
        /*adopizationPanel.add(new JLabel("Select adopization function:"));
        
        //give the user a choice of none or one of the most widely-used adopization functions
        adopizationSelector = new Choice();
        adopizationSelector.add("none");
        adopizationSelector.add("Bartlett");
        adopizationSelector.add("Hann");
        adopizationSelector.add("Welch");
        
        adopizationPanel.add(adopizationSelector);*/
        
        contentPane.add(adopizationPanel);
        
        //Add the plot and cancel buttons
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        
        performFit = new JButton("Perform FT");
        performFit.setToolTipText("Take the Fourier tranform of the selected data.");
        performFit.addActionListener(this);
        performFit.setBackground(Color.ORANGE);
        buttonPanel.add(performFit);
        
        close = new JButton("Close");
        close.setToolTipText("Close the Fourier transform window.");
        close.addActionListener(this);
        buttonPanel.add(close);
        
        contentPane.add(buttonPanel);
        
        /*JPanel adopizationInfoPanel = new JPanel();
        adopizationInfoPanel.setLayout(new FlowLayout());
        
        JButton infoButton = new JButton("About adopization");
        infoButton.setToolTipText("More information about the adopization functions.");
        infoButton.addActionListener(this);
        adopizationInfoPanel.add(infoButton);
        
        adopizationInfoPanel.add(new JLabel(""));
        adopizationInfoPanel.add(new JLabel(""));
        adopizationInfoPanel.add(new JLabel(""));
        
        contentPane.add(adopizationInfoPanel);*/
        
        progressBar = new JProgressBar(0, 100);
    }
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        
        if(command.equals("Perform FT"))
        {
            Complex[] result = transform();
            output(result);
            this.dispose();
        }
        
        else if(command.equals("Set adopization function"))
        {
            adopizationFunction = JOptionPane.showInputDialog("Enter the apodization function to be applied to the data.\n" +
                "The function must be in terms of the variable x.", adopizationFunction);
        }
        
        else if(command.equals("Close"))
            this.dispose();
                
        else 
            System.out.println("Error in user interface.");
    }
    
    public void itemStateChanged(ItemEvent e)
    {
        if(((String)(e.getItem())).equals("Real"))
            imaginaryDataSelector.setEnabled(false);
        else if(((String)(e.getItem())).equals("Complex"))
            imaginaryDataSelector.setEnabled(true);
        else
            System.out.println("Error in user interface.");
    }
    
    /**This method finds the current DataTable from the dictionary and selects from that DataTable the DataColumn with the same index 
     *as the selected DataColumn name in the Choice box (in this class). It then performs the Fourier transform on the selected column or columns
     *and puts the result in a new DataTable. 
     */
    public Complex[] transform()
    {
        System.out.println("FFT_GUI.transform() called");
        Complex[] transformed;
        progressBar.setString("Fast Fourier Tranform beginning");
        progressBar.setValue(0);
        progressBar.setIndeterminate(false);
        progressBar.setVisible(true);
        
        if(typeSelector.getSelectedItem().equals("Real"))
        {
            System.out.println("Selected column is " + realDataSelector.getSelectedItem());
            DataColumn real = ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).
                    getColumn(realDataSelector.getSelectedIndex());
            if(real==null)
                System.out.println("Error: real data column is null");
            RealFourierTransform rft = new RealFourierTransform(real);
            System.out.println("Transform initialized");
            progressBar.setString("Applying adopization function");
            progressBar.setValue(10);
            if(timeDataSelector.getSelectedIndex()==0)      //no time data selected
                rft.fft.applyWindowFunction(adopizationFunction, null, parent, parent.functionsTable, parent.dataTableVariables);
            else
                rft.fft.applyWindowFunction(adopizationFunction, 
                        ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(timeDataSelector.getSelectedItem()),
                        parent, parent.functionsTable, parent.dataTableVariables);
            progressBar.setString("Implementing FFT algorithm on real data");
            progressBar.setValue(20);
            transformed = rft.realTransform();  
        }
        
        else if(typeSelector.getSelectedItem().equals("Complex"))
        {
            DataColumn real = ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).
                    getColumn(realDataSelector.getSelectedIndex());
            DataColumn imaginary = ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).
                    getColumn(imaginaryDataSelector.getSelectedIndex());
            FastFourierTransform fft = new FastFourierTransform(real, imaginary);
            System.out.println("Transform initialized");
            progressBar.setString("Applying adopization function");
            progressBar.setValue(10);
            if(timeDataSelector.getSelectedIndex()==0)      //no time data selected
                fft.applyWindowFunction(adopizationFunction, null, parent, parent.functionsTable, parent.dataTableVariables);
            else
                fft.applyWindowFunction(adopizationFunction, 
                        ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(timeDataSelector.getSelectedItem()),
                        parent, parent.functionsTable, parent.dataTableVariables);
            progressBar.setString("Implementing FFT algorithm on complex data");
            progressBar.setValue(20);
            transformed = fft.transform();
        }
        
        else
        {
            System.out.println("There appears to be an error in the user interface.");
            transformed = new Complex[]{new Complex(0,0)};
        }
        
        progressBar.setVisible(false);
        
        return transformed;
    }
    
    //Takes the complex array of transformed values and puts it into a new DataColumn (or two DataColumns for full complex data
    //and adds that to the DataTable
    private void output(Complex[] transformed)
    {
        /*String outputType = resultTypeSelector.getSelectedItem();
        DataTable selectedDataTable = (DataTable)parent.dictionary.getElement(parent.getDataTableIndex());
        if(outputType.equals("Full complex data"))
        {*/
        ConcreteDataColumn realComp = new ConcreteDataColumn((realDataSelector.getSelectedItem() + "_fft_r"), (realDataSelector.getSelectedItem() + "_fft_r"), "", new Date());
        ConcreteDataColumn imagComp = new ConcreteDataColumn((realDataSelector.getSelectedItem() + "_fft_i"), realDataSelector.getSelectedItem() + "_fft_i", "", new Date());
        ConcreteDataColumn zeroErrorBars = new ConcreteDataColumn();
        
        for(int i=0; i<transformed.length; i++)
        {
            realComp.addElement(new Double(transformed[i].realPart()));
            imagComp.addElement(new Double(transformed[i].imaginaryPart()));
            zeroErrorBars.addElement(new Double(0.0));
        }

        //According to documentation elsewhere, use of "null"  indicates that no error bars are used... so... we shall see.
        MultiColumn real = new MultiColumn(realComp, zeroErrorBars, zeroErrorBars);
        MultiColumn imag = new MultiColumn(imagComp, zeroErrorBars, zeroErrorBars);
        
       //Create a new DataTable to hold these columns (because of the zero padding, they are longer than regular columns)
        
        //First it is necessary to find the location for the new DataTable (in case the user decides to save the GUI):
        
        DataTable newTable;
        
        if(JOptionPane.showConfirmDialog(this, "The newly created data columns must be placed in a new data table.\n" +
                "Would you like to choose a location for the data table, so that it can be saved later?", "Select location?",
                JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
            newTable = new DataTable(getFilepath());
        else
            newTable = new DataTable();
        
        //Make the DataTable all presentable by giving it some description:
        newTable.setTitle("FT on " + realDataSelector.getSelectedItem());
        String comment = "Fourier transform on column " + realDataSelector.getSelectedItem();
        if(adopizationFunction.length()>0);
            comment += " with adopization function: " + adopizationFunction;
        newTable.setComment(comment);
        
       //Put columns into the DataTable:
        newTable.addColumn(real);
        newTable.addColumn(imag);
        newTable.addColumn(getFrequencyColumn(transformed.length));       //Adds frequency bins derived from the time data
        //add the new data table to the list of data tables:
        parent.dictionary.addElement(newTable);
        //add to displayed list of data tables, too:
        parent.dataTableList.add(newTable.getTitle());
        parent.dataTableList.select(parent.dataTableList.getItemCount()-1);
        //reload the list to display changes
        parent.reloadColumnList();
        
        //Notify the user:
        String output = "The results of the Fourier transform on ";
        if(typeSelector.getSelectedItem().equals("Real"))
            output += realDataSelector.getSelectedItem();
        else if(typeSelector.getSelectedItem().equals("Complex"))
            output += realDataSelector.getSelectedItem() + " and " + imaginaryDataSelector.getSelectedItem();
        output += "\nhave been successfully stored in the new data table " + newTable.getTitle() + '.';
        
        JOptionPane.showMessageDialog(null, output);
    }
    
    //returns the filepath used as the argument to the constructor for the DataTable newTable (which holds the FFT'd data)
    private String getFilepath()
    {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
        String filepath = "";
        if(fileChooser.showDialog(this, "Select")==JFileChooser.APPROVE_OPTION)
            filepath = fileChooser.getSelectedFile().getAbsolutePath();
        return filepath;
    }
    
    private LinearColumn getFrequencyColumn(int size)
    {
        if(!timeDataSelector.getSelectedItem().equals("[none]"))
        {
            DataColumn timeData = ((DataTable)(parent.dictionary.getElement(parent.getDataTableIndex()))).getColumn(timeDataSelector.getSelectedItem());
            String timeUnits = timeData.getUnits();
            String freqUnits = "freq";

            //Try to guess the units for the frequency column by searching for some probable frequency units:
            if(timeUnits.indexOf("bins")>=0)
                freqUnits = "1/(bins)";
            else if(timeUnits.indexOf("s")>=0)
                freqUnits = "MHz";
            else if(timeUnits.indexOf("ns")>=0)
                freqUnits = "GHz";
            else if(timeUnits.indexOf("ms")>=0)
                freqUnits = "kHz";

            //Find the frequency interval by taking the difference between the first two elements of the time column and inverting:
            double step = Math.abs(1.0/(Double.parseDouble((String)timeData.getValue(1))) -
                    Double.parseDouble((String)timeData.getValue(0)));
            return new LinearColumn("Frequency", "FREQ", freqUnits, 0.0, step, size, new Date());
        }
        else        //the user has indicated that none of the columns in fact contain the time data
        {
            return new LinearColumn("Frequency", "FREQ", "", 0, 1, size, new Date());
        }
    }
}
