package musr.client.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import musr.client.datacolumn.*;
import musr.client.bestfit.*;
import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;
import musr.client.spline.*;

public class SplineGUI extends JFrame implements ActionListener
{
    //original input data
    private DataColumn xColumn;
    private DataColumn yColumn;
    
    //resulting splines
    private MultiColumn xSplines;
    private MultiColumn ySplines;
    
    //the "parent" from which this class is being called
    private GUI parent;
    
    //used to set the weight and multiplicity of the spline
    private JSlider weightSlider;
    private JSpinner multiplicitySpinner;
    
    //used to set the size of the window
   private static final int WIDTH = 360;
    private static final int HEIGHT = 200;
    
    public SplineGUI(GUI parentGUI)
    {
        //initialize the window and all its fields
        super("Cubic Splines");
        this.setSize(WIDTH, HEIGHT);
	this.setLocationRelativeTo(null);
        
        Container contentPane = this.getContentPane();
        GridBagConstraints constr = new GridBagConstraints();
        contentPane.setLayout(new GridBagLayout());
        
        //initialize the x and y columns
        parent = parentGUI;
        
        int xIndex = parent.xAxisList.getSelectedIndex();
        int yIndex = parent.yAxisList.getSelectedIndex();
        int tableIndex = parent.layoutChoice.getSelectedIndex();

        xColumn = (DataColumn) ((DataTable)parent.xAxisDataTable.elementAt(tableIndex)).getColumn(xIndex);
        yColumn = (DataColumn) ((DataTable)((Vector)parent.yAxisDataTable.elementAt(tableIndex)).elementAt(xIndex)).getColumn(yIndex);
        
        //initialize the buttons
        JButton findSplinesButton = new JButton("Find Splines");
        findSplinesButton.setBackground(Color.ORANGE);
        findSplinesButton.addActionListener(this);
        
        JButton closeButton = new JButton("Close");
        closeButton.addActionListener(this);
        
        //initialize the slider
        weightSlider = new JSlider(0,10,10);
        weightSlider.setMajorTickSpacing(2);
        weightSlider.setMinorTickSpacing(1);
        weightSlider.setPaintTicks(true);
        
        Hashtable labelTable = new Hashtable();
        for(int i=0; i<=5; i++)
            labelTable.put(new Integer(2*i), new JLabel(Double.toString(i/5.0)));
        
        weightSlider.setLabelTable(labelTable);
        weightSlider.setPaintLabels(true);
        weightSlider.setSnapToTicks(true);
        
        //initialize the spinner
        multiplicitySpinner = new JSpinner(new SpinnerNumberModel(1,1,4,1));
        
        //put all the features on the window, with some labels
        constr.gridy = 0;
        constr.gridx = 0;
        contentPane.add(new JLabel("Selected x column:"), constr);
        constr.gridx = 1;
        contentPane.add(new JLabel(xColumn.getLabel()), constr);
        
        constr.gridy++;
        constr.gridx = 0;
        contentPane.add(new JLabel("Selected y column:"), constr);
        constr.gridx = 1;
        contentPane.add(new JLabel(yColumn.getLabel()), constr);
       
        constr.gridy++;
        constr.gridx = 0;
        contentPane.add(new JLabel("Set point weights:"), constr);
        constr.gridx = 1;
        contentPane.add(weightSlider, constr);        
        
        /*constr.gridy++;
        constr.gridx = 0;
        contentPane.add(new JLabel("Set point multiplicities:"), constr);
        constr.gridx = 1;
        contentPane.add(multiplicitySpinner, constr);*/
        
        constr.gridy++;
        constr.gridx = 0;
        contentPane.add(findSplinesButton, constr);
        constr.gridx = 1;
        contentPane.add(closeButton, constr);
    }
    
    private void findSplines()
    {
        BSpline spliner = new BSpline(xColumn, yColumn, /*((Integer)multiplicitySpinner.getValue()).intValue()*/1, 
                weightSlider.getValue()/10.0);
        DataColumn[] splines = spliner.getSplines();
        xSplines = (MultiColumn)splines[0];
        ySplines = (MultiColumn)splines[1];
    }
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("Find Splines"))
        {
            findSplines();
            //create a new data table to hold the splines:
            DataTable newTable;

            if(JOptionPane.showConfirmDialog(this, "The newly created data columns must be placed in a new data table.\n" +
                    "Would you like to choose a location for the data table, so that it can be saved later?", "Select location?",
                    JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                newTable = new DataTable(getFilepath());
            else
                newTable = new DataTable();
        
            newTable.setTitle("Spline interpolation on " + yColumn.getName());
            newTable.setComment("Spline interpolation on " + yColumn.getName() + " against " + xColumn.getName());
            newTable.addColumn(xSplines);
            newTable.addColumn(ySplines);
            
            parent.dictionary.addElement(newTable);
            //add to displayed list of data tables, too:
            parent.dataTableList.add(newTable.getTitle());
            parent.dataTableList.select(parent.dataTableList.getItemCount()-1);
            //reload the list to display changes
            parent.reloadColumnList();
            
            JOptionPane.showMessageDialog(null, "The spline interpolation has successfully been stored in the new data table " + 
                    newTable.getTitle());
            this.dispose();
        }
        else if(command.equals("Close"))
        {
            this.dispose();
        }
        else
            System.out.println("An error has occurred in the cubic spline user interface.");
    }
    
    //returns the filepath used as the argument to the constructor for the DataTable newTable (which holds the spline data)
    private String getFilepath()
    {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
        String filepath = "";
        if(fileChooser.showDialog(this, "Select")==JFileChooser.APPROVE_OPTION)
            filepath = fileChooser.getSelectedFile().getAbsolutePath();
        return filepath;
    }
}