package musr.client.plot2d;
import java.applet.*;
import java.awt.*;

/**
 * It sets up all the axis information in its superclass Axis and then, using all that information,
 * it draws the appropriate axis in a logarithmic domain.
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class LogAxis extends Axis implements java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID
	/* Constructor methods */
        // Note that I am using the Math.round method for min, just ot make sure that it doesn't return a value like: 2.999999998 (which has hapened)
	public LogAxis(double min, double max, int numTics, int pixMin, int pixMax, PlotOptionsFrame optionVariables) throws Exception {
                        super(min, max, numTics, pixMin, pixMax, true, optionVariables);
	}
	public LogAxis() { super(); }

	/* Instance Methods */

	public void drawAxis(Graphics g) {

	  try {

                         float percentage = (float)0.6; // use float value because Math.round is used later to generate integer

                         /* Create Horizontal Logarithmic Axis */

	    if (orient.equals("horizontal")) {

            		  // Set the Fonts
                  Font font = new Font( optionVariables.numbersFontType,
                                        optionVariables.numbersFontStyle,
                                        optionVariables.numbersFontSize );
                  g.setFont(font);
                  FontMetrics fMetric = getFontMetrics(font);

			// Draw horizontal line
			g.setColor(optionVariables.borderlineColor);
			if (ticOutput.equals("dash"))
				g.drawLine(pixMin, appletHeight-pixDist, pixMax, appletHeight-pixDist);

			// Draw Ticks and write Labels numTics times
			for(int i=0; i<numTics; i++) {

				// Draw Tics
				// Don't draw the edge Tics.
 				// When ticOutput = "dash", draw regular tics on the line;
				// -Tics can be drawn above or below the axis ("up" or "down")
 				// When ticOutput = "dot", draw small dots inside the plot.
				int x = pix(tics[i].getValue(), true);
                                                                                        // -------------------------------------------
                                                                                        // This part of code is to determine whether to put in DETAILED power of ten LABELS
                                                                                        // The integer value is used to know how many main Labels there is
                                                                                        // the boolean value determines whether to put the labels at all for this 'for' loop:
                                                                                        //   note that the condition here is the same as the one for main label displaying bit not as where
                                                                                        //           the sub-labels in the code below are displayed (i.e. there is no condition in that part of code)
			                      int detailParam = 0;
                                                                                        boolean detailPar = false;
                                                                                        for(int d=0; d<numTics; d++) {
				  int xtemp = pix(tics[d].getValue(), true);
                                                                                          if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=xtemp && xtemp<=pixMax)) { // this is the same 'if' as the one below for label
                                                                                            detailParam++;
                                                                                            detailPar = true;
                                                                                          }
                                                                                        }
                                                                                        // ---------------------------------------------

				if ((i!=0) && (i!=numTics-1) && (pixMin<=x && x<=pixMax)) {
					if (ticOutput.equals("dash")) {
						if (ticSide.equals("up")) g.drawLine( x, appletHeight-(pixDist-5), x, appletHeight-(pixDist) );
						else if (ticSide.equals("down")) g.drawLine( x, appletHeight-pixDist, x, appletHeight-(pixDist+5) );
						else
							Tools.fail("Error in horizontal LogAxis: wrong String for ticSide: "+ ticSide);
					}
					else if (ticOutput.equals("dot"))
						g.drawOval(x, appletHeight-pixDist, 1, 1);
					else
						Tools.fail("Error in horizontal LogAxis: wrong String for ticOutput: " + ticOutput);
				}


				// Draw subTics:
				// if ticOutput is "dash"
				// For every Tic, draw seven logTics (don't add logTics to the last Tic)
				// If j==5, make a larger logTic in between Tics
				if (appletWidth > 150+pixMin-pixMax+appletWidth) {
                                  if (ticOutput.equals("dash")) {
                                    for(int j=2; j<10; j++) {
                                      int subTic = pix( tics[i].getValue() + ticSpacing * Tools.log10(j) , true);
                                      // make sure the tics are within the plot
                                      if (pixMin<=subTic && subTic<=pixMax) {
                                        if (i!=numTics-1) {
                                                                                                              // Put the 'detail' label: (e.g. 2x10^3..) if needed
                                                                                                              if (detailPar)
                                                                                                                if (labelOutput) {
                                                                                                                  // If there is only one (or none) main label, so draw all the sub-labels
                                                                                                                  if (detailParam<=1) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                  }
                                                                                                                  // there is two main labels, so draw only the second and the fifth sub-label (2x10^; 5x10^)
                                                                                                                  else if (detailParam==2) {
                                                                                                                    if (j==2 || j==5) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                    }
                                                                                                                  }
                                                                                                                  // there is three main labels, so draw only the third sub-label (3x10^)
                                                                                                                  else if (detailParam==3) {
                                                                                                                    if (j==3) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                    }
                                                                                                                  }
                                                                                                                  // if there is more than three, don't draw any
                                                                                                                  else {}
                                                                                                                }


                                          if (j==5) {
                                            if (ticSide.equals("up"))
                                              g.drawLine( subTic, appletHeight-(pixDist-3),
                              		            subTic, appletHeight-pixDist );
                                            if (ticSide.equals("down"))
                                              g.drawLine( subTic, appletHeight-pixDist,
                                                          subTic, appletHeight-(pixDist+3) );
                                          }
                                          else
                                            if (appletWidth > 250+pixMin-pixMax+appletWidth) {
                                              if (ticSide.equals("up"))
                                                g.drawLine( subTic, appletHeight-(pixDist-2),
                                                            subTic, appletHeight-pixDist );
                                              if (ticSide.equals("down"))
                                                g.drawLine( subTic, appletHeight-pixDist,
                                                            subTic, appletHeight-(pixDist+2) );
                                            }
                                        }
                                      }
                                    }
                                  }
				}

                              // Write Labels
                              // make sure they are within the plot
                              if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=x && x<=pixMax)) {
                                drawNumber(i,1,pix( tics[i].getValue(), true),tics,orient,false, g); //note: -1 is a dummy variable
                                  // if the Label is equal to 0.0, draw a vertical dashed line there
                      /*  Note: there is no need to do this in log axis
                                  if (tics[i].getValue()==0) {
                                    if (optionVariables.yZeroAxisMenuItem.getState()) {
                                      Graphics2D g2 = (Graphics2D) g;
                                      g2.setStroke(dashed);
                                      g2.drawLine(x, appletHeight-optionVariables.downDistance, x, optionVariables.upDistance);
                                      g2.setStroke(new BasicStroke());
                                    }
                                  } */
				}
			}
		}

		/* Create Vertical Logarithmic Axis */

		else if (orient.equals("vertical")) {

                  // Set the Fonts
                  Font font = new Font( optionVariables.numbersFontType,
                                        optionVariables.numbersFontStyle,
                                        optionVariables.numbersFontSize );
                  g.setFont(font);
                  FontMetrics fMetric = getFontMetrics(font);

			// Draw vertical line
			g.setColor(optionVariables.borderlineColor);
			if (ticOutput.equals("dash"))
			  g.drawLine(pixDist, pixMin, pixDist, pixMax);

			// Draw Ticks and write Labels numTics times
			for(int i=0; i<numTics; i++) {

				// Draw Tics
				// Don't draw the edge tics
 				// When ticOutput = "dash", draw regular tics on the line;
				// -Tics can be drawn left or right the axis ("left" or "right")
 				// When ticOutput = "dot", draw small dots inside the plot.
				int y = appletHeight - pix(tics[i].getValue(), false);
//System.out.println("tics["+i+"] = "+tics[i].getValue());
                                                                                        // -------------------------------------------
                                                                                        // This part of code is to determine whether to put in DETAILED power of ten LABELS
                                                                                        // The integer value is used to know how many main Labels there is
                                                                                        // the boolean value determines whether to put the labels at all for this 'for' loop:
                                                                                        //   note that the condition here is the same as the one for main label displaying bit not as where
                                                                                        //           the sub-labels in the code below are displayed (i.e. there is no condition in that part of code)
			                      int detailParam = 0;
                                                                                        boolean detailPar = false;
                                                                                        for(int d=0; d<numTics; d++) {
				  int ytemp = appletHeight - pix(tics[d].getValue(), false);
                                                                                          if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=ytemp && ytemp<=pixMax)) { // this is the same 'if' as the one below for label
                                                                                            detailParam++;
                                                                                            detailPar = true;
                                                                                          }
                                                                                        }
                                                                                        // ---------------------------------------------
//System.out.println("numTics: "+numTics+" i: "+i+" tics Value: "+tics[i].getValue()+" pix(tics): "+pix(tics[i].getValue(), false));
//System.out.println("detail Parameter: "+detailParam);
//System.out.println("i      "+i+" pixMin: "+pixMin+" pixMax: "+pixMax+" y: "+y);
				if ((i!=0) && (i!=numTics-1) && (pixMin<=y && y<=pixMax)) {
//System.out.println("i for: "+i+" pixMin: "+pixMin+" pixMax: "+pixMax+" y: "+y);
					if (ticOutput.equals("dash")) {
						if (ticSide.equals("left")) g.drawLine(pixDist-5, y, pixDist, y);
						else if (ticSide.equals("right")) g.drawLine(pixDist, y, pixDist+5, y);
						else
							Tools.fail("Error in vertical LogAxis: wrong String for ticSide: "
												 + ticSide);
					}
					else if (ticOutput.equals("dot"))
						g.drawOval(pixDist, y, 1, 1);
					else
						Tools.fail("Error in vertical LogAxis: wrong String for ticOutput: "
											 + ticOutput);
				}

				// Draw subTics:
				// if ticOutput is "dash"
				// For every Tic, draw seven logTics (don't add logTics to the last Tic)
				// If j==5, make a larger logTic in between Tics
				if (appletHeight > 150+pixMin-pixMax+appletHeight) {
                                                                                                    if (ticOutput.equals("dash")) {
                                                                                                      for(int j=2; j<10; j++) {
                                                                                                        int subTic = appletHeight - pix( tics[i].getValue() + ticSpacing * Tools.log10(j) , false);
                                                                                                        // Make sure the tics are within the graph
                                                                                                        if (pixMin<=subTic && subTic<=pixMax) {
                                                                                                          if (i!=numTics-1) {
                                                                                                              // -----------
                                                                                                              // Put the 'detail' label: (e.g. 2x10^3..) if needed
                                                                                                              if (detailPar)
                                                                                                                if (labelOutput) {
                                                                                                                  // If there is only one (or none) main label, so draw all the sub-labels
                                                                                                                  if (detailParam<=1) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                  }
                                                                                                                  // there is two main labels, so draw only the second and the fifth sub-label (2x10^; 5x10^)
                                                                                                                  else if (detailParam==2) {
                                                                                                                    if (j==2 || j==5) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                    }
                                                                                                                  }
                                                                                                                  // there is three main labels, so draw only the third sub-label (3x10^)
                                                                                                                  else if (detailParam==3) {
                                                                                                                    if (j==3) {
                                                                                                                    drawNumber(i,j,subTic,tics,orient,true, g);
                                                                                                                    }
                                                                                                                  }
                                                                                                                  // if there is more than three, don't draw any
                                                                                                                  else {}
                                                                                                                }
                                                                                                            if (j==5) {
                                                                                                              if (ticSide.equals("left"))
                                                                                                                g.drawLine(pixDist-3, subTic, pixDist, subTic);
                                                                                                              if (ticSide.equals("right"))
                                                                                                                g.drawLine(pixDist, subTic, pixDist+3, subTic);
                                                                                                            }
                                                                                                            else
                                                                                                              if (appletHeight > 250+pixMin-pixMax+appletHeight) {
                                                                                                                if (ticSide.equals("left"))
                                                                                                                  g.drawLine(pixDist-2, subTic, pixDist, subTic);
                                                                                                                if (ticSide.equals("right"))
                                                                                                                  g.drawLine(pixDist, subTic, pixDist+2, subTic);
                                                                                                              }
                                                                                                          }
                                                                                                        }
                                                                                                      }
                                                                                                    }
				}

				// Write labels
                                                                                         // make sure they are within the graph
                                                                                         if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=y && y<=pixMax)) {
                                                                                           drawNumber(i,1,appletHeight - pix( tics[i].getValue() , false), tics, orient, false, g); //note: -1 is a dummy variable
                                                                                                          // if the Label is equal to 0.0, draw a horizontal dashed line there
                      /*  Note: there is no need to do this in log axis
                                                                                                          if (tics[i].getValue()==0) {
                                                                                                            if (optionVariables.xZeroAxisMenuItem.getState()) {
                                                                                                              Graphics2D g2 = (Graphics2D) g;
                                                                                                              g2.setStroke(dashed);
                                                                                                              g2.drawLine(optionVariables.leftDistance, y, appletWidth - optionVariables.rightDistance, y);
                                                                                                              g2.setStroke(new BasicStroke());
                                                                                                            }
                                                                                                          }  */
				}
			}
		}
		else Tools.fail("Error in LogAxis: wrong String for orient: " + orient);
	  }
	  catch (Exception e) {System.err.println(e.getMessage());}
	}

        /**
         * note: ticsLabels is to indicate if there is a number "Nx" in front of the number 10
         */
        public void drawNumber(int i, int j, int subTic, Tic[] tics, String orient, boolean ticsLabels, Graphics g) {
          Font font = new Font( optionVariables.numbersFontType,
                                        optionVariables.numbersFontStyle,
                                        optionVariables.numbersFontSize );
          g.setFont(font);
          FontMetrics fMetric = getFontMetrics(font);
          int intStr = new Double(tics[i].getValue()).intValue();
          String str = Long.toString( intStr );
          float percentage = (float)0.6; // use float value because Math.round is used later to generate integer

          if (orient.equals("horizontal")) {
            // draw the numbers as 10 raised to the power of intStr
            if ( intStr>=4 || intStr<=-3 ) {
              if (ticsLabels)
                g.drawString( j+"x10", subTic-fMetric.stringWidth("Nx10"+"1")/2, appletHeight-(pixDist-28) );
              else
                g.drawString( "10", subTic-fMetric.stringWidth("10"+"1")/2, appletHeight-(pixDist-28) );
              // change the size of font
              font = new Font( optionVariables.numbersFontType, optionVariables.numbersFontStyle,
                                       Math.round(optionVariables.numbersFontSize*percentage) );
              g.setFont(font);
              fMetric = getFontMetrics(font);

              // Shift the y Coordinate slightly 'up' (by 70%)
              int shift = (int)(font.getSize()*(float)0.4);
              // draw the superscript
              if (ticsLabels)
                g.drawString( str, subTic+fMetric.stringWidth("Nx10")-fMetric.stringWidth("12"), appletHeight-(pixDist-25+shift) );
              else
                g.drawString( str, subTic+fMetric.stringWidth("10")-fMetric.stringWidth("1"), appletHeight-(pixDist-25+shift) );
              // Shift the font size back
              font = new Font( optionVariables.numbersFontType, optionVariables.numbersFontStyle,
                                       Math.round(optionVariables.numbersFontSize) );
              g.setFont(font);
              fMetric = getFontMetrics(font);
            } else {
              // Just write as a regular number
              double tempValue = j*Math.pow(10, (double)intStr);
              str = String.valueOf(tempValue);
              if (str.substring(str.length()-2).equals(new String(".0"))) str = str.substring(0,str.length()-2);
              g.drawString( str , subTic-fMetric.stringWidth(str)/2, appletHeight-(pixDist-28) );
            }
          } else if (orient.equals("vertical")) {
            // draw the numbers as 10 raised to the power of intStr
            if ( intStr>=4 || intStr<=-3 ) {
              if (ticsLabels)
                g.drawString( j+"x10", pixDist-fMetric.stringWidth("Nx10")-fMetric.stringWidth(str)-7, subTic + fMetric.getAscent()*3/8); // use Ascent instead of Height
              else
                g.drawString( "10", pixDist-fMetric.stringWidth("10")-fMetric.stringWidth(str)-7, subTic + fMetric.getAscent()*3/8); // use Ascent instead of Height
              // change the size of font
              font = new Font( optionVariables.numbersFontType, optionVariables.numbersFontStyle,
                                       Math.round(optionVariables.numbersFontSize*percentage) );
              g.setFont(font);
              fMetric = getFontMetrics(font);

              // Shift the y Coordinate slightly 'up' (by 70%)
              int shift = (int)(font.getSize()*(float)0.4);
              // draw the superscript
              if (ticsLabels)
                g.drawString( str, pixDist-fMetric.stringWidth(str)-12, subTic-shift);
              else
                g.drawString( str, pixDist-fMetric.stringWidth(str)-10, subTic-shift);
              // Shift the font size back
              font = new Font( optionVariables.numbersFontType, optionVariables.numbersFontStyle,
                                       Math.round(optionVariables.numbersFontSize) );
              g.setFont(font);
              fMetric = getFontMetrics(font);
            // Just write as a regular number
            } else {
              // Just write as a regular number
              double tempValue = j*Math.pow(10, (double)intStr);
              str = String.valueOf(tempValue);
              if (str.substring(str.length()-2).equals(new String(".0"))) str = str.substring(0,str.length()-2);
              g.drawString(str , pixDist-fMetric.stringWidth("1")
                                   -fMetric.stringWidth(String.valueOf(tempValue))-7, subTic + fMetric.getAscent()*3/8); // (don't) use Ascent instead of Height
            }

              g.setFont(font);
              fMetric = getFontMetrics(font);
              }

        }

}
