package musr.client.plot2d;
import musr.client.datacolumn.*;
import musr.client.gui.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.swing.*;
import java.lang.*;
//import java.awt.geom.*;
import java.util.*;

/**
 * This class is used to actually 'draw' a 2D plot using other neccessary classes such as:
 * Axis (and it's subclasses), PlotOPtionsFrame, Series, Column Classes..
 * In order to successfully draw the graphs with Plot2D, the following methods must first be called:
 * setPlotTitle, setPlotOptionsFrame, loadXY (as many as you want)
 *
 * Note: -there are more comments in the code itself about how certain parts of the class are implemented
 * -this class only plots the graph but does not display the button to access
 * the PlotOptionsFrame. Since the class is passed the PlotOptionsFrame parameter,
 * it has nothing to do with it's changes so the class that calls Plot2D will
 * take care of this option.
 *
 * Manual plot setting:
 * Note: When back button is pressed while in manual setting, if the stack is not empty,
 *  the stack will be emptied one by one but it won't be displayed since we are viewing manually what we want.
 *  It is still possible to zoom-in but whenever we move the plot around, it will reload the manual setting
 *  and the zoomed-in settings will be recorded in the stack so when we switch to automatic again, these are
 *  the settings that it will loaded. To get rid of them, just press refresh or keep pressing the back button
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class PlotApplet extends Applet implements ActionListener, MouseListener, MouseMotionListener, KeyListener, java.io.Serializable {//, Printable {

  static final long serialVersionUID = 0L; // SUID
	/* Instance Variables */

	/**
         *  Common variables
         */
	private PlotOptionsFrame optionVariables;

        /**
         * This is the GUI where the console is called from
         */
        public Frame parent;

        /**
         * This is the Panel where this Applet is called from
         */
         public Plot2D plot2D;

	private int minPixX, maxPixX, minPixY, maxPixY;
        /**
         * Number of points in graphs (or elemtns in columns)
         */
	private Vector graphPoints;
        /**
         * Number of loaded graphs
         */
        private int countGraphs;
        private Vector minX, maxX;
        private Vector minY, maxY;
        private String plotTitle;

        private Series ser;

        // These are used for Printing, to select a page format
        Dialog printPageFormatDialog;
        CheckboxGroup pageFormatGroup = new CheckboxGroup();
        Checkbox portraitCheckbox = new Checkbox("PORTRAIT",pageFormatGroup,true);
        Checkbox landscapeCheckbox = new Checkbox("LANDSCAPE",pageFormatGroup, false);
        Checkbox revLandscapeCheckbox = new Checkbox("REVERSE LANDSCAPE",pageFormatGroup, false);
        Button pageFormatButton = new Button("Choose Format");

	/** Automatically initialized */
	private Axis X, Y;
	/** Automatically initialized */
	private double absMinY, absMaxY;
	/** Automatically initialized */
	private double absMinX, absMaxX;
	/** Automatically initialized */
	private int numTicsX, numTicsY;
	/** Automatically initialized
         * Has height and width of the Applet */
	private Dimension dimens;
	/** Automatically initialized */
        private Font font;
	/** Automatically initialized */
 	private FontMetrics fMetric;

        /** Zoom feature */
        private boolean zoomOn;
        /** Dummy variable used in the MouseReleased() Method */
        private boolean mouseReleasedSecond;
        /** This value is used to draw a box when mouse is pressed and released on the graph (zoom feature) */
        int zoomBoxX1 = 0;
        /** This value is used to draw a box when mouse is pressed and released on the graph (zoom feature) */
        int zoomBoxY1 = 0;
        /** This value is used to draw a box when mouse is pressed and released on the graph (zoom feature) */
        int zoomBoxX2 = 0;
        /** This value is used to draw a box when mouse is pressed and released on the graph (zoom feature) */
        int zoomBoxY2 = 0;
        /**Temporary parameter that has a value assigned to it when a box is 'selected' in the plot (zoom feature) */
        private double zoomAbsMinX;
        private double zoomAbsMaxX;
        private double zoomAbsMinY;
        private double zoomAbsMaxY;


        /* Constructor method */
        public PlotApplet(Frame paren, PlotOptionsFrame opFrame, Plot2D plot2D) {
          minX = new Vector();
          maxX = new Vector();
          minY = new Vector();
          maxY = new Vector();
          ser = new Series();
          plotTitle = "";

          graphPoints = new Vector();
          minPixX=100; maxPixX=650;
          minPixY=50; maxPixY=500;
          countGraphs=0;

          optionVariables = opFrame;
          this.parent = paren;
          zoomOn=false;
          mouseReleasedSecond=false;
          this.plot2D = plot2D;

          pageFormatButton.addActionListener(this);

          this.addMouseMotionListener(this);

    this.setFocusable(true);
          this.addKeyListener(this);

        }

	/**
         * This Method initializes the Applet, this is not really called
         */
	public void init() {
            // Set the size of the applet that will appear
            //dimens.setSize(400,400);
            //this.setSize(dimens);
        }

        /**
        *  We override this method so that it doesn't clear the background
        * before calling paint().  No clear is necessary, since paint() overwrites
        * everything with an image.  Causes less flickering this way.
        */
        public void update(Graphics g) {
          g.clearRect(0, 0, this.getSize().width, this.getSize().height);
          paint(g);
        }

    /**
     * This method sets the looks of the Panel,
     * writes various information around the plot such as title and units
     * creates the X and Y axis and then uses the drawGraph method to display them.
     * Note: this method gets called whenever the panel (Plot2D) becomes visible (or is moved around)
     */
    public void paint(Graphics g) {

	  try {

            this.setBackground(optionVariables.backgroundColor);

            drawInfo(g);
//            updateStack();

            /* Create X Axis */
            drawXAxis(g);

            /* Create Y Axis */
            drawYAxis(g);

            /* Draw the graph */
            // A Series object was loaded with X and Y values
            drawGraph(g);

            drawCaption(g);

          } catch (Exception e) {System.err.println(e.getMessage());}
        }

        ////* Plot2D Methods *//////


        /**
         * This method updates the stack for the zoom feature.
         * the min and max have already been set in the loadXY method
         */
/*        public void updateStack() {
          try {
          // Determine absMinX and absMaxX, absMinY and absMaxY, overall range for all plots
            if (zoomOn) {
              // if there is a zoom action, fill the stack
              optionVariables.stack.push(new PlotBorders(optionVariables.xMinText.getText().trim(),
                                                         optionVariables.xMaxText.getText().trim(),
                                                         optionVariables.yMinText.getText().trim(),
                                                         optionVariables.yMaxText.getText().trim()));
              zoomOn = false;
            }
            // there has been no zoom in the past or the stack has been emptied by back button
            if (optionVariables.stack.size()==0) { // graph loaded for the first time
                optionVariables.stack.push(new PlotBorders(optionVariables.xMinText.getText().trim(),
                                                           optionVariables.xMaxText.getText().trim(),
                                                           optionVariables.yMinText.getText().trim(),
                                                           optionVariables.yMaxText.getText().trim()));
            }

          } catch (Exception e) {System.err.println(e.getMessage());}
        }
*/
        /**
         * This method draws various information around the plot
         */
        public void drawInfo(Graphics g) {

          try {
          // Get the dimensions of the Applet
          dimens = new Dimension(this.getSize());
          // Set the edges of the plot in the Applet
          minPixX=optionVariables.leftDistance;
          maxPixX=dimens.width-optionVariables.rightDistance;
          minPixY=optionVariables.downDistance;
          maxPixY=dimens.height-optionVariables.upDistance;

          /* Write various information around the plot */

          // Set the Fonts
          font = new Font( optionVariables.titleFontType,
                           optionVariables.titleFontStyle,
                           optionVariables.titleFontSize );
          g.setFont(font);
          fMetric = getFontMetrics(font);

          // Set the Color
          g.setColor(optionVariables.borderlineColor);

          // See whether user defined one's own title for the plot
          String newPlotTitle = "";
          if (optionVariables.titleText.getText().equals("")) newPlotTitle = plotTitle;
          else newPlotTitle = optionVariables.titleText.getText();

          // Get the pixel width of the plot title
          int titleWidth = getStringWidth(newPlotTitle, g, "title");

//System.out.println("titleWidth: "+titleWidth);
/*
                   // Calculate the lenght of this title (by omitting the \U and \D) and resize the font to fit into the graph
         int titleLength = 0;
         for (int j=0; j<newPlotTitle.length()-2; j++) { // subtract 2 because the if statement looks two chars ahead into the title
           if (!(newPlotTitle.substring(j,j+2).equals("\\U")) && !(newPlotTitle.substring(j,j+2).equals("\\D"))) // don't count these
              titleLength = titleLength + 1;
            else j++; // skip one char here
          }
          titleLength = titleLength + 3; // Add 3 instead of two just to be on the safe side (2 would be added because the for statement above cut short)
*/
          // This is the formula that approximates the size of the Title Font:   so that it automatically fits nicely above the graph
          if (optionVariables.adjustToFitMenuItem.getState()==true) {
            if (titleWidth!=0) {
              int newTitleSize = Math.round( (float) 420*optionVariables.titleFontSize/titleWidth);  // 420 is the approximate size of the title that I want
              // Don't let the font size become to big:
              if (newTitleSize>24) newTitleSize = 24;
              optionVariables.titleFontSize = newTitleSize;
              optionVariables.titleSizeLabel.setText( String.valueOf(newTitleSize));
              // Finally, rename the menubox item
              optionVariables.titleSizeMenu.setLabel("Title Font Size ("+String.valueOf(newTitleSize)+")");
//System.out.println("newtitleSize: "+newTitleSize);
            }
            // Get the NEW pixel width of the plot title
            titleWidth = getStringWidth(newPlotTitle, g, "title");
          }

//System.out.println("newtitleWith: "+newTitleWidth);

          // Make sure to erase any double drawing of the title since the title size algorithn sligthly changes values
          g.setColor(optionVariables.backgroundColor);
          g.fillRect( minPixX/2, (dimens.height-maxPixY)/4, maxPixX, (dimens.height-maxPixY)/2);
          g.setColor(optionVariables.borderlineColor);

          // Write the title above the plot
          drawString( newPlotTitle,
                           (int)Math.round(minPixX+(maxPixX-minPixX)/2-titleWidth/2), // Recalculate the place where the title starts
                           optionVariables.upDistance*3/5, g , "title");


          // Set the Fonts (proportionally to the Applet height)
          font = new Font( optionVariables.labelFontType,
			    optionVariables.labelFontStyle,
                            optionVariables.labelFontSize );
          g.setFont(font);
          fMetric = getFontMetrics(font);

          // See whether user defined one's own X Label for the plot
          String newXLabel = "";
          // If not, take the X Axis Label that was last added
          int lastLabelX = ((Vector)ser.dataX).size()-1; // Subtract 1 since vector starts counting from 0
          if (optionVariables.xLabelText.getText().equals("")) newXLabel = ser.getLabelX(lastLabelX);
          else newXLabel = optionVariables.xLabelText.getText();

          // Get the pixel width of the plot's x label
          int xLabelWidth = getStringWidth(newXLabel, g, "label");

          // Write Label below X Axis
          drawString( newXLabel,
                           (int)Math.round(minPixX+(maxPixX-minPixX)/2-xLabelWidth/2),
                           dimens.height-optionVariables.downDistance/6 , g, "label");

          g.setFont(font);
          fMetric = getFontMetrics(font);

          // See whether user defined one's own Y Label for the plot
          String newYLabel = "";
          // If not, take the Y Axis Label that was last added
          int lastLabelY = ((Vector)ser.dataY).size()-1; // Subtract 1 since vector starts counting from 0
          if (optionVariables.yLabelText.getText().equals("")) newYLabel = ser.getLabelY(lastLabelY);
          else newYLabel = optionVariables.yLabelText.getText();

          // Get the pixel width of the plot's y label
          int yLabelWidth = getStringWidth(newYLabel, g, "label");

          // Write Label left of Y  Axis
          Graphics2D g2 = (Graphics2D) g;
          g2.rotate( - Math.PI / 2 );
          g2.translate( (int)Math.round(-(dimens.height-maxPixY)-(maxPixY-minPixY)/2-yLabelWidth/2),
                        optionVariables.leftDistance/3);
          drawString(newYLabel, 0, 0, g2, "label");

          // return to the old graphics setting
          g2.translate( (int)Math.round((dimens.height-maxPixY)+(maxPixY-minPixY)/2+yLabelWidth/2),
                        -optionVariables.leftDistance/3);
          g2.rotate( Math.PI / 2);

          } catch (Exception e) {
            System.err.println(e.getMessage());
        //    ((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with displaying Title or Labels around the plot" );
          }
        }

        /**
         * This method calculates the width of the given string, with all the sub and super scripts adjusted
         */
        public int getStringWidth(String fullString, Graphics g, String type) {
          // this is the value to return
          int stringWidth = 0;
          // Create a StringTokenizer to traverse through and look for the '\' delimiter
          StringTokenizer tokenizer = new StringTokenizer(fullString, "\\", false);
          String line = "";
          // Just in case they are in some other mode, reset the Fonts
          if (type.equals("title"))
            font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                              optionVariables.titleFontSize );
          else if (type.equals("label"))
            font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                              optionVariables.labelFontSize );
          g.setFont(font);
          fMetric = getFontMetrics(font);
          // Just in case the whole string starts with 'U' or 'D', measur the first token (it could be the whole title)
          if (tokenizer.hasMoreTokens()) {
              line = tokenizer.nextToken();
              // Measure..
              stringWidth+=fMetric.stringWidth(line);
          }
          float percentage = (float)0.6; // use float value because Math.round is used later to generate integer
          // Now take care of the super- and subscripts:
          while (tokenizer.hasMoreTokens()) {
            line = tokenizer.nextToken();
            // If it is superscript, the first letter will be "U" (from '\U')
            if (line.substring(0,1).equals("U")) {
              line = line.substring(1);
              // Decrease the size of Font
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    Math.round(optionVariables.titleFontSize*percentage) );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    Math.round(optionVariables.labelFontSize*percentage) );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Measure ..
              stringWidth+=fMetric.stringWidth(line);
              // We're not done! Since every superscript ends with '\D', reset fonts and draw the next token too
              if (tokenizer.hasMoreTokens()) // just in case..
                line = tokenizer.nextToken().substring(1); // substring() to get rid of the letter 'D'
              // Resize the Font to it's original Size
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    optionVariables.titleFontSize );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    optionVariables.labelFontSize );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Measure ..
              stringWidth+=fMetric.stringWidth(line);
            }
            else if (line.substring(0,1).equals("D")) {
              line = line.substring(1);
              // Increase the size of Font
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    Math.round(optionVariables.titleFontSize*percentage) );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    Math.round(optionVariables.labelFontSize*percentage) );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Measure ..
              stringWidth+=fMetric.stringWidth(line);
              // We're not done! Since every subscript ends with '\U', reset fonts and draw the next token too
              if (tokenizer.hasMoreTokens()) // just in case..
                line = tokenizer.nextToken().substring(1); // substring() to get rid of the letter 'U'
              // Resize the Font to it's original Size
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                  optionVariables.titleFontSize );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                  optionVariables.labelFontSize );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Measure ..
              stringWidth+=fMetric.stringWidth(line);
            }
            // This is the case when the whole string is without super or subscripts
            else {
              // Measure ..
              stringWidth+=fMetric.stringWidth(line);
            }
          }
          return stringWidth;
        }

        /**
         * This method draws the given string on the canvas.
         * It looks for '\U' and '\D' and has the sub- and superscript feature
         * notes:
         *  this tokenizer does not take into account a line that starts with \U or \D
         *  the parameter 'type' will set up fonts for the title or for the label
         */
         public void drawString(String fullString, int xCoord, int yCoord, Graphics g, String type) {
          //fullString
          // Create a StringTokenizer to traverse through and look for the '\' delimiter
          StringTokenizer tokenizer = new StringTokenizer(fullString, "\\", false);
          String line = "";
          // Just in case they are in some other mode, reset the Fonts
          if (type.equals("title"))
            font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                              optionVariables.titleFontSize );
          else if (type.equals("label"))
            font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                              optionVariables.labelFontSize );
          g.setFont(font);
          fMetric = getFontMetrics(font);
          // Just in case the whole string starts with 'U' or 'D', write out the first token (it could be the whole title)
          if (tokenizer.hasMoreTokens()) {
              line = tokenizer.nextToken();
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
          }
          float percentage = (float)0.6; // use float value because Math.round is used later to generate integer
          // Now take care of the super- and subscripts:
          while (tokenizer.hasMoreTokens()) {
            line = tokenizer.nextToken();
            // If it is superscript, the first letter will be "U" (from '\U')
            if (line.substring(0,1).equals("U")) {
              line = line.substring(1);
              // Shift the y Coordinate slightly 'up' (by 70%)
              int shift = (int)(font.getSize()*(float)0.5);  // Note that this calculation is based on the font size before it is resized (because it is \U)
              yCoord-=shift;
//System.out.println("shift U: "+shift);
              // Decrease the size of Font
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    Math.round(optionVariables.titleFontSize*percentage) );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    Math.round(optionVariables.labelFontSize*percentage) );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Draw the superscript
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
              // We're not done! Since every superscript ends with '\D', reset fonts and draw the next token too
              if (tokenizer.hasMoreTokens()) // just in case..
                line = tokenizer.nextToken().substring(1); // substring() to get rid of the letter 'D'
              // Shift the y Coordinate back
              yCoord+=shift;
              // Resize the Font to it's original Size
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    optionVariables.titleFontSize );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    optionVariables.labelFontSize );
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Draw the superscript
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
            }
            else if (line.substring(0,1).equals("D")) {
              line = line.substring(1);
              // Increase the size of Font
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                    Math.round(optionVariables.titleFontSize*percentage) );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                    Math.round(optionVariables.labelFontSize*percentage) );
              // Shift the y Coordinate slightly 'down'
              int shift = (int)font.getSize()/2;  // Note that this calculation is based on the font size after it has been resized (because it is \U)
              yCoord+= shift;
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Draw the subscript or the regular line (in case it is after a superscript)
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
              // We're not done! Since every subscript ends with '\U', reset fonts and draw the next token too
              if (tokenizer.hasMoreTokens()) // just in case..
                line = tokenizer.nextToken().substring(1); // substring() to get rid of the letter 'U'
              // Resize the Font to it's original Size
              if (type.equals("title"))
                font = new Font( optionVariables.titleFontType,optionVariables.titleFontStyle,
                                  optionVariables.titleFontSize );
              else if (type.equals("label"))
                font = new Font( optionVariables.labelFontType,optionVariables.labelFontStyle,
                                  optionVariables.labelFontSize );
              // Shift the y Coordinate back
              yCoord-=shift;
              g.setFont(font);
              fMetric = getFontMetrics(font);
              // Draw the superscript
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
            }
            // This is the case when the whole string is without super or subscripts
            else {
              g.drawString(line, xCoord, yCoord);
              // Update the x coordinate to point to the end of the line that has just been drawn
              xCoord+=fMetric.stringWidth(line);
            }
          }
         }

        /**
         * This method creates and draws the x axis
         */
        public void drawXAxis(Graphics g) {

          try {

              // Note: there are two types of min and max values that are being 'sent in'. This depends on
              //  whether the 'limits' settings are set, then those speciffic values are provided. Otherwise,
              //  the auto-scaled values (absMinX,..) are provided.
              double tMinX = 0; double tMaxX = 0;
              if (optionVariables.xMinText.getText().trim().equals("")) tMinX = absMinX;
              else tMinX = Double.parseDouble(optionVariables.xMinText.getText().trim());
              if (optionVariables.xMaxText.getText().trim().equals("")) tMaxX = absMaxX;
              else tMaxX = Double.parseDouble(optionVariables.xMaxText.getText().trim());

              double roundMin = 1; // dummy value
              double roundMax = 1;
              // Determine and set the value of numTicsX
              // -"LinearAxis" || "Inverse Axis" numTicsX is  proportional to the Applet width
              // -"LogAxis" numTicsX is always set so that Tics are integer values in sequence
              if (optionVariables.axisTypeX.equals("LinearAxis") ||
        		optionVariables.axisTypeX.equals("InverseAxis")) {
                if (maxPixX>minPixX)
                  // This is a pretty 'flexible' and random formula, the essential part is actually the first integer number
                  numTicsX = 3 + (maxPixX-minPixX)/(minPixX-maxPixX+dimens.width);
              }
              else if (optionVariables.axisTypeX.equals("LogAxis")) {
                // note: Log Axis cannot take negative or zero values, so here those will be 'truncated'
//Also, the user could be informed of this 'truncation'
                if (optionVariables.xMinText.getText().trim().equals(""))
                  if (tMinX<=0.0) tMinX=1.0E-7;
                if (optionVariables.xMaxText.getText().trim().equals(""))
                  if (tMaxX<=0.0) tMaxX=1.0E-7;
  //            if (optionVariables.xMinText.getText().trim().equals("") && optionVariables.xMaxText.getText().trim().equals("")) {
                  numTicsX = new Double( Math.abs(Math.ceil(Tools.log10(tMaxX))-
                                       Math.floor(Tools.log10(tMinX))) ).intValue() + 1;
                 roundMin =  Tools.log10(tMinX);
                 roundMax =  Tools.log10(tMaxX);
    //          } else // calculate for exact values
    //            numTicsX = new Double( Math.abs(Tools.log10(tMaxX) - Tools.log10(tMinX)) ).intValue() + 1;
            }

              // Create X Axis
              if (optionVariables.axisTypeX.equals("LinearAxis"))
                X = new LinearAxis(tMinX, tMaxX, numTicsX, minPixX, maxPixX, optionVariables);
              else if (optionVariables.axisTypeX.equals("LogAxis"))
                X = new LogAxis(roundMin, roundMax, numTicsX, minPixX, maxPixX, optionVariables);
              else if (optionVariables.axisTypeX.equals("InverseAxis"))
                X = new InverseAxis(tMinX, tMaxX, numTicsX, minPixX, maxPixX, optionVariables);
              else
                Tools.fail("Error in Plot2D: wrong String for optionVariables.axisTypeX: " +
                            optionVariables.axisTypeX);

              X.setAppletWidthAndHeight(dimens);
              X.setLabelOutput(true);
              X.setOrient("horizontal");
              X.setTicSide("down");
              X.setPixDist(minPixY);
              X.drawAxis(g);

              X.setLabelOutput(false);
              X.setTicSide("up");
              X.setPixDist(maxPixY);
              X.drawAxis(g);

          } catch (Exception e) {
            System.err.println(e.getMessage());
            }
        }

        /**
         * This method creates and draws the y axis and the tics within the plot
         */
        public void drawYAxis(Graphics g) {

          try {
            // Note: there are two types of min and max values that are being 'sent in'. This depends on
            //  whether the 'limits' settings are set, then those speciffic values are provided. Otherwise,
            //  the auto-scaled values (absMinY,..) are provided.
            double tMinY = 0; double tMaxY = 0;
            if (optionVariables.yMinText.getText().trim().equals("")) tMinY = absMinY;
            else tMinY = Double.parseDouble(optionVariables.yMinText.getText().trim());
            if (optionVariables.yMaxText.getText().trim().equals("")) tMaxY = absMaxY;
            else tMaxY = Double.parseDouble(optionVariables.yMaxText.getText().trim());

              double roundMin = 1; // dummy value
              double roundMax = 1;
            // Determine and set the value of numTicsY
            // -"LinearAxis" numTicsY is  proportional to the Applet height
            // -"LogAxis" numTicsY is always set so that Tics are integer values in sequence
            if (optionVariables.axisTypeY.equals("LinearAxis") ||
              optionVariables.axisTypeY.equals("InverseAxis")) {
                if (maxPixY>minPixY)
                  // This is a pretty 'flexible' and random formula, the essential part is actually the first integer number
                  numTicsY = 4 + (maxPixY-minPixY)/(minPixY-maxPixY+dimens.height);
            }
            else if (optionVariables.axisTypeY.equals("LogAxis")) {
              // note: Log Axis cannot take negative or zero values, so here those will be 'truncated'
//Also, the user could be informed of this 'truncation'
              if (optionVariables.yMinText.getText().trim().equals(""))
                if (tMinY<=0.0) tMinY=1.0E-7;
              if (optionVariables.yMaxText.getText().trim().equals(""))
                if (tMaxY<=0.0) tMaxY=1.0E-7;
    //          if (optionVariables.xMinText.getText().trim().equals("") && optionVariables.xMaxText.getText().trim().equals("")) {
                  numTicsY = new Double( Math.abs(Math.ceil(Tools.log10(tMaxY))-
                                       Math.floor(Tools.log10(tMinY))) ).intValue() + 1;
                 roundMin =  Tools.log10(tMinY);
                 roundMax =  Tools.log10(tMaxY);
//System.out.println("roundMIn: "+roundMin+"tMinY: "+tMinY);
     //         } else // calculate for exact values
      //          numTicsY = new Double( Math.abs(Tools.log10(tMaxY) - Tools.log10(tMinY)) ).intValue() + 1;
            }

            // Create Y Axis
            // Send min and max pixel values reversed!
            if (optionVariables.axisTypeY.equals("LinearAxis"))
              Y = new LinearAxis(tMinY, tMaxY, numTicsY, dimens.height-maxPixY, dimens.height-minPixY, optionVariables);
            else if (optionVariables.axisTypeY.equals("LogAxis")) {
              Y = new LogAxis(roundMin, roundMax, numTicsY, dimens.height-maxPixY, dimens.height-minPixY, optionVariables);
            }
            else if (optionVariables.axisTypeY.equals("InverseAxis"))
              Y = new InverseAxis(tMinY, tMaxY, numTicsY, dimens.height-maxPixY, dimens.height-minPixY, optionVariables);
            else
              Tools.fail("Error in Plot2D: wrong String for optionVariables.axisTypeY: " + optionVariables.axisTypeY);

            Y.setAppletWidthAndHeight(dimens);
            Y.setLabelOutput(true);
            Y.setOrient("vertical");
            Y.setTicSide("right");
            Y.setPixDist(minPixX);
            Y.drawAxis(g);

            Y.setLabelOutput(false);
            Y.setTicSide("left");
            Y.setPixDist(maxPixX);
            Y.drawAxis(g);
/*
            if (optionVariables.dotFiducialsMenuItem.getState()) {
              Y.setTicOutput("dot");
              for(int i=1; i<X.getNumTics()-1; i++) {
                if (optionVariables.axisTypeX.equals("InverseAxis"))
                  Y.setPixDist( minPixX + new Double ( (maxPixX-minPixX)*
                                (1/X.getTic(i).getValue()-1/X.getTic(X.getNumTics()-1).getValue())/
                                (1/X.getTic(0).getValue()-1/X.getTic(X.getNumTics()-1).getValue()) ).intValue() );
                else	Y.setPixDist(X.pix(X.getTic(i).getValue()));
                  Y.drawAxis(g);
              }
            }
*/
          } catch (Exception e) {
            System.err.println(e.getMessage());
          }
        }


	/** This method draws all the graphs which are stored in the
         *  Series Class with their error bars which are also stored in the Series Classes.
         *  It has two big parts:
         *          -connecting the points with lines
         *          -drawing a certain symbol and the error bars at the points
         *
         * <Addiditonal info:>
         * Created Series Objects have in them:
         *   -two vectors of arrays: dataY of size dataY.size() and dataX of size dataY.size()
         *         Each array in these vectors holds X or Y values with size:
	 *		((DataColumn)dataX.elementAt(i)).getSize() and ((DataColumn)dataY.elementAt(i)).getSize().
	 * Here, values in them are called to draw the graph line.
         *
	 *	 the main 'for' loop is called countGraph times, to draw each graph.
         *
	 *  Inside the loop:
	 *	 -Variable 'numberX' corresponds to the Frame# of the graph which is stored in PlotOptionsFrame
	 *	  and 'numberY' counts the graph to be plotted in the given Frame
	 *	 -drawGraph will: draw a line in between points or put symbols for graph points. Code for both of
	 *	  these two options is quite similar.
	 *	  It finds the two appropriate array of numbers, xval[] and yval[] (of a certain graph) and
	 *		recalculates their values to accomodate the 'graphType' settings (linear, logarithmic, inverse)
	 *	 -After each point (or line) is drawn, error bars are drawn on all for sides of the point.
	 */
	public void drawGraph(Graphics g) {

		try {
                // This is the main 'for' loop (for each graph):
                for(int j=0; j<countGraphs; j++) {

                  // Find the PlotOptionsFrame numberOfX-1 and numberOfY-1 values for this loop
                  int numberX=0;
                  int numberY=0;
                  loop: for(int m=0; m<((DataTable) ((GUI)parent).xAxisDataTable.elementAt(((GUI)parent).layoutChoice.getSelectedIndex())).columnCount(); m++) {
                    for (int n=0; n<((DataTable)((Vector) ((GUI)parent).yAxisDataTable.elementAt(((GUI)parent).layoutChoice.getSelectedIndex())).elementAt(m)).columnCount(); n++) {
                      if(numberY==j) {
                        numberX=m;
                        numberY=n;
                        break loop;
                      }
                    numberY++;
                    }
                  }
                  // Connect points with lines if wanted
                  if ( ((Boolean) ((Vector)optionVariables.graphTypeLine.elementAt(numberX)).elementAt(numberY)).booleanValue()) {

                      double xVal[] = new double[Tools.objToInt(graphPoints.elementAt(j))];
                      double yVal[] = new double[Tools.objToInt(graphPoints.elementAt(j))];

                      xVal[0] = ser.getValueX(j, 0);
                      yVal[0] = ser.getValueY(j, 0);

                      int xGraph1, xGraph2, yGraph1, yGraph2;
                      xGraph1 = xGraph2 = yGraph1 = yGraph2 = 0;
                      for(int i=1; i<Tools.objToInt(graphPoints.elementAt(j)); i++) {
                       try {
                        int xGraph3, xGraph4, yGraph3, yGraph4;
                        xGraph3 = xGraph4 = yGraph3 = yGraph4 = 0;

                        xVal[i] = ser.getValueX(j, i);
                        yVal[i] = ser.getValueY(j, i);

                        if (optionVariables.axisTypeX.equals("LinearAxis")) {
                          xGraph1 = X.pix(xVal[i-1], true); xGraph2 = X.pix(xVal[i], true);
                        }
                        else if (optionVariables.axisTypeX.equals("LogAxis")) {
                          // If the value is negative, don't calculate it, and later below, don't draw it
                          if (xVal[i-1]>0)
                           if (xVal[i]>0) {
                            xGraph1 = X.pix(Tools.log10(xVal[i-1]), true);
                            xGraph2 = X.pix(Tools.log10(xVal[i]), true);
                          }
                        }
                        else if (optionVariables.axisTypeX.equals("InverseAxis")) {
                          // flip the values since they are inverse
                          if (xVal[i]!=0)
                           if (xVal[i-1]!=0) {
                            xGraph1 = X.pix(1/xVal[i], true);
                            xGraph2 = X.pix(1/xVal[i-1], true);
                          } else {
                            System.out.println("Error: attempt to use inverse axis with a zero data value");
                          }
                        }
                        if (optionVariables.axisTypeY.equals("LinearAxis"))	{
                          yGraph1 = Y.pix(yVal[i-1], false); yGraph2 = Y.pix(yVal[i], false);
                        }
                        else if (optionVariables.axisTypeY.equals("LogAxis")) {
                          // If the value is negative,  don't calculate it, and later below, don't draw it
                          if (yVal[i-1]>0)
                           if (yVal[i]>0) {
                            yGraph1 = Y.pix(Tools.log10(yVal[i-1]), false);
                            yGraph2 = Y.pix(Tools.log10(yVal[i]), false);
                          }
                        }
                        else if (optionVariables.axisTypeY.equals("InverseAxis")) {
                          // flip the values since they are inverse
                          if (yVal[i]!=0 || yVal[i-1]!=0) {
                            yGraph1 = Y.pix(1/yVal[i], false);
                            yGraph2 = Y.pix(1/yVal[i-1], false);
                          } else {
                            System.out.println("Error: attempt to use inverse axis with a zero data value");
                          }
                        }

                        // The pixel values of X and Y must be whithin the Plot
                        if( xGraph1>=minPixX && xGraph1<=maxPixX &&
                        xGraph2>=minPixX && xGraph2<=maxPixX &&
                        yGraph1>=minPixY && yGraph1<=maxPixY &&
                        yGraph2>=minPixY && yGraph2<=maxPixY ){
                          // Set the Color
                          g.setColor( (Color)((Vector)optionVariables.graphColor.elementAt(numberX)).elementAt(numberY) );
                          // Draw the graph line
                          // note: if the values for either x or y axis are negative when it is logarithmic axis, don't draw the line
                          if ( !(optionVariables.axisTypeX.equals("LogAxis") && (xVal[i-1]<=0 || xVal[i]<=0)) )
                            if ( !(optionVariables.axisTypeY.equals("LogAxis") && (yVal[i-1]<=0 || yVal[i]<=0)) )
                              g.drawLine( xGraph1, dimens.height - yGraph1,xGraph2, dimens.height - yGraph2 );

                        }
                       } catch (Exception e) {
                       //((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with loading the X and Y Axes Data" );
                       break;
                      }
                      }
                  }
                  // Draw symbols for graph points if wanted and then draw the error bars if wanted
                  String command = ((String)((Vector)optionVariables.graphTypeOther.elementAt(numberX)).elementAt(numberY));
                  if ( command.equals("dot") ||
                       command.equals("plus") ||
                       command.equals("cross") ||
                       command.equals("star") ||
                       command.equals("circle") ||
                       command.equals("square") ||
                       command.equals("triangle") ||
                       command.equals("nabla") ||
                       command.equals("diamond") ||
                       command.equals("heart") ||
                       command.equals("none") ) {


                    double xVal[] = new double[Tools.objToInt(graphPoints.elementAt(j))];
                    double yVal[] = new double[Tools.objToInt(graphPoints.elementAt(j))];
                    for(int i=0; i<Tools.objToInt(graphPoints.elementAt(j)); i++) {
                     try {


                      int xGraph1, yGraph1;
                      xGraph1 = yGraph1 = 0;
                      int xGraph3, xGraph4, yGraph3, yGraph4;
                      xGraph3 = xGraph4 = yGraph3 = yGraph4 = 0;

                      // Get X and Y values
                      xVal[i] = ser.getValueX(j, i);
                      yVal[i] = ser.getValueY(j, i);

              // if the values for either x or y axis are negative when it is logarithmic axis, don't draw the point nor the errors
              if ( !(optionVariables.axisTypeX.equals("LogAxis") && xVal[i]<=0) )
                if ( !(optionVariables.axisTypeY.equals("LogAxis") && yVal[i]<=0) ) {

                      if (optionVariables.axisTypeX.equals("LinearAxis"))	xGraph1 = X.pix(xVal[i], true);
                      else if (optionVariables.axisTypeX.equals("LogAxis"))	xGraph1 = X.pix(Tools.log10(xVal[i]), true);
                      else if (optionVariables.axisTypeX.equals("InverseAxis"))
                          // flip the values since they are inverse
                          if (xVal[i]!=0) {
                            xGraph1 = X.pix(1/xVal[i], true);
                          } else {
                            System.out.println("Error: attempt to use inverse axis with a zero data value");
                          }
                      if (optionVariables.axisTypeY.equals("LinearAxis"))	yGraph1 = Y.pix(yVal[i], false);
                      else if (optionVariables.axisTypeY.equals("LogAxis"))	yGraph1 = Y.pix(Tools.log10(yVal[i]), false);
                      else if (optionVariables.axisTypeY.equals("InverseAxis"))
                          // flip the values since they are inverse
                          if (yVal[i]!=0) {
                            yGraph1 = Y.pix(1/yVal[i], false);
                          } else {
                            System.out.println("Error: attempt to use inverse axis with a zero data value");
                          }
                      // The pixel values of X and Y must be whithin the Plot
                      if( (xGraph1>=minPixX) && (xGraph1<=maxPixX) && (yGraph1>=minPixY) && (yGraph1<=maxPixY) ) {

                        // This is where the symbol drawing starts
                        if (!command.equals("none")) {

                          // Set the size of the Symbol
                          int k = Integer.parseInt( ((String)((Vector)optionVariables.graphSymbolSize.elementAt(numberX)).elementAt(numberY)) );
                          // Set the Color
                          g.setColor( (Color)((Vector)optionVariables.graphColor.elementAt(numberX)).elementAt(numberY) );

                          // Draw graph dots
                          if ( command.equals("dot") )
                           g.drawOval( xGraph1, dimens.height - yGraph1, 1, 1 );
                          // Draw graph pluses
                          else if ( command.equals("plus") ) {
                            g.drawLine( xGraph1-k, dimens.height - yGraph1, xGraph1+k, dimens.height - yGraph1 );
                            g.drawLine( xGraph1, dimens.height - yGraph1-k, xGraph1, dimens.height - yGraph1+k );
                          }
                          // Draw graph crosses
                          else if ( command.equals("cross") ) {
                            // do this adjustment to make the symbol size the same as all the others
                            k=k*3/4;
                            g.drawLine( xGraph1-k, dimens.height - yGraph1-k, xGraph1+k, dimens.height - yGraph1+k );
                            g.drawLine( xGraph1-k, dimens.height - yGraph1+k, xGraph1+k, dimens.height - yGraph1-k );
                          }
                          // Draw graph stars
                          else if ( command.equals("star") ) {
                            double a = 0.726543*k;
                            double b = 0.449028*k;
                            double m = 0.363271*k;
                            double x = 0.587786*k;

                            double r = 0.309017*k;
                            double R = 0.381965*k;
                            double y = 0.427051*k;
                            double n = 0.118033*k;
                            int xStar[] = { xGraph1, xGraph1+(int)Math.round(b/2.0), xGraph1+(int)Math.round(b/2.0+a), xGraph1+(int)Math.round(m)
                                                , xGraph1+(int)Math.round(x) , xGraph1 , xGraph1+(int)Math.round(-x) , xGraph1+(int)Math.round(-m)
                                                , xGraph1+(int)Math.round(-b/2.0-a) , xGraph1+(int)Math.round(-b/2.0) , xGraph1 };
                            int yStar[] = { dimens.height - yGraph1-k, dimens.height - yGraph1-(int)Math.round(r),
                                                dimens.height - yGraph1-(int)Math.round(r), dimens.height - yGraph1-(int)Math.round(-n),
                                                dimens.height - yGraph1-(int)Math.round(-R-y), dimens.height - yGraph1-(int)Math.round(-R),
                                                dimens.height - yGraph1-(int)Math.round(-R-y), dimens.height - yGraph1-(int)Math.round(-n),
                                                dimens.height - yGraph1-(int)Math.round(r), dimens.height - yGraph1-(int)Math.round(r),
                                                dimens.height - yGraph1-k
                                                };
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillPolygon( xStar, yStar, 11 );
                            else
                              g.drawPolygon( xStar, yStar, 11 );
                          }
                          // Draw graph circles
                          else if ( command.equals("circle") ) {
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() ) {
                              // do this adjustment to make the symbol size the same as all the others
                              k+=2;
                              g.fillOval( xGraph1-(k+1)/2, dimens.height - yGraph1-(k+1)/2, k+1, k+1 );
                            }
                            else {
                              // do this adjustment to make the symbol size the same as all the others
                              k+=2;
                              g.drawOval( xGraph1-(k+1)/2, dimens.height - yGraph1-(k+1)/2, k+1, k+1 );
                            }
                          }
                          // Draw graph squares
                          else if ( command.equals("square") )
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillRect( xGraph1-(k+1)/2, dimens.height - yGraph1-(k+1)/2, k+1, k+1 );
                            else
                              g.drawRect( xGraph1-(k+1)/2, dimens.height - yGraph1-(k+1)/2, k+1, k+1 );
                          // Draw graph triangles
                          else if ( command.equals("triangle") ) {
                            // do an integer adjustment to make the symbol size the same as all the others
                            //k-=1;
                            Double sideVal = new Double(Math.ceil(0.65*k));
                            int xTriangle[] = {xGraph1- sideVal.intValue(), xGraph1+sideVal.intValue(),
                                               xGraph1, xGraph1-sideVal.intValue()};
                            int yTriangle[] = {dimens.height - yGraph1+(k/2), dimens.height - yGraph1+(k/2),
                                               dimens.height - yGraph1-k, dimens.height - yGraph1+(k/2)};
                            //k+=1; //restart the value
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillPolygon( xTriangle, yTriangle, 4);
                            else
                              g.drawPolygon( xTriangle, yTriangle, 4);
                          }
                          // Draw graph nablas (upside-down triangles)
                          if ( command.equals("nabla") ) {
                            // do an integer adjustment to make the symbol size the same as all the others
                            //k-=1;
                            Double sideVal = new Double(Math.ceil(0.65*k));
                            int xNabla[] = {xGraph1-sideVal.intValue(), xGraph1+sideVal.intValue(),
                                            xGraph1, xGraph1-sideVal.intValue()};
                            int yNabla[] = {dimens.height - yGraph1-k/2, dimens.height - yGraph1-k/2,
                                               dimens.height - yGraph1+k, dimens.height - yGraph1-k/2};
                            //k+=1; //restart the value
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillPolygon( xNabla, yNabla, 4);
                            else
                              g.drawPolygon( xNabla, yNabla, 4);
                          }
                          // Draw graph diamonds
                          if ( command.equals("diamond") ) {
                            // do the integer adjustment to make the symbol size the same as all the others
                            k-=2;
                            int xDiamond[] = {xGraph1-k, xGraph1, xGraph1+k, xGraph1, xGraph1-k};
                            int yDiamond[] = {dimens.height - (yGraph1), dimens.height - (yGraph1)-k,
                                              dimens.height - (yGraph1), dimens.height - (yGraph1)+k, dimens.height - (yGraph1)};
                            k+=2; //restart the value
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillPolygon( xDiamond, yDiamond, 5);
                            else
                              g.drawPolygon( xDiamond, yDiamond, 5);
                          }
                          // Draw graph hearts
                          else if ( command.equals("heart") ) {
                            // do an integer adjustment to make the symbol size the same as all the others
                            k-=4;
                            int xHeart[] = {xGraph1, xGraph1-k, xGraph1-3*k/2, xGraph1-k,
                                            xGraph1-k/2, xGraph1, xGraph1+k/2, xGraph1+k, xGraph1+3*k/2, xGraph1+k, xGraph1};
                            int yHeart[] = {dimens.height - (yGraph1)+k, dimens.height - (yGraph1),
                                            dimens.height - (yGraph1)-k/2, dimens.height - (yGraph1)-k,
                                            dimens.height - (yGraph1)-k, dimens.height - (yGraph1)-k/2,
                                            dimens.height - (yGraph1)-k, dimens.height - (yGraph1)-k,
                                            dimens.height - (yGraph1)-k/2, dimens.height - (yGraph1),
                                            dimens.height - (yGraph1)+k};
                            k+=4; //restart the valueg.drawPolygon( xHeart, yHeart, 11);
                            if ( ((Boolean) ((Vector)optionVariables.graphSymbolFilled.elementAt(numberX)).elementAt(numberY)).booleanValue() )
                              g.fillPolygon( xHeart, yHeart, 11);
                            else
                              g.drawPolygon( xHeart, yHeart, 11);
                          }
                        } //if !"none"


                        // Draw error bars if wanted
                        if (optionVariables.errorBarsMenuItem.getState()) {

                          // Check if there is a line drawn or if there are symbols requested (!none)
                          // If not, then there is no point in drawing the error bars
                          if ( ((Boolean) ((Vector)optionVariables.graphTypeLine.elementAt(numberX)).elementAt(numberY)).booleanValue() || !command.equals("none") ) {
                              // Draw X graph error bars
                              // Left
                              double leftError = Tools.objToDoub(((DataColumn) ser.getDataX(j)).getErrorMinus(i));
                              if (optionVariables.axisTypeX.equals("LinearAxis"))
                                xGraph3 = X.pix(xVal[i]- leftError, true);
                              else if (optionVariables.axisTypeX.equals("LogAxis"))
                                if (xVal[i]-leftError>0)
                                  xGraph3 = X.pix(Tools.log10(xVal[i]-leftError), true);
                                else xGraph3 = 0;
                              else if (optionVariables.axisTypeX.equals("InverseAxis"))
                                if (xVal[i]-leftError!=0) {
                                  xGraph3 = X.pix(1/(xVal[i]-leftError), true);
                                } else {
                                  System.out.println("Error: attempt to use inverse axis with a zero data value");
                                  xGraph3 = dimens.width;
                                }
                                if (xGraph3>=minPixX) { // if the error is within the box, draw it
                                  g.drawLine( xGraph3, dimens.height-yGraph1, xGraph1, dimens.height-yGraph1 );
                                  // Draw edge tic of error bar if wanted
                                  if (optionVariables.errorBarTicsMenuItem.getState())
                                    g.drawLine( xGraph3, dimens.height-yGraph1-1, xGraph3, dimens.height-yGraph1+1 );
                                }
                                else { // draw only until the edge of the left side of the plot box
                                  g.drawLine(minPixX, dimens.height-yGraph1, xGraph1, dimens.height-yGraph1);
                                }
                              // Right
                              double rightError = Tools.objToDoub(((DataColumn) ser.getDataX(j)).getErrorPlus(i));
                              if (optionVariables.axisTypeX.equals("LinearAxis"))
                               xGraph4 = X.pix(xVal[i]+rightError, true);
                              else if (optionVariables.axisTypeX.equals("LogAxis"))
                                if (xVal[i]+rightError>0)
                                  xGraph4 = X.pix(Tools.log10(xVal[i]+rightError), true);
                                else xGraph4 = 0;
                              else if (optionVariables.axisTypeX.equals("InverseAxis"))
                                if (xVal[i]+rightError!=0) {
                                  xGraph4 = X.pix(1/(xVal[i]+rightError), true);
                                } else {
                                  System.out.println("Error: attempt to use inverse axis with a zero data value");
                                  xGraph4 = dimens.width;
                                }
                                if (xGraph4<=maxPixX) { //If the right error is within box draw it
                                  g.drawLine( xGraph4, dimens.height-yGraph1, xGraph1, dimens.height-yGraph1 );

                                  // Draw edge tic of error bar if wanted
                                  if (optionVariables.errorBarTicsMenuItem.getState())
                                    g.drawLine( xGraph4, dimens.height-yGraph1-1,	xGraph4, dimens.height-yGraph1+1 );
                                } else { // draw the error bar only until the right edge of the plot box
                                  g.drawLine(maxPixX, dimens.height-yGraph1, xGraph1, dimens.height-yGraph1 );
                                }
                              // Draw Y graph error bars
                              // Up
                              double upError = Tools.objToDoub(((DataColumn) ser.getDataY(j)).getErrorPlus(i));
                              if (optionVariables.axisTypeY.equals("LinearAxis"))
                                yGraph3 = Y.pix(yVal[i]+upError, false);
                              else if (optionVariables.axisTypeY.equals("LogAxis"))
                                if (yVal[i]+upError>0)
                                  yGraph3 = Y.pix(Tools.log10(yVal[i]+upError), false);
                                else yGraph3 = 0;
                              else if (optionVariables.axisTypeY.equals("InverseAxis"))
                                if (yVal[i]+upError!=0) {
                                  yGraph3 = Y.pix((1/yVal[i]+upError), false);
                                } else {
                                  System.out.println("Error: attempt to use inverse axis with a zero data value");
                                  yGraph3 = dimens.height;
                                }
                                if (yGraph3<=maxPixY) { // If the upper error bar is within the box, draw it whole
                                  g.drawLine( xGraph1, dimens.height-yGraph3, xGraph1, dimens.height-yGraph1 );
                                  // Draw edge tic of error bar if wanted
                                  if (optionVariables.errorBarTicsMenuItem.getState())
                                    g.drawLine( xGraph1-1, dimens.height-yGraph3, xGraph1+1, dimens.height-yGraph3 );
                                } else { // draw the tic only until the upper edge of the plot box
                                  g.drawLine(xGraph1, dimens.height-maxPixY, xGraph1, dimens.height-yGraph1 );
                                }
                              // Down
                              double downError = Tools.objToDoub(((DataColumn) ser.getDataY(j)).getErrorMinus(i));
                              if (optionVariables.axisTypeY.equals("LinearAxis"))
                                yGraph4 = Y.pix(yVal[i]-downError, false);
                              else if (optionVariables.axisTypeY.equals("LogAxis"))
                                if (yVal[i]-downError>0)
                                  yGraph4 = Y.pix(Tools.log10(yVal[i]-downError), false);
                                else yGraph4 = 0;
                              else if (optionVariables.axisTypeY.equals("InverseAxis"))
                                if (yVal[i]-downError!=0) {
                                  yGraph4 = Y.pix((1/yVal[i]-downError), false);
                                } else {
                                  System.out.println("Error: attempt to use inverse axis with a zero data value");
                                  yGraph4 = dimens.height;
                                }
                                if (yGraph4>=minPixY) { // if the lower error bar is in the box draw it
                                  g.drawLine( xGraph1, dimens.height-yGraph4, xGraph1, dimens.height-yGraph1 );
                                  // Draw edge tic of error bar if wanted
                                  if (optionVariables.errorBarTicsMenuItem.getState())
                                    g.drawLine( xGraph1-1, dimens.height-yGraph4, xGraph1+1, dimens.height-yGraph4 );
                                } else { // draw the error only until the edge of the bottom of the plot box
                                  g.drawLine( xGraph1, dimens.height-minPixY, xGraph1, dimens.height-yGraph1 );
                                }
                          } // if graph line drawn or !none
                        } //if error bars

                      } // if pixel (points and error bars)
                     }
                    } catch (Exception e) {
                       //((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with loading the X and Y Axes Data" );
                       break;
                    }
                   } // 'for' loop (for points and error bars)
                  } // 'if' command for the negative log values
                  else  Tools.fail("Error in Plot2D: wrong String for graphTypeOther. ");
                } // main 'for' loop for graphs
              } catch (Exception e) {
                System.err.println(e.getMessage());
              }
            }


      public void drawCaption(Graphics g) {
        try {
          String captionsText = optionVariables.captionsTextArea.getText();
          if (!captionsText.equalsIgnoreCase("")) {

            StringTokenizer lineTokenizer = new StringTokenizer(captionsText, "\n", false);
            while (lineTokenizer.hasMoreTokens()) {
              String lineTemp = lineTokenizer.nextToken();

              StringTokenizer captionTokenizer = new StringTokenizer(lineTemp, ";", false);
              int numTokens = captionTokenizer.countTokens();

              // Format: "caption; x; y; show; angle; size; color; type; style;"
              // Retreive the variables
              String caption = captionTokenizer.nextToken();
              double captionX = Double.parseDouble( captionTokenizer.nextToken().trim() );
              double captionY = Double.parseDouble( captionTokenizer.nextToken().trim() );
              Boolean showTemp = new Boolean(captionTokenizer.nextToken().trim());
              boolean captionShow = showTemp.booleanValue();
              double captionAngle = Double.parseDouble( captionTokenizer.nextToken().trim() );
              int captionSize = Integer.parseInt( captionTokenizer.nextToken().trim() );
              String colorTemp = captionTokenizer.nextToken().trim();
              Color captionColor = PlotOptionConstants.COLOR[0];
              for (int i = 0; i<PlotOptionConstants.COLOR.length; i++) {
                if (optionVariables.COLORS[i].toUpperCase().equals(colorTemp.toUpperCase().trim())) {// note that I am comparing to the PlotOptionsFrame List since it contains Strings (not colors)
                  captionColor = PlotOptionConstants.COLOR[i];
                }
              }
              String captionType = captionTokenizer.nextToken().trim();
              String styleTemp = captionTokenizer.nextToken().trim();
              int captionStyle = PlotOptionConstants.FONT_STYLE[0];
              for (int i = 0; i<PlotOptionConstants.FONT_STYLE.length; i++) {
                if (optionVariables.FONT_STYLES[i].equals(styleTemp.toUpperCase().trim())) {// note that I am comparing to the PlotOptionsFrame List since it contains Strings (not colors)
                  captionStyle = PlotOptionConstants.FONT_STYLE[i];
                }
              }

              if (captionShow) {
                // Set the Color
                g.setColor(captionColor);
                // Set the Fonts
                Font font = new Font( captionType, captionStyle, captionSize);
                g.setFont(font);
                FontMetrics fMetric = getFontMetrics(font);

                // Write Label left of Y  Axis
                Graphics2D g2 = (Graphics2D) g;
                g2.translate( X.pixel(captionX,true) , dimens.height-Y.pixel(captionY,false));
                g2.rotate( Math.toRadians(captionAngle));
                g2.drawString(caption, 0, 0);

                // return to the old graphics setting
                g2.rotate( - Math.toRadians(captionAngle));
                g2.translate( -X.pixel(captionX,true) , -dimens.height+Y.pixel(captionY,false));
              }
            }

          }
        } catch (Exception e) {
          System.err.println("Error: Plot Caption settings (drawCaption Method):"+e.getMessage());
       //   ((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with the Caption on the plot, check your settings" );
        }
      }


                /**
                 * This method finds the error value for a certain point
                 * which is in the j-th vector (which is a certain DataColumn)and i-th array value of the
                 * Series class and looks for the error which is oriented
                 * in the 'str' direction.
                 *
                 * The String sent determines the error wanted and it can be: xLeft, xRight, yUp, yDown
                 */
                 public double FindGraphError(String str, int j, int i) throws Exception {
				// Get X errors
				if (str.equals("xLeft")) return Tools.objToDoub(((DataColumn) ser.getDataX(j)).getErrorMinus(i));
				else if (str.equals("xRight")) return Tools.objToDoub(((DataColumn) ser.getDataX(j)).getErrorPlus(i));
				// Get Y errors
				else if (str.equals("yUp"))	return Tools.objToDoub(((DataColumn) ser.getDataY(j)).getErrorPlus(i));
				else if (str.equals("yDown"))	return Tools.objToDoub(((DataColumn) ser.getDataY(j)).getErrorMinus(i));
				else Tools.fail("Error in Plot2D method FindGraphError: wrong String for str: "+ str);
				return 666.0;    // Dummy return
		}


////
                /*
		public void setPlotOptionsFrame(PlotOptionsFrame optionVariables) {
				this.optionVariables = optionVariables;
                                // 'adjust' the stack since the new plot has been loaded
                                if (optionVariables.stack.size()==0) {} // do nothing if for any reason, the stack is empty
                                else
                                  // in case the min and max values have changed, that will be the first stack setting
                                  optionVariables.stack.setElementAt(new PlotBorders(absMinX, absMaxX, absMinY, absMaxY),0);
		}*/

		/** Method for the Plot Title */
		public void setPlotTitle(String title) { plotTitle = new String(title); }
		/** Method for the Plot Title */
		public String getPlotTitle() { return plotTitle; }

		/**


              /** This method loads values for graphing column values X and Y axis.
                 *  It also includes two error values for both X and Y values
                 *  If some error bars are not wanted, they shold be sent as null
                 *  X and Y values, X errors, Y errors are each loaded into three Series Classes
                 */
                 public void loadXY( DataColumn dataX, DataColumn dataY) {
                try {

                 // Check if the plot is being printed with a Monotonic X Axis
                 // If so: order all the dataColumns using the Sort() Class with a reference to X axis (dataX)
                 if (optionVariables.monotonicMenuItem.getState()){
                   // Create a new DataTable (just for the purpose of ordering all these columns) and send it to the Sort Class
                   DataTable sortDataTable = new DataTable("foopath"); // filepath irrelevant since this DataTable is used temporalily
                   sortDataTable.addColumn((DataColumn)dataX);
                   sortDataTable.addColumn((DataColumn)dataY);
                   Sort sort = new Sort(sortDataTable, 0);
                   // This is yet another dummy default datatable that is used to load the sorted columns
                   DataTable sortedDataTable = sort.sortAll();
                   // Exchange the new sorted values into the existing columns
                   dataX = (DataColumn)sortedDataTable.getColumn(0);
                   dataY = (DataColumn)sortedDataTable.getColumn(1);
                 }

                 if (dataX.getSize()==0 || dataY.getSize()==0)
                   Tools.fail("Error: in plot2D, DataColumns sent is empty");
                 countGraphs++;
                 // Set the number of elements in columns dataX and dataY
                 graphPoints.addElement( Tools.intToObj(dataX.getSize()) );
                 // Set the minimal and maximal values of columns dataX and dataY
                 Object tempMinX = searchMin(dataX);
                 this.minX.addElement(tempMinX);
                                //if (absMinX) absMinX=Tools.objToDoub(tempMinX);
                                //else {
                                  // NOTE! If this is the first graph (countGraphs==1),
                                  // then absMinX hasn't been created jet so load whatever the new min value is
                                  if ( absMinX>Tools.objToDoub(tempMinX) || countGraphs==1)
                                    absMinX=Tools.objToDoub(tempMinX);
                                //}
                                Object tempMaxX = searchMax(dataX);
				this.maxX.addElement(tempMaxX);
                                //if (absMaxX==null) absMaxX=Tools.objToDoub(tempMaxX);
                                //else {
                                  // NOTE! If this is the first graph (countGraphs==1),
                                  // then absMaxX hasn't been created jet so load whatever the new max value is
                                  if ( absMaxX<Tools.objToDoub(tempMaxX) || countGraphs==1)
                                    absMaxX=Tools.objToDoub(tempMaxX);
                                //}
                                // if Min and Max are still equal (horizontal line)
                                if (absMinX==absMaxX) {
                                  absMaxX++;
                                    if (absMinX-1<=0) {
                                      absMinX+=1;
                                      absMinX/=10;
                                  }
                                }
                                Object tempMinY = searchMin(dataY);
                                this.minY.addElement(tempMinY);
                                //if (absMinY==null) absMinY=Tools.objToDoub(tempMinY);
                                //else {
                                  // NOTE! If this is the first graph (countGraphs==1),
                                  // then absMaxX hasn't been created jet so load whatever the new max value is
                                  if ( absMinY>Tools.objToDoub(tempMinY) || countGraphs==1)
                                    absMinY=Tools.objToDoub(tempMinY);
                                //}
                                Object tempMaxY = searchMax(dataY);
                                this.maxY.addElement(tempMaxY);
                                //if (absMaxY==null) absMaxY=Tools.objToDoub(tempMaxY);
                                //else {
                                  // NOTE! If this is the first graph (countGraphs==1),
                                  // then absMaxX hasn't been created jet so load whatever the new max value is
                                  if ( absMaxY<Tools.objToDoub(tempMaxY) || countGraphs==1)
                                    absMaxY=Tools.objToDoub(tempMaxY);
                                //}
                                // if Min and Max are still equal (horizontal line)
                                if (absMinY==absMaxY) {
                                  absMaxY++;
                                  if (absMinY-1<=0) {
                                    absMinY+=1;
                                    absMinY/=10;
                                  }
                                }

                              // Load the Series Class with data
                              ser.LoadXY(dataX, dataY);

               }
               catch (Exception e) {
                       System.err.println(e.getMessage());
            }
		}

		public boolean isEmpty() {
				return (countGraphs==0);
		}

                /**
                 * This method prints the graphic contents of Plot2D
                 */
             /*   public int print(Graphics g, PageFormat pf, int pi) throws PrinterException {
                  if (pi >= 1) {
                    return Printable.NO_SUCH_PAGE;
                  }
                  // now assign this page format value
                  if (portraitCheckbox.getState()==true) pf.setOrientation(PageFormat.PORTRAIT);
                  if (landscapeCheckbox.getState()==true) pf.setOrientation(PageFormat.LANDSCAPE);
                  if (revLandscapeCheckbox.getState()==true) pf.setOrientation(PageFormat.REVERSE_LANDSCAPE);

                  g.setClip(0,0,dimens.width,dimens.height);
System.out.println("width: "+dimens.width+" height: "+dimens.height);
                  this.paint(g);
                  return Printable.PAGE_EXISTS;
                }

                /**
                 * This method searches the DataColumn to find the minimal value
                 */
                public Object searchMin(DataColumn data) {
                  Object min = data.getValue(0);
                    for(int i=1; i<data.getSize(); i++) {
                      try {
                        if ( Tools.objToDoub(min) > Tools.objToDoub(data.getValue(i)) ) min = data.getValue(i);
                       } catch (Exception e) {
                    //   ((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with loading the X and Y Axes Data" );
                       break;
                      }
                    }
                  return min;
                }

                /**
                 * This method searches the DataColumn to find the maximal value
                 */
                public Object searchMax(DataColumn data) {
                  Object max = data.getValue(0);
                  for(int i=1; i<data.getSize(); i++) {
                    try {
                      if ( Tools.objToDoub(max) < Tools.objToDoub(data.getValue(i)) ) max = data.getValue(i);
                    } catch (Exception e) {
               //        ((GUI)parent).popAlertDialog(" ","   Alert: ", " There is a problem with loading the X and Y Axes Data" );
                       break;
                    }
                  }
                  return max;
                }

                /*
                * Event Handelers for Zoom Feature
                */


                 /**
                  * These are KeyListener Methods
                  */
                  public void keyPressed(KeyEvent e) {
                    if (e.getKeyCode()==e.VK_Z) {
                      // Make sure that the last row is empty (in case the user was playing around with it

                      if (!optionVariables.captionsTextArea.getText().endsWith("\n") && !optionVariables.captionsTextArea.getText().trim().equals(""))
                       optionVariables.captionsTextArea.append("\n");

                      // Format: "caption; x; y; show; angle; size; color; type; style;"
                     optionVariables.captionsTextArea.append("Enter_Caption_Here; "+ (String)plot2D.xCoordinate.getText()+
                                                                                 "; "+(String)plot2D.yCoordinate.getText()+
                                                                                "; true; 0"+ // show; angle
                                                                                 "; 15"+ // size
                                                                                 "; black"+ // color
                                                                                 "; "+optionVariables.labelFontType+ // type
                                                                                 "; plain;"+ // style
                                                                                 "\n");
                     ((GUI)parent).captionsFrameDisplay();
System.out.println("optionVariables.labelFontStyle "+optionVariables.labelFontStyle);
         // System.out.println(optionVariables.captionsTextArea.getText().endsWith("\n"));

                    }
                  }
                  public void keyReleased(KeyEvent e) {}
                  public void keyTyped(KeyEvent e) {}

                /**
                 * This method detects the mouse events for zoom-in
                 */
                  public void mousePressed(MouseEvent e) {
                    Graphics g = getGraphics();
                    // This Box draws over a box that the user might have already selected (basically, erases it)
                    g.setColor(optionVariables.backgroundColor);
                      //Color cTemp = Color.OPAQUE;// new Color(1.0f, 0.0f, 0.0f);//, 0.5f);
                      //g.setColor(cTemp);
                    g.drawLine(zoomBoxX2,zoomBoxY2,zoomBoxX2,zoomBoxY1);
                    g.drawLine(zoomBoxX2,zoomBoxY2,zoomBoxX1,zoomBoxY2);
                    g.drawLine(zoomBoxX2,zoomBoxY1,zoomBoxX1,zoomBoxY1);
                    g.drawLine(zoomBoxX1,zoomBoxY2,zoomBoxX1,zoomBoxY1);


                    zoomBoxX1 = e.getX();
                    zoomBoxY1 = e.getY();

                    g.setColor(optionVariables.borderlineColor);
/*                    g.drawString("(" + Double.toString(Tools.fraction(X.value(zoomBoxX1),2))
				 +", " + Double.toString(Tools.fraction((Y.value(maxPixY) + Y.value(minPixY) - Y.value(zoomBoxY1)),2)) + ")",
                                 zoomBoxX1+6, zoomBoxY1);*/

                    // the value for y axis is 'flipped' (min is max and vice versa) so fix that
                    // this is due the fact that applet plot base is in the upper left corner

/* Note! (Bug) The Tools.fraction function doesn't work since GUI plots involve extremely small numbers
                Solution: design a function that slides the floating point to the first number and expresses the number
                in 10-n..., This function can be designed within the Tools Class. */
                  /*zoomAbsMinX=Tools.fraction(X.value(zoomBoxX1),2);
                    zoomAbsMinY= Tools.fraction((Y.value(maxPixY) + Y.value(minPixY) - Y.value(zoomBoxY1)),2);
                  */
                    zoomAbsMinX=X.value(zoomBoxX1, true);
                    zoomAbsMinY=Y.value(dimens.height - zoomBoxY1, false);

//System.out.println("zoomX1 "+zoomAbsMinX+" zoomY1 "+zoomAbsMinY);
                  }

                  public void mouseReleased(MouseEvent e) {
                    // This makes sure that the zoom will be performed (when the "zoom in" button is pressed) only
                    // if there is a box drawn
                    zoomOn = true; //turn the zoom on

                    Graphics g = getGraphics();
                    zoomBoxX2 = e.getX();
                    zoomBoxY2 = e.getY();

                    // Draw a box to where the zoom can be performed by pressing the 'Zoom In' Button
                    // note: the zoomBoxX1 and zoomBoxY1 values have been obtained when mouse was pressed
                    g.drawLine(zoomBoxX2,zoomBoxY2,zoomBoxX2,zoomBoxY1);
                    g.drawLine(zoomBoxX2,zoomBoxY2,zoomBoxX1,zoomBoxY2);
                    g.drawLine(zoomBoxX2,zoomBoxY1,zoomBoxX1,zoomBoxY1);
                    g.drawLine(zoomBoxX1,zoomBoxY2,zoomBoxX1,zoomBoxY1);

                    // the value for y axis is 'flipped' (min is max and vice versa) so that is 'fixed'
                    // this is due the fact that applet plot base is in the upper left corner

                    // Note: The mouseReleased method ets called twice when the mouse is released
                    //       therefore: ignore it the first time
                    if (!mouseReleasedSecond) { //this is the first one
                      mouseReleasedSecond = true;
                    }
                    else
                    {

                      // We don't know which way the user will move the mouse before releasing
                      // for this reason create temporary variables

/* See Note! in the 'moouse Pressed event..*/
                    /*double tempAbsX = Tools.fraction(X.value(zoomBoxX2),2);
                      double tempAbsY = Tools.fraction((Y.value(maxPixY) + Y.value(minPixY) - Y.value(zoomBoxY2)),2);*/
                      double tempAbsX = X.value(zoomBoxX2, true);
                      double tempAbsY = Y.value(dimens.height - zoomBoxY2, false);

//System.out.println("zoomX2 "+tempAbsX+"  "+zoomAbsMinX);
                      if (tempAbsX>zoomAbsMinX) zoomAbsMaxX=tempAbsX;
                      else { zoomAbsMaxX=zoomAbsMinX; zoomAbsMinX=tempAbsX; }
                      // if Min and Max are equal (mouse released at the same spot)
//                      else { zoomAbsMaxX++; if (zoomAbsMinX-1<=0) { zoomAbsMinX+=1; zoomAbsMinX/=10; } }
                      if (tempAbsY>zoomAbsMinY) zoomAbsMaxY=tempAbsY;
                      else { zoomAbsMaxY=zoomAbsMinY; zoomAbsMinY=tempAbsY; }
                      // if Min and Max are equal (mouse released at the same spot)
//                     else { zoomAbsMaxY++; if (zoomAbsMinY-1<=0) { zoomAbsMinY+=1; zoomAbsMinY/=10; } }
//System.out.println("zoomX2 "+tempAbsX+"  "+zoomAbsMinX);

                      mouseReleasedSecond = false; // Restart this value
                    }
                  }

                // These methods are not used, but have to be implemented since PlotApplet implements MouseListener
                  public void mouseExited(MouseEvent e) {}
                  public void mouseEntered(MouseEvent e) {}
                  public void mouseClicked(MouseEvent e) {}

                  // MouseMotionListener
                  public void mouseDragged(MouseEvent e) {
                    Graphics g = getGraphics();
                    // Draw the X axis 'cursor' (above and below)
                    if (e.getX()>minPixX && e.getX()<maxPixX && e.getY()<(int)dimens.getHeight()-minPixY && e.getY()>(int)dimens.getHeight()-maxPixY) {

                      // Fill the X and Y Coordinate TextFields in the plot2D Panel
                      double xValue = X.value(e.getX(), true);
                      double yValue = Y.value((int)dimens.getHeight()-e.getY(), false);
                      String xValueString = "";
                      String yValueString = "";
                      // Display x
                      if (optionVariables.axisTypeX.equals("LogAxis")) {
                        xValueString = "10e"+String.valueOf(xValue);
                      } else {
                        xValueString = String.valueOf(xValue);
                      }
                      if (xValueString.length()>8) xValueString = xValueString.substring(0,8);
                      plot2D.xCoordinate.setText(xValueString);
                      // Display y
                      if (optionVariables.axisTypeY.equals("LogAxis")) {
                        yValueString = "10e"+String.valueOf(yValue);
                      } else {
                        yValueString = String.valueOf(yValue);
                      }
                      if (yValueString.length()>8) yValueString = yValueString.substring(0,8);
                      plot2D.yCoordinate.setText(yValueString);


                      // Draw the Cursors
                      g.setColor(optionVariables.backgroundColor);  // Set to background color
                         // X axis clear
                      g.fillRect(minPixX, (int)dimens.getHeight()-maxPixY-7, maxPixX-minPixX, 6 ); // Clear the up cursor
                      g.fillRect(minPixX, (int)dimens.getHeight()-minPixY+1, maxPixX-minPixX, 7 ); // Clear the down cursor
                         // Y axis clear
                      g.fillRect(minPixX-7, (int)dimens.getHeight()-maxPixY, 6, (int)maxPixY-minPixY ); // Clear the left cursor
                      g.fillRect(maxPixX+1, (int)dimens.getHeight()-maxPixY, 7, (int)maxPixY-minPixY ); // Clear the right cursor
                      g.setColor(optionVariables.borderlineColor);  // Set to borderline color
                         // X axis cursor
                      g.drawLine( e.getX(), (int)dimens.getHeight()-maxPixY-7, e.getX(), (int)dimens.getHeight()-maxPixY-2 ); // this is the up 'cursor'
                      g.drawLine( e.getX(), (int)dimens.getHeight()-minPixY+2, e.getX(), (int)dimens.getHeight()-minPixY+7 ); // this is the down 'cursor'
                         // Y axis cursor
                      g.drawLine( minPixX-7 , e.getY(), minPixX-2, e.getY() ); // this is the left 'cursor'
                      g.drawLine( maxPixX+ 2 , e.getY(), maxPixX+7, e.getY() ); // this is the right 'cursor'

                      // Draw the box
                    //  g.drawLine(zoomBoxX1, zoomBoxY1, e.getX(), e.getY()); ///....
                    }
                  }
                  public void mouseMoved(MouseEvent e) {
                    Graphics g = getGraphics();
                    // Draw the X axis 'cursor' (above and below)
                    if (e.getX()>minPixX && 
                            e.getX()<maxPixX && 
                            e.getY()<(int)dimens.getHeight()-minPixY && 
                            e.getY()>(int)dimens.getHeight()-maxPixY) {

                      // Fill the X and Y Coordinate TextFields in the plot2D Panel
                      double xValue = X.value(e.getX(), true);
                      double yValue = Y.value((int)dimens.getHeight()-e.getY(), false);
                      String xValueString = "";
                      String yValueString = "";
                      // Display x
                      if (optionVariables.axisTypeX.equals("LogAxis")) {
                        xValueString = "10e"+String.valueOf(xValue);
                      } else {
                        xValueString = String.valueOf(xValue);
                      }
                      if (xValueString.length()>8) xValueString = xValueString.substring(0,8);
                      plot2D.xCoordinate.setText(xValueString);
                      // Display y
                      if (optionVariables.axisTypeY.equals("LogAxis")) {
                        yValueString = "10e"+String.valueOf(yValue);
                      } else {
                        yValueString = String.valueOf(yValue);
                      }
                      if (yValueString.length()>8) yValueString = yValueString.substring(0,8);
                      plot2D.yCoordinate.setText(yValueString);

/* This is another algorythm, not very good, but there are some good ideas..
                    try {
                      int xNumFraction = -1;
                      int yNumFraction = -1;
                      // Caluculate the fraction for x
                      if (Tools.log10(Math.abs(xValue))>=1) xNumFraction=1;
                      else if (Tools.log10(Math.abs(xValue))<1 && Tools.log10(Math.abs(xValue))>=0) xNumFraction=2;
                      else if (Tools.log10(Math.abs(xValue))<0) {
                        int i = 0;
                        double tempxValue = xValue;
                        while ( tempxValue>=0) {
                          tempxValue = tempxValue*10;
                          i++;
                        }
                        xNumFraction=i+5;
                      }
                      plot2D.xCoordinate.setText( String.valueOf( Tools.fraction(xValue, xNumFraction) ) );
                      // Caluculate the fraction for y
                      if (Tools.log10(Math.abs(yValue))>=1) yNumFraction=1;
                      else if (Tools.log10(Math.abs(yValue))<1 && Tools.log10(Math.abs(yValue))>=0) yNumFraction=2;
                      else if (Tools.log10(Math.abs(yValue))<0) {
                        int i = 0;
                        double tempyValue = yValue;
                        while ( tempyValue>=0) {
                          tempyValue = tempyValue*10;
                          i++;
                        }
                        yNumFraction=i+5;
                      }
                      plot2D.yCoordinate.setText( String.valueOf( Tools.fraction(yValue, yNumFraction) ) );
                     } catch (Exception ee) {System.err.println(ee.getMessage());} */

                      // Draw the Cursors
                      g.setColor(optionVariables.backgroundColor);  // Set to background color
                         // X axis clear
                      g.fillRect(minPixX, (int)dimens.getHeight()-maxPixY-7, maxPixX-minPixX, 6 ); // Clear the up cursor
                      g.fillRect(minPixX, (int)dimens.getHeight()-minPixY+1, maxPixX-minPixX, 7 ); // Clear the down cursor
                         // Y axis clear
                      g.fillRect(minPixX-7, (int)dimens.getHeight()-maxPixY, 6, (int)maxPixY-minPixY ); // Clear the left cursor
                      g.fillRect(maxPixX+1, (int)dimens.getHeight()-maxPixY, 7, (int)maxPixY-minPixY ); // Clear the right cursor
                      g.setColor(optionVariables.borderlineColor);  // Set to borderline color
                         // X axis cursor
                      g.drawLine( e.getX(), (int)dimens.getHeight()-maxPixY-7, e.getX(), (int)dimens.getHeight()-maxPixY-2 ); // this is the up 'cursor'
                      g.drawLine( e.getX(), (int)dimens.getHeight()-minPixY+2, e.getX(), (int)dimens.getHeight()-minPixY+7 ); // this is the down 'cursor'
                         // Y axis cursor
                      g.drawLine( minPixX-7 , e.getY(), minPixX-2, e.getY() ); // this is the left 'cursor'
                      g.drawLine( maxPixX+ 2 , e.getY(), maxPixX+7, e.getY() ); // this is the right 'cursor'
                    }
                  }

                // Handle events on the Buttons
                  public void actionPerformed(ActionEvent e) {
                    String command = ((Button)e.getSource()).getLabel();
                    Graphics g = getGraphics();

                    // If a 'box' has been selected with the mouse in the plot, it will zoom into that area
                    // Otherwise, it will just redraw the same plot
                    // Note: since the zoom option is being performed, there will be no auto-scalin anymore,
                    //  i.e. all four (min, max) values will have speciffic limit values
                    if (command.equals("Zoom In")) {
                      if (zoomOn) {
                        // Put the limit values that are about to be replaced onto the stack
                        optionVariables.stack.push(new PlotBorders(optionVariables.xMinText.getText().trim(),
                                                                   optionVariables.xMaxText.getText().trim(),
                                                                   optionVariables.yMinText.getText().trim(),
                                                                   optionVariables.yMaxText.getText().trim() ));
                        // Update the min and max values with the values obtained from the selected box in the plot
                        // if the axis is Logarithmic, transfer the log-ed values back to teir original values
                        if (optionVariables.axisTypeX.equals("LinearAxis")) {
                          optionVariables.xMinText.setText(String.valueOf(zoomAbsMinX));
                          optionVariables.xMaxText.setText(String.valueOf(zoomAbsMaxX));
                        }
                        else if (optionVariables.axisTypeX.equals("LogAxis")) {
                          optionVariables.xMinText.setText(String.valueOf(Math.pow(10, zoomAbsMinX)));
                          optionVariables.xMaxText.setText(String.valueOf(Math.pow(10, zoomAbsMaxX)));
                        }
                        if (optionVariables.axisTypeY.equals("LinearAxis")) {
                          optionVariables.yMinText.setText(String.valueOf(zoomAbsMinY));
                          optionVariables.yMaxText.setText(String.valueOf(zoomAbsMaxY));
                        }
                        else if (optionVariables.axisTypeX.equals("LogAxis")) {
                          optionVariables.yMinText.setText(String.valueOf(Math.pow(10, zoomAbsMinY)));
                          optionVariables.yMaxText.setText(String.valueOf(Math.pow(10, zoomAbsMaxY)));
                        }
                        // Reset Zoom Box values
                        zoomAbsMinX = zoomAbsMaxX = zoomAbsMinY = zoomAbsMaxY = 0;
                        //
                        update(g);
                        // disable the zoom feature until the nex box is selected on the graph
                        zoomOn = false;
                      }
                    }
                    else if (command.equals("Zoom Back")) {
                      if (optionVariables.stack.size()==0) {} // do nothing if for any reason, the stack is empty
                      else {
                        // take the new value off the stack
                        PlotBorders tempBorders = (PlotBorders) optionVariables.stack.peek();
                        optionVariables.xMinText.setText(tempBorders.getXMin());
                        optionVariables.xMaxText.setText(tempBorders.getXMax());
                        optionVariables.yMinText.setText(tempBorders.getYMin());
                        optionVariables.yMaxText.setText(tempBorders.getYMax());
                        // get rid of that value in the stack
                        optionVariables.stack.pop();

                        // Reset Zoom Box values
                        zoomAbsMinX = zoomAbsMaxX = zoomAbsMinY = zoomAbsMaxY = 0;
                        update(g);
                      }
                    }
/*                    else if (command.equals("Refresh")) {
                      if (optionVariables.stack.size()==0) {} // do nothing if the for any reason the stack is empty
                      else {
                        PlotBorders tempBorders = (PlotBorders) optionVariables.stack.firstElement(); //get the initial plot values
                        absMinX = tempBorders.getXMin();
                        absMaxX = tempBorders.getXMax();
                        absMinY = tempBorders.getYMin();
                        absMaxY = tempBorders.getYMax();

                        optionVariables.stack.clear(); // empty the stack

                        update(layer1.getGraphics());
                      }
                    }
                    else if (command.equals("Print")) {
                      PrinterJob printJob = PrinterJob.getPrinterJob();
                      printJob.setPrintable(PlotApplet.this);

                      // First, Pop-up a Dialog and let the user select a page format
                      Frame tempFrame = new Frame();
                      printPageFormatDialog = new Dialog(tempFrame, "Please choose page format:", true);
                      printPageFormatDialog.setLayout(new GridLayout(4,1));
                      printPageFormatDialog.add(portraitCheckbox);
                      printPageFormatDialog.add(landscapeCheckbox);
                      printPageFormatDialog.add(revLandscapeCheckbox);
                      printPageFormatDialog.add(pageFormatButton);

                      printPageFormatDialog.setSize(new Dimension(600,130));
                      printPageFormatDialog.setLocation(100,100);
                      printPageFormatDialog.show();  // display the dialog and block until answered

                      // Now check for frame width and height to make sure the graph is not too big for the page
                      // Note: I have estimated Max. width to be: 1300 and Max. height to be 1250 for PORTRAIT
                      //        and vice versa for LANDSCAPE and REVERSE LANDSCAPE page formats
                      int maxWidth = 1000;
                      int maxHeight = 900;
                      if (portraitCheckbox.getState()==true) {
                        // if the width is too big, just 'shrink it to fit in the upper area of page
                        if (dimens.width>maxWidth) dimens.setSize(maxWidth, maxHeight-200);
  System.out.println("width: "+maxWidth+" "+maxHeight);
                        // if the height is too bih too, then fit it in the whole page
                        if (dimens.height>maxHeight) dimens.setSize(maxWidth, maxHeight);
                      } else { // it is landscape mode
                        // here hight is actually width (and vice versa) so if too big, adjust it appropriately
                        if (dimens.width>maxHeight) dimens.setSize(maxHeight, maxWidth);
                        if (dimens.height>maxWidth) dimens.setSize(maxWidth, maxHeight);
                      }
                      // Finally, open the print dialog for the user to make settings
                      // Note: this print command will call GUI's print() method as well!
                      if (printJob.printDialog()) { // opens the print dialog for the user to make settings
                        try {
                          printJob.print();
                        } catch (Exception ex) {
                          ex.printStackTrace();
                        }
                      }

//                      Frame printFrame = new Frame();
//                      PrintJob myJob = this.getPrintJob(printFrame, "My Print",null);
                    }
                    // When printing, the user has to choose page format, this button is used for that purpose
                    else if (command.equals("Choose Format")) {
                      printPageFormatDialog.hide();
                    }*/

                  }

}
