package musr.client.xml;
import musr.client.datacolumn.*;
import musr.client.gui.*;
import musr.client.plot2d.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import java.util.Date;

/**
 *  This class exports the Data Tables from muView in XML format
 *
 * @author Ivan Tucakov
 */
public class ExportXML {

       /**
        * This is the GUI where the console is called from
        */
       public Frame parent;
       public FileDialog d;
       public DataTable dataTable;

       public ExportXML(Frame paren) {
         this.parent = paren;
         d = new FileDialog(paren, "Save selected DataTable into XML file", FileDialog.SAVE);

        }

        /**
         * This method is used to create and save the XML file on the hard drive
         */
        public void createXML() {
                    String filename = "";
                    try {
                      d.setFile("DataTable.xml");
                      d.setLocation(300,300);
                      d.show();  // display the dialog and block until answered
                      filename = d.getFile();   // Get the user's response
                      if ((filename != null) || (filename.length() != 0)) {           // If user didn't click "Cancel."
                          try {
                            // load the datatable if it exists
                            if ( (((GUI)parent).dataTableList.getSelectedIndex()!=-1) && (((GUI)parent).dataTableList.getItemCount() !=0) ) {

                              int dataTableKey = ((GUI)parent).getDataTableIndex();
                              dataTable = (DataTable) ((GUI)parent).dictionary.getElement(dataTableKey);

                              /*** This is where the file is written in *****/
                              // Use FileWriter and BufferedWriter to write the file
                              BufferedWriter outFile = new BufferedWriter( new FileWriter(d.getDirectory()+filename) );   // Create a stream to write

                              // Start the file
                              outFile.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>"); outFile.newLine();
                              outFile.newLine();
                              outFile.write("<!-- Go to http://www.ltg.ed.ac.uk/~richard/xml-check.html"); outFile.newLine();
                              outFile.write("     to validate the XML. Online version of this file at"); outFile.newLine();
                              outFile.write("     http://musr.triumf.ca/~angus/Sample.xml . -->"); outFile.newLine();
                              outFile.newLine();
                              outFile.write("<!-- This is a DTD (Document Type Definition) which specifies"); outFile.newLine();
                              outFile.write("     acceptable formatting. It can be placed in an external file.-->"); outFile.newLine();
                              outFile.newLine();
                              outFile.write("<!DOCTYPE compilation [ ");
                              outFile.newLine();

                              outFile.write("<!ELEMENT compilation (header,dataTable+)> <!-- At least one dataTable -->"); outFile.newLine();
                              outFile.newLine();
                              outFile.write("<!ELEMENT header (title, subtitle?, abstract?, comments?)> <!-- Only title required -->");
                              outFile.write("<!ELEMENT title (#PCDATA)>");
                              outFile.write("<!ELEMENT subtitle (#PCDATA)>");
                              outFile.write("<!ELEMENT abstract (#PCDATA)>");
                              outFile.write("<!ELEMENT comments (#PCDATA)>");
                              outFile.newLine();
                              outFile.write("<!ELEMENT dataTable (source?, colDefns, dataRows)>");
                              outFile.write("<!ELEMENT source (#PCDATA)>");
                              outFile.write("<!ELEMENT colDefns (column+)> <!-- At least one label -->");
                              outFile.write("<!ELEMENT column EMPTY>");
                              outFile.write(" <!ATTLIST column");
                              outFile.write("     type  (virtual|real) #REQUIRED");
                              outFile.write("      name  CDATA          #REQUIRED");
                              outFile.write("      label CDATA          #REQUIRED>");
                              outFile.newLine();
                              outFile.write("  <!ELEMENT dataRows (virtualdata, realdata)>");
                              outFile.newLine();
                              outFile.write("  <!ELEMENT virtualdata (var+)> <!-- At least one point -->");
                              outFile.newLine();
                              outFile.write("  <!ELEMENT realdata (row+)> <!-- At least one point -->");
                              outFile.write("  <!ELEMENT row (var+)> <!-- At least one var -->");
                              outFile.write("  <!ATTLIST row details CDATA #IMPLIED>");
                              outFile.newLine();
                              outFile.write("<!ELEMENT var EMPTY>");
                              outFile.write("<!ATTLIST var");
                              outFile.write("      name  CDATA #REQUIRED");
                              outFile.write("      begin CDATA #IMPLIED");
                              outFile.write("      incr  CDATA #IMPLIED");
                              outFile.write("      size  CDATA #IMPLIED");
                              outFile.write("      value CDATA #IMPLIED");
                              outFile.write("      posErr CDATA #IMPLIED");
                              outFile.write("      negErr CDATA #IMPLIED>");
                              outFile.newLine();
                              outFile.write("]>"); outFile.newLine();
                              outFile.newLine();
                              outFile.write("<!-- This is the start of the actual XML. -->"); outFile.newLine();
                              outFile.newLine();

                              outFile.write("<compilation>"); outFile.newLine();
                                outFile.write("<header>"); outFile.newLine();
                                if (dataTable.getTitle()!=null) {
                                  outFile.write("<title>"+dataTable.getTitle()+"</title>"); outFile.newLine(); }
                                if (dataTable.getAbstract()!=null) {
                                  outFile.write("<abstract>"+dataTable.getAbstract()+"</abstract>"); outFile.newLine(); }
                                if (dataTable.getComment()!=null) {
                                  outFile.write("<comments>"+dataTable.getComment()+"</comments>"); outFile.newLine(); }
                                outFile.write("</header>"); outFile.newLine();
                                outFile.newLine();

                                // DataTable
                                outFile.write("<dataTable>"); outFile.newLine();
                                  // Source
                                  outFile.write("<source>"); outFile.newLine();
                                    outFile.write("<![CDATA[ "); outFile.newLine();

                                    // Input all the info in the TableHistory of this DataTable
                                  //  for (int i=0; i< dataTable.getTableHistory().getRows(); i++) {
                                      outFile.write(dataTable.getTableHistory().getText()); outFile.newLine();
                                  //  }

                                    outFile.write("   ]]>"); outFile.newLine();
                                    outFile.write("</source>"); outFile.newLine();
                                      outFile.newLine();

                                  // Load the Columns
                                  outFile.write("<colDefns>"); outFile.newLine();
                                    // Traverse through all the columns
                                    for (int i=0; i<dataTable.columnCount(); i++) {
                                      // In case any of these fields contain the quote mark ", subtitute it for \"
                                   //   String type = dataTable.getColumn(i).getType();
                                   //   while (type.indexOf("\"")!=-1 ) type substring(type.indexOf("\"",type.indexOf("\""+1)
                                      outFile.write("<column type=\""+dataTable.getColumn(i).getType()+
                                                                     "\" name=\""+dataTable.getColumn(i).getName()+
                                                                     "\" label=\""+dataTable.getColumn(i).getLabel()+"\" />"); outFile.newLine(); }
                                    outFile.write("</colDefns>"); outFile.newLine();
                                  outFile.newLine();

                                  // Load the Data
                                  outFile.write("<dataRows>"); outFile.newLine();
                                    // Load Virtual Data
                                    outFile.write("<virtualData>"); outFile.newLine();
                                    // Traverse through all the columns and find the virtual (linear) ones
                                    for (int i=0; i<dataTable.columnCount(); i++) {
                                      if (dataTable.getColumn(i).getType().equals(new String("virtual"))) {
                                          outFile.write("<var name=\""+((LinearColumn)dataTable.getColumn(i)).getName()+
                                                                   "\" begin=\""+((LinearColumn)dataTable.getColumn(i)).begin+
                                                                   "\" step=\""+((LinearColumn)dataTable.getColumn(i)).step+
                                                                   "\" size=\""+((LinearColumn)dataTable.getColumn(i)).size+"\" />"); outFile.newLine();
                                      }
                                    }
                                    outFile.write("</virtualData>"); outFile.newLine();
                                    outFile.newLine();

                                    // Load Real Data
                                    outFile.write("<realData>"); outFile.newLine();
                                    // Traverse through all the rows
                                    for (int j=0; j<dataTable.getColumn(0).getSize(); j++) {
                                      // This field determines if a new row has been started. This is so that the line <row> wouldn't be written too many times
                                      boolean rowStarted = true;
                                      // Traverse through all the columns and find the real ones
                                      for (int i=0; i<dataTable.columnCount(); i++) {
                                        if (dataTable.getColumn(i).getType().equals(new String("real"))) {
                                          // Start a new Row (unless it was started already)
                                          if (rowStarted == true) {
                                            outFile.write("<row>"); outFile.newLine();
                                          }
                                          // Make it false and from now on it will not draw <row> until it reaches the end of the file
                                          rowStarted = false;
                                              outFile.write("<var name=\""+((DataColumn)dataTable.getColumn(i)).getName()+
                                                                "\" value=\""+((DataColumn)dataTable.getColumn(i)).getValue(j)+
                                                                "\" posErr=\""+((DataColumn)dataTable.getColumn(i)).getErrorPlus(j)+
                                                                "\" negErr=\""+((DataColumn)dataTable.getColumn(i)).getErrorMinus(j)+"\" />"); outFile.newLine();
                                        }
                                      }
                                      // Close this row and move on to the next one
                                      // note: I check for the rowStarted to be false because it will be false ONLY IF drew at least one row.
                                      if (rowStarted == false) {
                                        outFile.write("</row>"); outFile.newLine();
                                        outFile.newLine();
                                      }
                                    }
                                    outFile.write("</realData>"); outFile.newLine();
                                    outFile.newLine();

                                  outFile.write("</dataRows>"); outFile.newLine();
                                  outFile.newLine();

                                outFile.write("</dataTable>"); outFile.newLine();
                                outFile.newLine();

                              outFile.write("</compilation>"); outFile.newLine();
                              outFile.newLine();

                              outFile.close();
                            }
                            // inform the user that the GUI is in the process of being saved
                            ((GUI)parent).statusTextField.setText("Please wait, creating the XML file... Time Left: 2");

                            // inform the user that the GUI is in the process of being saved
                            ((GUI)parent).statusTextField.setText("Please wait, creating the XML file... Time Left: 1");
                          }

                          // Print out exceptions.
                          catch (IOException ioe) { System.out.println("I/O Exception in file writing: "+ioe); }
                        }
                      } catch (Exception ce) {
                               System.err.println(ce.getMessage());
                      }
                      if (filename != null)
                        ((GUI)parent).statusTextField.setText("The XML file has been created and saved:  "+filename);
                      else ((GUI)parent).statusTextField.setText("");
        }


}