package musr.client.gui;
import musr.client.datacolumn.*;
import musr.client.dbreader.*;
import musr.client.xml.*;
import musr.client.csv.*;
import musr.client.function.*;
import musr.client.plot2d.*;
import musr.client.dbchecker.*;
import musr.server.GUI_RMIServer;
import musr.server2.GUI_RMIServer2;
import musr.spect.*;
import com.sun.image.codec.jpeg.*;  // for image capture (printing)
import java.awt.*;       // for Frame()
import java.applet.Applet;
import java.awt.image.*;
import java.awt.List;
import java.awt.event.*;
import java.awt.print.*;
import java.net.URL;
import java.util.*;      // for Vector(), Hashtable()
import java.io.*;
import java.lang.*;      // for Long()
import java.security.*; // Access Control
//import javax.print;

import java.rmi.Naming;
import java.rmi.RemoteException;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.imageio.*;

//for printing to PS
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;


/**
 * Plotting GUI
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class GUI extends JFrame implements Printable {


  /*## Main Buffer ##*/

  /**  Getting the db objects through musr RMI server */
  GUI_RMIServer obj = null;
  /**  Getting the db objects through cmms RMI server2 */
  GUI_RMIServer2 obj2 = null;

  /** This variable is used to open a local file
   *  This field is made static because the bufferedreader which this class implements is not
   *  serializable and therefore can't be copied (cloned) by ObjectCloner */
  private /*static*/ musr.client.dbreader.FileLineReader inFile = null;

  /** Colection of (DataTable)-s or anything else
   * This is where all the imported DataTables/columns are stored */
  public Dictionary dictionary = new Dictionary();
  /** This variable is used to treat parameters and constants as case-sensitive or case-insensitive */
  //public boolean caseOn = false; // make it case-insensitive by default
  /** These variables are used to calculate the size of the symbols in the graph.
   *  The calculation is size=nPix/m where nPix is the width of the plot in pixels and m = max(N,Nmax) where N is
   *  the number of points for the graph that is being plotted withiin this plot.
   */
   public long Nmax = 20;
   public long nPix = 300;


   /**
    * this variable is used for file loading.
    * When the user first loads a file, the directory path to that file is saved into this variable and then used when other files want to be loaded.
    * It can also be set using the 'cd' command in the DBConsole
    * Note that the default setting will be set to the /tmp (\\temp\) directory
    */
    public String dirPath = "/";

  /** The Data Variables from the db file and any other constants are stored in this class
   *  the values in it will be loaded everytime a new DataTable is imported, overwriting any previous
   *  values from some other previous DataTables (if any)
   *  It is also possible to define the constant variables manually */
  public Constants dataTableVariables = new Constants();

  /** This is a collection of 'custom' functions
   * (For now) they only take one parameter and when defined, the parameter should be x.*/
  public Functions functionsTable = new Functions();

  /**
   * THese variables hold the value of the size of the graph that will be plotted.
   * THe value is in centimeters.
   *  THese variables can be changed by the user in the plot setting menu otion.
   * Two vaues are for the Landscape mode and two are for Portrait mode.
   */

   public int plotWidthLand = 15;
   public int plotHeightLand = 20;

   public int plotWidthPort = 15;
   public int plotHeightPort = 12;

   /**
    *  This variable holds the plot format option for printing: portrait(1), landscape (0) or reverse-landscape(2)
    */
    public PageFormat pageForm = new PageFormat();
    public CheckboxGroup pageFormGroup = new CheckboxGroup();
    public Checkbox pageFormCheckL = new Checkbox("Landscape", pageFormGroup, true);
    public Checkbox pageFormCheckP = new Checkbox("Portrait", pageFormGroup, false);

    /**
     * This creates the Object that holds the instances and settings for printing of the plot
     */
     // This is what is used to print the GUI Plot
     PrinterJob printJob;
     String printerName = "";

     /**
      * This variable holds the dimentions of the User's screen
      */
      Toolkit tk = Toolkit.getDefaultToolkit();
      Dimension screenDimen;

  /*## Ploting Columns Buffer ##*/
  /**
   * When columns are added/removed from the ColumnsList, the columns
   * and information about them are saved in this variable
   */
  public Vector xAxisDataTable = new Vector(); // For each Frame there is a x-Axis DataTable of Columns created by the user to plot
  public Vector yAxisDataTable = new Vector();  // for each x-Axis column there is a y-Axis DataTable of Columns created by the user to plot
  Vector titlexAxis = new Vector();      // The title of the xAxis (Vector of Strings)
  Vector titleyAxis = new Vector();      //    (Vector of Strings)
  Vector namexAxis = new Vector();      // The name of the DataTable from dataTableList (Vector of Strings) - used for keeping track which datatable it came from
  Vector plotsRecord = new Vector();     // Plotting Records of columns in x-Axis List (PlotRecord-s)
  Vector plotsOptions = new Vector();    // Plotting Options, for each layout frame (PlotOptionsFrame)
		                                             // first one is added when layoutChoice is filled
  /*## Plots ##*/
  /** There is a seperate plot for each created Frame, this is a Vector of Plot2D's  */
  Vector plots = new Vector();
  // Create a plot Applet for printing
  Vector plotsPrint;

  /** This variable is only used to temporarly read in data before it is put into the dataTable */
  public Vector tempLabels = new Vector();
  /** This variable is only used to temporarly read in data before it is put into the dataTable */
  public Vector tempNames = new Vector();
  /** Holds the DataColumns from the GUI temporarily and performs various operations in it*/
  public DataTable tempDataColumns;
  /** This variable is only used to temporarly read in data before it is put into the dataTable */
  public Vector tempRowDescription = new Vector();

  /** Used to store negative errors of all the columns to have as a reference when evaluating expressions */
  Vector tempNegErr = new Vector();
  /** Used to store positive errors of all the columns to have as a reference when evaluating expressions */
  Vector tempPosErr = new Vector();
  /** Used to store errors calculation: sqrt(posErr^2 + negErr^2) of all the columns to have as a reference when evaluating expressions */
  Vector tempErr = new Vector();

  /*Holds the content pane of the main JFame of the GUI*/
  Container contentPane;
  
  // Menus used
  MenuBar menuBar;
    Menu fileMenu;
      MenuItem openMenuItem;
      MenuItem saveMenuItem;
      MenuItem savePNGMenuItem;
      MenuItem resetMenuItem;

    Menu loadMUDSpectrum;
      MenuItem integralDataMenuItem;
      MenuItem differentialDataMenuItem;

      MenuItem importFileMenuItem;
      MenuItem importLocalFileMenuItem;
      MenuItem openBlankMenuItem;
    Menu importMenu;
      MenuItem importXMLFileMenuItem;
      MenuItem importCSVFileMenuItem;
      MenuItem importFileMenuItem2;
      MenuItem importLocalFileMenuItem2;
      //MenuItem importURLMenuItem;
      MenuItem removeMenuItem;
    Menu exportMenu;
      MenuItem xmlMenuItem;
      MenuItem dbMenuItem;
      MenuItem csvMenuItem;
      MenuItem quitMenuItem;
    Menu viewMenu;
      MenuItem displayMenuItem;
      MenuItem optionsMenuItem;
      MenuItem dataTableMenuItem;
      MenuItem showMenuItem;
      MenuItem spreadSheetMenuItem;
      MenuItem funTableMenuItem;
      MenuItem conTableMenuItem;
    Menu toolsMenu;
        Menu dataMenu;
            MenuItem bestFitMenuItem;
            MenuItem fourierMenuItem;
            MenuItem splinesMenuItem;
      MenuItem dbCheckerItem;
      MenuItem commandLineConsoleItem;
      MenuItem consoleInstructionsItem;
      MenuItem framesMenuItem;
      MenuItem createFunMenuItem;
      MenuItem createConMenuItem;

      MenuItem titleLabelMenuItem;
      MenuItem axesMenuItem;
      MenuItem plotMenuItem;
      
   Menu globalDefaultsMenu;
      public CheckboxMenuItem caseMenuItem;

      MenuItem repackMenuItem;
      MenuItem symbSizeMenuItem;
      MenuItem printSetupMenuItem;
      MenuItem printMenuItem;
      MenuItem postScriptPrintMenuItem;
    public Menu optionsMenu;
      MenuItem axisTypesMenuItem;
      MenuItem graphTypesMenuItem;
      MenuItem titleSetupMenuItem;
      MenuItem limitsMenuItem;
      MenuItem captionsMenuItem;
      Menu optionsTextMenu;
      Menu optionsAxesMenu;
      Menu optionsPointsMenu;
      Menu optionsPlotMenu;
    Menu helpMenu;
      MenuItem manualMenuItem;
      MenuItem aboutMenuItem;

  // Components used
  JPanel dataTablePanel;
    public ScrollPane tableListScrollbar;
    public List dataTableList;
    List columnList;
//    Button addDataTableButton;
    JButton removeDataTableButton;
    JButton showDataTableInfoButton;
    JButton refreshDataTableButton;
    JButton columnButton;
    JButton statsButton;
    JButton fourierButton;
    JPanel framesPanel;
      CheckboxGroup gridGroup; // this is in the Frames Window
        Checkbox gridAutoCheckbox;
        Checkbox gridManualCheckbox;
      Choice gridXChoice;
      Choice gridYChoice;
      public Choice layoutChoice;
      Button layoutAddButton;
      Button layoutRemoveButton;
      JLabel layoutLabel;
    JPanel statusPanel;
      public TextField statusTextField;
    JPanel selectPanel;
      JPanel loadPanel;
        public List xAxisList;
        List yAxisList;
      JPanel buttonsPanel;
        JButton addXButton;
        JButton removeXButton;
        JButton addYButton;
        JButton removeYButton;
      JPanel buttonsPanel2;
        JButton conditionalPlotButton;
        JButton bestFitButton;
        JButton splineButton;
        JButton displayPlotButton;
        JButton repackDataButton;
      Checkbox invertedCheckbox;
      Checkbox visibleCheckbox;
	GridBagConstraints gridConstr = new GridBagConstraints();
    // These are used for file opening from musr server
    Dialog openFileDialog;
    TextField openFileTextField1 = new TextField("/home/itucakov/public_html/dbfiles/");
    TextField openFileTextField2 = new TextField("beauty.db");
    TextField openFileTextField3 = new TextField("/tmp/");
    TextField openFileTextField4 = new TextField("beauty.db");
    Button openFileButton = new Button("Open musr File");
    Button openFileButton2 = new Button("Cancel musr Open");
    // These are used for file opening from cmms server
    Dialog openCFileDialog;
    TextField openCFileTextField1 = new TextField("/home/itucakov/public_html/dbfiles/");
    TextField openCFileTextField2 = new TextField("beauty.db");
    TextField openCFileTextField3 = new TextField("/tmp/");
    TextField openCFileTextField4 = new TextField("beauty.db");
    Button openCFileButton = new Button("Open cmms File");
    Button openCFileButton2 = new Button("Cancel cmms Open");

    // This Dialog will contain one OK button and will be used for Displaying any kind of information,
    // error warnings or anything that would only have one OK button.
    Dialog infoDialog = new Dialog(GUI.this,"", true);
    Button OKButton = new Button("OK");

    CSVReader csvReader;        //Used to open .csv files
    
    // These are used for defining a new constant or a new function, defining sutomatic-symbol-size-setup options
    Dialog defDialog;
    TextField defTextField1 = new TextField();
    TextField defTextField2 = new TextField();
    TextField defTextField3 = new TextField();
    Button defConButton = new Button("Define Constant");
    Button defFunButton = new Button("Define Function");
    Button symbSizeButton = new Button("Settings OK");
    Button defButton2 = new Button("Cancel Def");

    // These are the fields and buttons for the Repack Dialog box
    Dialog repackDialog;
    TextField repackField = new TextField();
    Button repackOKButton = new Button("OK");
    Button repackCancelButton = new Button("Cancel");
   
    // These are used for Printing, to select a plot size for the appropriate page format
    Dialog printPageFormatDialog;
    Button pageFormatButton = new Button("Choose Plot Size Format");

    // Windows used
    PlotFrame plotFrame = new PlotFrame();               // called from displayMenuItem MenuItem
    Dialog showDialog = new Dialog(GUI.this);            // called from abstact and comment Item's in the menu
    public DBConsole console = new DBConsole(GUI.this, "Command Line Console"); // called from 'command' MenuItem
    public MUDSpectrumFrame mudSpectFrame;

    // this Frame will contain list of all the loaded Data Tables and their Columns
    public JFrame dataTableFrame;
    public ScrollPane tableFrameScrollbar;
    public JPanel tablePanel;
    GridBagLayout tablePanelGridBag;
    GridBagConstraints tablePanelConstraints;
    Button dataTableButton = new Button("Close Data Table Window");
    // This frame will contain x, y-axis addition and plot options
    // This frame will contain the new Frames set-up options
    public JFrame framesFrame = new JFrame("Frames");
    Button framesButton = new Button("Close Frames Window");


    // This Dialog can be used for any occasion when GUI wants to ask the user do do something or not
    public Dialog warningDialog = new Dialog(GUI.this, "Warning", true);
    Button warningDialogYesButton = new Button("Yes");
    Button warningDialogNoButton = new Button("No");
    boolean warningDialogAnswer = false;

    // Sets up the Filter for *.db and *. file opening
/*    private class DImportLocalFilter implements FilenameFilter {
      public boolean accept(File dir, String name) {
        name = name.toLowerCase();
        if (name.endsWith(".db")) return true;
        if (name.endsWith(".")) return true;
        return false;
      }
    };
    DImportLocalFilter dImportLocalFilter;
*/

    // This is the dialog that will be opened when loading a .db file
    // note: I have created this object here (instead of creating it every time a new .db file is imported) so that the
    // directories from which .db files are imported would be 'remembered' in the FileDialog everytime it is reopened
    JFileChooser dImportLocal = new JFileChooser();
    musr.client.gui.ExtensionFileFilter dbFilter = new ExtensionFileFilter();

    // This is to open saved GUI files
    JFileChooser dOpenGUI = new JFileChooser();

/* Example:
     JFileChooser chooser = new JFileChooser(); // Note: source for ExtensionFileFilter can be found in the SwingSet demo Extension
     FileFilter filter = new ExtensionFileFilter();
     filter.addExtension("jpg");
     filter.addExtension("gif");
     filter.setDescription("JPG & GIF Images");
     chooser.setFileFilter(filter);
     int returnVal = chooser.showOpenDialog(parent);
     if(returnVal == JFileChooser.APPROVE_OPTION)
      { System.out.println("You chose to open this file: " + chooser.getSelectedFile().getName()); }
*/
    // Event Listeners used
    private class ActionListenerS implements java.awt.event.ActionListener, java.io.Serializable {
      public void actionPerformed(ActionEvent e) {}
    };
    ActionListenerS menuBarListener;
    ActionListenerS buttonListener;
    private class ItemListenerS implements java.awt.event.ItemListener, java.io.Serializable {
      public void itemStateChanged(ItemEvent e) {}
    };
    ItemListenerS listListener;
    ItemListenerS displayOptionsListener;


    /**
     * This varibale loads the initial GUI state right after the GUI is created, before (and if) the default.GUI state is loaded
     * It is used to reset the whole GUI if the user ever decides to.
     * Note that this variable is not saved into a SaveGUI Object. There is no need to do that since the
     *   GUI will always create and contain this Object in one unchanging state.
     */
     SaveGUI resetGUI;
     
     //Used to store "this" when call to "this" needed in inner classes
     GUI thisGUI;

  /**
   * Creates the whole GUI, and sets up event listeners
   * passes a GUI_RMIApplet variable which is the parent Applet that loads this GUI.
   */
  public GUI(GUI_RMIApplet guiApp) {
    super("View - Main Window");  // set frame's title

    thisGUI = this;
    /** Get the size of the screen */
    screenDimen = tk.getScreenSize();

    /** This is the Applet that loads this GUI */
    final GUI_RMIApplet guiApplet = guiApp;
    this.setBackground(Color.lightGray);

    // Set a default font
    this.setFont(new Font("SansSerif", Font.PLAIN, 12));

    contentPane = this.getContentPane();
    
    // Create the first Frame where all the plots will be loaded and create the plot options for it
    // Put the options menus into the GUI menus (see below)
    layoutChoice = new Choice();
    layoutChoice.add(Long.toString(layoutChoice.getItemCount()+1));
    plotsOptions.addElement(new PlotOptionsFrame());         // Create the first PlotOptionsFrame !!
    ((PlotOptionsFrame) plotsOptions.lastElement()).setParentGUI(this);

    // Initialize the PageFormat for printing  to Landscape
    pageForm.setOrientation(0);

    // This is used to load MUD data
    mudSpectFrame = new MUDSpectrumFrame(GUI.this);

    // Set the Filter in the File Dialog for db file oppening to load only *.db files and *. files
//old:    dImportLocal.setFilenameFilter(dImportLocalFilter);

    //Create a new MenuItem to act as a spacer between frequently used items
    //Dr Brewer requested more spacing between these items
    MenuItem blankMenuItem1 = new MenuItem("");
    MenuItem blankMenuItem2 = new MenuItem("");    
    blankMenuItem1.setEnabled(false);
    blankMenuItem2.setEnabled(false);
    
    // Create a MenuBar and tell the frame about it.
    menuBar = new MenuBar();
    this.setMenuBar(menuBar);
    // Add file menu to MenuBar.
    fileMenu = new Menu("File");
    menuBar.add(fileMenu);
    // Add items to the File menu
    // setting shortcut,action command and listener(defined below).
        openBlankMenuItem = new MenuItem("New Data Table", new MenuShortcut(KeyEvent.VK_N));
        openBlankMenuItem.setActionCommand("openBlankDataTable");
    fileMenu.add(openBlankMenuItem);
    fileMenu.add(blankMenuItem1);
    importMenu = new Menu("Load Data");
      loadMUDSpectrum = new Menu("MUD Spectra from '.msr' File");
      integralDataMenuItem = new MenuItem("TI Data");
      integralDataMenuItem.setActionCommand("loadmudspectint");
      loadMUDSpectrum.add(integralDataMenuItem);
      differentialDataMenuItem = new MenuItem("TD Data");
      differentialDataMenuItem.setActionCommand("loadmudspect");
      loadMUDSpectrum.add(differentialDataMenuItem);
    importMenu.add(loadMUDSpectrum);
    importMenu.add(new MenuItem("-")); // this is a separator
      importFileMenuItem = new MenuItem("Import db from 'musr.triumf.ca'"/*, new MenuShortcut(KeyEvent.VK_F)*/);
      importFileMenuItem.setActionCommand("importFileMusr");
//    fileMenu.add(importFileMenuItem);
      importFileMenuItem2 = new MenuItem("Import db from 'cmms.triumf.ca'");
      importFileMenuItem2.setActionCommand("importFileCmms");
//    fileMenu.add(importFileMenuItem2);
      importLocalFileMenuItem = new MenuItem("Data Table from '.db' File", new MenuShortcut(KeyEvent.VK_I));
      importLocalFileMenuItem.setActionCommand("importLocalFile");
    importMenu.add(importLocalFileMenuItem);
      importXMLFileMenuItem = new MenuItem("Data Table from '.xml' File");
      importXMLFileMenuItem.setActionCommand("importXMLFile");
    importMenu.add(importXMLFileMenuItem);
      importCSVFileMenuItem = new MenuItem("Data Table from '.csv' File");
      importCSVFileMenuItem.setActionCommand("importCSVFile");
    importMenu.add(importCSVFileMenuItem);
    fileMenu.add(importMenu);
    fileMenu.add(blankMenuItem2); 
       exportMenu = new Menu("Save Data");
    fileMenu.add(exportMenu);
      dbMenuItem = new MenuItem("Data Table to '.db' File");
      dbMenuItem.setActionCommand("exportdb");
      exportMenu.add(dbMenuItem);
      xmlMenuItem = new MenuItem("Data Table to '.xml' File");
      xmlMenuItem.setActionCommand("xml");
      exportMenu.add(xmlMenuItem);
      csvMenuItem = new MenuItem("Data Table to '.csv' File");
      csvMenuItem.setActionCommand("exportcsv");
      exportMenu.add(csvMenuItem);
    fileMenu.add(new MenuItem("-"));    //a separator

    openMenuItem = new MenuItem("Load State", new MenuShortcut(KeyEvent.VK_O));
      openMenuItem.setActionCommand("open GUI");
    fileMenu.add(openMenuItem);   
      saveMenuItem = new MenuItem("Save State", new MenuShortcut(KeyEvent.VK_S));
      saveMenuItem.setActionCommand("save GUI");
    fileMenu.add(saveMenuItem);
    fileMenu.add(new MenuItem("-"));    //a separator
        
    printMenuItem = new MenuItem("Print", new MenuShortcut(KeyEvent.VK_P));
        printMenuItem.setActionCommand("print");
        fileMenu.add(printMenuItem);
      savePNGMenuItem = new MenuItem("Save Plot", new MenuShortcut(KeyEvent.VK_E));
      savePNGMenuItem.setActionCommand("savePNG");
    fileMenu.add(savePNGMenuItem);
    //fileMenu.add(new MenuItem("-")); // this is a separator
    /*  importURLMenuItem = new MenuItem("Import DataTable from  URL (not functional)", new MenuShortcut(KeyEvent.VK_U));
    importURLMenuItem.setActionCommand("importURL");
    fileMenu.add(importURLMenuItem);*/

      removeMenuItem = new MenuItem("Remove Data Table", new MenuShortcut(KeyEvent.VK_R));
      removeMenuItem.setActionCommand("remove");
//    fileMenu.add(removeMenuItem);
    fileMenu.add(new MenuItem("-")); // this is a separator

      resetMenuItem = new MenuItem("Global Reset", new MenuShortcut(KeyEvent.VK_R));
      resetMenuItem.setActionCommand("reset GUI");    
    fileMenu.add(resetMenuItem);
      quitMenuItem = new MenuItem("Quit");
      quitMenuItem.setActionCommand("quit");
    fileMenu.add(quitMenuItem);
    
    // Add View to MenuBar
    viewMenu = new Menu("View");
    menuBar.add(viewMenu);
      displayMenuItem = new MenuItem("Plot", new MenuShortcut(KeyEvent.VK_D));
      displayMenuItem.setActionCommand("display");
    viewMenu.add(displayMenuItem);
    viewMenu.add(new MenuItem("-")); // this is a separator
      dataTableMenuItem = new MenuItem("Data Tables", new MenuShortcut(KeyEvent.VK_G));
      dataTableMenuItem.setActionCommand("datatable");
    viewMenu.add(dataTableMenuItem);
      showMenuItem = new MenuItem("Data Information");//, new MenuShortcut(KeyEvent.VK_A));
      showMenuItem.setActionCommand("show");
    viewMenu.add(showMenuItem);
      spreadSheetMenuItem = new MenuItem("Data Spreadsheet");//, new MenuShortcut(KeyEvent.VK_E));
      spreadSheetMenuItem.setActionCommand("spreadsheet");
    viewMenu.add(spreadSheetMenuItem);
    viewMenu.add(new MenuItem("-")); // this is a separator
      funTableMenuItem = new MenuItem("Functions");//, new MenuShortcut(KeyEvent.VK_));
      funTableMenuItem.setActionCommand("funtable");
    viewMenu.add(funTableMenuItem);
      conTableMenuItem = new MenuItem("Constants");//, new MenuShortcut(KeyEvent.VK_));
      conTableMenuItem.setActionCommand("contable");
    viewMenu.add(conTableMenuItem);
    viewMenu.add(new MenuItem("-")); // this is a separator
    
      framesMenuItem = new MenuItem("Frames");
      framesMenuItem.setActionCommand("frames");
    viewMenu.add(framesMenuItem);


    // Add Options to MenuBar
    optionsMenu = new Menu("Options");
      menuBar.add(optionsMenu);
    addOptionsToMenuBar(plotsOptions, optionsMenu);

    // Add Tools to MenuBar
    toolsMenu = new Menu("Tools");
    
    menuBar.add(toolsMenu);
    
    dataMenu = new Menu("Data Processing");
    toolsMenu.add(dataMenu);
        bestFitMenuItem = new MenuItem("Polynomial Fit");
        bestFitMenuItem.setActionCommand("bestfit");
    dataMenu.add(bestFitMenuItem);
        fourierMenuItem = new MenuItem("Fourier Transform");
        fourierMenuItem.setActionCommand("fourier");
    dataMenu.add(fourierMenuItem);
        splinesMenuItem = new MenuItem("Spline Interpolation");
        splinesMenuItem.setActionCommand("splines");
    dataMenu.add(splinesMenuItem);        
        repackMenuItem = new MenuItem("Repack Data Table");//, new MenuShortcut(KeyEvent.VK_));
        repackMenuItem.setActionCommand("repack");
    dataMenu.add(repackMenuItem);
    toolsMenu.add(new MenuItem("-"));   //this is a separator
    
      commandLineConsoleItem = new MenuItem("Command Line Console", new MenuShortcut(KeyEvent.VK_L));
      commandLineConsoleItem.setActionCommand("command");
    toolsMenu.add(commandLineConsoleItem);
    toolsMenu.add(new MenuItem("-")); // this is a separator
     
      createFunMenuItem = new MenuItem("Define Function", new MenuShortcut(KeyEvent.VK_F));//, new MenuShortcut(KeyEvent.VK_));
      createFunMenuItem.setActionCommand("function");
    toolsMenu.add(createFunMenuItem);
      createConMenuItem = new MenuItem("Define Constant", new MenuShortcut(KeyEvent.VK_C));//, new MenuShortcut(KeyEvent.VK_));
      createConMenuItem.setActionCommand("constant");
    toolsMenu.add(createConMenuItem);
    toolsMenu.add(new MenuItem("-")); // this is a separator
        
      symbSizeMenuItem = new MenuItem("Symbol Size Setup");//, new MenuShortcut(KeyEvent.VK_));
      symbSizeMenuItem.setActionCommand("symbol size");
    toolsMenu.add(symbSizeMenuItem);
      printSetupMenuItem = new MenuItem("Plot Size Setup");//, new MenuShortcut(KeyEvent.VK_));
      printSetupMenuItem.setActionCommand("plot size setup");
    toolsMenu.add(printSetupMenuItem);
      toolsMenu.add(new MenuItem("-")); // this is a separator
      
      dbCheckerItem = new MenuItem("Check '.db' file");
      dbCheckerItem.setActionCommand("dbcheck");
    toolsMenu.add(dbCheckerItem);
      postScriptPrintMenuItem = new MenuItem("Print as PostScript");
      postScriptPrintMenuItem.setActionCommand("printPS");
    //toolsMenu.add(postScriptPrintMenuItem);

    // Add Help to MenuBar
    helpMenu = new Menu("Help");
    menuBar.add(helpMenu);
      manualMenuItem = new MenuItem("View Manual", new MenuShortcut(KeyEvent.VK_H));
      manualMenuItem.setActionCommand("manual");
    helpMenu.add(manualMenuItem);
      consoleInstructionsItem = new MenuItem("Console Help");
      consoleInstructionsItem.setActionCommand("command instructions");
    helpMenu.add(consoleInstructionsItem);
    helpMenu.add(new MenuItem("-")); // this is a separator
      aboutMenuItem = new MenuItem("About");
      aboutMenuItem.setActionCommand("about");
    helpMenu.add(aboutMenuItem);


    // Assign a BorderLayout manager with margins for this frame.
    this.setLayout(new BorderLayout(10, 10));

    // Create all the Panels for this frame
    dataTablePanel = new JPanel();
    dataTablePanel.setLayout(new GridBagLayout());
    dataTablePanel.setBackground(Color.lightGray);
    dataTablePanel.setSize((int)screenDimen.getWidth()*(5/8)*(27/40), (int)screenDimen.getHeight()*(2/5)*(5/6));


    // dataTableFrame Frame settings
    dataTableFrame = new JFrame("Data Tables - Frame #"+String.valueOf(layoutChoice.getSelectedIndex()+1));

    // Create Scrollbar pane (for the purpose if the window is not big enough for all the components in it
     tableFrameScrollbar = new ScrollPane();
     tablePanel = new JPanel();
     tablePanelGridBag = new GridBagLayout();
     tablePanelConstraints = new GridBagConstraints();
     tablePanel.setLayout(tablePanelGridBag);

    tablePanel.setBackground(Color.lightGray);
    tablePanelConstraints.fill = GridBagConstraints.BOTH;

    tablePanelConstraints.weightx = 1.0;
    tablePanelGridBag.setConstraints(dataTablePanel, tablePanelConstraints);
    tablePanel.add(dataTablePanel,tablePanelConstraints); // add the Panel with Data Tables and Columns to the Frame

    selectPanel = new JPanel();
    selectPanel.setLayout(new BorderLayout(10, 10));
    selectPanel.setBackground(Color.lightGray);
    tablePanel.add(selectPanel, tablePanelConstraints);

    statusPanel = new  JPanel();
    statusPanel.setBackground(Color.LIGHT_GRAY);
    //statusPanel.setLayout(new BorderLayout(5, 10));
    //tablePanelConstraints.weightx = 0.0; // new row
    contentPane.add(statusPanel);//, tablePanelConstraints);


//    dataTableFrame.getContentPane().add(dataTableButton, "South");
    tableFrameScrollbar.add(tablePanel);
    dataTableFrame.getContentPane().add(tableFrameScrollbar);
    dataTableFrame.setSize((int)screenDimen.getWidth()*5/8, (int)screenDimen.getHeight()*2/5);        //    dataTableFrame.setSize(1200, 450);
    dataTableFrame.setLocation(50, (int)screenDimen.getHeight()/2);

    // Some default values for the constranints object
    gridConstr.fill = GridBagConstraints.BOTH;
    gridConstr.insets = new Insets(5,5,5,5);
    gridConstr.anchor = GridBagConstraints.CENTER;

    // Create Scrollbar pane (for the purpose if the window is not big enough for all the components in it
    //tableListScrollbar = new ScrollPane();
    // Fill the DataTable Panel
    dataTableList = new List(6, false);
//    dataTableList.setSize((int)screenDimen.getWidth()*(6/8)*(18/40), (int)screenDimen.getHeight()*(2/5)*(1/3));
    dataTableList.setBackground(Color.lightGray);
		gridConstr.gridx = 0; gridConstr.gridy = 0;
    // I added spaces in the end so that the dataTable List (under it) would be wider
    dataTablePanel.add(new JLabel("Data Tables:           "), gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 1;
    dataTablePanel.add(new JLabel("(filename; title)"), gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 2;
    //tableListScrollbar.add(dataTableList);
    dataTablePanel.add(dataTableList, gridConstr);

    // add the button that removes the selected DataTable
//    addDataTableButton = new Button("Import DataTable");
//		gridConstr.gridx = 0; gridConstr.gridy = 3;
//    dataTablePanel.add(addDataTableButton, gridConstr);

    showDataTableInfoButton = new JButton("Show Data Table Info");
    showDataTableInfoButton.setBackground(Color.LIGHT_GRAY);
    showDataTableInfoButton.setToolTipText("Show header information on the selected data table.");
                gridConstr.gridx = 0; gridConstr.gridy = 3;
    dataTablePanel.add(showDataTableInfoButton, gridConstr);
    
    // add the button that removes the selected DataTable
    removeDataTableButton = new JButton("Remove Data Table");
    removeDataTableButton.setBackground(Color.LIGHT_GRAY);
    removeDataTableButton.setToolTipText("Remove the selected data table.");
		gridConstr.gridx = 0; gridConstr.gridy = 4;
    dataTablePanel.add(removeDataTableButton, gridConstr);
    
    refreshDataTableButton = new JButton("Refresh Data Table");
    refreshDataTableButton.setBackground(Color.LIGHT_GRAY);
    refreshDataTableButton.setToolTipText("Refresh the selected data table.");
                gridConstr.gridx = 0; gridConstr.gridy = 5;
    dataTablePanel.add(refreshDataTableButton, gridConstr);


    columnList = new List(6, false);
//    columnList.setSize((int)screenDimen.getWidth()*(6/8)*(9/40), (int)screenDimen.getHeight()*(2/5)*(1/3));
    columnList.setBackground(Color.lightGray);
		gridConstr.gridx = 1; gridConstr.gridy = 0;
		dataTablePanel.add(new JLabel("Columns:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 1;
		dataTablePanel.add(new JLabel("(name = label)"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 2;
    dataTablePanel.add(columnList, gridConstr);

    // add the button that displays the contents of the selected column
    columnButton = new JButton("View Column Data");
    columnButton.setBackground(Color.LIGHT_GRAY);
    columnButton.setToolTipText("Display the contents of the selected column.");
		gridConstr.gridx = 1; gridConstr.gridy = 3;
    dataTablePanel.add(columnButton, gridConstr);
    
    //Add button that displays statistics of the selected column
    statsButton = new JButton("View Column Statistics");
    statsButton.setBackground(Color.LIGHT_GRAY);
    statsButton.setToolTipText("Display statistics nn the contents of the selected column.");
                    gridConstr.gridx = 1; gridConstr.gridy =4;
    dataTablePanel.add(statsButton, gridConstr);
    
    
    //Add button that performs a Fourier transform on column data
    /*fourierButton = new JButton("Fourier Transform");
    fourierButton.setBackground(Color.LIGHT_GRAY);
    fourierButton.setToolTipText("Take the Fourier transform of a data column.");
                    gridConstr.gridx = 1; gridConstr.gridy = 5;
    dataTablePanel.add(fourierButton, gridConstr);*/
    

    // Fill the Info Panel

    // Create the Display Options Panel and fill it
    framesPanel = new JPanel();
    framesPanel.setBackground(Color.LIGHT_GRAY);
    framesPanel.setLayout(new GridBagLayout());
    layoutLabel = new JLabel("Frame");
      gridConstr.gridx = 0; gridConstr.gridy = 0; gridConstr.gridwidth=1; gridConstr.gridheight=1;
    framesPanel.add(layoutLabel, gridConstr);
      gridConstr.gridx = 0; gridConstr.gridy = 1;
    framesPanel.add(layoutChoice, gridConstr);
		layoutAddButton = new Button("add");
		gridConstr.gridx = 1; gridConstr.gridy = 0; gridConstr.gridwidth=2; gridConstr.gridheight=1;
    framesPanel.add(layoutAddButton, gridConstr);
		layoutRemoveButton = new Button("remove");
		gridConstr.gridx = 1; gridConstr.gridy = 1; gridConstr.gridwidth=2; gridConstr.gridheight=1;
    framesPanel.add(layoutRemoveButton, gridConstr);
		gridGroup = new CheckboxGroup();
		gridAutoCheckbox = new Checkbox("auto",gridGroup,false);
		gridConstr.gridx = 0; gridConstr.gridy = 2; gridConstr.gridwidth=1; gridConstr.gridheight=1;
    framesPanel.add(gridAutoCheckbox, gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 2;
    framesPanel.add(new JLabel("Rows"), gridConstr);
		gridConstr.gridx = 2; gridConstr.gridy = 2;
    framesPanel.add(new JLabel("Columns"), gridConstr);
		gridManualCheckbox = new Checkbox("manual",gridGroup,true);
		gridConstr.gridx = 0; gridConstr.gridy = 3;
    framesPanel.add(gridManualCheckbox, gridConstr);
		gridXChoice = new Choice();
		gridXChoice.add("*");
		for(int i=1; i<=10; i++) gridXChoice.add(Long.toString(i));
		gridConstr.gridx = 1; gridConstr.gridy = 3;
    framesPanel.add(gridXChoice, gridConstr);
		gridYChoice = new Choice();
		gridYChoice.add("*");
		for(int i=1; i<=10; i++) gridYChoice.add(Long.toString(i));
		gridConstr.gridx = 2; gridConstr.gridy = 3;
    framesPanel.add(gridYChoice, gridConstr);
		gridYChoice.select(1);
      visibleCheckbox = new Checkbox("visible", true);
		gridConstr.gridx = 0; gridConstr.gridy = 4;
    framesPanel.add(visibleCheckbox, gridConstr);
      invertedCheckbox = new Checkbox("x <-> y", false);
		gridConstr.gridx = 1; gridConstr.gridy = 4;
    framesPanel.add(invertedCheckbox, gridConstr);


    // Frames Frame settings
    framesFrame.setBackground(Color.lightGray);
    framesFrame.add(framesPanel, "Center");
    framesFrame.add(framesButton, "South");
    framesFrame.setSize(330,200);

    // Fill the Select (Plot) Panel

    // Create the Load Panel and fill it
    loadPanel = new JPanel();
    loadPanel.setBackground(Color.LIGHT_GRAY);
    loadPanel.setLayout(new GridBagLayout());
    selectPanel.add(loadPanel, "North");
    xAxisList = new List(6, false);
    xAxisList.setSize((int)screenDimen.getWidth()*(6/8)*(4/40), (int)screenDimen.getHeight()*(2/5)*(1/3));
    xAxisList.setBackground(Color.lightGray);
    yAxisList = new List(6, false);
    xAxisList.setSize((int)screenDimen.getWidth()*(6/8)*(4/40), (int)screenDimen.getHeight()*(2/5)*(1/3));
    yAxisList.setBackground(Color.lightGray);

		gridConstr.gridx = 0; gridConstr.gridy = 0;
    /*loadPanel.add((JLabel) ((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).layoutFrameLabel, gridConstr);*/
		gridConstr.gridx = 0; gridConstr.gridy = 1; gridConstr.gridwidth=1; gridConstr.gridheight=1;
    loadPanel.add(new JLabel("x-axis:"), gridConstr);
                gridConstr.gridx = 0; gridConstr.gridy = 2;
    loadPanel.add(new JLabel("independent variable(s)"), gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 3;
    loadPanel.add(xAxisList, gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 0;
    loadPanel.add(new JLabel(""), gridConstr); // this is just empty space
		gridConstr.gridx = 1; gridConstr.gridy = 1;
    loadPanel.add(new JLabel("y-axis:"), gridConstr);
                gridConstr.gridx = 1; gridConstr.gridy = 2;
    loadPanel.add(new JLabel("dependent variable(s)"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 3;
    loadPanel.add(yAxisList, gridConstr);

    // Create the Buttons Panel and fill it
    buttonsPanel = new JPanel();
    buttonsPanel.setBackground(Color.LIGHT_GRAY);
    buttonsPanel.setLayout(new GridBagLayout());
    selectPanel.add(buttonsPanel, "Center");
		addXButton = new JButton("add X");
                addXButton.setBackground(Color.LIGHT_GRAY);
                addXButton.setToolTipText("Add a data column to the list of x columns.");
		gridConstr.gridx = 0; gridConstr.gridy = 0;
    buttonsPanel.add(addXButton, gridConstr);
		addYButton = new JButton("add Y");
                addYButton.setBackground(Color.LIGHT_GRAY);
                addYButton.setToolTipText("Add a data column to this list of y columns.");
		gridConstr.gridx = 1; gridConstr.gridy = 0;
    buttonsPanel.add(addYButton, gridConstr);
		removeXButton = new JButton("remove X");
                removeXButton.setBackground(Color.LIGHT_GRAY);
                removeXButton.setToolTipText("Remove the selected column from the list of x columns.");
		gridConstr.gridx = 0; gridConstr.gridy = 1;
    buttonsPanel.add(removeXButton, gridConstr);
		removeYButton = new JButton("remove Y");
                removeYButton.setBackground(Color.LIGHT_GRAY);
                removeYButton.setToolTipText("Remove the selected column from this list of y columns.");
		gridConstr.gridx = 1; gridConstr.gridy = 1;
    buttonsPanel.add(removeYButton, gridConstr);

    // Create the buttonsPanel2 and fill it
    buttonsPanel2 = new JPanel();
    buttonsPanel2.setBackground(Color.LIGHT_GRAY);
    buttonsPanel2.setLayout(new GridLayout(2,1));
    
    JPanel plotPanel = new JPanel();
    plotPanel.setLayout(new FlowLayout());
    plotPanel.setBackground(Color.LIGHT_GRAY);
                displayPlotButton = new JButton("Display Plot");
                displayPlotButton.setBackground(Color.ORANGE);
                displayPlotButton.setToolTipText("Display a plot of the selected x column and the corresponding y columns.");
                //gridConstr.gridx = 0; gridConstr.gridy = 0;
    plotPanel.add(displayPlotButton);
                conditionalPlotButton = new JButton("Conditional Plot");
                conditionalPlotButton.setBackground(Color.LIGHT_GRAY);
                conditionalPlotButton.setToolTipText("Create a conditional plot of the selected x column and the corresponding y columns.");
                //gridConstr.gridx = 1; gridConstr.gridy = 0;
    plotPanel.add(conditionalPlotButton);
    buttonsPanel2.add(plotPanel);
    selectPanel.add(buttonsPanel2, "South");
    
    JPanel dataToolPanel = new JPanel();
    dataToolPanel.setBackground(Color.LIGHT_GRAY);
    dataToolPanel.setLayout(new GridLayout(1,1));
                //gridConstr.gridx = 0; gridConstr.gridy = 1;
    //dataToolPanel.add(new JLabel(""));
    
    JPanel dataToolButtonsPanel = new JPanel();
    dataToolButtonsPanel.setBackground(Color.LIGHT_GRAY);
    dataToolButtonsPanel.setLayout(new GridLayout(4,1));
    dataToolButtonsPanel.setBorder(BorderFactory.createTitledBorder("Data processing"));
    //dataToolButtonsPanel.add(new JLabel("Data processing:"));
                bestFitButton = new JButton("Find Best Fit");
                bestFitButton.setBackground(Color.LIGHT_GRAY);
                bestFitButton.setToolTipText("Fit the selected x and y columns to a polynomial curve.");
    //dataToolButtonsPanel.add(bestFitButton);
                fourierButton = new JButton("Fourier Transform");
                fourierButton.setBackground(Color.LIGHT_GRAY);
                fourierButton.setToolTipText("Take the Fourier transform of a data column");
    //dataToolButtonsPanel.add(fourierButton);
                splineButton = new JButton("Spline Interpolation");
                splineButton.setBackground(Color.LIGHT_GRAY);
                //System.out.println("Spline Interpolation button initialized");
                splineButton.setToolTipText("Use cubic splines to interpolate between data points.");
                //gridConstr.gridx = 1; gridConstr.gridy = 1;
    //dataToolButtonsPanel.add(splineButton);
                repackDataButton = new JButton("Repack Data");
                repackDataButton.setBackground(Color.LIGHT_GRAY);
                repackDataButton.setToolTipText("Repack the data to eliminate repeated bins and minimize error bars.");
                //gridConstr.gridx = 2; gridConstr.gridy = 1;"D
    //dataToolButtonsPanel.add(repackDataButton);
    
    //dataToolPanel.add(dataToolButtonsPanel);

    tablePanelConstraints.gridx = 3;
    tablePanelConstraints.gridy = 0;
    tablePanelConstraints.insets.top = 50;
    tablePanelConstraints.insets.bottom = 50;
    tablePanel.add(dataToolPanel, tablePanelConstraints);
    tablePanelConstraints.insets.top = 0;       //reset to defaults
    tablePanelConstraints.insets.bottom = 0;

    //*** Fill the Status Panel ***
    statusTextField = new TextField(60);
    statusTextField.setBackground(Color.lightGray);
    statusTextField.setEditable(false);
    statusPanel.add(statusTextField, "Center");

    setDisplayOptionsEnabled(false);

    // since there is one Frame from the start, there should be one of each of the variables below
    xAxisDataTable.addElement(new DataTable(new String("")));
    titlexAxis.addElement(new Vector());
    namexAxis.addElement(new Vector());
    yAxisDataTable.addElement(new Vector());
    titleyAxis.addElement(new Vector());
    plotsRecord.addElement(new Vector());


    // The frame that will display the plots has already been created. Set it's size now to fit the screen
    // Once width has been found, based on that size, determine the size of the height (which will be 0.8 times)
    //   smaller, thus it will correspond to the landscape printout ratio.
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    // Change the size of the plot frame to the size of the landscape printout + additional 75
    plotFrame.setSize((int)plotWidthPort*28 + 150,(int)plotHeightPort*28 + 150 + 75);//((int)(d.width*0.6), (int)(d.width*0.6*0.8));
    // Finally, set the plotFrame to be nonresizable.
    //*Warning*
    // This will cause that none of the code which applies to smaller plot-sizes will be used or seen on the screen
    plotFrame.setResizable(false);


    // Display the Main GUI window
    dataTableFrame.show();

/* This is designed so that when the user wants to open the file, it would load the dialog in /tmp/. I am not going to do that (at least for now)
    if (System.getProperty("os.name").indexOf("Windows") != -1)  // it is in windows os
      dOpenGUI.setCurrentDirectory(new File("c:\\temp\\"));
    else // it is in unix or mac os x
      dOpenGUI.setCurrentDirectory(new File("/tmp/"));
*/

    // Set the dirPath to look on the /tmp directory as defualt
    if (System.getProperty("os.name").indexOf("Windows") != -1)  // it is in windows os
      dirPath="c:\\temp\\";
    else // it is in unix or mac os x
      dirPath="/tmp/";

		/*
		 * Event Handelers
		 */

		// Handle events in the MenuBar
		menuBarListener = new ActionListenerS() {
                  public void actionPerformed(ActionEvent e) {
                      //System.out.println("The following command was received:\n" + e.getActionCommand());
                    String command = e.getActionCommand();
                    String filename = "";

                    /* Open a saved GUI state */
                       if (command.equals("open GUI")) {
                        // Clear the info text field
                        statusTextField.setText("");
                      loadGUI("load", true, plotsOptions);
                    }

                    /* save the GUI state
                      Note: since the Object values point to a same object it is necessary to clone each one of them*/
                    else if (command.equals("save GUI")) {
                      saveGUI();
                    }
                    
                    else if(command.equals("savePNG"))
                    {
                         //save the current plot as a PNG image   
                         savePlotPNG();
                    }

                    /* Globally Reset the Whole GUI stte to the Initial Default State */
                    else if (command.equals("reset GUI")) {
                      /*warningDialog.setLayout(new BorderLayout());
                      warningDialog.setSize(350, 70);
                      warningDialog.add(new JLabel("Are you sure you want to Reset the GUI? This will erase all your work."), "North");
                      warningDialog.add(warningDialogYesButton, "West");
                      warningDialog.add(warningDialogNoButton, "East");
                      warningDialog.pack();

                      warningDialog.show(); // this will block everything until answered, the outcome is taken care of in the button listener..
                      if (warningDialogAnswer) {
                        loadGUI("res", true, plotsOptions);
                        plotFrame.hide();
                      }*/
                        if(JOptionPane.showConfirmDialog(null,"Are you sure you want to reset? This will erase all your work.",
                                "Please confirm", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
                            ==JOptionPane.YES_OPTION)
                        {
                            loadGUI("res", true, plotsOptions);
                            plotFrame.hide();                            
                        }
                    }

                    /* This opens the new frame for loading MUD Spectrums for time-differential mode data*/
                    else if (command.equals("loadmudspect")) {
                      loadMUDSpectrum(MUDSpectrumFrame.DIFFERENTIAL_MODE);
                    }
                    
                    /* This command opens a new frame for loading MUD Spectrums for time-integral mode data */
                    else if(command.equals("loadmudspectint"))
                    {
                        loadMUDSpectrum(MUDSpectrumFrame.INTEGRAL_MODE);
                    }
                    
                    /* Import DataTable from File in musr.triumf.ca (through RMI) */
                    // this function should comunicate with the musr.triumf.ca server, open a browser of options that
                    // server sends it, and then download a selected DataTable
                    // here DBFileReader reads in the sent file and then sends the DataTable it read from the file
                    else if (command.equals("importFileMusr")) {
                      importFileMusr();
                    }

                    /* Import DataTable from File in cmms.triumf.ca (through RMI) */
                    // this function should comunicate with the cmms.triumf.ca server, open a browser of options that
                    // server sends it, and then download a selected DataTable
                    // here DBFileReader reads in the sent file and then sends the DataTable it read from the file
                    else if (command.equals("importFileCmms")) {
                      importFileCmms();
                    }

                    /* Import DataTable from Local File */
                    // This function just loads a file from the machine where GUI is loaded (client machine)
                    else if (command.equals("importLocalFile")) {
                      importLocalFile();
                    }

                    /* Import DataTable from Local File */
                    // This function just loads a file from the machine where GUI is loaded (client machine)
                    else if (command.equals("openBlankDataTable")) {
                      openBlankDataTable();
                    }
                                              
                    /* Import DataTable from Local File */
                    // This function just loads a file from the machine where GUI is loaded (client machine)
                    else if (command.equals("importXMLFile")) {
                      importXMLFile();
                    }
                    
                    else if(command.equals("importCSVFile"))
                    {
                        importCSVFile();
                    }

                    /* Remove DataTable */
                    // Removes the selected DataTable
                    else if (command.equals("remove")) 
                    {
                        if(JOptionPane.showConfirmDialog(null, "Are you sure you want to remove the selected data table?",
                                "Please confirm", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                            removeDataTable();
                    }

                    /* Export the selected DataTable into XML file */
                    else if (command.equals("xml")) {
                      // make sure there is at least one DataTable or if there is a DataTable selected
                      if (dataTableList.getItemCount()!=0 || dataTableList.getSelectedIndex()!=-1) {
                        ExportXML exportXML = new ExportXML(GUI.this);
                        exportXML.createXML();
                      } else {
                        popAlertDialog("   Warning: ","    There is no DataTable selected, " ,"   Please Select a DataTable. ");
                      }
                    }

                    /* Export the selected DataTable into DB file */
                    else if (command.equals("exportdb")) {
                      // make sure there is at least one DataTable or if there is a DataTable selected
                      if (dataTableList.getItemCount()!=0 || dataTableList.getSelectedIndex()!=-1) {
                        ExportDB ExportDB = new ExportDB(GUI.this);
                        ExportDB.exportDB();
                      } else {
                        popAlertDialog("   Warning: ","    There is no DataTable selected, " ,"   Please Select a DataTable. ");
                      }
                    }
                    
                    else if(command.equals("exportcsv"))
                        if((dataTableList.getItemCount()!=0)||(dataTableList.getSelectedIndex()!=-1))
                        {
                            ExportCSV exportCSV = new ExportCSV(GUI.this);
                            exportCSV.exportCSV();
                        }
                            else
                                popAlertDialog("   Warning: ","    There is no DataTable selected, " ,"   Please Select a DataTable. ");

                    /* Quit */
                    // exit the program
                    else if (command.equals("quit")) 
                    {
                        //show a dialog to allow the user to confirm the quit
                        if(JOptionPane.showConfirmDialog(null, "Are you sure you want to exit View?", "Quitting View", JOptionPane.YES_NO_OPTION) == 
                                JOptionPane.YES_OPTION)
                        {
                              // The Main GUI Window and the plot frame doesn't close right away
                              dataTableFrame.hide();
                              plotFrame.hide();
                              mudSpectFrame.hide();
                              // quit everything now
                              disposeAll();
                        }
                    }

                      /* Display Plot */
                      // Displays all the layout Frames and the plots that were added to them. See below for details
                      // dislay doesn't catch exceptions from plotting. The error is printed on the console/xterm
                      // that is something that can be implemented
                      else if (command.equals("display")) {
                        display();
                      }

                      /* Data Tables and Columns */
                      // Display all the loaded Data Tables with a list of Columns for the selected Data Table
                      else if (command.equals("datatable")) {
                        dataTableFrame.show();
                      }

                    
                                        
                      //Check the formatting of the .db file
                      else if(command.equals("dbcheck"))
                      {
                        JFileChooser fileChooser = new JFileChooser();
                        if(fileChooser.showOpenDialog(GUI.this)==JFileChooser.APPROVE_OPTION)
                        {
                            DBDiagnosticGUI checker = new DBDiagnosticGUI(fileChooser.getSelectedFile());
                            checker.setVisible(true);
                        }
                      }

                      /* Show */
                      // Show the title, abstract and comments of the selected DataTable in a seperate Dialog
                      else if (command.equals("show")) {
                        if (dictionary.size()>0) {
                          int dataTableKey = getDataTableIndex();
                          DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
                          /*InfoDialog d = new InfoDialog(GUI.this, "Show", -1,-1);
                          d.addLine("Title:");
                          d.addLine(selectedDataTable.getTitle());
                          d.addLine("");
                          d.addLine("Abstract:");
                          d.addLine(selectedDataTable.getAbstract());
                          d.addLine("");
                          d.addLine("Comment:");
                          d.addLine(selectedDataTable.getComment());
                          d.setLocation(200,200);
                          d.show();*/
                          String output = "Abstract:\n" + selectedDataTable.getAbstract() + "\nComment:\n" + selectedDataTable.getComment();
                          JOptionPane.showMessageDialog(GUI.this,output,selectedDataTable.getTitle(),JOptionPane.INFORMATION_MESSAGE);
                        }
                        else {
                          statusTextField.setText(" # You must select a DataColumn!");
                        }
                      }

                      /* Table Spreadsheet */
                      else if (command.equals("spreadsheet")) {
                        // there is always a selected DataTable so no need to woory about that 'constraint'
                        int dataTableKey = getDataTableIndex();
                        DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
                        // check if the selected DataTable has columns loaded
                        if (selectedDataTable.columnCount()!=0) {
                          /*InfoDialog d = new InfoDialog(GUI.this, "DataTable Spreadsheet",
                                                        selectedDataTable.getColumn(0).getSize(), //row number
                                                        selectedDataTable.columnCount()); //column number
//System.out.println("rows: "+selectedDataTable.getColumn(0).getSize()+"columns: "+selectedDataTable.columnCount());
                          d.addDataTable(selectedDataTable);
                          d.setLocation(200,200);
                          d.show();*/
                          
                          displayTable(selectedDataTable);
                        }
                        else {
                          statusTextField.setText(" # The DataTable contains no columns!");
                        }
                      }

                      /* Functions Table */
                      else if (command.equals("funtable")) {
                        /*InfoDialog d = new InfoDialog(GUI.this, "Functions Table", -1,-1);
                        for(int i=0; i<functionsTable.getSize(); i++)
                          d.addLine(functionsTable.getFunctionName(i).toString()+" = "+
                                     functionsTable.getFunctionValue(i).toString());
                        d.setLocation(200,200);
                        d.show();*/
                        
                        showFunctionTable(GUI.this);
                      }

                      /* Constants Table */
                      else if (command.equals("contable")) {
                        /*InfoDialog d = new InfoDialog(GUI.this, "Constants Table", -1,-1);
                        for(int i=0; i<dataTableVariables.getSize(); i++) {
                          if (dataTableVariables.getConstantName(i).toString().indexOf(new String("!---"))==-1)
                            d.addLine(dataTableVariables.CONSTANT.get(i)+" = "+
                                     dataTableVariables.CONSTANT_VALUE.get(i) +"  "+
                                     dataTableVariables.CONSTANT_DESCRIPTION.get(i) );
                          else
                            d.addLine("!----------------- "+dataTableVariables.CONSTANT_DESCRIPTION.get(i) );
                        }
                        d.setLocation(200,200);
                        d.setSize(350,500);
                        d.show();*/
                        
                        showConstantTable(GUI.this);
                      }

                      /* Open the command line input console */
                      else if (command.equals("command")) {
                        console.setLocationRelativeTo(GUI.this);
                        console.setVisible(true);
                      }

                    /*Bring up the user interface for a polynomial fit*/
                      else if(command.equals("bestfit"))
                      {
                            BestFitGUI fit = new BestFitGUI(GUI.this);
                            fit.setVisible(true);
                      }
                    /*Bring up the user interface for a Fourier transform*/
                      else if(command.equals("fourier"))
                      {
                            FFT_GUI fourier = new FFT_GUI(thisGUI);
                            fourier.setVisible(true);
                      }
                    /*Bring up the user interface for spline interpolation*/
                      else if(command.equals("splines"))
                      {
                            SplineGUI spline = new SplineGUI(GUI.this);
                            spline.setVisible(true);
                      }
                    
                      /* Open the command line input console Instructions (this is actually the MANUAL.html page scrolled down) */
                      else if (command.equals("command instructions")) {
                        // This is old code, no need to use it because of the AppletContext code below
                        /*InfoDialog manDialog = new InfoDialog(GUI.this, "GUI MANUAL", -3, -2);
                        manDialog.setLocation(200,200);
                        manDialog.show();
                        */

                        /*try {
                        URL manual = new java.net.URL("http://cmms.triumf.ca/~itucakov/html/MANUAL.html#4.1_Command_Line_Console");
                        guiApplet.getAppletContext().showDocument(manual,"newwindow");
                        } catch (java.net.MalformedURLException me) {
                            System.err.println("Error reading Console Instructions for AppletContext: "+ me.getMessage());
                        }
                      }
                      else if (command.equals("frames")) {
                        framesFrame.setLocation(200,200);
                        framesFrame.show();*/
                        
                        //open the command-line console, type 'help', and press enter for the help functions
                        console.setLocationRelativeTo(GUI.this);
                        console.setVisible(true);
                        console.append(console.getHelpText("prompt")+"\n"); //displays help menu instructions
                        
                      }

                      /* Define Function */
                      else if (command.equals("function")) {
                        defDialog = new Dialog(GUI.this, "Define a new function", true);
                        defDialog.setLayout(new GridLayout(3,2));
                        defTextField1.setText("");
                        defTextField2.setText("");
                        defTextField3.setText("");
                        defDialog.add(new JLabel("Name:"));
                        defDialog.add(defTextField1);
                        defDialog.add(new JLabel("Expression (in terms of parameter x):"));
                        defDialog.add(defTextField2);
                        defDialog.add(defFunButton);
                        defDialog.add(defButton2);

                        defDialog.setSize(new Dimension(600,160));
                        defDialog.setLocation(100,100);
                        defDialog.show();  // display the dialog and block until answered
                      }

                      /* Define Constant */
                      else if (command.equals("constant")) {
                        defDialog = new Dialog(GUI.this, "Define a new Constant (Note: UPPERCASE)", true);
                        defDialog.setLayout(new GridLayout(4,2));
                        defTextField1.setText("");
                        defTextField2.setText("");
                        defDialog.add(new JLabel("Constant name:"));
                        defDialog.add(defTextField1);
                        defDialog.add(new JLabel("Constant value:"));
                        defDialog.add(defTextField2);
                        defDialog.add(new JLabel("Function description (optional):"));
                        defDialog.add(defTextField3);
                        defDialog.add(defConButton);
                        defDialog.add(defButton2);

                        defDialog.setSize(new Dimension(600,130));
                        defDialog.setLocation(100,100);
                        defDialog.show();  // display the dialog and block until answered

                      }

                      /* Repack the DataTable */
                      else if (command.equals("repack")) {
                        repackDialog = new Dialog(GUI.this, "Repack", true);
                        repackDialog.setLayout(new GridLayout(3,2));
                        repackField.setText("1");
                        repackDialog.add(new JLabel("Enter the integer value to repack the Data Table:"));
                        repackDialog.add(repackField);
                        repackDialog.add(new JLabel(("This will merge adjacent bins as specified.")));
                        repackDialog.add(new JLabel(("")));
                        repackOKButton.setActionCommand("Repack OK"); 
                        repackCancelButton.setActionCommand("Repack Cancel");
                        repackDialog.add(repackOKButton);
                        repackDialog.add(repackCancelButton);

                        repackDialog.setSize(new Dimension(625,90));
                        repackDialog.setLocation(100,100);
                        repackDialog.show();  // display the dialog and block until answered

                      }

                      /* This is used to change parameters of the algorithm used for automatic-symbol-size setting */
                      else if (command.equals("symbol size")) {
                        defDialog = new Dialog(GUI.this, "Automatic Symbol Size Settings", true);
                        defDialog.setLayout(new GridLayout(4,2));
                        defTextField1.setText(Long.toString(nPix));
                        defTextField2.setText(Long.toString(Nmax));
                        defDialog.add(new JLabel("Symbol size = 0.5*nPix/max(N,Nmax),"));
                        defDialog.add(new JLabel(" where N = # of graph points"));
                        defDialog.add(new JLabel("nPix:"));
                        defDialog.add(defTextField1);
                        defDialog.add(new JLabel("Nmax:"));
                        defDialog.add(defTextField2);
                        defDialog.add(symbSizeButton);
                        defDialog.add(defButton2);

                        defDialog.setSize(new Dimension(600,130));
                        defDialog.setLocation(100,100);
                        defDialog.show();  // display the dialog and block until answered

                      }

                      /* Choose a page format */
                      else if (command.equals("plot size setup")) {
                        // Create four textfields that will be used to set the plot width and height
                        TextField widthPortraitText = new TextField();
                        TextField widthLandscapeText = new TextField();
                        TextField heightPortraitText = new TextField();
                        TextField heightLandscapeText = new TextField();
                        // Pop-up a Dialog and let the user select a page format
                        printPageFormatDialog = new Dialog(GUI.this, "Plot Size Settings for different Page Formats:", true);
                        printPageFormatDialog.setLayout(new GridBagLayout());

                        // Setup the options for portrait print which include width and height of plot in cm
                        gridConstr.gridx = 0; gridConstr.gridy = 0;
                        printPageFormatDialog.add(new JLabel("PORTRAIT"), gridConstr);
                        gridConstr.gridx = 1; gridConstr.gridy = 0;
                        if (pageForm.getOrientation()==1) pageFormCheckP.setState(true);
                        printPageFormatDialog.add(pageFormCheckP, gridConstr);
                        ////
                        gridConstr.gridx = 0; gridConstr.gridy = 1;
                        printPageFormatDialog.add(new JLabel("Width (cm) (15): "), gridConstr);
                        gridConstr.gridx = 1; gridConstr.gridy = 1;
                        widthPortraitText.setColumns(5);
                        widthPortraitText.setText(Long.toString(plotWidthPort)); // Default value is 15
                        printPageFormatDialog.add(widthPortraitText, gridConstr);
                        gridConstr.gridx = 3; gridConstr.gridy = 1;
                        printPageFormatDialog.add(new JLabel("Height (cm) (12): "), gridConstr);
                        gridConstr.gridx = 4; gridConstr.gridy = 1;
                        heightPortraitText.setColumns(5);
                        heightPortraitText.setText(Long.toString(plotHeightPort)); // Default value is 12
                        printPageFormatDialog.add(heightPortraitText, gridConstr);

                        gridConstr.gridx = 0; gridConstr.gridy = 2;
                        printPageFormatDialog.add(new JLabel("LANDSCAPE"), gridConstr);
                        gridConstr.gridx = 1; gridConstr.gridy = 2;
                        if (pageForm.getOrientation()==0) pageFormCheckL.setState(true);
                        printPageFormatDialog.add(pageFormCheckL, gridConstr);
                        ///////
                        gridConstr.gridx = 0; gridConstr.gridy = 3;
                        printPageFormatDialog.add(new JLabel("Width (cm) (15): "), gridConstr);
                        gridConstr.gridx = 1; gridConstr.gridy = 3;
                        widthLandscapeText.setColumns(5);
                        widthLandscapeText.setText(Long.toString(plotWidthLand)); // Default value is 15
                        printPageFormatDialog.add(widthLandscapeText, gridConstr);
                        gridConstr.gridx = 3; gridConstr.gridy = 3;
                        printPageFormatDialog.add(new JLabel("Height (cm) (19): "), gridConstr);
                        gridConstr.gridx = 4; gridConstr.gridy = 3;
                        heightLandscapeText.setColumns(5);
                        heightLandscapeText.setText(Long.toString(plotHeightLand)); // Default value is 19
                        printPageFormatDialog.add(heightLandscapeText, gridConstr);

                        gridConstr.gridx = 0; gridConstr.gridy = 4;
                        printPageFormatDialog.add(new JLabel("When you print, the plot will be "), gridConstr);
                        gridConstr.gridx = 1; gridConstr.gridy = 4;
                        printPageFormatDialog.add(new JLabel("adjusted to the values above"), gridConstr);

                        gridConstr.gridx = 1; gridConstr.gridy = 5;
                        printPageFormatDialog.add(new JLabel("Note: input integer values"), gridConstr);

                        gridConstr.gridx = 0; gridConstr.gridy = 6;
                        printPageFormatDialog.add(pageFormatButton, gridConstr);

                        printPageFormatDialog.setSize(new Dimension(600,300));
                        printPageFormatDialog.setLocation(100,100);
                        printPageFormatDialog.show();  // display the dialog and block until answered

                      }

                      /* Print out all the plots that are viewed */
                      else if (command.equals("print")) {
                        printPlot();
                      }
                    
                    /*
                     *Print the plot(s) to a PostScript file
                     */
                      else if (command.equals("printPS"))
                      {
                        //System.out.println("Listener fired for printToPostScript");
                        printToPostScript();
                      }

                    /*
                      //open windows for the various plot options
                      else if(command.equals("titlelabeloptions"))
                      {
                        showTitleLabelOptions();
                      }
                      else if(command.equals("axisoptions"))"Pin
                      {
                        showAxisOptions();
                      }
                      else if(command.equals("plotoptions"))
                      {
                        showPlotOptions();
                      }
                     **/
                    
                      else if (command.equals("axistypes")) {
                        // Create and display Axis Types Frame
                        // Contains the log,linear and inverse settings for the axis
                        final JFrame aFrame = new JFrame("Axis Types");
                        aFrame.setBackground(Color.lightGray);
                        // add the Frame Label
                        aFrame.getContentPane().add(
                          new JLabel(" Data File Components - "+String.valueOf(layoutChoice.getSelectedIndex()+1)+" "), "North");
                        // Add the Axis Types Panel
                        aFrame.getContentPane().add(((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).axisTypesPanel, "Center");
                        Button aButton = new Button("Close");
                        aFrame.getContentPane().add(aButton, "South");
                        aFrame.setSize(200,300);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerTypes = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Close"))
                          aFrame.dispose();
                          display();
                         }
                        };
                        aButton.addActionListener(buttonlistenerTypes);

                        aFrame.setLocation(200,200);
                        aFrame.show();
                      }

                      else if (command.equals("graphtypes")) {
                        // Create and display the GraphTypesPanel in aFrame
                        final JFrame aFrame = new JFrame("Point Types & Sizes");
                        aFrame.setBackground(Color.lightGray);
                        // add the Frame Label
                        aFrame.getContentPane().add(
                          new JLabel(" Data File Components - "+String.valueOf(layoutChoice.getSelectedIndex()+1)+"  "), "North");
              // Fill the typeColorTextField in PlotOptionsFrame with appropriate numbers:
              //                       (int)type.(int)color     (note: '-2' because there is types: 'none' and 'dot' before all other)
              PlotOptionsFrame tempPOF = ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex()));
              tempPOF.typeColorTextField.setText(
                                        String.valueOf(tempPOF.gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(tempPOF.graphColorChoice.getSelectedIndex()) );
                        
              /*tempPOF.graphPointSpinner.setValue(PlotOptionsFrame.POINT_TYPE[0]);
              tempPOF.graphColourSpinner.setValue(PlotOptionConstants.COLOR[0]);*/
              
                        // Add the graphTypesPanel
                        aFrame.getContentPane().add(tempPOF.graphTypesPanel, "Center");
                        Button aButton = new Button("Close");
                        aFrame.getContentPane().add(aButton, "South");
                        aFrame.setSize(250,400);
                        aFrame.pack();
                        aFrame.isEnabled();

                        ActionListenerS buttonlistenerPoint = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                           if (e.getActionCommand().equals("Close")) {

                             // First, make sure that if the user entered a new 'numerical' value for type and color, to update that info
                             PlotOptionsFrame tempPlotOptions = (PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex());
                             if ( tempPlotOptions.graphNumberChoice.getItemCount()!=0 ) {
                               // If the typeColorTextField is not empty, load its values into appropriate group and graph vectors:  (int)type+2.(int)color
                               TextField tempTextField = tempPlotOptions.typeColorTextField;
                               if (!(tempTextField.getText().trim().equals(""))) {
                                 int tempTypeIndex = 0;
                                 int tempColorIndex = 0;
                                 if (tempTextField.getText().trim().indexOf(".")!=-1 ) { // the form is "(int)type.(int)color"
                                   tempTypeIndex = Integer.parseInt( tempTextField.getText().trim().substring(0,tempTextField.getText().trim().indexOf(".")) );
                                   tempColorIndex = Integer.parseInt( tempTextField.getText().trim().substring(tempTextField.getText().trim().indexOf(".")+1) );
                                 } else { // the form is "(int)type"
                                     tempTypeIndex = Integer.parseInt( tempTextField.getText().trim() );
                                     // Calculate the total number of graphs
                                     int tempCount = 0;
                                     for (int x =0; x<tempPlotOptions.groupNumberChoice.getSelectedIndex(); x++) {
                                       for (int y=0; y<((DataTable) ((Vector) yAxisDataTable.elementAt(layoutChoice.getSelectedIndex())).elementAt(x)).columnCount();y++) {// count them all
                                         tempCount = tempCount+1;
                                       }
                                     }
                                     tempCount = tempCount+tempPlotOptions.graphNumberChoice.getSelectedIndex(); // add last group items
                                     tempColorIndex = (tempCount)%14; // divide using MOD, since there are no more than 14 types of graph Symbols
                                 }
                                 /*// Set Symbol Type   - note: '+2' is because there is already 'none' and 'dot' items in the gTypeOtherChoice
                                 ((Vector)tempPlotOptions.graphTypeOther.elementAt(tempPlotOptions.groupNumberChoice.getSelectedIndex())).
                                                                            setElementAt( tempPlotOptions.POINT_TYPE[tempTypeIndex] ,
                                                                                                 tempPlotOptions.graphNumberChoice.getSelectedIndex() );
                                 System.out.println(tempPlotOptions.groupNumberChoice.getSelectedIndex());
                                 System.out.println("GUI value:\t\t\t" + tempPlotOptions.POINT_TYPE[tempTypeIndex]);
                                 // Set Symbol (Graph) Color
                                 ((Vector)tempPlotOptions.graphColor.elementAt(tempPlotOptions.groupNumberChoice.getSelectedIndex())).
                                                                   setElementAt(PlotOptionConstants.COLOR[tempColorIndex],
                                                                                       tempPlotOptions.graphNumberChoice.getSelectedIndex() );*/
                                 
                                 // Set Symbol Type   - note: '+2' is because there is already 'none' and 'dot' items in the gTypeOtherChoice
                                 //System.out.println(tempPlotOptions.groupNumberChoice.getSelectedIndex());
                                 //System.out.println(tempTypeIndex);
                                 //System.out.println(tempPlotOptions.graphNumberChoice.getSelectedIndex());
                                 ((Vector)tempPlotOptions.graphTypeOther.elementAt(tempPlotOptions.groupNumberChoice.getSelectedIndex())).
                                                                            setElementAt( tempPlotOptions.POINT_TYPE[tempTypeIndex] ,
                                                                                                 tempPlotOptions.graphNumberChoice.getSelectedIndex() );
                                 // Set Symbol (Graph) Color
                                 ((Vector)tempPlotOptions.graphColor.elementAt(tempPlotOptions.groupNumberChoice.getSelectedIndex())).
                                                                   setElementAt(PlotOptionConstants.COLOR[tempColorIndex],
                                                                                       tempPlotOptions.graphNumberChoice.getSelectedIndex() );

              // Fill the typeColorTextField with appropriate numbers: (int)type.(int)color     (note: '-2' because there is types: 'none' and 'dot' before all other)
              tempPlotOptions.typeColorTextField.setText( String.valueOf(tempPlotOptions.gTypeOtherChoice.getSelectedIndex()-2)+
                                                                "."+String.valueOf(tempPlotOptions.graphColorChoice.getSelectedIndex()) );
                               }
                               aFrame.dispose();
                               display();
                             }
                           }
                         }
                        };
                        aButton.addActionListener(buttonlistenerPoint);

                        aFrame.setLocation(200,200);
                        aFrame.show();
                      }

                      else if (command.equals("titlesetup")) {
                        // Contains settings to change the plot title and labels
                        final JFrame aFrame = new JFrame("Title and Label, Manual Setup");
                        aFrame.setBackground(Color.lightGray);
                        ScrollPane aFrameScrollbar = new ScrollPane();
                        JPanel aPanel = new JPanel();
                        aPanel.setLayout(new BorderLayout());

                        // add the Frame Label
                        aPanel.add(
                          new JLabel("    ***  FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)+"   ***"), "North");
                        // Add the titleSetupPanel
                        aPanel.add(((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).titleSetupPanel, "Center");
                        Button aButton = new Button("Close");
                        aPanel.add(aButton, "South");

                        aFrameScrollbar.add(aPanel);
                        aFrame.getContentPane().add(aFrameScrollbar);

                        aFrame.setSize(700,250);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerTitle = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Close"))
                          aFrame.dispose();
                          display();
                         }
                        };
                        aButton.addActionListener(buttonlistenerTitle);

                        aFrame.setLocation(200,200);
                        aFrame.show();
                      }
                      else if (command.equals("captionsetup")) {
                        captionsFrameDisplay();
                      }

                      else if (command.equals("limits")) {
                        // Contains the Min-Max settings for the plot
                        final JFrame aFrame = new JFrame("Limits Setup");
                        aFrame.setBackground(Color.lightGray);
                        // add the Frame Label
                        aFrame.getContentPane().add(
                          new JLabel("FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)), "North");
                        // Add the minMaxPanel
                        aFrame.getContentPane().add(((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).minMaxPanel, "Center");
                        JPanel buttonPanel = new JPanel();
                        buttonPanel.setLayout(new GridBagLayout());
                        buttonPanel.setBackground(Color.LIGHT_GRAY);
                        // this Button displays teh plot
                        Button displayButton = new Button("Display Plot");
                          gridConstr.gridx = 0; gridConstr.gridy = 0;
                          buttonPanel.add(displayButton, gridConstr);
                        // this Button clears all the textfield values
                        Button clearButton = new Button("Clear Limits");
                          gridConstr.gridx = 1; gridConstr.gridy = 0;
                          buttonPanel.add(clearButton, gridConstr);
                        // This Button closes this Frame
                        Button aButton = new Button("Close");
                          gridConstr.gridx = 0; gridConstr.gridy = 3;
                          buttonPanel.add(aButton, gridConstr);
                        aFrame.getContentPane().add(buttonPanel, "South");
                        aFrame.setSize(300,400);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerLimits = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Display Plot")) {
                            display();
                          }
                          else if (e.getActionCommand().equals("Close")) {
                            aFrame.dispose();
                            display();
                          }
                          else if(e.getActionCommand().equals("Find Best Fit"))
                          {
                              if(JOptionPane.showConfirmDialog(GUI.this, "This feature is still under development.\n" +
                                      "While it appears to be stable, there may be bugs.\n" +
                                      "Are you you want to use this feature?", "Under construction", JOptionPane.YES_NO_OPTION,
                                      JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION)
                              {
                                  BestFitGUI fitter = new BestFitGUI(GUI.this);
                                  fitter.setVisible(true);
                              }
                          }
                          else if(e.getActionCommand().equals("Spline Interpolation"))
                          {
                              System.out.println("Spline Interpolation picked up by listener at line 1444");
                                if(JOptionPane.showConfirmDialog(GUI.this, "This feature is still under development.\n" +
                                      "While it appears to be stable, there may be bugs.\n" +
                                      "Are you you want to use this feature?", "Under construction", JOptionPane.YES_NO_OPTION,
                                      JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION)
                              {
                                  SplineGUI spline = new SplineGUI(GUI.this);
                                  spline.setVisible(true);
                              }
                          }
                          else if (e.getActionCommand().equals("Clear Limits")) {
                            // Clear the TextFields in PlotOptionsframe's MinMaxPanel
                            clearMinMaxLimits();
                          }
                         }
                        };
                        aButton.addActionListener(buttonlistenerLimits);
                        clearButton.addActionListener(buttonlistenerLimits);
                        displayButton.addActionListener(buttonlistenerLimits);

                        aFrame.setLocation(200,200);
                        aFrame.show();
                      }
                   
                      else if (command.equals("status")) {
                        /*// Add a Label with a number that will tell the user
                        //   which Frame of graphs this window refers to (the Label is the 0th component in the frame)
                        final JFrame aFrame = new JFrame("Status Info");
                        aFrame.setBackground(Color.lightGray);
                        // add the Frame Label
                        aFrame.getContentPane().add(
                          new JLabel("    ***  FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)+"   ***"), "North");
                        // Add the statusInfoPanel
                        aFrame.getContentPane().add(((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).statusInfoPanel, "Center");
                        Button aButton = new Button("Close");
                        aFrame.getContentPane().add(aButton, "South");
                        aFrame.setSize(300,400);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerStatus = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Close"))
                          aFrame.dispose();
                         }
                        };
                        aButton.addActionListener(buttonlistenerStatus);

                        aFrame.setLocation(200,200);
                        aFrame.show();*/
                        
                        PlotOptionsFrame currPOF = (PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex());
                        
                        String output = "";
                        output = output + "\nColours:";
                        output = output + "\n\tBackground:\t" + currPOF.backgroundColorLabel.getText();
                        output = output + "\n\tBorders:\t" + currPOF.borderlineColorLabel.getText();
                        output = output + "\nFonts:";
                        output = output + "\n\tLabels:\t" 
                                + currPOF.labelTypeLabel.getText() + ", " + currPOF.labelStyleLabel.getText() + ", " + currPOF.labelSizeLabel.getText();
                        output = output + "\n\tTitles:\t"
                                + currPOF.titleTypeLabel.getText() + ", " + currPOF.titleStyleLabel.getText() + ", " + currPOF.titleSizeLabel.getText();;
                        output = output + "\n\tNumbers:\t"
                                + currPOF.numbersTypeLabel.getText() + ", " + currPOF.numbersStyleLabel.getText() + ", " + currPOF.numbersSizeLabel.getText();;
                        
                        JOptionPane.showMessageDialog(GUI.this, output, "FRAME #" + String.valueOf(layoutChoice.getSelectedIndex()+1) + " status",
                                JOptionPane.INFORMATION_MESSAGE);
                      }

                      else if (command.equals("manual")) {
                        // This is old code, no need to use it because of the AppletContext code below
                        /*InfoDialog manDialog =
                          new InfoDialog(GUI.this, "GUI MANUAL", -2, -2);
                        manDialog.setLocation(200,200);
                        manDialog.show();
                        */
                        //System.out.println("manual");
                        try {
                        URL manual = new java.net.URL("http://musr.org/muview/zz_java_gui.pdf");
                        guiApplet.getAppletContext().showDocument(manual,"newwindow");
                        } catch (java.net.MalformedURLException me) {
                            System.err.println("Error reading Console Instructions for AppletContext: "+ me.getMessage());
                        }
                      }

                      else if (command.equals("about")) 
                      {
                            String message = "View - SR Graphical User Interface\n" +
                                    "Created by Ivan Tucakov, Jess Brewer, Jacob Cosman and the SR development team\n" +
                                    "Version 1.2 (summer 2009)";
                            JOptionPane.showMessageDialog(null, message, "View 1.2", JOptionPane.INFORMATION_MESSAGE);
                      }
                    
                    /*
                      //open windows for the various plot options
                      else if(command.equals("titlelabeloptions"))
                      {
                        JOptionPane.showMessageDialog(null,"Titles and labels");
                      }
                      else if(command.equals("axisoptions"))
                      {
                        JOptionPane.showMessageDialog(null,"Axes");
                      }
                      else if(command.equals("plotoptions"))
                      {
                        JOptionPane.showMessageDialog(null,"Plot");
                      }
                     **/
                    }
              };
    openMenuItem.addActionListener(menuBarListener);
    saveMenuItem.addActionListener(menuBarListener);
    savePNGMenuItem.addActionListener(menuBarListener);
    resetMenuItem.addActionListener(menuBarListener);
    loadMUDSpectrum.addActionListener(menuBarListener);
    importFileMenuItem.addActionListener(menuBarListener);
    importFileMenuItem2.addActionListener(menuBarListener);
    importLocalFileMenuItem.addActionListener(menuBarListener);
    openBlankMenuItem.addActionListener(menuBarListener);
    importXMLFileMenuItem.addActionListener(menuBarListener);
    importCSVFileMenuItem.addActionListener(menuBarListener);
    //importURLMenuItem.addActionListener(menuBarListener); Closed for good! It's just a matter of time when I will erase it
    removeMenuItem.addActionListener(menuBarListener);
    xmlMenuItem.addActionListener(menuBarListener);
    dbMenuItem.addActionListener(menuBarListener);
    csvMenuItem.addActionListener(menuBarListener);
    quitMenuItem.addActionListener(menuBarListener);

    displayMenuItem.addActionListener(menuBarListener);
    optionsMenuItem.addActionListener(menuBarListener);
    dataTableMenuItem.addActionListener(menuBarListener);
    showMenuItem.addActionListener(menuBarListener);
    spreadSheetMenuItem.addActionListener(menuBarListener);
    funTableMenuItem.addActionListener(menuBarListener);
    conTableMenuItem.addActionListener(menuBarListener);
    
    fourierMenuItem.addActionListener(menuBarListener);
    splinesMenuItem.addActionListener(menuBarListener);
    bestFitMenuItem.addActionListener(menuBarListener);

    dbCheckerItem.addActionListener(menuBarListener);
    commandLineConsoleItem.addActionListener(menuBarListener);
    consoleInstructionsItem.addActionListener(menuBarListener);
    createFunMenuItem.addActionListener(menuBarListener);
    createConMenuItem.addActionListener(menuBarListener);
    repackMenuItem.addActionListener(menuBarListener);
    symbSizeMenuItem.addActionListener(menuBarListener);
    printMenuItem.addActionListener(menuBarListener);
    postScriptPrintMenuItem.addActionListener(menuBarListener);
    printSetupMenuItem.addActionListener(menuBarListener);

    axisTypesMenuItem.addActionListener(menuBarListener);
    graphTypesMenuItem.addActionListener(menuBarListener);
    titleSetupMenuItem.addActionListener(menuBarListener);
    limitsMenuItem.addActionListener(menuBarListener);
    captionsMenuItem.addActionListener(menuBarListener);
    framesMenuItem.addActionListener(menuBarListener);                
               
    /*
    titleLabelMenuItem.addActionListener(menuBarListener);
    axesMenuItem.addActionListener(menuBarListener);
    plotMenuItem.addActionListener(menuBarListener);
     **/
       
    manualMenuItem.addActionListener(menuBarListener);
    aboutMenuItem.addActionListener(menuBarListener);


    // Handle events on the Lists
    listListener = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        ItemSelectable selectedItem = e.getItemSelectable();
          // DataTable List selected
          if (selectedItem == dataTableList) {
            // if nothing is selected, don't do anything to avoid problems
            if (dataTableList.getSelectedIndex()==-1 && dataTableList.getItemCount()>0) {} // this is a bug where the item is not selected
            else {
              String selected = dataTableList.getSelectedItem();
              tempDataColumns = ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex()));
              reloadColumnList();
            }
          }
          // Column List selected
          else if (selectedItem == columnList) {
            // if nothing is selected, don't do anything to avoid problems
            if (columnList.getSelectedIndex()==-1 && columnList.getItemCount()>0) {} // this is a bug where the item is not selected
            else {
              String selected = columnList.getSelectedItem();
            }
          }
          // x-axis List selected
          else if (selectedItem == xAxisList) {
            // if nothing is selected, don't do anything to avoid problems
            if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) {} // this is a bug where the item is not selected
            else {
              reloadyAxisList();
              int selectedIndex = xAxisList.getSelectedIndex();
              if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) // this is a bug where the item is not selected
              refreshDisplayOptions(selectedIndex);
              String selected = ( (DataColumn) ((DataTable) xAxisDataTable.elementAt(layoutChoice.getSelectedIndex()))
                       .getColumn(xAxisList.getSelectedIndex()) ).getName();
            }
          }
          // y-axis List selected
          else if (selectedItem == yAxisList) {
            // if nothing is selected, don't do anything to avoid problems
            if (yAxisList.getSelectedIndex()==-1 && yAxisList.getItemCount()>0) {} // this is a bug where the item is not selected
            else {
              String selected = yAxisList.getSelectedItem();
            }
         }
      }
    };
    dataTableList.addItemListener(listListener);
    columnList.addItemListener(listListener);
    xAxisList.addItemListener(listListener);
    yAxisList.addItemListener(listListener);


    // Handle events on the Display Options Checkboxes and the Choice
    displayOptionsListener = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
				ItemSelectable selectedItem = e.getItemSelectable();
				// Reload xAxisList and yAxisList with the axises that respond to the chosen Frame layout
 				if (selectedItem == layoutChoice) {
                                  reloadxAxisList();
                                  reloadyAxisList();
                                  // Change the label in the Plot Select Window to the new frame
                                  ((JLabel)
                                    ((PlotOptionsFrame)plotsOptions.
                                      elementAt(layoutChoice.getSelectedIndex())).layoutFrameLabel )
                                        .setText("    ***  FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)+"   ***");
		    		}
				// disEnable manual Choices
				else if (selectedItem == gridAutoCheckbox) {
					gridXChoice.setEnabled(false);
					gridYChoice.setEnabled(false);
				}
				// enable manual Choices
				else if (selectedItem == gridManualCheckbox) {
					gridXChoice.setEnabled(true);
					gridYChoice.setEnabled(true);
				}
				// Verify that gridYChoice is not zero
 				else if (selectedItem == gridXChoice) {
					if (gridXChoice.getSelectedIndex()==0 && gridYChoice.getSelectedIndex()==0) {
						gridXChoice.select(1);
						statusTextField.setText(" # Use auto Grid for (*,*)!");
					}
				}
				// Verify that gridXChoice is not zero
 				else if (selectedItem == gridYChoice) {
					if (gridYChoice.getSelectedIndex()==0 && gridXChoice.getSelectedIndex()==0) {
						gridYChoice.select(1);
						statusTextField.setText(" # Use auto Grid for (*,*)!");
					}
				}
 				// set Visible option for the selected x-axis
 				else if (selectedItem == visibleCheckbox) {
					int selectedxAxisIndex = xAxisList.getSelectedIndex();
					boolean state = visibleCheckbox.getState();
					((PlotRecord) ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).elementAt(selectedxAxisIndex)).setVisible(state);
				}
 				// set Inverted option for the selected x-axis
 				else if (selectedItem == invertedCheckbox) {
					int selectedxAxisIndex = xAxisList.getSelectedIndex();
					boolean state = invertedCheckbox.getState();
					((PlotRecord) ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).elementAt(selectedxAxisIndex)).setInverted(state);
				}
      }
    };
    invertedCheckbox.addItemListener(displayOptionsListener);
    visibleCheckbox.addItemListener(displayOptionsListener);
    layoutChoice.addItemListener(displayOptionsListener);
    gridAutoCheckbox.addItemListener(displayOptionsListener);
    gridManualCheckbox.addItemListener(displayOptionsListener);
    gridXChoice.addItemListener(displayOptionsListener);
    gridYChoice.addItemListener(displayOptionsListener);


    // Handle events on the Buttons
    // Add/Removing Columns and their information in/from x-axis/y-axis Lists
    buttonListener = new ActionListenerS() {
      public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        // add an item to layoutChoice.
        // clear xAxisList
        if (command.equals("add")) {
          String layoutName = Long.toString(layoutChoice.getItemCount()+1);
          statusTextField.setText("> Adding Frame "+layoutName+"... ");
          layoutChoice.add(layoutName);
          plotsOptions.addElement(new PlotOptionsFrame());
          layoutChoice.select(layoutChoice.getItemCount()-1);
          // Change the label in the Plot Select Window to the newly selected frame
          ((JLabel)
            ((PlotOptionsFrame)plotsOptions.
               elementAt(layoutChoice.getSelectedIndex())).layoutFrameLabel )
                 .setText("    ***  FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)+"   ***");

                                        xAxisDataTable.addElement(new DataTable(new String("")));
                                        titlexAxis.addElement(new Vector());
                                        namexAxis.addElement(new Vector());
                                        yAxisDataTable.addElement(new Vector());
                                        titleyAxis.addElement(new Vector());

                                        plotsRecord.addElement(new Vector());

                                        xAxisList.removeAll();
                                        yAxisList.removeAll();
                                        setDisplayOptionsEnabled(false);

                                      statusTextField.setText("Frame "+layoutName+" adding done.");
        }
        // removes the selected layaut frame and all the plots in it,
        else if (command.equals("remove")) {
          if (layoutChoice.getItemCount() > 1) {   // if this is not the last choice
            int selectedLayoutIndex = layoutChoice.getSelectedIndex();
            //Display the action:
            String layoutName = layoutChoice.getSelectedItem();
            statusTextField.setText("> You have removed Frame "+layoutName+".");
/* this might not be necessary to do:
						// go through all the x-axis columns
						for (int i=0; i<xAxisList.getItemCount(); i++)
                                                  removexAxisColumn(i--);
*/
                                                xAxisDataTable.removeElementAt(selectedLayoutIndex);
                                                yAxisDataTable.removeElementAt(selectedLayoutIndex);
                                                titlexAxis.removeElementAt(selectedLayoutIndex);
                                                namexAxis.removeElementAt(selectedLayoutIndex);
                                                titleyAxis.removeElementAt(selectedLayoutIndex);
                                                plotsRecord.removeElementAt(selectedLayoutIndex);

                                                plotsOptions.removeElementAt(selectedLayoutIndex);  // remove layout frame's PlotOptionFrame
                                                layoutChoice.remove(layoutChoice.getItemCount()-1);    // remove Last Choice
                                                layoutChoice.select(0);
                                                // Change the label in the Plot Select Window to the newly selected frame
                                                ((JLabel)
                                                  ((PlotOptionsFrame)plotsOptions.
                                                    elementAt(layoutChoice.getSelectedIndex())).layoutFrameLabel )
                                                      .setText("    ***  FRAME #"+String.valueOf(layoutChoice.getSelectedIndex()+1)+"   ***");


                                                // Reduce the layout settings by one for plots that are higher in order than selected frame
                                                // plotsRecord vector has already been 'cut' so just change the layout value in it
                                                // note: both tempSize and selectedLayoutIndex have old values
                                                // note: no other values in plotsRecord need to be changed
                                                int tempSize = layoutChoice.getItemCount();
                                                if (selectedLayoutIndex < tempSize)
                                                  for (int i=selectedLayoutIndex; i<tempSize; i++)
                                                    for (int j=0; j<((Vector)plotsRecord.elementAt(i)).size(); j++)
                                                      ((PlotRecord)((Vector)plotsRecord.elementAt(i)).elementAt(j)).setLayoutPosition(i);
                                                          reloadxAxisList();            // reload xAlisList
                                                          reloadyAxisList();            // reload yAlisList
                                      }
                                      else  {
                                          statusTextField.setText(" # Cannot Remove. This is the last Frame!");
                                        }
                              }

                              
                            else if(command.equals("Show DataTable Info"))
                            {
                                showDataTableInfo();
                            }
        
                            else if(command.equals("Refresh Data Table"))
                            {
                                refreshDataTable();
                            }

                              // add a column in x-axis List
                              else if (command.equals("add X")) {
                                int selectedColumnIndex = columnList.getSelectedIndex();             // get selected item
                                int selectedLayoutIndex = layoutChoice.getSelectedIndex();
                                if ((selectedColumnIndex != -1) && (selectedLayoutIndex != -1)) {
                                  addX(selectedColumnIndex, selectedLayoutIndex);
                                }
                                else {
                                  statusTextField.setText(" # You must select a column in Columns List ");
                                }
                              }
                              // remove a column in x-axis List
                              else if (command.equals("remove X")) {
                                int selectedxAxisIndex = xAxisList.getSelectedIndex();   // get selected xAxis item
                                int selectedLayoutIndex = layoutChoice.getSelectedIndex(); // get selected Layout Frame index
                                if ((selectedxAxisIndex != -1) && (selectedLayoutIndex != -1)) {
                                  // remove the selected X column (note: this function will also call removeXOptions method for plotsOptions)
                                  removexAxisColumn(selectedxAxisIndex);
                                  if (xAxisList.getItemCount()>0) {
                                    if (selectedxAxisIndex==0) xAxisList.select(0);    // select other entry in list
                                    else xAxisList.select(selectedxAxisIndex-1);
                                    refreshDisplayOptions(xAxisList.getSelectedIndex());// refresh Display Options
                                  }
                                  else setDisplayOptionsEnabled(false);                  // options not usable any more

                                  reloadyAxisList();                                     // reload yAxisList
                                }
                                else {
                                statusTextField.setText(" # You must select a column in x-Axis List");
                                }
                              }
                              // add a column in y-axis List
                              else if (command.equals("add Y")) {
                                int selectedColumnIndex = columnList.getSelectedIndex(); // get selected items
                                int selectedxAxisIndex = xAxisList.getSelectedIndex();
                                int selectedLayoutIndex = layoutChoice.getSelectedIndex();
                                if ((selectedColumnIndex != -1) && (selectedxAxisIndex != -1)) {
                                  addY(selectedColumnIndex,selectedxAxisIndex,selectedLayoutIndex);
                                }
                                else {
                                  statusTextField.setText(" # Columns in Columns and x-axis Lists must be selected");
                                }
                              }
                              // remove a column in y-axis List
                              else if (command.equals("remove Y")) {
                                int selectedxAxisIndex = xAxisList.getSelectedIndex();    // get selected items
                                int selectedyAxisIndex = yAxisList.getSelectedIndex();
                                int selectedLayoutIndex = layoutChoice.getSelectedIndex();
                                if ((selectedyAxisIndex != -1) && (selectedxAxisIndex != -1)) {
                                  yAxisList.remove(selectedyAxisIndex);                   // remove from the List
                                  if (yAxisList.getItemCount()>0)                         // select other entry
                                    if (selectedyAxisIndex==0) yAxisList.select(0);
                                    else yAxisList.select(selectedyAxisIndex-1);
                                  // record the change
                                  ((DataTable) ((Vector) yAxisDataTable.elementAt(selectedLayoutIndex)).elementAt(selectedxAxisIndex)).removeColumnAt(selectedyAxisIndex);
                                 ((Vector) ((Vector) titleyAxis.elementAt(selectedLayoutIndex)).elementAt(selectedxAxisIndex)).removeElementAt(selectedyAxisIndex);
                                  // update the PlotOptionsFrame
                                  ((PlotOptionsFrame)plotsOptions.elementAt(selectedLayoutIndex)).removeYOptions(selectedxAxisIndex, selectedyAxisIndex);
                                }
                                else {
                                  statusTextField.setText(" # You must select a column in x-Axis and y-Axis List ");
                                }
                              }
                              else if (command.equals("Conditional Plot")) {
                                  // Open the command line input console
                                  console.setVisible(true);
                                  console.jta.append("Enter a conditional plot command (eg. plot T AL if T>20 & T<100) \n");
                              }
                              else if(command.equals("Find Best Fit"))
                              {
                                BestFitGUI fit = new BestFitGUI(GUI.this);
                                fit.setVisible(true);
                              }
                              else if(command.equals("Spline Interpolation"))
                              {
                                  SplineGUI spline = new SplineGUI(GUI.this);
                                  spline.setVisible(true);
                              }
                              else if(command.equals("Repack Data"))
                              {
                                    DataRepacker repacker = new DataRepacker(GUI.this);
                                    repacker.setVisible(true);
                              }
                                else if (command.equals("Display Plot")) {
                                  display();
                                }
                                // This is used for importing a db file at musr.triumf.ca through the RMI
                                else if (command.equals("Open musr File")) {
                                  openFileDialog.hide();
                                  // This is the path and name of the file that is requested
                                  String fileName = openFileTextField1.getText()+openFileTextField2.getText();
                                  // This is the path and name to where to put the file on local machine
                                  String fileSave = openFileTextField3.getText()+openFileTextField4.getText();
                                  // THis is the packet that teh file is contained within for RMI use
                                  FilePacket packet = null;

                                  // This is where DataTable will be read into from the file that is to be sent from the server
                                  statusTextField.setText("> Selected file: "+ openFileTextField2.getText() + ". Importing data...");

                                  try {
                                    obj = (GUI_RMIServer)Naming.lookup("//musr.triumf.ca/GUI_RMIServer");
                                    // transfer the db file from remote server
                                    packet = (FilePacket)
                                                obj.senddbFile(openFileTextField1.getText(),openFileTextField2.getText());
// name of the file // System.out.println("Packet: "+packet.getName().substring(packet.getName().lastIndexOf("/")));
                                    }
                                    catch (Exception ex) {
                                      System.out.println(" GUI_RMI exception, " + ex.getMessage() + ".");
                                     // write this exception in the status field of the GUI
                                    statusTextField.setText("Importing data through RMI interrupted.");
                                     // also pop-up an alert about this
                                     popAlertDialog("  ERROR: RMI connection failure occured",
                                                             "   make sure that the rmiregistry on your computer is active" ,
                                                             "   and that the file name and directory path on musr.triumf.ca server are correct.");
                                    }
                                  /*
                                    // Similar code to this is also contained within the GUI_RMIServerImpl
                                    // here it was used to just load the file directly
                                    if ((openFileTextField2.getText() == null) || (openFileTextField2.getText().length() == 0))
                                      System.out.println("File name is invalid");
                                    File f = new File(openFileTextField1.getText(),openFileTextField2.getText()); // Create a file object
                                    try {
                                      inFile = new FileLineReader(new FileReader(f));
                                    } catch (IOException ioe) { System.err.println("I/O Exception inFile (Open File (RMI)): "+ ioe.getMessage()); }
                                    finally {   // Always be sure to close the input stream!
                                      try { if (inFile != null) inFile.close(); }
                                      catch (IOException ioe) {
                                      System.err.println("I/O Exception DBFileReader file input stream not closed: "+ ioe.getMessage());
                                      }
                                    }
                                    */
                           DBFileReader dbFileReader = null;
                           try {
                                    //
                                    // In order to support folder separators for multiple operating systems, save 'fileSave' as File
                                    File fileSaveFile = new File(fileSave);
                                    // Write this file to the copmuter.
                                    packet.writeTo(new FileOutputStream(fileSaveFile));
                                    // And then read it back in
                                    inFile = new FileLineReader(new FileReader(fileSaveFile));
                                    dbFileReader = new DBFileReader(GUI.this, dataTableVariables, functionsTable);
                                    dbFileReader.loadStream(inFile, fileSave);                             // load the data from the file
                                    // Save the filepath into the DataTable for future reference (throw out the filename first)
                                    // Note that this is not the RMI filepath, it is the new filepath on the local machine
                                    ((DataTable)dictionary.getElement(dictionary.size()-1)).filepath = fileSave;

                              } catch (Exception ex) { System.err.println(" Exception inFile: "+ ex.getMessage());
                                     // write this exception in the status field of the GUI
                                    statusTextField.setText("Importing data through RMI - interrupted.");
                                    // also pop-up an alert about this
                                    popAlertDialog(" ERROR:", "   The path for the file to be saved on your machine doesn't exist","   or the loaded file didn't download properly");
                                     }
                              finally {   // Always be sure to close the input stream!
                                try { if (inFile != null) inFile.close(); }
                                  catch (IOException ioe) {
                                    System.err.println("I/O Exception DBFileReader file input stream not closed: "+ ioe.getMessage());
                                  }
                              }

                                  statusTextField.setText("  Done with file download.");
// ********************** saving speciffic fields into default.gui

                              // Save directory path and filename settings (that user could have changed) into the default.gui (if it exists)
                        SaveGUI load_obj = (SaveGUI)getDefaultGUI();

                        if (load_obj!=null) { // if file tmp(temp)/default.gui file exists
                          load_obj.openFileTextField1.setText(openFileTextField1.getText());
                          load_obj.openFileTextField2.setText(openFileTextField2.getText());
                          load_obj.openFileTextField3.setText(openFileTextField3.getText());
                          load_obj.openFileTextField4.setText(openFileTextField4.getText());
                        }

                        saveDefaultGUI(load_obj);
// **************************
                                }
                                // This is used for imprting a db file from musr.triumf.ca through the RMI
                                else if (command.equals("Cancel musr Open")) {
                                  openFileDialog.hide();
                                }
                                // This is used for importing a db file at cmms.triumf.ca through the RMI
                                else if (command.equals("Open cmms File")) {
                                  openCFileDialog.hide();
                                  // This is the path and name of the file that is requested
                                  String fileName = openCFileTextField1.getText()+openCFileTextField2.getText();
                                  // This is the path and name to where to put the file on local machine
                                  String fileSave = openCFileTextField3.getText()+openCFileTextField4.getText();
                                  // THis is the packet that teh file is contained within for RMI use
                                  FilePacket packet = null;

                                  // This is where DataTable will be read into from the file that is to be sent from the server
                                  statusTextField.setText("> Selected file: "+ openFileTextField2.getText() + ". Importing data...");

                                  try {
                                    obj2 = (GUI_RMIServer2)Naming.lookup("//cmms.triumf.ca/GUI_RMIServer2");
                                    // transfer the db file from remote server
                                    packet = (FilePacket)
                                                obj2.senddbFile(openCFileTextField1.getText(),openCFileTextField2.getText());
// name of the file // System.out.println("Packet: "+packet.getName().substring(packet.getName().lastIndexOf("/")));
                                    }
                                    catch (Exception ex) {
                                      System.out.println(" GUI_RMI exception, " + ex.getMessage() + ".");
                                     // write this exception in the status field of the GUI
                                    statusTextField.setText("Importing data through RMI interrupted.");
                                     // also pop-up an alert about this
                                     popAlertDialog("ERROR: RMI connection failure occured",
                                                             " make sure that the rmiregistry on your computer is active",
                                                            " and that the file name and directory path on cmms.triumf.ca server are correct." );
                                    }
                           DBFileReader dbFileReader = null;
                           try {
                                    // In order to support folder separators for multiple operating systems, save 'fileSave' as File
                                    File fileSaveFile = new File(fileSave);
                                    // Write this file to the computer.
                                    packet.writeTo(new FileOutputStream(fileSaveFile));
                                    // And then read it back in
                                    inFile = new FileLineReader(new FileReader(fileSaveFile));
                                    dbFileReader = new DBFileReader(GUI.this, dataTableVariables, functionsTable);
                                    dbFileReader.loadStream(inFile, fileSave);                             // load the data from the file
                                    // Save the filepath into the DataTable for future reference (throw out the filename first)
                                    // Note that this is not the RMI filepath, it is the new filepath on the local machine
                                    ((DataTable)dictionary.getElement(dictionary.size()-1)).filepath = fileSave;

                              } catch (Exception ex) { System.err.println(" Exception inFile: "+ ex.getMessage());
                                     // write this exception in the status field of the GUI
                                    statusTextField.setText("Importing data through RMI - interrupted.");
                                    // also pop-up an alert about this
                                    popAlertDialog(" ERROR:", "   The path for the file to be saved on your machine doesn't exist", "   or the loaded file didn't download properly");
                                     }
                              finally {   // Always be sure to close the input stream!
                                try { if (inFile != null) inFile.close(); }
                                  catch (IOException ioe) {
                                    System.err.println("I/O Exception DBFileReader file input stream not closed: "+ ioe.getMessage());
                                  }
                              }

                                  statusTextField.setText("  Done with file download.");
// ********************** saving speciffic fields into default.gui

                              // Save directory path and filename settings (that user could have changed) into the default.gui (if it exists)
                        SaveGUI load_obj = (SaveGUI)getDefaultGUI();

                        load_obj.openFileTextField1.setText(openFileTextField1.getText());
                        load_obj.openFileTextField2.setText(openFileTextField2.getText());
                        load_obj.openFileTextField3.setText(openFileTextField3.getText());
                        load_obj.openFileTextField4.setText(openFileTextField4.getText());

                        saveDefaultGUI(load_obj);
// **************************

                                }
                                // This is used for imprting a db file from cmms.triumf.ca through the RMI
                                else if (command.equals("Cancel cmms Open")) {
                                  openCFileDialog.hide();
                                }
                                else if (command.equals("Define Function")) {
                                  defDialog.hide();
                                  String caseString = defTextField1.getText();
                                  functionsTable.setFunction(caseString, defTextField2.getText());
                                  statusTextField.setText("New Function has been added to the Functions Table: "+
                                                           caseString+" = "+defTextField2.getText());
                                }
                                else if (command.equals("Define Constant")) {
                                  defDialog.hide();
                                  // check for case sensitivity first
                                  String caseString = "";
                                  caseString = defTextField1.getText();
                                  if (Character.isDigit(defTextField2.getText().charAt(0))) { // if the value is a number
                                    dataTableVariables.setConstant(caseString,
                                                                   defTextField2.getText(), defTextField3.getText() );
                                    statusTextField.setText("New Constant has been added to the Constants Table: "+
                                                             caseString+" = "+
                                                             Double.parseDouble(defTextField2.getText())+"  "+ defTextField3.getText() );
                                  } else statusTextField.setText("Constant value has to be a number! Constant \""+
                                                                  caseString+"\" hasn't been added to the Table.");
                                }
                                else if (command.equals("Settings OK")) {
                                  defDialog.hide();
                                  if (Character.isDigit(defTextField1.getText().charAt(0)) &&
                                      Character.isDigit(defTextField2.getText().charAt(0))) { // if the values are a number
                                        nPix = Long.parseLong(defTextField1.getText());
                                        Nmax = Long.parseLong(defTextField2.getText());
                                    statusTextField.setText("The Automatic Symbol Size parameters have been updated.");
                                  } else statusTextField.setText("Error: the value you entered has to be an integer number");
                                }
                                else if (command.equals("Cancel Def")) {
                                  defDialog.hide();
                                }
                                else if (command.equals("Repack OK")) {
                                  if( !repackField.getText().trim().equals(new String("")) ) { // command REPACK WITH a packBIns number has been entered
                                    // since there is a new datatable being created, create a new reference date to feed into it's columns
                                    Date date = new Date();
                                    int packBins = Integer.parseInt( repackField.getText().trim() );
                                    int dataTableKey = getDataTableIndex();
                                    DataTable dt = (DataTable)dictionary.getElement(dataTableKey);
                                    DataTable returnDT = new DataTable(dt.filepath);
                                    returnDT.setTitle(dt.getTitle()+" repacked("+repackField.getText().trim()+")");
                                    int dtSize = dt.columnCount();
                                    for (int i =0; i<dtSize; i++) {
                                      //ConcreteDataColumn
                                      returnDT.addColumn( repack(packBins, (DataColumn)dt.getColumn(i), date ));
                                    }
                                    addDataTable(returnDT, false, "" );
                                    repackDialog.hide();
                                  } else { // just do nothing (for now)
                                  }
                                }
                                else if (command.equals("Repack Cancel")) {
                                  repackDialog.hide();
                                }
                                // This event will import a Data Table into the dataTableList by asking the user what kind of import to perform
                                else if (command.equals("Import DataTable")) {
                                  importDataTable();
                                }
                                // This event will remove the selected Data Table in the dataTableList
                                else if (command.equals("Remove Data Table")) 
                                {
                                    if(JOptionPane.showConfirmDialog(null, "Are you sure you want to remove the selected data table?",
                                        "Please confirm", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
                                        removeDataTable();
                                }
                                // This option displays the data contents of the selected columun (for a certain data table)
                                else if (command.equals("View Column Data")) {
                                  ViewColumnData tempcol = new ViewColumnData(GUI.this);
                                  tempcol.showTable();
                                }
                                else if (command.equals("Close Data Table Window")) {
                                  dataTableFrame.hide();
                                }
                                else if (command.equals("Close Frames Window")) {
                                  framesFrame.hide();
                                }
                                else if (command.equals("OK")) {
                                  infoDialog.hide();
                                }
                          // When printing, the user can choose plot size for a page format, this button is used for that purpose
                                else if (command.equals("Choose Plot Size Format")) {
                                  printPageFormatDialog.hide();
                                  // Assign the landscape or portrait format
                                  if (pageFormCheckP.getState()) pageForm.setOrientation(1);
                                  else if (pageFormCheckL.getState()) pageForm.setOrientation(0);
                                  // Assing the plot width and height settings in case user changed them
                                  // Note: They should be integer values
                                  // note: component value is counted from zero and on (i.e number 2 is the 3rd component)
                                  plotWidthPort = Long.valueOf(
                                      ((TextField) printPageFormatDialog.getComponent(2)).getText() ).intValue();
                                  plotHeightPort = Long.valueOf(
                                      ((TextField) printPageFormatDialog.getComponent(4)).getText() ).intValue();
                                  plotWidthLand = Long.valueOf(
                                      ((TextField) printPageFormatDialog.getComponent(8)).getText() ).intValue();
                                  plotHeightLand = Long.valueOf(
                                      ((TextField) printPageFormatDialog.getComponent(10)).getText() ).intValue();

//####################  This very same code is called in print() method, probably unnecesarily
    plotFrame.setResizable(true);
          // Change the size of the plot frame (on the screen) to the size of the chosen size printout + additional 75
          if (pageForm.getOrientation()==pageForm.PORTRAIT)
            plotFrame.setSize((int)plotWidthPort*28 + 150,(int)plotHeightPort*28 + 150 + 75);
          else
            plotFrame.setSize((int)plotHeightLand*28 + 150,(int)plotWidthLand*28 + 150 + 75);
    // Finally, set the plotFrame to be nonresizable.
    //*Warning*
    // This will cause that none of the code which applies to smaller plot-sizes will be used or seen on the screen
    plotFrame.setResizable(false);

                        // Now check for frame width and height to make sure the graph is not too big for the page
                        // Approximately, the ratio of the plot box would be around 4x5
                        // Note: I have estimated Max. width to be: 570 and Max. height to be 710 for PORTRAIT
                        //        and vice versa for LANDSCAPE and REVERSE LANDSCAPE page formats
                        //   I also estimated that the ratio between centimeters and pixels is 28. So the Widthts and heights
                        //     below will be calculated from the settings that were done in "plot size setup". There, default max.
                        //     values will be pre-set so that they match the maximums below.

                        int maxWidth = 570;  // 15(cm)*28 + 150(edges) = 570 (width should not be bigger than 570)
                        int maxHeight = 720;  // 20(cm)*28 + 150(edges) = 720
                        int niceHeight = 490;  // 12(cm)*28 + 150(edges) = 486  (520 is for 13 cm..)
                        int tempw = 570;//some random value
                        int temph = 520;//some random value
                        if (plotsPrint!=null) {
                          if (plotsPrint.size()!=0) {//just in case the vector is not empty
                          /* Note, this code is in case we want to print the plot as it appears on the screen
                          instead, calculate out the print setup values (below..)
                            tempw = ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth();
                            for(int i=0; i<plotsPrint.size(); i++)
                              temph += ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getHeight();
                          */
                          }
                          // If PageFormat is Portrait
                          if (pageForm.getOrientation()==PageFormat.PORTRAIT) { //Portrait = 1
                            tempw = plotWidthPort*28 + 150;
                            temph = plotHeightPort*28 + 150;
                            // just fit the one graph in the upper area of the Frame
//System.out.println("tempw:"+tempw+"temph:"+temph+"vector size:"+plotsPrint.size());
                            if (plotsPrint.size()==1) {
                              // resize it only if the plot is bigger than the page
                              if (tempw>maxWidth) {
                                ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(maxWidth, niceHeight);
                                }
                              // resize to the value set by the user
                              // note: this code would be unnecessary in case where the print would be obtained from the
                              //       plot size off the screen (the disabled code above)
                              else  ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(tempw, temph);
                            // if there are more than one graphs, then fit them in the whole page
                            } else
                                for(int i=0; i<plotsPrint.size(); i++)
                                  ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).setSize(maxWidth, maxHeight/plotsPrint.size());
                          } else { // it is landscape mode
                            tempw = plotWidthLand*28 + 150;
                            temph = plotHeightLand*28 + 150;
                            // just fit the one graph into the whole area of the Frame
                            if (plotsPrint.size()==1) {
                              // resize it only if the plot is bigger than the page
                              if (temph>=maxHeight)
                                ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(maxHeight, maxWidth);
                              // resize to the value set by the user
                              // note: this code would be unnecessary in case where the print would be obtained from the
                              //       plot size off the screen (the disabled code above)
                              else ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(temph, tempw);

                            // if there are more than one graphs, then fit them in the whole page
                            } else
                                for(int i=0; i<plotsPrint.size(); i++)
                                  ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).setSize(maxHeight, maxWidth/plotsPrint.size());
                          }
                        }

                        // After all this, resize the Frame to fit the new Graph sizes
                  //this won't work, create a new Applet class!!
                        //  plotFrame.setSize(maxHeight, maxWidth);

          if(plotsPrint.size()!=0) {// if there are plots displayed in the GUI
            // Since the plot will be done one above the other, the width will stay the same
            // The height depends on the number of plots.
            temph = 0;
            for(int i=0; i<plotsPrint.size(); i++) {
              temph += ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight();
            }
//System.out.println("height: "+temph+" width: "+((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth()+"\n");
            // Set the 'range' of Graphics
            if (plotsPrint.size()!=0) // just in case the Vector is empty
             ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getGraphics().
                      setClip(0,0,((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth(),temph);
            for(int i=0; i<plotsPrint.size(); i++) {
              ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).paint(((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getGraphics());
              // translate the graphing down by the height of each plot
              ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getGraphics().
                      translate(0,((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight());//0,plotFrame.getHeight()/plotsPrint.size());
            }
          }
//####################
                                  display();

                                }
        
            else if(command.equals("View Column Statistics"))
            {
                ShowStatistics stats = new ShowStatistics(GUI.this);
                stats.setVisible(true);
            }
        
            else if(command.equals("Fourier Transform"))
            {
                FFT_GUI fourier = new FFT_GUI(thisGUI);
                fourier.setVisible(true);
            }
                                else if (command.equals("Yes")) { // this is for warningDialogYesButton
                                  warningDialogAnswer = true;
                                  warningDialog.hide();
                                }
                                else if (command.equals("No")) { // this is for warningDialogNoButton
                                  warningDialogAnswer = false;
                                  warningDialog.hide();
                                }

			}
    };
    layoutAddButton.addActionListener(buttonListener);
    layoutRemoveButton.addActionListener(buttonListener);
    addXButton.addActionListener(buttonListener);
    removeXButton.addActionListener(buttonListener);
    addYButton.addActionListener(buttonListener);
    removeYButton.addActionListener(buttonListener);
    conditionalPlotButton.addActionListener(buttonListener);
    bestFitButton.addActionListener(buttonListener);
    splineButton.addActionListener(buttonListener);
    displayPlotButton.addActionListener(buttonListener);
    repackDataButton.addActionListener(buttonListener);
    openFileButton.addActionListener(buttonListener);
    openFileButton2.addActionListener(buttonListener);
    openCFileButton.addActionListener(buttonListener);
    openCFileButton2.addActionListener(buttonListener);
    defConButton.addActionListener(buttonListener);
    defFunButton.addActionListener(buttonListener);
    repackOKButton.addActionListener(buttonListener);
    repackCancelButton.addActionListener(buttonListener);
    symbSizeButton.addActionListener(buttonListener);
    defButton2.addActionListener(buttonListener);
//    addDataTableButton.addActionListener(buttonListener);
    removeDataTableButton.addActionListener(buttonListener);
    showDataTableInfoButton.addActionListener(buttonListener);
    refreshDataTableButton.addActionListener(buttonListener);
    columnButton.addActionListener(buttonListener);
    statsButton.addActionListener(buttonListener);
    fourierButton.addActionListener(buttonListener);
    dataTableButton.addActionListener(buttonListener);
    framesButton.addActionListener(buttonListener);
    OKButton.addActionListener(buttonListener);
    pageFormatButton.addActionListener(buttonListener);
    warningDialogYesButton.addActionListener(buttonListener);
    warningDialogNoButton.addActionListener(buttonListener);


    // Arrange to detect window close events
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				disposeAll();
			}
		});


		// pack all the components in the frame
		this.pack();

		// Open the Frame where graphs will be plotted
		//		plotFrame.show();

    // Just before (and if) a default.gui state is loaded, create a resetGUI Object to have it around in case the user decides to reset the whole GUI
    loadGUI("res", true, plotsOptions); // Do this in case for some reason, the browser loads some old setting (frin cashe, or something like that)
    resetGUI = createSaveGUI();  // then, save that state
    // In case there is a previously saved default state in the directory GUI was started from, load it in
    loadGUI("def", true, plotsOptions);
	}

        private void showDataTableInfo()
        {
                String output = "";

                if(dataTableList.getSelectedIndex()>-1)
                {
                    DataTable table = (DataTable)(dictionary.getElement(getDataTableIndex()));
                    if(table!=null)
                    {
                        output = "Displaying information for data table: ";
                        output += table.getTitle();
                        output += "\n\n";
                        
                        if((table.getComment()!=null)&&(!table.getComment().equals(""))&&(!table.getComment().equalsIgnoreCase("null")))
                        {
                            output += "Comment:\n";
                            output += table.getComment();
                            output += "\n\n";
                        }
                        
                        if((table.getAbstract()!=null)&&(!table.getAbstract().equals(""))&&(!table.getAbstract().equalsIgnoreCase("null")))
                        {
                            output += "Abstract:\n";
                            output += table.getAbstract();
                            output += "\n\n";
                        }
                        
                        if((table.getColumns()!=null)&&(table.getColumns().size()>0))
                        {
                            output += "This table contains the following column(s):\n";
                            Vector columns = table.getColumns();
                            DataColumn currentColumn;
                            for(int i=0; i<columns.size(); i++)
                            {
                                currentColumn = (DataColumn)(columns.elementAt(i));
                                output += currentColumn.getName() + " (" + currentColumn.getLabel() + ")";
                                
                                if(currentColumn instanceof MultiColumn)
                                    output += " - data values with error bars\n";
                                else if(currentColumn instanceof ConcreteDataColumn)
                                    output += " - data values without error bars\n";
                                else if(currentColumn instanceof LinearColumn)
                                    output += " - constructed column of linearly increasing values\n";
                                else if(currentColumn instanceof PoissonDataColumn)
                                    output += " - data values with constructed Poisson error bars\n";
                            }
                        }    
                    }
                }
                else 
                {
                    output = "No DataTables have been loaded, so there is no information to be displayed.\n";
                    output += "Please load a DataTable, and try again.";
                }

                JOptionPane.showMessageDialog(null, output);
        }


        /**
         * This method is called when columnButton is pressed.
         * It will display the data contents of the selected column
         * Functionality now replaced by ViewColumnData.
         */
         public void createAndDisplayNewColumn() {
           // there is always a selected DataTable so no need to worry about that 'constraint'
           int dataTableKey = getDataTableIndex();
           DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
           // check if the selected DataTable has columns loaded
           if (selectedDataTable.columnCount()!=0) {
             // get the index of the selected column
             int columnIndex = columnList.getSelectedIndex();
             // create the components that will be used to display the data
             JTextArea jta = new JTextArea();
             jta.setEditable(false); // this is where the past commands are displayed
             JScrollPane jsp= new JScrollPane(jta); // use the scroll pane if the textarea can't fit the Component
             // Put the Name of the column at the top
             jta.append("Column: "+selectedDataTable.getColumn(columnIndex).getName()+"\n");
             // Add all values with the error bars if there are any
             for(int i=0; i<selectedDataTable.getColumn(columnIndex).getSize(); i++)
              {
                String value = ((DataColumn)selectedDataTable.getColumn(columnIndex)).getValue(i).toString();
                String plusErr = "", minusErr = "";
                //If errors are zero, don't display it, just leave it blank in between the two commas
                if ( Double.parseDouble(selectedDataTable.getColumn(columnIndex).getErrorPlus(i).toString())!=0.0 )
                  plusErr = selectedDataTable.getColumn(columnIndex).getErrorPlus(i).toString();
                if ( Double.parseDouble(selectedDataTable.getColumn(columnIndex).getErrorMinus(i).toString())!=0.0 )
                  minusErr = selectedDataTable.getColumn(columnIndex).getErrorMinus(i).toString();
                jta.append(value+ ", " +plusErr+ ", " +minusErr+"\n");
              }
             final JFrame dataFrame = new JFrame("Column: "+selectedDataTable.getColumn(columnIndex).getName());
             dataFrame.getContentPane().add(jsp, "Center");
             // Create a button that will close and delete this frame
             Button colButt = new Button("Close");
             dataFrame.getContentPane().add(colButt,"South");
             dataFrame.pack();
             dataFrame.isEnabled();
             //dataFrame.setSize(50,400);
             ActionListenerS buttonlistenerColumn = new ActionListenerS() {
              public void actionPerformed(ActionEvent e) {
                  if (e.getActionCommand().equals("Close"))
                    dataFrame.dispose();
              }
             };
             colButt.addActionListener(buttonlistenerColumn);
             dataFrame.setSize(150,400);
             dataFrame.setLocation(200,200);
             dataFrame.show();
           }
           else {
             statusTextField.setText(" # The DataTable contains no columns!");
           }
         }

 	/**
         * en(diss)ables the display options componets and title
         * if there is no x-axis selected, its's options should be dissabled
         */
	public void setDisplayOptionsEnabled(boolean b) {
		invertedCheckbox.setEnabled(b);
		visibleCheckbox.setEnabled(b);
	}


         /**
         * sets display options and the Title with given positon of x-axis
         * whenever a x-axis is selected, its options are refreshed with this method
         */
         public void refreshDisplayOptions(int selectedxAxisIndex) {
           if ( !((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).isEmpty() || selectedxAxisIndex==-1) {
             if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) // this is a bug where the item is not selected
               selectedxAxisIndex = xAxisList.getItemCount()-1;
             PlotRecord tempRecord = (PlotRecord) ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).elementAt(selectedxAxisIndex);
             invertedCheckbox.setState(tempRecord.inverted);
             visibleCheckbox.setState(tempRecord.visible);
           }
	}

      /**
         * Note: the arguments that are passed are for calculating the symbol size of a new graph
         *   Nmax variable is used to calculate the size of the symbols in the graph.
         *   The calculation is size=0.5*(n/m) where n is the width of the plot in pixels and m = max(N,Nmax) where N is
         *   the number of points for the graph that is being plotted within this plot. 0.5 is because the symbol size
         *   will actually be twice the size of this value.
         */
         public int graphSymbolSize(long nPix, int Npoints) {
          // Calculate the symbol size which is to be passed to the setNumberOfGraphs() method in plotOptionsFrame
          // Note: this value will actually be used only if there IS a new column added, otherwise, these are all 'dummy' numbers
          long m = Math.max(Nmax, Npoints); //get the number of points first
          int symbSize = Math.round(Math.round(0.5*(nPix/m)));
          if (symbSize==0) symbSize=1; // in case it gets rounded to zero, take the minimal value:1.

          return symbSize;
        }

        /**
         * Removes a column in xAxisList at the given position
         * This function is defined seperately because it is called from removeX, removeDataTable() method and removeFrame 'buttons'
         * note: actually it is not called from removeFrame button since it might not be necessary (since it is an option that is never used)
         */
        public void removexAxisColumn(int selectedxAxisIndex) {
          xAxisList.remove(selectedxAxisIndex);                  // remove from list
          // record the changes
          ((DataTable)xAxisDataTable.elementAt(layoutChoice.getSelectedIndex())).removeColumnAt(selectedxAxisIndex);
          ((Vector)yAxisDataTable.elementAt(layoutChoice.getSelectedIndex())).removeElementAt(selectedxAxisIndex);
          ((Vector)titlexAxis.elementAt(layoutChoice.getSelectedIndex())).removeElementAt(selectedxAxisIndex);
          ((Vector)namexAxis.elementAt(layoutChoice.getSelectedIndex())).removeElementAt(selectedxAxisIndex);
          ((Vector)titleyAxis.elementAt(layoutChoice.getSelectedIndex())).removeElementAt(selectedxAxisIndex);

          ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).removeElementAt(selectedxAxisIndex);
          // update the fields in PlotOptionsFrame

          // remove X fields in the PlotOptionsFrame
          ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).removeXOptions(selectedxAxisIndex);
        }

        /**
         * clears xAxisList and then fills it with x-axis corresponding to the selected Frame (layoutChoice)
         * whenever a new Frame (layout Choice) is selected its corresponding xAxisList has to be reloaded
         */
 	public void reloadxAxisList() {
                int tempXFirst = 0; // PlotsRecord index of the first x-axis element in the list
                if (xAxisList.getItemCount() > 0) {
                  xAxisList.removeAll();         // empty xAxisList
                }
                if (layoutChoice.getItemCount() > 0) {
                  int selectedLayoutIndex = layoutChoice.getSelectedIndex();      // get selected Frame
                  // fill xAxis List with elements from the selected Frame
                  for(int i = 0; i < ((DataTable)xAxisDataTable.elementAt(selectedLayoutIndex)).columnCount(); i++) {
                          // the name of the column (to add to the Axis List)
                          String name = ((DataColumn) ((DataTable)xAxisDataTable.elementAt(selectedLayoutIndex)).getColumn(i)).getName();
                          // get the dataTableName from the xAxisDataTable
                          // for the purposes of recovering the filename (listItem) from the DataTableList AND to know whether it is sorted or not
                          String dataTableName = (String)((Vector)namexAxis.elementAt(selectedLayoutIndex)).elementAt(i);
                          // Get the DataTable Name from the DataTableList, traverse through it and compare the title to the content to get filename
                          //     note: the string in DataTableList is :  ("File: "+fileName+"; Title: "+title);
                          String filename = "";
                          for(int j=0; j<dictionary.size(); j++) {
                            // check if the full Name (filename; Title; (SORTED.. )) matches this xAxis to determmine what to write into the xAxisList
//System.out.println("reloadxAxisList: dataTableLIst item: "+dataTableList.getItem(j));
//System.out.println("dataTableName: "+ dataTableName);
//System.out.println("index: "+dataTableList.getItem(j).indexOf( dataTableName));
                            if ( dataTableList.getItem(j).indexOf( dataTableName)!=-1) {
                              // find the filename in the datatable from the dictionary
                              filename = new File(((DataTable)dictionary.getElement(j)).filepath).getName();
                              // Check in the DataTable list to see if this Column is from a 'sorted' datatable (sorted datatables will have: 'SORTED ON <column name>' at the end
                              if (dataTableList.getItem(j).indexOf(" SORTED")>-1)   // note: I could have also checked this in the dataTableName
                              filename = filename+dataTableList.getItem(j).substring(dataTableList.getItem(j).indexOf(" SORTED")) ;  // append it to the name
                              break;
                            }
                          }
                          xAxisList.add(name+"  ("+filename+")");
                   }
                }
                if (xAxisList.getItemCount()!=0) { // If this Frame contains x-axis elements
                  xAxisList.select(0);
                  refreshDisplayOptions(0);
                  setDisplayOptionsEnabled(true);
                } else setDisplayOptionsEnabled(false);     // options not usable any more
	}

 	/**
         * clears yAxisList and then fills it with y-axis corresponding to the selected x-axis
         * whenever a new xAxis is selected its corresponding yAxisList has to be reloaded
         */
      public void reloadyAxisList() {
        if (yAxisList.getItemCount() > 0) yAxisList.removeAll();         // empty yAxisList
        if (xAxisList.getItemCount() > 0) {
          int selectedxAxisIndex = xAxisList.getSelectedIndex();              // get selected items
          if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) // this is a bug where the item is not selected
            selectedxAxisIndex = xAxisList.getItemCount()-1;
          DataTable selectedyAxisDataTable = (DataTable) ((Vector) yAxisDataTable.
          elementAt(layoutChoice.getSelectedIndex())).elementAt(selectedxAxisIndex);
          for(int i = 0; i < selectedyAxisDataTable.columnCount(); i++) {          // fill yAxis List
            // Set the color to the same color as the plot color
            PlotOptionsFrame plotsOptionsTemp = (PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex());
            //yAxisList.getGraphics().setColor((Color) ((Vector)plotsOptionsTemp.graphColor.elementAt(plotsOptionsTemp.groupNumberChoice.getSelectedIndex()))
                                                              //          .get(plotsOptionsTemp.graphNumberChoice.getSelectedIndex()) );
            // Add the Y Axis
            yAxisList.add(((DataColumn) selectedyAxisDataTable.getColumn(i)).getName());
          }
          //yAxisList.getGraphics().setColor(Color.black); // reset it back to black
          yAxisList.select(0);
        }
      }

 	/**
         * clears columnList and then fills it with columns corresponding to the selected dataTable
         * whenever a new dataTable is selected its corresponding columnList has to be reloaded
         */
          public void reloadColumnList() {
          if (columnList.getItemCount() > 0) columnList.removeAll();
          int dataTableKey = getDataTableIndex();
          if (dataTableList.getItemCount() > 0 && dataTableKey!=-1) {
            DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
            for(int i=0; i<selectedDataTable.columnCount(); i++) {
              DataColumn nextColumn = selectedDataTable.getColumn(i);
              columnList.add(nextColumn.getName()+ " = " + nextColumn.getLabel());
            }
            columnList.select(0);
          }
        }

        /**
         *
         */
        public void addX(int selectedColumnIndex, int selectedLayoutIndex) {
          if (xAxisList.getItemCount() == 0) setDisplayOptionsEnabled(true); // display options now usable
          // get the column from the DataTable/Dictionary and add it to the x-axis List
          int dataTableKey = getDataTableIndex();
          DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
          DataColumn selectedColumn = selectedDataTable.getColumn(selectedColumnIndex);
          String name = selectedColumn.getName();
          String tabl = Long.toString(dataTableList.getSelectedIndex()+1);
          // Get the DataTable Name from the Data Table filepath
          String listItem = selectedDataTable.filepath.substring(selectedDataTable.filepath.lastIndexOf("/")+1);
//System.out.println("addx method, filepath: "+ listItem);
          // Check in the DataTable list to see if this Column is from a 'sorted' datatable (sorted datatables will have: 'SORTED ON <column name>' at the end
          if (dataTableList.getSelectedItem().indexOf(" SORTED")>-1)
            listItem = listItem+dataTableList.getSelectedItem().substring(dataTableList.getSelectedItem().indexOf(" SORTED")) ;  // append it to the name
          xAxisList.add(name+"  ("+listItem+")");
//System.out.println("addx method - name+  (+listItem+): "+ name+"  ("+listItem+")");
          xAxisList.select(xAxisList.getItemCount()-1);
          yAxisList.removeAll();
          // record the changes
          // selected Column doesn't have to be cloned !!!!!!!!!!
          // for each Layout add new columns and new titles into the vectors
          ((DataTable) xAxisDataTable.elementAt(selectedLayoutIndex)).addColumn(selectedColumn);
          ((Vector) titlexAxis.elementAt(selectedLayoutIndex)).addElement(selectedDataTable.getTitle());
          // add the dataTableList Item but omit the number (#:) athe the beginning because that instance can change
          ((Vector) namexAxis.elementAt(selectedLayoutIndex)).addElement(dataTableList.getSelectedItem().substring( dataTableList.getSelectedItem().indexOf(":")+1 ));

          ((Vector) yAxisDataTable.elementAt(selectedLayoutIndex)).addElement(new DataTable(new String("")));
          ((Vector) titleyAxis.elementAt(selectedLayoutIndex)).addElement(new Vector());

          ((Vector) plotsRecord.elementAt(selectedLayoutIndex)).addElement(new PlotRecord(selectedLayoutIndex,selectedDataTable.getTitle()));
//System.out.println("xAxisList.getSelectedIndex() "+xAxisList.getSelectedIndex());
          // create new Display Options and display them
          if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) // this is a bug where the item is not selected
            refreshDisplayOptions(xAxisList.getItemCount()-1);
          else
            refreshDisplayOptions(xAxisList.getSelectedIndex());

          // Add X fields in the PlotOptionsFrame
          ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).AddXOptions();
        }

        /**
         *
         */
        public void addY(int selectedColumnIndex, int selectedxAxisIndex, int selectedLayoutIndex) {
          // get the selected DataTable from the dictionary

          int dataTableKey = getDataTableIndex();
          DataTable selectedDataTable = (DataTable) dictionary.getElement(dataTableKey);
          // if y-axis(that is added) comes from the same DataTable where x-axis(where it's added) came from
          String fileName = selectedDataTable.filepath.substring(selectedDataTable.filepath.lastIndexOf("/")+1);
//System.out.println("filePath: "+selectedDataTable.filepath+"filename:"+fileName);
//  System.out.println("       xAxisList.getItem(selectedxAxisIndex) "+xAxisList.getItem(selectedxAxisIndex));
//  System.out.println("       fileName "+fileName);
          if (xAxisList.getItem(selectedxAxisIndex).indexOf(fileName)!=-1) { // check if the filename is contained in the ListColumn
          // get the column from the DataTable/Dictionary and add it to the y-axis List
           DataColumn selectedColumn = selectedDataTable.getColumn(selectedColumnIndex);
           yAxisList.add(selectedColumn.getName());
           yAxisList.select(yAxisList.getItemCount()-1);
           // record the change
           ((DataTable) ((Vector) yAxisDataTable.elementAt(selectedLayoutIndex)).elementAt(selectedxAxisIndex)).addColumn(selectedColumn);
           ((Vector) ((Vector) titleyAxis.elementAt(selectedLayoutIndex)).elementAt(selectedxAxisIndex)).addElement(selectedDataTable.getTitle());

           // Add Y fields in the PlotOptionsFrame
           ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).
                                                           AddYOptions(selectedxAxisIndex, graphSymbolSize(nPix, selectedColumn.getSize()));

           }
           else {
            statusTextField.setText(" # x-axis and y-axis must be from the same DataTable!");
           }
        }

        /**
         * This method creates new columns that are loaded into the X and Y axis based on the given String s that
         * carries certain conditions. The string is extracted from the command:
         *  plot A B if CONDITION
         *  where the CONDITION is extracted from the command (i.e. the String s doesn't contain 'plot A B if')
         *  Once the columns have been created, the method loads them by calling the addXYCond() method.
         *  It returns true or false depending on whether the column was loaded or not (with the addXYCond() method)
         */
        public boolean findCondColumn(String s, int columnX, int columnY) {
                    // Since the 'st' tokenizer 'skips' only blank space, it is necessary to create a new
                    // StringTokenizer which will give out all the tokens, including delimiters <,>,=,&,|, ,
                    // Note: in order to allow the user to type the whole command with blank spaces in between
                    //      the words as much as one wants, the code below might be a little 'robust'
                    // Note: the user is required to pupt & or | if there are two conditions
                    // Create it with the initial 's' string but truncate everything until the end of 'if'
                    StringTokenizer pst = new StringTokenizer(
                            s.substring(s.toUpperCase().indexOf("IF")+2), "<>=&|",true);

                    // these are the 'column filling' determining parameters
                    boolean firstBoolExpr = false;
                    boolean secondBoolExpr = false;

                    // These are the two columns the DATA will be taken from:
                    DataColumn realColumnX =
                        (DataColumn)((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(columnX);
                    DataColumn realColumnY =
                        (DataColumn)((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(columnY);
                    // Now create new X and Y axis COLUMNS (which will be filled using the conditions generated above)
                    ConcreteDataColumn condColumnX = new ConcreteDataColumn(realColumnX.getName(),realColumnX.getLabel(),
                                                                                                                  realColumnX.getUnits(), realColumnX.getDate());
                    ConcreteDataColumn condColumnXminErr = new ConcreteDataColumn();
                    ConcreteDataColumn condColumnXplusErr = new ConcreteDataColumn();
                    ConcreteDataColumn condColumnY = new ConcreteDataColumn(realColumnY.getName(),realColumnY.getLabel(),
                                                                                                                  realColumnY.getUnits(), realColumnY.getDate());
                    ConcreteDataColumn condColumnYminErr = new ConcreteDataColumn();
                    ConcreteDataColumn condColumnYplusErr = new ConcreteDataColumn();

                    // these are the parmeters that will be loaded and then using them, the conditions will be checked
                    String firstExpr1 = ""; String firstExpr2 = ""; String firstExprCond = "";
                    String logicSymbol = "";
                    String secondExpr1 = ""; String secondExpr2 = ""; String secondExprCond = "";
                    // 'Load' the values
                    if (pst.hasMoreTokens()) firstExpr1 = pst.nextToken(); // get the first expression 1
                    if (pst.hasMoreTokens()) {
                      String tmp= "";
                      tmp = pst.nextToken();
                      // this means the first expression doesn't contain <,>,=, and there is another condition too
                      if (tmp.equals("&") || tmp.equals("|")) {
                        logicSymbol=tmp;
                        if(pst.hasMoreTokens())
                          secondExpr1 = pst.nextToken(); // get the second expresion 1, skip the blank lines
                        // this means that the second expression contains <,>,=, with another expression right after that
                        if (pst.hasMoreTokens()) {
                          secondExprCond = pst.nextToken(); // get the conditional symbol for second expression
                          secondExpr2 = pst.nextToken(); // get the second expression 2
                        }
                      // this means the first expression contains <,>,=, with another expression right after that
                      } else if (tmp.equals(">") || tmp.equals("<") || tmp.equals("=")){
                        firstExprCond = tmp; // get the conditional symbol for first expression
                        firstExpr2 = pst.nextToken(); // get the first expression 2

                        // There could be another conditional expression after the first one
                        if (pst.hasMoreTokens()) {
                          String temp = "";
                          temp = pst.nextToken();
                          // this means the first expression contained <,>,=, and there is another condition too
                          if (temp.equals("&") || temp.equals("|")) {
                            logicSymbol=temp;
                            if (pst.hasMoreTokens()) secondExpr1 = pst.nextToken(); // get the second expresion 1
                            // this means that the second expression contains <,>,=, with another expression right after that
                            if (pst.hasMoreTokens()) {
                              secondExprCond = pst.nextToken(); // get the conditional symbol for second expression
                              secondExpr2 = pst.nextToken(); // get the second expression 2
                            }
                          }
                          // This means that the user didn't type '&' or '|' in and the second condition exists.
                          // In this case report an ERROR
                          else {
                            console.jta.append("You didn't type '&' or '|' in the expression, anything could happen..\n");
                          }
                        }

                      }
                      // This means that the user didn't type '&' or '|' in and the second condition exists.
                      // In this case report an ERROR
                      else {
                        console.jta.append("You didn't type '&' or '|' in the expression, anything could happen..\n");
                      }
                    }
                    // Fill the temp error vectors in case there are references to them when calling evaluateExpr()
                    if (dataTableList.getSelectedIndex()!=-1 &&
                        ( (firstExpr1.toUpperCase().indexOf("ERR"))!=-1 || (firstExpr2.toUpperCase().indexOf("ERR"))!=-1 ||
                        (secondExpr1.toUpperCase().indexOf("ERR"))!=-1 || (secondExpr2.toUpperCase().indexOf("ERR"))!=-1 ) )
                      fillTempErrors( (DataTable) dictionary.getElement(dataTableList.getSelectedIndex()) );
                    // LOAD these columns using the given conditions. (THere are 6 combinations)
                    for (int i=0; i<realColumnX.getSize(); i++) {
                      boolean test = false;
                      // Only the first expression, without a conditional symbo provided
                      if (!firstExpr1.equals("") && firstExpr2.equals("") &&
                          secondExpr1.equals("") && secondExpr2.equals("")) {
                        if (evaluateExpr(firstExpr1,i, -1, "")>0) test = true;
                      }
                      // Only the first expression, with a conditional symbol provided
                      else if (!firstExpr1.equals("") && !firstExpr2.equals("") &&
                               secondExpr1.equals("") && secondExpr2.equals("")) {
                        if (checkCondition(firstExpr1, firstExprCond, firstExpr2, i)) test=true;
                      }
                      // The first expression, without a conditional symbol provided
                      // and second too, without conditional symb.
                      else if (!firstExpr1.equals("") && firstExpr2.equals("") &&
                              !secondExpr1.equals("") && secondExpr2.equals("")) {
                        if (logicSymbol.equals("&")) {
                          if (evaluateExpr(firstExpr1,i, -1, "")>0 &&
                              evaluateExpr(secondExpr1,i, -1, "")>0) test = true;
                          }
                        if (logicSymbol.equals("|"))
                          if (evaluateExpr(firstExpr1,i, -1, "")>0 ||
                              evaluateExpr(secondExpr1,i, -1, "")>0) test = true;
                      }
                      // The first expression, with a conditional symbol provided
                      // and second too, without conditional symb.
                      else if (!firstExpr1.equals("") && !firstExpr2.equals("") &&
                              !secondExpr1.equals("") && secondExpr2.equals("")) {
                        if (logicSymbol.equals("&"))
                          if (checkCondition(firstExpr1, firstExprCond, firstExpr2, i)
                                                && evaluateExpr(secondExpr1,i, -1, "")>0) test = true;
                        if (logicSymbol.equals("|"))
                          if (checkCondition(firstExpr1, firstExprCond, firstExpr2, i)
                                                || evaluateExpr(secondExpr1,i, -1, "")>0) test = true;
                      }
                      // The first expression, without a conditional symbol provided
                      // and second too, with a conditional symb.
                      else if (!firstExpr1.equals("") && firstExpr2.equals("") &&
                              !secondExpr1.equals("") && !secondExpr2.equals("")) {
                        if (logicSymbol.equals("&"))
                          if (evaluateExpr(firstExpr1,i, -1, "")>0
                                && checkCondition(secondExpr1, secondExprCond, secondExpr2, i)) test = true;
                        if (logicSymbol.equals("|"))
                          if (evaluateExpr(firstExpr1,i, -1, "")>0
                                || checkCondition(secondExpr1, secondExprCond, secondExpr2, i)) test = true;
                      }
                      // The first expression, with a conditional symbol provided
                      // and second too, with a conditional symb.
                      else if (!firstExpr1.equals("") && !firstExpr2.equals("") &&
                              !secondExpr1.equals("") && !secondExpr2.equals("")) {
                        if (logicSymbol.equals("&"))
                          if (checkCondition(firstExpr1, firstExprCond, firstExpr2, i)
                                && checkCondition(secondExpr1, secondExprCond, secondExpr2, i)) test = true;
                        if (logicSymbol.equals("|"))
                          if (checkCondition(firstExpr1, firstExprCond, firstExpr2, i)
                                || checkCondition(secondExpr1, secondExprCond, secondExpr2, i)) test = true;
                      }
                      // ADD to the columns
                      if (test) {
                        condColumnX.addElement(realColumnX.getValue(i));
                        condColumnXminErr.addElement(realColumnX.getErrorMinus(i));
                        condColumnXplusErr.addElement(realColumnX.getErrorPlus(i));
                        condColumnY.addElement(realColumnY.getValue(i));
                        condColumnYminErr.addElement(realColumnY.getErrorMinus(i));
                        condColumnYplusErr.addElement(realColumnY.getErrorPlus(i));
                      }
                    }
                    // The COLUMNS have (finally!!) been created, now create the MultiColumns and LOAD them into the GUI
                    MultiColumn condXMultiColumn = new MultiColumn((ConcreteDataColumn) condColumnX,
                                                                                               (ConcreteDataColumn) condColumnXplusErr,
                                                                                               (ConcreteDataColumn) condColumnXminErr);
                    MultiColumn condYMultiColumn = new MultiColumn((ConcreteDataColumn) condColumnY,
                                                                                               (ConcreteDataColumn) condColumnYplusErr,
                                                                                               (ConcreteDataColumn) condColumnYminErr);
                    return addXYCond(condXMultiColumn, condYMultiColumn);

        }

        /**
         * This method is used by the CONDITIONAL PLOT. Once the columns have been created, they are loaded into the GUI
         *  NOTE: the code below 'resembles' the code which is used in addX() and addY() methods.
         *  It returns true or false depending on whether a column was loaded or not
         */
        public boolean addXYCond(MultiColumn condColumnX, MultiColumn condColumnY) {

          boolean result = false;
          if (!condColumnX.isEmpty()) { //in case the conditions were such so that no data was loaded, don't load
            if (xAxisList.getItemCount() == 0) setDisplayOptionsEnabled(true); // display options now usable
            // add the x column to the x-axis List
            int dataTableKey = getDataTableIndex();
            String name = condColumnX.getName();
            // Get the DataTable Name from the Data Table List
            // the list item has this structure: "File: filename; Title: .."  We want filename.
            String listItem = "";
            listItem = dataTableList.getSelectedItem().substring(dataTableList.getSelectedItem().indexOf("File:")+6 , // this will go to the end of "File:"
                                                                                       dataTableList.getSelectedItem().indexOf(";"));
//System.out.println(" listItem:"+listItem);
            xAxisList.add(name+"  ("+listItem+")");
            xAxisList.select(xAxisList.getItemCount()-1);
            yAxisList.removeAll();
            // record the changes
            // selected Column doesn't have to be cloned !!!!!!!!!!
            // for each Layout add new columns and new titles into the vectors
            ( (DataTable) xAxisDataTable
                  .elementAt( layoutChoice.getSelectedIndex() ) ).addColumn(condColumnX);
            ((Vector) titlexAxis.elementAt(layoutChoice.getSelectedIndex()))
                  .addElement( ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getTitle() );
            ((Vector) namexAxis.elementAt(layoutChoice.getSelectedIndex()))
                  .addElement( dataTableList.getSelectedItem() );
            ((Vector) yAxisDataTable.elementAt( layoutChoice.getSelectedIndex() ))
                  .addElement(new DataTable(new String("")));
            ((Vector) titleyAxis.elementAt( layoutChoice.getSelectedIndex() ))
                  .addElement(new Vector());
            ((Vector) plotsRecord.elementAt( layoutChoice.getSelectedIndex() ))
                .addElement(new PlotRecord( layoutChoice.getSelectedIndex(), ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getTitle()));
            // create new Display Options and display them
            if (xAxisList.getSelectedIndex()==-1 && xAxisList.getItemCount()>0) // this is a bug where the item is not selected
              refreshDisplayOptions(xAxisList.getItemCount()-1);
            else
              refreshDisplayOptions(xAxisList.getSelectedIndex());

            // Add X fields in the PlotOptionsFrame
            ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).AddXOptions();

            // add the y column to the y-axis List
            yAxisList.add(condColumnY.getName());
            yAxisList.select(yAxisList.getItemCount()-1);
            // record the change
            ((DataTable) ((Vector) yAxisDataTable.elementAt( layoutChoice.getSelectedIndex() ))
                .elementAt(xAxisList.getItemCount()-1)).addColumn(condColumnY);
            ((Vector) ((Vector) titleyAxis.elementAt( layoutChoice.getSelectedIndex() ))
                .elementAt(xAxisList.getItemCount()-1))
                  .addElement(((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getTitle());

           // Add Y fields in the PlotOptionsFrame
           ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).
                                                           AddYOptions(xAxisList.getItemCount()-1, graphSymbolSize(nPix, condColumnX.getSize()));

            result = true;
          }
          return result;
        }

        /** This method evaluates the expression using Ari's function class
         *  it finds all the parameters in the expression and exchanges them with
         *  their values from the Constants (dataTableVariables) table.
         *  Note: Ari's function works with a parameter x, the final expression in
         *  this method will not contain any parameters
         */
/*        private double evaluateExpr(String expr) {
//System.out.println("expr sent to evaluateExpr: "+expr);
          String param = ""; // this is the parameter we are loking for
          StringTokenizer tokenizer; // parse the line
          String newExpr = expr; // this will be the final expression with no parameters in it
          // note: the right parenthesis '(' is left out from the delimiters for checking function calls
          // Note: the delimiters are set to true, so that it would be possible to get '(' and check for function calls
          tokenizer = new StringTokenizer(expr, "()+-/*^ ", true);
          // In order to avoid the bug of having a number that contains exponential letter E within it put into
          // the string and then having to search for a constatn or column named E, use this number (strPointer) so that I
          // wouldn't have to traverse through the string from the start, therefore I wouldn't exchange the wrong letter
          int strPointer = -1;
          while (tokenizer.hasMoreTokens()) {
            String tempString = tokenizer.nextToken();
            char tempChar = tempString.charAt(0);
            if (Character.isLetter(tempChar)) { // if it is an expression (not a number or an arithmetic sign)
              // if it exists in the constants table, then we have found the parameter
              tempString =tempString.trim();
              String findValue = "";
              int indx = -1;
              // First check for function calls, in case someone named a variable same as some of the functions
              // This is done by looking up the next token, if it is '(' then it is a function call for sure
              // Note: calling nextToken will not affect the overall parsing of the expression!
              // make sure it is not the last element since there is a nextToken call, and check if it is not a function
              // Note: if the first condition in the if statement below is true, then the second one won't be evaluated,
              //       that's really important since if the tokenizer is at the end method nextToken can't be called
              if (!tokenizer.hasMoreTokens() || !tokenizer.nextToken().equals("(")) {
                // check for case sensitivity first
                String caseString = "";
                if (!caseOn) caseString = tempString.toUpperCase();
                else caseString = tempString;
                if ((indx = dataTableVariables.indexOf(caseString))!=-1) { // a constant value was found
                  findValue=((Object)Tools.doubToObj(dataTableVariables.getConstantValue(indx))).toString();
                  param = tempString;
                  // now change the occurence of that parameter to the value of this parameter from the table
                  StringBuffer buff = new StringBuffer(newExpr);
                  // This will point to the index at the end of the token that is exchanged in the string
                  // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                  int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
                  buff.replace( newExpr.indexOf(param,strPointer),
                                newExpr.indexOf(param,strPointer)+param.length(), "("+findValue+")" );
                  strPointer = tmpStrPointer;
                  newExpr = buff.toString();
                }
              }

            }
          }
          // Create the function with this new expression (newExpr) and evaluate it
          Function fun = new Function(newExpr);
//System.out.println("This is the new expression:"+newExpr);
          return fun.evaluate(-1, caseOn, functionsTable); //-1 is a dummy variable, since teh expressions doesn't contain x
        }
         */


  /**
   * This method evaluates the expression that is sent to it.
   * Note: this method recognizes columns name calls in the expression.
   * The 'expr' is the expression and rowIndex is the row to look into once the column has been found.
   *
   *  Important! it is essential to call the method fillTempErrors(); befote calling this method IF the expression contains ERR references
   */
/*   private double evaluateExpr(String expr, int rowIndex) {
//System.out.println("expr sent to evaluateExpr: "+expr);
          String param = ""; // this is the parameter(s) we are loking for
          StringTokenizer tokenizer; // parse the line
          String newExpr = expr; // this will be the final expression with no parameters in it
          // Note: the delimiters are set to true, so that it would be possible to get '(' and check for function calls
          tokenizer = new StringTokenizer(newExpr, "()+-/*^ ", true);
          // In order to avoid the bug of having a number that contains exponential letter E within it put into
          // the string and then having to search for a constant or column named E, use this number (strPointer) so that I
          // wouldn't have to traverse through the string from the start, therefore I wouldn't exchange the wrong letter
          int strPointer = -1;

          while (tokenizer.hasMoreTokens()) {
            String tempString = tokenizer.nextToken();
            char tempChar = tempString.charAt(0);
            if (Character.isLetter(tempChar)) { // if it is an expression (not a number or an arithmetic sign)
              // this means that we found some parameter. Find it (in Columns) and evaluate with it's rowIndex value
              tempString = tempString.trim();
              String findValue = ""; // this value could be any non-numeric token, including sqrt,sqr..
              int columnIndex = -1;
              // First check for function calls, in case someone named a variable same as some of the functions
              // This is done by looking up the next token, if it is '(' then it is a function call for sure
              // Note: calling nextToken will not affect the overall parsing of the expression!
              // make sure it is not the last element since there is a nextToken call and check if it is not a function
              // Note: if the first condition in the if statement below is true, then the second one won't be evaluated,
              //       that's really important since if hte tokenizer is at the end method nextToken can't be called
              if (!tokenizer.hasMoreTokens() || !tokenizer.nextToken().equals("(")) {
                int stringIndex = tempString.indexOf(";");
                // if it is a column name and not an error name (reference), it won't contain ; since error reference has a format ABC;ERR
                if (stringIndex == -1) {
                  // check for case sensitivity first
                  String caseString = "";
                  if (!caseOn) caseString = tempString.toUpperCase();
                  else caseString = tempString;
                  // finally, find the column name stringIndex
                  if ( (DataColumn)((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(caseString)!=null ) {
                    param = tempString;
                    // this is the value to exchange in the expression
                    findValue = ((DataColumn)((DataTable)
                              dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(caseString)).getValue(rowIndex).toString();
//System.out.println("Mod: change value (regular): "+findValue);
                    // now exchange the occurence of that parameter to the value of that parameter from the table
                    StringBuffer buff = new StringBuffer(newExpr);
                    // This will point to the index at the end of the token that is exchanged in the string
                    // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                    int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
//System.out.println("newExpr: "+newExpr);
//System.out.println("Mod: trim: "+newExpr.indexOf(param,strPointer)+" to "+newExpr.indexOf(param,strPointer)+param.length());
                    // param (ie tempString) was initially extracted from newExpr so find it in there now again
                    int first, second = 0;
                    first = newExpr.indexOf(param,strPointer);
                    second = newExpr.indexOf(param,strPointer)+param.length();
                    buff.replace( first, second, "("+findValue+")");
                    newExpr = buff.toString();
                    strPointer=tmpStrPointer;
//System.out.println("this is the newExpr for MOD (with column parameter), strPointer: "+newExpr+", "+strPointer);
                  }
                // It is an error reference. It can be in three different formats: ABC;POSERR, ABC;NEGERR or ABC;ERR
                } else if (stringIndex != -1) {
                  // check for case sensitivity first and extract the column name from the string at the same time
                  String caseString = "";
                  if (!caseOn) caseString = tempString.substring(0,stringIndex-1).toUpperCase();
                  else caseString = tempString.substring(0,stringIndex-1);
System.out.println("tempString.toUpperCase(): "+tempString.toUpperCase());
System.out.println("tempString.toLowerCase(): "+tempString.toLowerCase());
System.out.println("tempString.substring(0,stringIndex-1): "+tempString.substring(0,stringIndex-1));
System.out.println("tempString.substring(stringIndex+1).toUpperCase(): "+tempString.substring(stringIndex+1).toUpperCase());
                  // find the column name index
                  if ( (DataColumn)((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(caseString)!=null ) {
                    // Get the index of the column
                    int columnIndex = ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getIndex(caseString);
                    // if it is a reference to the positive error, find that error from the column (that is also given)
                    if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("POSERR")) ) {
                      // this is the positive error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                      findValue = ((ConcreteDataColumn) tempPosErr.get(columnIndex) ).getValue(rowIndex).toString();
                    } else if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("NEGERR")) ) {
                      // this is the negative error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                      findValue = ((ConcreteDataColumn) tempNegErr.get(columnIndex) ).getValue(rowIndex).toString();
                     } else if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("ERR")) ) {
                       // this is the error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                       findValue = ((ConcreteDataColumn) tempErr.get(columnIndex) ).getValue(rowIndex).toString();
                     }
                     param = tempString;
                     // now exchange the occurence of that parameter to the value of that parameter from the table
                     StringBuffer buff = new StringBuffer(newExpr);
                     // This will point to the index at the end of the token that is exchanged in the string
                     // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                     int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
                     // param (ie tempString) was initially extracted from newExpr so find it in there now again
                     int first, second = 0;
                     first = newExpr.indexOf(param,strPointer);
                     second = newExpr.indexOf(param,strPointer)+param.length();
                     buff.replace( first, second, "("+findValue+")");
                     newExpr = buff.toString();
                     strPointer=tmpStrPointer;
                  }
                }
              }
            }
          }
          // Create the function with this new expression (newExpr) and evaluate it
          Function fun = new Function(newExpr);
//System.out.println("This is the new expression:"+newExpr);
          return fun.evaluate(-1, caseOn, functionsTable); //-1 is a dummy variable, since the expression doesn't contain x
        }
   */

        /** This is a method for the "command-line" DATA,
         *  It loads the data values of all the columns for a certain row. It is called as many times as there are rows.
         *  It also loads the variables into the dataTableVariables object and keeps it as a reference
         *  note: the line that is passed to it doesn't contain any backslashes
         *  it also calls the method evaluateExpr(String)
         */
        public void setDataVariables(String line) {
          // parse the line
          StringTokenizer tokenizer;
          tokenizer = new StringTokenizer(line, "=,", true);
          String delimToken  = "";

          // Fill the temp error vectorst in case there are references to them when calling evaluateExpr()
          if (dataTableList.getSelectedIndex()!=-1 && (line.toUpperCase().indexOf("ERR"))!=-1)
            fillTempErrors( (DataTable) dictionary.getElement(dataTableList.getSelectedIndex()) );

          // this line contains 'columnCount' number of column data values plus the comment at the end
          // Read each one of those sets of data (every three commas)
          // Note: the line can end anywhere along the line, in that case fill the rest of the fields with zeros
          for(int i=0;i<tempDataColumns.columnCount();i++) {

            // In case the line has 'ended early', fill the value with zero
            // Note: since this is a DATA generated column, this DataColumn will be of type Multicolumn
            if (!tokenizer.hasMoreElements()) {
                ((ConcreteDataColumn) ((MultiColumn) tempDataColumns.getColumn(i)).getValueColumn() ).addElement("0");
            } else {
              Object exprToken = tokenizer.nextToken().trim(); // first label
//System.out.println("label: "+exprToken);
              delimToken = tokenizer.nextToken().trim();
//System.out.println("delimiter: "+delimToken);
              if (delimToken.equals("=")) { // if next token is =
                // evaluate the value and load it into the Constants table and into the DataTable
                String expr = tokenizer.nextToken();
                int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                double newExpr = evaluateExpr(expr, index, -1, "");
                // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                  System.out.println("Error: Division with zero exception: in column "+tempNames.get(i).toString()+
                                    ", expression: "+ expr );
                dataTableVariables.setConstant(exprToken.toString(), Double.toString(newExpr), new String("") );
                ((ConcreteDataColumn) ((MultiColumn) tempDataColumns.getColumn(i)).getValueColumn() ).addElement(Tools.doubToObj(newExpr));
                tokenizer.nextToken(); // skip the comma
//System.out.println("expression evaluated: "+newExpr);
              } else if (delimToken.equals(",")) { // it is a comma
                // evaluate the label value and load it into the DataTable
                int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                double newExpr = evaluateExpr(exprToken.toString(), index, -1, "");
                // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                  System.out.println("Error: Division with zero exception: in column "+tempNames.get(i).toString()+
                                    ", expression: "+ exprToken.toString() );
                // load the value
                ((ConcreteDataColumn) ((MultiColumn) tempDataColumns.getColumn(i)).getValueColumn() ).addElement(Tools.doubToObj(newExpr));
              }
            }

            // load the 'plus' error if there is any
            // this value is loaded into minus error if there is no value for minus error in the db file
            Object tempPlusValue = "0"; // it WILL get some value in the code below!
            // In case the line has 'ended early', fill the value with zero
            if (!tokenizer.hasMoreElements()) {
                ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorPlusColumn()).addElement("0");
            } else {
              Object plusToken = tokenizer.nextToken().trim();
//System.out.println("plusToken: "+plusToken);
              // if the first error exists (i.e. if the next token is an expression, not comma)
              if ( !plusToken.equals(",")) {
                if ( !plusToken.equals("")) {// if it is not a blank string + trim()

                  delimToken = tokenizer.nextToken().trim();
//System.out.println("delimiter: "+delimToken);
                  if (delimToken.equals("=")) { // if next token is =
                    // evaluate the value and load it into the Constants table and into the DataTable
                    String expr = tokenizer.nextToken();
                    int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                    double newExpr = evaluateExpr(expr, index, -1, "");
                    // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                    if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                      System.out.println("Error: Division with zero exception: in plus error column "+tempNames.get(i).toString()+
                                    ", expression: "+ expr );
                    dataTableVariables.setConstant(plusToken.toString(), Double.toString(newExpr), new String(""));
//System.out.println("plus Constant value: "+dataTableVariables.getConstantValue(plusToken.toString()));
                    ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorPlusColumn()).addElement(Tools.doubToObj(newExpr));
                    tempPlusValue = Tools.doubToObj(newExpr);
                    tokenizer.nextToken(); // skip the comma
                  } else if (delimToken.equals(",")) { // it is a comma
                    // evaluate the label value and load it into the DataTable
                    int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                    double newExpr = evaluateExpr(plusToken.toString(),index, -1, "");
//System.out.println("new Expr plus error :"+newExpr);
                    // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                    if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                      System.out.println("Error: Division with zero exception: in plus error column "+tempNames.get(i).toString()+
                                    ", expression: "+ plusToken.toString() );
                    // load the value
                    ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorPlusColumn()).addElement(Tools.doubToObj(newExpr));
                    tempPlusValue = Tools.doubToObj(newExpr);
                  }
                // these two else-s used to load tempPlusErrorColumns with null at i to increase plotting efficiency, but...
                } else { ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorPlusColumn()).addElement("0"); // load zero value
                            tempPlusValue = "0";
                            tokenizer.nextToken();
                // also, since it is a blank line, we have to skip the comma after it
                tokenizer.nextToken();
                }
              } else { ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorPlusColumn()).addElement("0");  // load zero value
                    tempPlusValue = "0";
              }
            }

            // load the 'minus' error if there is any
            // In case the line has 'ended early', fill the value with zero
            if (!tokenizer.hasMoreElements()) {
                ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorMinusColumn()).addElement("0");
            } else {
              Object minusToken = tokenizer.nextToken().trim();

              // if the first error exists (i.e. if the next token is an expression, not comma)
              if ( !minusToken.equals(",")) {
                if (!minusToken.equals("")) {// if it is not a blank string + trim()

                  delimToken = tokenizer.nextToken().trim();
                  if (delimToken.equals("=")) { // if next token is =
                    // evaluate the value and load it into the Constants table and into the DataTable
                    String expr = tokenizer.nextToken();
                    int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                    double newExpr = evaluateExpr(expr, index, -1, "");
                    // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                    if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                      System.out.println("Error: Division with zero exception: in minus error column "+tempNames.get(i).toString()+
                                    ", expression: "+ expr );
                    dataTableVariables.setConstant(minusToken.toString(), Double.toString(newExpr), new String(""));
                    ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorMinusColumn()).addElement(Tools.doubToObj(newExpr));
                    tokenizer.nextToken(); // skip the comma
                  } else if (delimToken.equals(",")) { // it is a comma
                    // evaluate the label value and load it into the DataTable
                    int index = tempDataColumns.getColumn(i).getSize()-1; // This will give the last row that just got filled
                    double newExpr = evaluateExpr(minusToken.toString(), index, -1, "");
                    // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
                    if (Double.isInfinite(newExpr) || Double.isNaN(newExpr))
                      System.out.println("Error: Division with zero exception: in minus error column "+tempNames.get(i).toString()+
                                    ", expression: "+ minusToken.toString());
                    // load the value
                    ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorMinusColumn()).addElement(Tools.doubToObj(newExpr));
                  }
                } else {
                  // if minus error doesn't exist, copy the plus error value into the minus error
                  ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorMinusColumn()).addElement(tempPlusValue);
                  // also, since it is a blank line, we have to skip the comma after it
                  tokenizer.nextToken();
                }
              } else {
                // if minus error doesn't exist, copy the plus error value into the minus error
                ((ConcreteDataColumn)((MultiColumn) tempDataColumns.getColumn(i)).getErrorMinusColumn()).addElement(tempPlusValue);
              }
            }
          }
          // Finally, read in the comment at the end of the line
          // In case the line has 'ended early', it means that there will be no comment at the end
          if (tokenizer.hasMoreElements()) {
            String tempComm = "";
            while (tokenizer.hasMoreTokens()) {
              tempComm = tempComm.concat(tokenizer.nextToken()+" ");
            }
          }

        }

        /**
         *
         * note: there is a Date supplied. This is a new date since repack function is used when creating a new DataTable
         *         also, depending on whether the column is virtual or real, appropriate new DataColumn will be generated
         */
        public DataColumn repack(int packing, DataColumn valueColumn, Date date) {

          DataColumn returnColumn = null;
          // If it is a Virtual Column, fill with a Virtual DataColumn
          if ( valueColumn.getType().equals(new String("virtual")) ) {
            double begin = ((LinearColumn)valueColumn).begin;
            int size = ((LinearColumn)valueColumn).size;
            double step = ((LinearColumn)valueColumn).step;
            // The new fields will be: size=(int)size/packing; step=step*packing
            LinearColumn newColumn = new LinearColumn(valueColumn.getName(),valueColumn.getLabel(),valueColumn.getUnits(),
                                                                                  begin , step*packing , (int) size/packing, date);
            returnColumn = (LinearColumn)newColumn;
          // If it is "poisson", fill it with a PoissonDataColumn
          } else if ( valueColumn.getType().equals(new String("poisson")) ) {
            PoissonDataColumn newColumn = new PoissonDataColumn(valueColumn.getName(),valueColumn.getLabel(),valueColumn.getUnits(), date);
            int j = 0;
            // the new size of the column will be: (int)valueColumn.getSize()/packing
            for (int k = 0; k<(int)valueColumn.getSize()/packing; k++) {
              double packValue = 0;
              double sumW = packing;
              for (int i=0; i<packing; i++) {
                double value = Tools.objToDoub( valueColumn.getValue(j) );
                packValue = packValue + value;
                 j=j+1;
              }
              packValue = packValue/sumW;
              newColumn.addElement( Tools.doubToObj(packValue) );
            }
            returnColumn = (PoissonDataColumn)newColumn;
          // If it is "real", fill it with ConcreteDataColumns
          // Note: this Column might be either MultiColumn or ConcreteDataColumn, that's why I will treat it here as a DataColumn
          } else if ( valueColumn.getType().equals(new String("real")) ) {
         // This part of the code generates ConcreteDataColumns if the valueColumn is of ConcreteDataColumn Type
            ConcreteDataColumn newColumn = new ConcreteDataColumn(valueColumn.getName(),valueColumn.getLabel(),valueColumn.getUnits(), date);
            ConcreteDataColumn newColumnMinusErr = new ConcreteDataColumn();
            ConcreteDataColumn newColumnPlusErr = new ConcreteDataColumn();
            int j = 0;
//System.out.println("packing: "+packing);
            // the new size of the column will be: (int)valueColumn.getSize()/packing
            for (int k = 0; k<(int)valueColumn.getSize()/packing; k++) {
              double packValue = 0;
              double packError = 0;
              double sumW = 0;
              for (int i=0; i<packing; i++) {
                double value = Tools.objToDoub( valueColumn.getValue(j) );
                double errMin = Math.abs( Tools.objToDoub( valueColumn.getErrorMinus(j) ) );
                double errPlus = Math.abs( Tools.objToDoub( valueColumn.getErrorPlus(j) ) );
//System.out.println("j: "+j+"i: "+i+" k: "+k);
//System.out.println("errPlus: "+errPlus);
//System.out.println("errMin: "+errMin);
                 j=j+1;
                // If error == 0, do a simple average
                if ( errMin == 0 || errPlus == 0 ) { // Note: this algorithm could be better. It doesn't take into account if only one error is zero
                  packValue = packValue + value;
                  // note: packError will remain zero
                  sumW = packing;
                }
                // else do a weighted average
                else {
                  double w = 1/Math.pow(errMin,2) + 1/Math.pow(errPlus,2);
                  packValue = packValue + value*w;
                  sumW = sumW + w;
                  packError = 1; // just change the value so that it woudl make it through the if condition below
                }
//System.out.println("sumW: "+sumW);
              }
              packValue = packValue/sumW;
//System.out.println("packValue: "+packValue);
              if ( packError!=0 ) // if packError is zero, then that means that the errors are both zero
                packError = 1/Math.sqrt(sumW);
//System.out.println("packError: "+packError);
              newColumn.addElement( Tools.doubToObj(packValue) );
              newColumnMinusErr.addElement( Tools.doubToObj(packError) );
              newColumnPlusErr.addElement( Tools.doubToObj(packError) );
            }
            returnColumn = new MultiColumn((ConcreteDataColumn)newColumn,
                                                                                (ConcreteDataColumn)newColumnPlusErr,
                                                                                (ConcreteDataColumn)newColumnMinusErr);
          }
            return returnColumn;
        }

        /**
         * This method modifies the Errors in the appropriate column with the given expression
         *
         *
         */
         public void modifyError(String expr, String modName) {
           // find which column is supposed to be modified, check for case sensitivity first
           int index = indexOfDataColumnNames(GUI.this, modName);
           // in case the column is a MultiColumn
           if ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("real")) ) {
             // 'Refresh' (of fill if it is called for the first time) the temporary error vectors
             if (dataTableList.getSelectedIndex()!=-1 && (expr.toUpperCase().indexOf("ERR"))!=-1)
               fillTempErrors((DataTable)dictionary.getElement(dataTableList.getSelectedIndex()));
             // find how many rows this column contains
             int rowCount = ((DataColumn)tempDataColumns.getColumn(index)).getSize();
             for(int i=0; i<rowCount; i++) {
               double exprEval = 0;
               // Evaluate the given expression which most probably contains references to ERROR values
               exprEval = evaluateExpr(expr,i, -1, "");
               // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
               if (Double.isInfinite(exprEval) || Double.isNaN(exprEval))
                 System.out.println("Error: Division with zero exception: MOD or GEN error column "+tempNames.get(i).toString()+", expression: "+ expr );
//System.out.println("GUI:modifyColumn: Tools.doubToObj(exprEval): "+Tools.doubToObj(exprEval) );
                 ((MultiColumn)tempDataColumns.getColumn(index)).setErrorPlusColumn(new Double(exprEval), i);
                 ((MultiColumn)tempDataColumns.getColumn(index)).setErrorMinusColumn(new Double(exprEval), i);
              }
           // if it is virual(Linear) or poisson, don't change anyting and display message in the console
           } else {
                console.jta.append(console.jtf.getText()+"\n Error: You can't modify Errors of Linear or Poisson Columns. \n");
                console.jtf.setText("DB:");
                console.newLineRecognizer=""; // 'restart' this value
           }
          // Now insert this datatable in the dictionary
          ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).setColumn( (DataColumn) tempDataColumns.getColumn(index), index);
         }


        /**
         *  This is a method for the "command-line" MODIFY, it modifies the column specified with modName
         *  It sets the column to calculated expr value using evaluateExpr() method. The expression will
         *   contain the appropriate parameter and could contain any other constant.
         *  If there is an "=" sign in the expression, load the value into the constants as well
         *
         *  It also calls the method evaluateErrorExpr() to calculate the errors
         */
        public void modifyColumn(String expr1, String modName) {

          // find which column is supposed to be modified, check for case sensitivity first
          int index = indexOfDataColumnNames(GUI.this, modName);
          // find how many rows this column contains
          int rowCount = ((DataColumn)tempDataColumns.getColumn(index)).getSize();
          for(int i=0; i<rowCount; i++) {
            // First, check the expr1.  In case this method was called when the GEN command was called while there were no columns loaded,
            // it will contain a variable 'i' which will take the value: rowCount+1. Exchange that now.
            String expr = expr1; // We don't want to override the original expression.
            if (dictionary.size()==0) {
              while (expr.toUpperCase().indexOf("I")!=-1) {
              expr = expr.substring(0, expr.toUpperCase().indexOf("I")) + String.valueOf(i+1) + expr.substring(expr.toUpperCase().indexOf("I")+1);
//System.out.println("  expr: "+expr);
              }
            }
            // ** Evaluate the VALUE **
            // Check whether expr contains "=" sign, in that case, load the constant with the calculated expr value
            Object constToken = null;
            if(expr.indexOf("=")!=-1) { // it contains the "=" sign
              // I am assuming the expr line has a form of (e.g.): X = y+1
              StringTokenizer tempTokenizer;
              tempTokenizer = new StringTokenizer(expr, "=", true);
              constToken = tempTokenizer.nextToken().trim(); // constant parameter
              String delimToken = tempTokenizer.nextToken().trim(); // "=" sign
              expr = tempTokenizer.nextToken(); // this is the rest of the line, the expression
            }
            // **Evaluate the ERROR first, before evaluating the value because if the value of column changes,
            // then that will effect the error value calculations if it were to be done after it.
            // The error will be estimated to be the same for both plus and minus.
            double exprEval = 0;
            if ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("real")) ) {
              exprEval = evaluateErrorExpr(expr,i);
            }
            // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
            if (Double.isInfinite(exprEval) || Double.isNaN(exprEval))
              System.out.println("Error: Division with zero exception: MOD or GEN error column "+tempNames.get(i).toString()+
                                   ", expression: "+ expr );
            // in case the column is a MultiColumn, change the zero values
            // Note: if it is virual(Linear) or poisson, no need to change anyting
            if ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("real")) ) {
              ((MultiColumn)tempDataColumns.getColumn(index)).setErrorPlusColumn(new Double(exprEval), i);
              ((MultiColumn)tempDataColumns.getColumn(index)).setErrorMinusColumn(new Double(exprEval), i);
            }
            // evaluate the expression
            exprEval = evaluateExpr(expr,i,-1,"");
            // EXCEPTION: if there is a division with zero in the expression, print it out for the user to know
            if (Double.isInfinite(exprEval) || Double.isNaN(exprEval))
              System.out.println("Error: Division with zero exception: MOD or GEN column (setting the constant) "+
                                   tempNames.get(index).toString()+", expression: "+ expr );
            // *** if the expr contained the "=" sign, the constToken Object!=null ***
            // Load the constant into the constants table
            if (constToken!=null) {
              dataTableVariables.setConstant(constToken.toString(), Double.toString(exprEval), new String(""));
            }
            if ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("real")) ) {
              ((ConcreteDataColumn) ((MultiColumn) tempDataColumns.getColumn(index)).getValueColumn() ).setElementAt(Tools.doubToObj(exprEval), i);
            } else if ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("poisson")) ) {
             ((PoissonDataColumn) tempDataColumns.getColumn(index)).setElementAt(Tools.doubToObj(exprEval), i);
            } else if  ( ((DataColumn)tempDataColumns.getColumn(index)).getType().equals(new String("virtual")) ) {
              // when it is the first row, change the 'begin' value
              if (i==0) {
                double exprEvaltemp = evaluateExpr(expr,i+1,-1,""); // note that i+1 is 1. Calculate this value first because the value 'begin' is about to get changed
                ((LinearColumn) tempDataColumns.getColumn(index)).begin = exprEval;
              // when it is the second row, change the 'size' value (which is the second value subtracted with the first value)
                ((LinearColumn) tempDataColumns.getColumn(index)).step = exprEvaltemp - ((LinearColumn) tempDataColumns.getColumn(index)).begin;
              }
            }
          }

          // This means that a Columns was created from scratch so we need to enter a new DataTable into the dictionary
          if ( dictionary.size()==0 ) {
                  //---------------------
                  // Now add this DataTable into the GUI
                  addDataTable(tempDataColumns, false, "");
                  // Reading the DataTable (otherwise it is not associated with it anymore)
                  tempDataColumns = (DataTable) dictionary.getElement( dataTableList.getItemCount()-1 );
                  //---------------------
          }


        //  In case the index points to an existing column, just exchange it
        int delIndex = dataTableList.getSelectedIndex();
        // Note: there should always be a DataTable in the dictionary
        if ( dictionary.size()>0 && delIndex==-1 ) {
          delIndex = dictionary.size()-1;  // Take the last element
          dataTableList.select(delIndex);  // select the last element (this doesn't usually happen)
        }
        if (((DataTable)dictionary.getElement(delIndex)).columnCount()>index)
           ((DataTable)dictionary.getElement(delIndex)).setColumn( (DataColumn) tempDataColumns.getColumn(index), index);
        // otherwise, there is a column missing (this was called from generateExpr method in the DBConsole or DBFileReader), so add a new column
        else if (((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).columnCount()==index)
          ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).addColumn( (DataColumn) tempDataColumns.getColumn(index));
        }

        /**
         * This method calculates the error for the given expression
         * if there are occurences of column parameters, include them unless their errors are zero.
         *
         * **** First, the method detects all the occurences of column names. Counts them and registers them into a vector
         * ***** Then, for every occurence of a column name, it calculates the expression with and error added to that column value
         * and adds it to the final error expression.
         * The full error formula looks like this: e.g. dz^2 = {[F(x+dxp)-F(x-dxm)]/2}^2 + {[F(y+dyp)-F(y-dym)]/2}^2 + ...
         *    and this method uses evaluateExpr() to 'handle' each one of the F( ) functions in the formula
         */
        public double evaluateErrorExpr(String expr, int rowIndex) {
//System.out.println("expr sent to evaluateExpr: "+expr);
          // This will be the final error value
          double result = 0;
          // **** Count the number of parameters in the expression. Everything will be 'revolving' around this number ****
          //     Note: the whole while loop below is for the purpose of calculating this number.
          int countParam = 0;
          // also, create a vector to load parameter name column indexes, size of the vector will be countParam
          Vector paramIndex = new Vector();

          StringTokenizer tokenizer; // parse the line
          // Note: the delimiters are set to true, so that it would be possible to get '(' and check for function calls
          tokenizer = new StringTokenizer(expr, "()+-*/^ ", true);
          while (tokenizer.hasMoreTokens()) {
            String tempString = tokenizer.nextToken();
            char tempChar = tempString.charAt(0);
            if (Character.isLetter(tempChar)) { // if it is an expression (not a number or an arithmetic sign)
              // this means that this parameter could be a column name.
              tempString = tempString.trim();
              String findValue = ""; // this value could be any non-numeric token, including sqrt,sqr..
              int columnIndex =  indexOfDataColumnNames(GUI.this, tempString);;
              // First check for function calls, in case someone named a variable same as some of the functions
              // This is done by looking up the next token, if it is '(' then it is a function call for sure
              // Note: calling nextToken will not affect the overall parsing of the expression!
              // make sure it is not the last element since there is a nextToken call and check if it is not a function
              // Note: if the first condition if the if statement below is true, then the second one won't be evaluated,
              //       that's really important since if the tokenizer is at the end, method nextToken can't be called
              if (!tokenizer.hasMoreTokens() || !tokenizer.nextToken().equals("(")) {
                // Only column values have error bars, so check if this parameter is a column value
                if (columnIndex!=-1) {
                  // we've found the parameter, increase countParam by one
                  countParam++;
//System.out.println("Error: countParam: "+countParam+", columnIndex: "+columnIndex);
                  paramIndex.addElement(Long.toString(columnIndex));
                }
              }
            }
          }
          // Fill the temp error vectors in case there are references to them when calling evaluateExpr()
          if (dataTableList.getSelectedIndex()!=-1 && (expr.toUpperCase().indexOf("ERR"))!=-1)
            fillTempErrors( (DataTable) dictionary.getElement(dataTableList.getSelectedIndex()) );
          // ***** Now calculate the appropriate error, the formula is a little robust  *****
          // Formula e.g. dz^2 = {[F(x+dxp)-F(x-dxm)]/2}^2 + {[F(y+dyp)-F(y-dym)]/2}^2 + ...
          for(int i=1; i<=countParam; i++) {
//System.out.println("Error: expr: "+expr);
            result = result +
              Math.pow( ( evaluateExpr(expr, rowIndex, i, ((DataColumn) tempDataColumns.getColumn(Tools.objToInt(paramIndex.get(i-1))) ).
                                                                                       getErrorPlus(rowIndex).toString()) -
                                 evaluateExpr(expr, rowIndex, i,
                          "-"+((DataColumn) tempDataColumns.getColumn(Tools.objToInt(paramIndex.get(i-1))) ).
                                                                                      getErrorMinus(rowIndex).toString()) )/2
                       ,2 );
//System.out.println("Error: result (i): "+result+" ("+i+")");
          }
          result = Math.sqrt(result);
          return result;
        }

        /** This method evaluates the expression using Ari's function class
         *  it finds all the parameters in the expression and exchanges them with
         *  their values from the:
         *    - *** the appropriate Column values.
         *    - **** Constants (dataTableVariables) table
         *    - ***** the appropriate Column errors (POSERR, NEGERR or ERR).
         *
         * - Since this function is called from the MODIFY command, the expression will contain
         *   a parameter reference to one of the columns, value rowIndex holds the value to the row of that column.
         *
         * - This method is also called from the evaluateErrorExpr() method (which calculates the partial derivative for the sent expression: expr)
         *   in this case, the fields paramNo and errorVal will hold actual values (paramNo!=-1 and errorVal!="")
         *   Basically, the calculation will evaluate the expression (expr) in such a way that it will add an error value (errorVal)
         *   to the occurence of coulumn value in the expression (expr) that is the parameter number: paramNo and won't do that to
         *   not to any other occurence of a column name.
         *  - The full error formula (that is used in evaluateErrorExpr() looks like this: e.g. dz^2 = {[F(x+dxp)-F(x-dxm)]/2}^2 + {[F(y+dyp)-F(y-dym)]/2}^2 + ...
         *    and this method is used to 'handle' each one of the F( ) functions in the formula
         *      Note: -in case the parameter's error is equal to zero, just return zero. This is not correct! I am
         *        doing this to avoid unnecessary computation. Because the two partial derivatives of a parameter
         *        whose error iz zero would cancel each other anyways (F(x+dx)-F(x-dx)=0 if dx=0)
         *
         *  Note: Ari's function works with a parameter x, the final expression in
         *  this method will not contain any parameters
         *  Note: the second (last) part (' else if' part) of the code is the same as in evaluateExpr() method
         *
         *  Important! it is essential to call the method fillTempErrors(); befote calling this method IF the expression contains ERR references
         *
         */
         public double evaluateExpr(String expr, int rowIndex, int paramNo, String errorVal) {
//System.out.println("expr sent to evaluateExpr: "+expr);
          String param = ""; // this is the parameter(s) we are loking for
          StringTokenizer tokenizer; // parse the line
          String newExpr = expr; // this will be the final expression with no parameters in it
          // Note: the delimiters are set to true, so that it would be possible to get '(' and check for function calls
          tokenizer = new StringTokenizer(newExpr, "()+-*/^ ", true);
          // In order to avoid the bug of having a number that contains exponential letter E within it put into
          // the string and then having to search for a constant or column named E, use this number (strPointer) so that I
          // wouldn't have to traverse through the string from the start, therefore I wouldn't exchange the wrong letter
          int strPointer = -1;

          // When the 'paramNo'-th column value parameter has been reached, add errorVal to it.
          // Use this parameter to keep track and know when to add the error value if needed (when paramNo!=-1).
          int countParam = 1;
          while (tokenizer.hasMoreTokens()) {
            String tempString = tokenizer.nextToken();
            char tempChar = tempString.charAt(0);
            if (Character.isLetter(tempChar)) { // if it is an expression (not a number or an arithmetic sign)
              // this means that we found some parameter. Find it and evaluate with it's rowIndex value
              tempString = tempString.trim();
              String findValue = ""; // this value could be any non-numeric token, including sqrt,sqr..
              int columnIndex =  indexOfDataColumnNames(GUI.this, tempString);;
              // First check for function calls, in case someone named a variable same as some of the functions
              // This is done by looking up the next token, if it is '(' then it is a function call for sure
              // Note: calling nextToken will not affect the overall parsing of the expression!
              // make sure it is not the last element since there is a nextToken call and check if it is not a function
              //         Note: if the first condition if the if statement below is true, then the second one won't be evaluated,
              //                  that's really important since if hte tokenizer is at the end method nextToken can't be called
              if (!tokenizer.hasMoreTokens() || !tokenizer.nextToken().equals("(")) {
                int stringIndex = tempString.indexOf(";");
                // First, I am checking for ":" because it could be a common mistake
                if ( tempString.indexOf(":") != -1)  {
                  console.jta.append(console.jtf.getText()+"\n Misspelling Error: use \";\" instead of \":\". \n");
                  console.jtf.setText("DB:");
                  console.newLineRecognizer=""; // 'restart' this value
                // if it is a column name and not an error name (reference), it won't contain ; since error reference has a format ABC;ERR
                } else if (stringIndex == -1) {
                  // *** Find the column name index ***
                  // Note: I am first checking for Column name parameters in case there is a same name in the constants table

                  if (columnIndex!=-1) {
                    param = tempString;
                    // This will be just a regular column value insert for the parameter
                    if (paramNo==-1 || paramNo!=countParam) {
                      // this is the value to exchange in the expression
                      findValue=((DataColumn)tempDataColumns.getColumn(columnIndex)).getValue(rowIndex).toString();
                      countParam++;
//System.out.println("Mod: change value (regular): "+findValue);
                    // this means paramNo!=-1 && paramNo==countParam, so add the errorVal to the value (this means this method was called from evaluateErrorExpr()
                    } else {
                      double val=Tools.objToDoub(((DataColumn)tempDataColumns.getColumn(columnIndex)).getValue(rowIndex))
                                 + Tools.objToDoub(errorVal);
                      findValue=Tools.doubToObj(val).toString();
                      countParam++;
//System.out.println("Mod: change value (error): "+findValue+" ("+errorVal+")");
                    }
                    // now exchange the occurence of that parameter to the value of that parameter from the table
                    StringBuffer buff = new StringBuffer(newExpr);
                    // This will point to the index at the end of the token that is exchanged in the string
                    // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                    int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
//System.out.println("newExpr: "+newExpr);
//System.out.println("[3725] Mod: frim: "+newExpr.indexOf(param,strPointer)+" to "+newExpr.indexOf(param,strPointer)+param.length());
                    // If caseOff is on, handle the expression parsing accordingly
                    int first, second = 0;
                    first = newExpr.indexOf(param,strPointer);
                    second = newExpr.indexOf(param,strPointer)+param.length();
                    buff.replace( first, second, "("+findValue+")");
                    newExpr = buff.toString();
                    strPointer=tmpStrPointer;
//System.out.println("[3733] this is the newExpr for MOD (with column parameter), strPointer: "+newExpr+", "+strPointer);

                  // **** it must exist in the CONSTANTS table, since it is not a function, nor a column value ****
                  // it also doesn't have error handling since it is just a constant
                  } else if ((columnIndex = dataTableVariables.indexOf(GUI.this, tempString))!=-1) { // a constant value was found
                    findValue=((Object)Tools.doubToObj(dataTableVariables.getConstantValue(GUI.this, columnIndex))).toString();
                    param = tempString;
                    // now change the occurence of that parameter to the value of this parameter from the table
                    StringBuffer buff = new StringBuffer(newExpr);
                    // This will point to the index at the end of the token that is exchanged in the string
                    // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                    int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
                    // If caseOff is on, handle the expression parsing accordingly
                    int first, second = 0;
                    first = newExpr.indexOf(param,strPointer);
                    second = newExpr.indexOf(param,strPointer)+param.length();
                    buff.replace( first, second, "("+findValue+")" );
                    newExpr = buff.toString();
                    strPointer=tmpStrPointer;
//System.out.println("[37752] this is the newExpr for MOD (with constant parameter), strPointer: "+newExpr+", "+strPointer);
                  }
                // ***** Check for the 'ERR' String *****
                // It is an error reference. It can be in three different formats: ABC;POSERR, ABC;NEGERR or ABC;ERR
                } else if (stringIndex != -1) {
                 if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("POSERR")) ||
                     tempString.substring(stringIndex+1).toUpperCase().equals(new String("NEGERR")) ||
                     tempString.substring(stringIndex+1).toUpperCase().equals(new String("ERR")) ) {
                    // check for case sensitivity first and extract the column name from the string at the same time
                    String caseString = "";
                    if (!caseMenuItem.getState()) caseString = tempString.toUpperCase().substring(0,stringIndex).toUpperCase();
                    else caseString = tempString.substring(0,stringIndex);



                    // find the column name index
                    if ( (DataColumn)((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getColumn(caseString)!=null ) {
                      // Get the index of the column
                      int colIndex = ((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getIndex(caseString);
                      // if it is a reference to the positive error, find that error from the column (that is also given)
                      if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("POSERR")) ) {
                        // this is the positive error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                        findValue = ((ConcreteDataColumn) tempPosErr.get(colIndex) ).getValue(rowIndex).toString();
                      } else if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("NEGERR")) ) {
                        // this is the negative error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                        findValue = ((ConcreteDataColumn) tempNegErr.get(colIndex) ).getValue(rowIndex).toString();
                       } else if (tempString.substring(stringIndex+1).toUpperCase().equals(new String("ERR")) ) {
                         // this is the error value to exchange in the expression, it is looked up in temp Vector since the actual error values might change
                         findValue = ((ConcreteDataColumn) tempErr.get(colIndex) ).getValue(rowIndex).toString();
                       }
                       param = tempString;
                       // now exchange the occurence of that parameter to the value of that parameter from the table
                       StringBuffer buff = new StringBuffer(newExpr);
                       // This will point to the index at the end of the token that is exchanged in the string
                       // +2 is for the brackets that are added, they are added in case the number has a + or - in front
                       int tmpStrPointer = newExpr.indexOf(param,strPointer)+findValue.length()+2;
                       // param (ie tempString) was initially extracted from newExpr so find it in there now again
                       int first, second = 0;
                       first = newExpr.indexOf(param,strPointer);
                       second = newExpr.indexOf(param,strPointer)+param.length();
                       buff.replace( first, second, "("+findValue+")");
                       newExpr = buff.toString();
                       strPointer=tmpStrPointer;
                    }
                  } else {
                    console.jta.append(console.jtf.getText()+"\n Misspelling Error: use POSERR, NEGERR or ERR. \n");
                    console.jtf.setText("DB:");
                    console.newLineRecognizer=""; // 'restart' this value
                  }
                }
              }
            }
          }
          // Create the function with this new expression (newExpr) and evaluate it
          newExpr = newExpr.trim();
          Function fun = new Function(newExpr);
          return fun.evaluate(GUI.this, -1, caseMenuItem.getState(), functionsTable, dataTableVariables); //-1 is a dummy variable, since the expression doesn't contain x
        }


  /**
   * This method is used by CONDITIONAL PLOT. It takes in the three 'tokens' from the conditional expression and
   * 'evaluates' it's validity. 'rowIndex' holds the row value of the columns.
   */
  public boolean checkCondition(String first, String delim, String second, int rowIndex) {
    // Fill the temp error vectors in case there are references to them when calling evaluateExpr()
    if (dataTableList.getSelectedIndex()!=-1 && ((first.toUpperCase().indexOf("ERR"))!=-1 || (second.toUpperCase().indexOf("ERR"))!=-1) )
      fillTempErrors( (DataTable) dictionary.getElement(dataTableList.getSelectedIndex()) );
    boolean result = false;
    if (delim.equals("=")) {
      result = evaluateExpr(first, rowIndex, -1, "")==evaluateExpr(second, rowIndex, -1, "");
    } else if (delim.equals("<")) {
        result = evaluateExpr(first, rowIndex, -1, "") < evaluateExpr(second, rowIndex, -1, "");
    } else if (delim.equals(">")) {
        result = evaluateExpr(first, rowIndex, -1, "") > evaluateExpr(second, rowIndex, -1, "");
    } else result=false;
    return result;
  }

        /**
         *  This Method adds the DataTable into the dictionary, The label into the DataTableList,
         *                     resets the console DataTable and reloads the ColumnList to show the new columns of the DataTable
         *  Arguments:
         *      importedDataTable - the new DataTable
         *      boolean sortedDataTable - determines whether the new DataTable to be created will be a sorted (in order to append 'sorted' at end)
         *      String sortRefColumn - this is the Column the DataTable was sorted to, use it to put it into the DataTable Name
         *           Note, the two values rigt above are called from the sortDataTable() Method
         */
        public void addDataTable(DataTable importedDataTable, boolean sortedDataTable, String sortRefColumn) {
          // Put the DataTable into the dictionary and the dataTable List
          String fileName = new File(importedDataTable.filepath).getName();
          String tableName = importedDataTable.getTitle();    
          if((tableName.equals(""))||(tableName==null)) //no name returned, so use fileName without the extension
              tableName = fileName.substring(0, fileName.lastIndexOf("."));
            if (!sortedDataTable) { // use regular name
            // The Name of the DataTable in the list is of the Format:
            //               <List order #>: File: <filename>; Title: <Title> SORTED ON <sortRefColumn>
            dictionary.addElement(importedDataTable);
            // note: I am extraction filename from filePath
            dataTableList.add(String.valueOf(dictionary.size() + ". " + fileName + "; " + tableName));  // Add it to the List
            dataTableList.select(dataTableList.getItemCount()-1);                // Select it
//System.out.println("Selected in dataTableList: "+dataTableList.getItemCount()+"-1");
          } else{
            // The Name of the DataTable in the list is of the Format:
            //               <List order #>: File: <filename>; Title: <Title>
//System.out.println("importedDataTable : "+importedDataTable.columnCount());
            dictionary.addElement(importedDataTable);
            // note: I am extraction filename from filePath
            dataTableList.add(String.valueOf(dictionary.size() + ". " + fileName +"; "+ tableName +" SORTED ON "+sortRefColumn ));  // Add it to the List
          }
/*
          // This part of code is actually not necessary for GUI functionality right now
          if (dbFileReader!=null) {
            // These values get 'filled' when commands are entered manually in the DBConsole
            // Therefore, if the newly loaded data table contains various 'information', reload it in the DBConsole
            console.tempDataColumns = dbFileReader.tempColumns;
            console.tempLabels = dbFileReader.tempLabels;
            console.tempNames = dbFileReader.tempNames;
            console.columnCount = dbFileReader.tempDataColumns.size();
          }
*/
            reloadColumnList();
        }

        /**
         * This Function opens the Frame for MUD Spectrums loading
         */
        public void loadMUDSpectrum(int dataType) {
          mudSpectFrame.setDataType(dataType);
          mudSpectFrame.setLocation((int)screenDimen.getWidth()/4, 50);
          mudSpectFrame.show();
        }

        /**
         *  This method imports a db file from musr.tirumf.ca through RMI
         * It is called from the MenuActionLIstener and from the Main GUI Window Button
         */
        public void importFileMusr() {
                      openFileDialog = new Dialog(GUI.this, "Open File from 'musr.triumf.ca' Server", true);
                      openFileDialog.setLayout(new GridLayout(8,2));
                      // Add the components to the FileDialog
                      // File path and name to look for on the musr server
                      openFileDialog.add(new JLabel("Enter file path and name to get from "));
                      openFileDialog.add(new JLabel("'musr.triumf.ca:'"));
                      openFileDialog.add(new JLabel("Directory:"));
                      openFileDialog.add(openFileTextField1);
                      openFileDialog.add(new JLabel("File Name:"));
                      openFileDialog.add(openFileTextField2);
                      // File path and name to save on the local machine
                      openFileDialog.add(new JLabel("Enter file path and name to save on your computer: "));
                      openFileDialog.add(new JLabel(""));
                      openFileDialog.add(new JLabel("Directory:"));
                      openFileDialog.add(openFileTextField3);
                      openFileDialog.add(new JLabel("File Name:"));
                      openFileDialog.add(openFileTextField4);
                      // RMI reminder message
                      openFileDialog.add(new JLabel("Note: make sure the rmiregistry is active on your computer "));
                      openFileDialog.add(new JLabel(""));
                      // Add buttons
                      openFileDialog.add(openFileButton);
                      openFileDialog.add(openFileButton2);

                      openFileDialog.setSize(new Dimension(700,350));
                      openFileDialog.setLocation(200,200);
                      openFileDialog.show();  // display the dialog and block until answered

                      // the rest of the code is in the "Open musr File" buttonListener!
        }
/**
         *  This method imports a db file from cmms.tirumf.ca through RMI
         * It is called from the MenuActionLIstener and from the Main GUI Window Button
         */
        public void importFileCmms() {

                      openCFileDialog = new Dialog(GUI.this, "Open File from 'cmms.triumf.ca' Server", true);
                      openCFileDialog.setLayout(new GridLayout(8,2));
                      // Add the components to the FileDialog
                      // File path and name to look for on the musr server
                      openCFileDialog.add(new JLabel("Enter file path and name to get from "));
                      openCFileDialog.add(new JLabel("'cmms.triumf.ca':"));
                      openCFileDialog.add(new JLabel("Directory:"));
                      openCFileDialog.add(openCFileTextField1);
                      openCFileDialog.add(new JLabel("File Name:"));
                      openCFileDialog.add(openCFileTextField2);
                      // File path and name to save on the local machine
                      openCFileDialog.add(new JLabel("Enter file path and name to save on your computer: "));
                      openCFileDialog.add(new JLabel(""));
                      openCFileDialog.add(new JLabel("Directory:"));
                      openCFileDialog.add(openCFileTextField3);
                      openCFileDialog.add(new JLabel("File Name:"));
                      openCFileDialog.add(openCFileTextField4);
                      // RMI reminder message
                      openCFileDialog.add(new JLabel("Note: make sure the rmiregistry is active on your computer "));
                      openCFileDialog.add(new JLabel(""));
                      // Add buttons
                      openCFileDialog.add(openCFileButton);
                      openCFileDialog.add(openCFileButton2);

                      openCFileDialog.setSize(new Dimension(700,350));
                      openCFileDialog.setLocation(200,200);
                      openCFileDialog.show();  // display the dialog and block until answered

                      // the rest of the code is in the "Open cmms File" buttonListener!

        }

        /**
         * This method imports a db file from the loacal machine
         * It is called from the MenuActionLIstener and from the Main GUI Window Button
         */
        public void importLocalFile(){

                      dbFilter.addExtension("db");
                      dbFilter.addExtension("dbc");
                      dbFilter.addExtension("cmd");
//                      dbFilter.setDescription("'.db' and '.' files");
                      dImportLocal.setFileFilter(dbFilter);

                      dImportLocal.setLocation(300,300);

                     int returnVal = dImportLocal.showOpenDialog(GUI.this); // display the dialog and block until answered
                     if(returnVal == JFileChooser.APPROVE_OPTION)  { // Load the file if the Dialog is not cancelled
                          String filename = dImportLocal.getSelectedFile().getName();
                          // Always re-save dirPath because the file is being loaded from the dialog box
                          dirPath = dImportLocal.getCurrentDirectory().getAbsolutePath()+"/";
//    System.out.println("dirPath: "+dirPath);
                          if (filename!=null) {
                            loadDBFile(dirPath, filename);
                          }
                        }
        }

        /**
         * This method imports a db file from the loacal machine
         * It is called from the MenuActionLIstener and from the Main GUI Window Button
         */
        public void openBlankDataTable() {

                        // Contains three fields, one for the Column name, the other for the amount of points and the last (optional) for the expression
                        final JFrame aFrame = new JFrame("Open new blank DataTable");
                        aFrame.setBackground(Color.lightGray);
                        ScrollPane aFrameScrollbar = new ScrollPane();
                        JPanel aPanel = new JPanel();
                        aPanel.setLayout(new BorderLayout());

                        final TextField genName = new TextField();
                        final TextField genNumber = new TextField();
                        final TextField genExpr = new TextField("i"); // Note that 'i' will be the default expression and will load the values like this: {1, 2, 3.... N}

                        JPanel openBlankDTPanel = new JPanel();
                        openBlankDTPanel.setLayout(new GridBagLayout());

                          gridConstr.gridx = 0; gridConstr.gridy = 0;
                        openBlankDTPanel.add(new JLabel("Enter NAME for new variable: "),gridConstr);
                          gridConstr.gridx = 1; gridConstr.gridy = 0;
                        genName.setColumns(50);
                        openBlankDTPanel.add(genName, gridConstr);
                          gridConstr.gridx = 0; gridConstr.gridy = 1;
                        openBlankDTPanel.add(new JLabel("How many new data points: "),gridConstr);
                          gridConstr.gridx = 1; gridConstr.gridy = 1;
                        genNumber.setColumns(50);
                        openBlankDTPanel.add(genNumber, gridConstr);
                          gridConstr.gridx = 0; gridConstr.gridy = 2;
                        openBlankDTPanel.add(new JLabel("Enter defining statement, X[i]: "),gridConstr);
                          gridConstr.gridx = 1; gridConstr.gridy = 2;
                        genExpr.setColumns(50);
                        openBlankDTPanel.add(genExpr, gridConstr);

                        // Add the three fields
                        aPanel.add(openBlankDTPanel, "Center");
                        Button aButton = new Button("Create");
                        aPanel.add(aButton, "South");

                        aFrameScrollbar.add(aPanel);
                        aFrame.getContentPane().add(aFrameScrollbar);

                        aFrame.setSize(700,250);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerBlankTD = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Create"))
                          aFrame.dispose();

                console.rowCount = Long.parseLong(genNumber.getText());
                // If there has been no data table loaded at all
                // ---------------
                if (tempDataColumns==null) tempDataColumns = new DataTable(new String(""));
                if (tempDataColumns.columnCount()==0) {
                  console.generateColumn(genExpr.getText(), genName.getText(), new Date() );
                } else {
                  console.generateColumn(genExpr.getText(), genName.getText(), ((DataColumn) tempDataColumns.getColumn(0)).getDate() );
                }
                // --------------
                // Update this dataTable's history (for the XML output later..)
                tempDataColumns.tableHistory.append(new String(" GEN "+genName+" "+genExpr+"\n"));

                         }
                        };
                        aButton.addActionListener(buttonlistenerBlankTD);

                        aFrame.setLocation(200,200);
                        aFrame.show();

        }

        /**
         * This method imports an XML file from the local machine
         * It is called from the MenuActionLIstener
         */
        public void importXMLFile(){
                      dbFilter.addExtension("xml");
                      dImportLocal.setFileFilter(dbFilter);

                      dImportLocal.setLocation(300,300);

                     int returnVal = dImportLocal.showOpenDialog(GUI.this); // display the dialog and block until answered
                     if(returnVal == JFileChooser.APPROVE_OPTION)  { // Load the file if the Dialog is not cancelled
                          String filename = dImportLocal.getSelectedFile().getName();
                          // Always re-save dirPath because the file is being loaded from the dialog box
                          dirPath = dImportLocal.getCurrentDirectory().getAbsolutePath()+"/";
    //System.out.println("dirPath: "+dirPath);
                          if (filename!=null) {
                            loadXMLFile(dirPath, filename);
                          }
                        }
        }
        
        public void importCSVFile()
        {
//System.out.println("entered importCSVFile");            
            dbFilter.addExtension("csv");
            dImportLocal.setFileFilter(dbFilter);
            dImportLocal.setLocation(300,300);
            int returnVal = dImportLocal.showOpenDialog(GUI.this);
            if(returnVal==JFileChooser.APPROVE_OPTION)
                    csvReader = new CSVReader(GUI.this);
            if(dImportLocal.getSelectedFile()!=null)
                csvReader.loadStream(dImportLocal.getSelectedFile().getPath());
        }

        /**
         * This method is an alternative way of importing DataTables.
         * It is called when the user presses the 'Import Data Table' button in the Main GUI Window
         * There are two ways to import a DataTable: locally and through RMI
         */
         public void importDataTable() {
         // Open a 'frame box' asking which type of import the user wants to do
           final JFrame aFrame = new JFrame("Import Data Table");
           aFrame.setBackground(Color.lightGray);
           aFrame.getContentPane().setLayout(new GridBagLayout());
           	gridConstr.gridx = 0; gridConstr.gridy = 1;
           Button localButton = new Button("Import Data Table from a local file");
           aFrame.getContentPane().add(localButton, gridConstr);
           	gridConstr.gridx = 0; gridConstr.gridy = 2;
           Button musrButton = new Button("Import Data Table from musr.triumf.ca (through RMI)");
           aFrame.getContentPane().add(musrButton, gridConstr);
           	gridConstr.gridx = 0; gridConstr.gridy = 3;
           Button cmmsButton = new Button("Import Data Table from cmms.triumf.ca (through RMI)");
           aFrame.getContentPane().add(cmmsButton, gridConstr);
           	gridConstr.gridx = 0; gridConstr.gridy = 4;
           Button aButton = new Button("Close");
           aFrame.getContentPane().add(aButton, gridConstr);
           aFrame.setSize(600,250);
           aFrame.isEnabled();
           ActionListenerS buttonlistenerTitle = new ActionListenerS() {
             public void actionPerformed(ActionEvent e) {
               if (e.getActionCommand().equals("Import Data Table from a local file")) {
                 aFrame.dispose();
                 importLocalFile();
               }
               if (e.getActionCommand().equals("Import Data Table from musr.triumf.ca (through RMI)")) {
                 aFrame.dispose();
                 importFileMusr();
               }
               if (e.getActionCommand().equals("Import Data Table from cmms.triumf.ca (through RMI)")) {
                 aFrame.dispose();
                 importFileCmms();
               }
               if (e.getActionCommand().equals("Close")) {
                 aFrame.dispose();
               }
            }
          };
          localButton.addActionListener(buttonlistenerTitle);
          musrButton.addActionListener(buttonlistenerTitle);
          cmmsButton.addActionListener(buttonlistenerTitle);
          aButton.addActionListener(buttonlistenerTitle);

                        aFrame.setLocation(200,200);
                        aFrame.show();
         }

        /**
         * This method is called from different parts of teh GUI: the button from the Main GUI Window, from the File Menu..
         * It Erases the selected DataTable and all the associated X and Y axis Columns in the Plot.
         */
         public void removeDataTable() {
             /*if(JOptionPane.showConfirmDialog(null, "Are you sure you want to remove this DataTable?", "Select an option", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
             {*/
              int selectedDataTableIndex = dataTableList.getSelectedIndex(); // get selected xAxis item\
              if (selectedDataTableIndex!=-1) { // if there are data tables in the List

                // ****** this part of code erases all the associated x-axes from the xAxisList
                        // this filename is for the purposes of erasing every X and Y axis in the Axis list associated with the DataTable to be erased
                        // The filename is extracted from the datatable in the dictionary, the index is from the selected datatable in the datatable List
                        DataTable selectedDataTable = (DataTable)dictionary.getElement(selectedDataTableIndex);
                        String fileName = selectedDataTable.filepath.substring(selectedDataTable.filepath.lastIndexOf("/")+1);
                        // OLD WAY: String fileName = dataTableList.getSelectedItem().substring(..6, dataTableList.getSelectedItem().indexOf(";"));
                        // Erase all the associated X and Y colums from the xAxisDataTable and yAxisDataTable
                        //  browse through the xAxis List and while there is a column that contains the title of this DataTable, erase it.
                        if (xAxisList.getItemCount() > 0) {
                          boolean noMore = false; // "noMore" means that there is no more x Axis Columns associated with this DataTable
                          int eraseIndex = -1;  // this is the index that needs to be erased
                          while (!noMore) { //means that there still might be more columns ( in the newly created (reduced) xAxisList)
                            eraseIndex = -1; //reset the eraseIndex Value
                            for (int i = 0; i<xAxisList.getItemCount(); i++) {

                              xAxisList.select(i);
                              Date checkDate = ((DataTable)xAxisDataTable.elementAt(layoutChoice.getSelectedIndex())).getColumn(i).getDate();
                              if ( selectedDataTable.getColumn(0).getDate().equals(checkDate) )  { // Note: the chosen column is 0 just to get one of the columns
                                eraseIndex = xAxisList.getSelectedIndex();
                                break;
                              }

/*
                              xAxisList.select(i);
                              // ***!!!***
                              // check if the DataTable FILENAME && TITLE are the same as in the selected x-axis and check if it is SORTED datatable or not
                              String sortedColumn = "not sorted data table";
                              if (dataTableList.getSelectedItem().indexOf(" SORTED")>-1) // this will 'capture' the String "SORTED ON *" from dataTableList Item
                                sortedColumn = dataTableList.getSelectedItem().substring(dataTableList.getSelectedItem().indexOf(" SORTED"));

                              if ((xAxisList.getSelectedItem().indexOf(fileName)!=-1) &&  // the filename
                                   (((DataTable)dictionary.getElement(dataTableList.getSelectedIndex())).getTitle().trim().equals(  // the title
                                          (Object)((PlotRecord) ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).elementAt(xAxisList.getSelectedIndex())).title.trim() )) ) {
                                // Check if the DataTable is NOT Sorted AND "SORTED ON *" is NOT in this x axis column
                                if ((sortedColumn.equals(new String("not sorted data table"))) &&  (xAxisList.getSelectedItem().indexOf("SORTED")==-1))  {
                                  eraseIndex = xAxisList.getSelectedIndex();
                                  break;
                                }
                                // Check if the DataTable is Sorted AND "SORTED ON *" is in this x axis column
                                if ((!sortedColumn.equals(new String("not sorted data table"))) && (xAxisList.getSelectedItem().indexOf(sortedColumn)!=-1)) {
                                  eraseIndex = xAxisList.getSelectedIndex();
                                  break;
                                }
                              }
*/


                            }
//System.out.println(xAxisList.getSelectedItem()+" "+fileName+" "+eraseIndex);
                            if (eraseIndex!=-1) {
//System.out.println("eraseIndex: "+eraseIndex);
                              removexAxisColumn(eraseIndex);
                              reloadxAxisList();
                              reloadyAxisList();
                            } else
                              noMore=true;
                            if (xAxisList.getItemCount() == 0) noMore=true; // this is in case the xAxisLIst empties out
                          }
                          if (xAxisList.getItemCount() > 0) // if there are still some elements, select the last one
                            xAxisList.select(xAxisList.getItemCount()-1);
                        }
            // ********* //

                        // Now remove the DataTable
                        int dataTableKey = getDataTableIndex();
                        statusTextField.setText("> DataTable " +	selectedDataTable.getTitle() + " removed.");
                        dataTableList.remove(selectedDataTableIndex);                // Remove dataTable from the List
                        dictionary.removeElement(dataTableKey);                      // Remove dataTable from the dictionary

                        // if there are no data tables left in the GUI, restart the tempDataColumns DataTable to an empty new Data Table
                        if (dictionary.size()==0) {
                          tempNames.removeAllElements();
                          tempLabels.removeAllElements();
                          tempDataColumns = new DataTable(new String(" "));
                          tempRowDescription.removeAllElements();
                        }

                        if (dataTableList.getItemCount()>0) {
                          // Since all the names of DataTables start with a number describing their order in list, shift this number down
                          //    in all the Names that are above the datatable that just got erased.
                          // Note that the first one in line is at the same place the previous datatable just got erased at.
//System.out.println("dictionary size:"+dictionary.size()+"selected index"+selectedDataTableIndex+1);
                          if (dictionary.size()>=selectedDataTableIndex+1) // if there are more datatables above
                            for (int i = 1; i<=dictionary.size()-selectedDataTableIndex; i++) {
                              String newName = dataTableList.getItem(selectedDataTableIndex+i-1); // starts from the same place where the data table was erased
                              // put the number and then add the rest of the old name (note: this code has in mind that numbers can be 1, 2, or more characters long)
                              newName = String.valueOf(selectedDataTableIndex+i)+newName.substring(newName.indexOf("."));
                              dataTableList.replaceItem(newName, selectedDataTableIndex+i-1); // add this to the same place it was taken from
//System.out.println(newName);
                            }

                          if (selectedDataTableIndex==0) dataTableList.select(0);    // if there is just one left, slect it
                          else dataTableList.select(selectedDataTableIndex-1);     // select the DataTable of order one less
                        }

                        reloadColumnList();                                          // reload Column List
               } // end of if (there is tables in the list)
         }

        /**
         * This method opens the appropriate db file. It uses the DBFileReader Class.
         * The argument that is passed is the full path name of the file directory+filename
         */
        public void loadDBFile(String dirName, String fileName) {

          statusTextField.setText("> Selected db file: "+ dirName + fileName + ". Importing data...");

//System.out.println("loaDBFile: "+dirName + fileName);

          // load the file (DataTable)
          DBFileReader dbFileReader = new DBFileReader(GUI.this, dataTableVariables, functionsTable);
          String f = "";
          try {
          f = dirName+fileName;
            if ((f == null) || (f.length() == 0)) return;
            inFile = new FileLineReader(new FileReader(f));   // Create a stream to read
            dbFileReader.loadStream(inFile, f);   // load the data from the stream/file
            // Save the filepath into the DataTable for future reference (throw out the filename first)
            //((DataTable)dictionary.getElement(dictionary.size()-1)).filepath = f;
            /*//  This code sets the security privileges for file opening
            try {
             final String dir = d.getDirectory();
             final String file = fileName;
             inFile = (FileLineReader) AccessController.doPrivileged(
               new PrivilegedExceptionAction() {
                 public Object run() throws IOException {
                   File f = new File(dir, file);           // Create a file object
                   return new FileLineReader(new FileReader(f));   // Create a stream to read
                  }
                }
              );
              dbFileReader.loadStream(inFile);  // load the data from the stream/file
            }
            catch (PrivilegedActionException pe) {
             throw (IOException) pe.getException();
            }*/

            // write that the file has been loaded. If it acttually hasn't the 'catch' below will overwrite that
            statusTextField.setText(" Done with file download.");

          } catch (IOException ioe) {
            // in console
            System.err.println("I/O Exception inFile (loadDBFile()): "+ ioe.getMessage()+"->"+inFile);
            // write this exception in the status field of the GUI
            statusTextField.setText("Error reading *.db: The file doesn't exist, the path is invalid or the file contains text errors");
            // pop alert
            popAlertDialog("Error reading from file.","  The file doesn't exist, the path is invalid or the file contains text errors.", "   Use DBDiagnostic to check .db files' validity.");
            // Display a message in the console window too
            console.jta.append("\n  *** ASK:  Cannot Open # 1 Indirect file: \n"+dirName+fileName+"\n");
          }
           finally {   // Always be sure to close the input stream!
            try { if (inFile != null) inFile.close(); }
            catch (IOException ioe) {
              System.err.println("I/O Exception DBFileReader file input stream not closed: "+ ioe.getMessage());
            }
          }
          // If the file was not found I will assume that DBFilesReader didn't manage to load Data into it's DataTable
          if (inFile!=null && (dictionary.size())!=0 && ((DataTable)dictionary.getElement(dictionary.size()-1)).columnCount()!=0 ) {

          } else {
            statusTextField.setText("Error reading db: The file doesn't exist, the path is invalid or the file contains text errors");
          }
        }


        /**
         * This method opens the appropriate XML file. It uses the DBFileReader Class.
         * The argument that is passed is the full path name of the file directory+filename
         */
        public void loadXMLFile(String dirName, String fileName) {

          statusTextField.setText("> Selected XML file: "+ dirName + fileName + ". Importing data...");

//System.out.println("loaDBFile: "+dirName + fileName);

          // load the file (DataTable)
          XMLFileReader xmlFileReader = new XMLFileReader(GUI.this, dataTableVariables, functionsTable);
          String f = "";
          try {
          f = dirName+fileName;
            if ((f == null) || (f.length() == 0)) return;
            inFile = new FileLineReader(new FileReader(f));   // Create a stream to read
            xmlFileReader.loadStream(inFile, f);   // load the data from the stream/file
            // write that the file has been loaded. If it acttually hasn't the 'catch' below will overwrite that
            statusTextField.setText(" Done with xml file download.");

          } catch (IOException ioe) {
            // in console
            System.out.println("I/O Exception inFile (loadXMLFile()): "+ ioe.getMessage()+"->"+inFile);
            // write this exception in the status field of the GUI
            statusTextField.setText("Error reading *.xml: The file doesn't exist, the path is invalid or the file contains text errors");
            // pop alert
            popAlertDialog(" ","   Error reading the file ", "   The file doesn't exist, the path is invalid or the file contains text errors");
            // Display a message in the console window too
            console.jta.append("\n  *** ASK:  Cannot Open # 1 Indirect file: \n"+dirName+fileName+"\n");
          }
           finally {   // Always be sure to close the input stream!
            try { if (inFile != null) inFile.close(); }
            catch (IOException ioe) {
              System.err.println("I/O Exception XMLFileReader file input stream not closed: "+ ioe.getMessage());
            }
          }
          // If the file was not found I will assume that DBFilesReader didn't manage to load Data into it's DataTable
          if (inFile!=null && (dictionary.size())!=0 && ((DataTable)dictionary.getElement(dictionary.size()-1)).columnCount()!=0 ) {

          } else {
            statusTextField.setText("Error reading xml: The file doesn't exist, the path is invalid or the file contains text errors");
          }
        }

        /**
         * This displays the Frame with the CaptionsTextArea in it
         */
        public void captionsFrameDisplay() {

                        // Contains settings to add or remove plot captions
                        final JFrame aFrame = new JFrame("Captions [Legends]");
                        aFrame.setBackground(Color.lightGray);
                        ScrollPane aFrameScrollbar = new ScrollPane();
                        JPanel aPanel = new JPanel();
                        aPanel.setLayout(new BorderLayout());

                        // add the Frame Label
                        JPanel instructionsPanel = new JPanel();
                        instructionsPanel.setLayout(new BorderLayout());
                        instructionsPanel.add(new JLabel("Caption format: caption; x; y; show; angle; size; color; type; style;"),BorderLayout.NORTH);
                        ActionListenerS captionExplanationListener = new ActionListenerS()
                        {
                            public void actionPerformed(ActionEvent e)
                            {
                                if(e.getActionCommand().equals("Click here for an explanation"))
                                {
                                    showCaptionsExplanation();
                                }
                            }
                        };
                        JButton explanationButton = new JButton("Click here for an explanation");
                        explanationButton.addActionListener(captionExplanationListener);
                        instructionsPanel.add(explanationButton, BorderLayout.SOUTH);
                        
                        aPanel.add(instructionsPanel, "North");
                        // Add the titleSetupPanel
                        aPanel.add(((PlotOptionsFrame)plotsOptions.
                              elementAt(layoutChoice.getSelectedIndex())).captionsTextArea, "Center");
                        Button aButton = new Button("Close");
                        aPanel.add(aButton, "South");

                        aFrameScrollbar.add(aPanel);
                        aFrame.getContentPane().add(aFrameScrollbar);

                        aFrame.setSize(700,250);
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerTitle = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Close"))
                          aFrame.dispose();
                          display();
                         }
                        };
                        aButton.addActionListener(buttonlistenerTitle);

                        aFrame.setLocation(200,200);
                        aFrame.show();
        }
        
        /**
         *Displays a window which explains the caption input system in greater detail.
         *In particular, this method explains what a caption is, what each of the terms is,
         *and what the valid range of input options is.
         */
        private void showCaptionsExplanation()
        {
            String output = "Captions are text labels on the graph area. To add a caption, specify the following parameters:\n" +
                    "Caption:\tThe text to be displayed.\n" +
                    "X:\tThe horizontal position, in pixels.\n" +
                    "Y:\tThe vertical position, in pixels.\n" +
                    "Show:\tWhether the caption is visible. Either \'true\' or \'false\'.\n" +
                    "Angle:\tThe angle at which the text is oriented, in degrees.\n" +
                    "Size:\tThe font size, in pt.\n" +
                    "Color:\tThe colour of the caption.\n" +
                    "Type:\tThe fond of the caption.\n" +
                    "Style:\tOne of \'plain\', \'bold\', or \'italic\'.\n";

            JOptionPane.showMessageDialog(null, output);
        }

  /** Returns the index of the first occurence of the object in the CONSTANT vector, if not there, return -1 */
  public int indexOfDataColumnNames(Frame parent, String name) {
    int index = -1;
//System.out.println(" ((GUI)parent).caseMenuItem.getState(): "+((GUI)parent).caseMenuItem.getState());
    for (int i=0; i<tempNames.size(); i++) {
      if ( !((GUI)parent).caseMenuItem.getState() ) {
//System.out.println(" tempNames.get(i).toString().toUpperCase(): "+tempNames.get(i).toString().toUpperCase());
        if ( tempNames.get(i).toString().toUpperCase().equals( name.toUpperCase() )  )
          index = i;
      } else {
        if ( tempNames.get(i).toString().equals( name )  )
          index = i;
      }
    }
    // if it doesn't exist, notify the user in the DBConsole
    //if (index!=-1)
     // console.jta.append("\n Note: The name:  "+name+" doesn't exist in the selected DataTable. \n");
    return index;
  }
        /**
         * This method opens the appropriate filename. file. It uses the LoadFileReader Class.
         * The argument that is passed is the full path name of the file directory+filename
         * THis is called when the '.' (command) file is opened form the GUI Menu (not the console)
         */
/*        public void loadLoadFile(String f) {

          statusTextField.setText("> Selected db file: "+ f + ". Importing data...");

          // load the file
          LoadFileReader loadFileReader = new LoadFileReader(GUI.this, dataTableVariables, functionsTable);
          try {
            if ((f == null) || (f.length() == 0)) return;
            inFile = new FileLineReader(new FileReader(f));   // Create a stream to read
            loadFileReader.loadStream(inFile);   // load the data from the stream/file   */
            /*//  This code sets the security privileges for file opening
            try {
             final String dir = d.getDirectory();
             final String file = fileName;
             inFile = (FileLineReader) AccessController.doPrivileged(
               new PrivilegedExceptionAction() {
                 public Object run() throws IOException {
                   File f = new File(dir, file);           // Create a file object
                   return new FileLineReader(new FileReader(f));   // Create a stream to read
                  }
                }
              );
              dbFileReader.loadStream(inFile);  // load the data from the stream/file
            }
            catch (PrivilegedActionException pe) {
             throw (IOException) pe.getException();
            }*/
/*
          } catch (IOException ioe) {
            System.err.println("I/O Exception inFile (loadFileReader): "+ ioe.getMessage());
            statusTextField.setText("Error: filename. was not loaded; The file doesn't exist or the path is invalid");
          }
           finally {   // Always be sure to close the input stream!
            try { if (inFile != null) inFile.close(); }
            catch (IOException ioe) {
              System.err.println("I/O Exception LoadFileReader file input stream not closed: "+ ioe.getMessage());
            }
          }
        }
*/
        // THis Functions Loads the GUI Options menu with Options that are all defined in the PlotOptionsFrame Class
        public void addOptionsToMenuBar(Vector plotsOptions, Menu optionsMenu) {

            /*titleLabelMenuItem = new MenuItem("Titles and labels");
            titleLabelMenuItem.addActionListener(menuBarListener);
            titleLabelMenuItem.setActionCommand("titlelabeloptions");
            optionsMenu.add(titleLabelMenuItem);
                    
            axesMenuItem = new MenuItem("Axes");
            axesMenuItem.addActionListener(menuBarListener);
            axesMenuItem.setActionCommand("axisoptions");
            optionsMenu.add(axesMenuItem);
            
            plotMenuItem = new MenuItem("Plot");
            plotMenuItem.addActionListener(menuBarListener);
            plotMenuItem.setActionCommand("plotoptions");
            optionsMenu.add(plotMenuItem);
             **/
                        


          // create Gloabl efaults subMenu
          //globalDefaultsMenu = new Menu("Global Defaults");
  //
//            globalDefaultsMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).detailedPowerTenMenuItem );  //

          //optionsMenu.add(globalDefaultsMenu);

          //optionsMenu.add(new MenuItem("-")); // this is a separator
          


          
          optionsTextMenu = new Menu("Text");          
                titleSetupMenuItem = new MenuItem("Title and Labels");
                titleSetupMenuItem.addActionListener(menuBarListener);
                titleSetupMenuItem.setActionCommand("titlesetup");
              optionsTextMenu.add(titleSetupMenuItem);
                captionsMenuItem = new MenuItem("Captions");
                captionsMenuItem.addActionListener(menuBarListener);
                captionsMenuItem.setActionCommand("captionsetup");
              optionsTextMenu.add(captionsMenuItem);
              optionsTextMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).labelFontMenu );
              optionsTextMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).titleFontMenu );
              optionsTextMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).numbersFontMenu );
          optionsMenu.add(optionsTextMenu);
              
          optionsAxesMenu = new Menu("Axes");
                axisTypesMenuItem = new MenuItem("Axis Types");
                axisTypesMenuItem.addActionListener(menuBarListener);
                axisTypesMenuItem.setActionCommand("axistypes");
              optionsAxesMenu.add(axisTypesMenuItem);
                limitsMenuItem = new MenuItem("Limits");
                limitsMenuItem.addActionListener(menuBarListener);
                limitsMenuItem.setActionCommand("limits");
              optionsAxesMenu.add(limitsMenuItem);
              optionsAxesMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).setupMenuA );
          optionsMenu.add(optionsAxesMenu);
              
          optionsPointsMenu = new Menu("Points");
                graphTypesMenuItem = new MenuItem("Types and Sizes");
                graphTypesMenuItem.addActionListener(menuBarListener);
                graphTypesMenuItem.setActionCommand("graphtypes");
            optionsPointsMenu.add(graphTypesMenuItem);
            optionsPointsMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).allPointsLineMenuItem );  //
            optionsPointsMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).symbolsFilledMenuItem );
          optionsMenu.add(optionsPointsMenu);
            
          optionsPlotMenu = new Menu("Plot");
            optionsPlotMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).setupMenuP );
            optionsPlotMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).setupMenuG );
            optionsPlotMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).colorMenu );
            optionsPlotMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).strictLimitsMenuItem ); 
            optionsPlotMenu.add( ((PlotOptionsFrame)plotsOptions.elementAt(layoutChoice.getSelectedIndex())).monotonicMenuItem );
                caseMenuItem = new CheckboxMenuItem("Set case-sensitive", false);
            optionsPlotMenu.add(caseMenuItem); 
          optionsMenu.add(optionsPlotMenu);
          
          optionsMenu.add(new MenuItem("-")); // this is a separator
            optionsMenuItem = new MenuItem("Status", new MenuShortcut(KeyEvent.VK_T));
            optionsMenuItem.setActionCommand("status");
            optionsMenuItem.addActionListener(menuBarListener);
            optionsMenu.add(optionsMenuItem);
      }

      /**
       * This function loads a saved GUI
       * -String loadType:
       *    If the passed loadType variable is "res", that means that the GUI will just reset itself to a SaveGUI state (which is default).
       *    If the passed loadType variable is "def", that means that the GUI will load the default.gui from the local directory.
       *    If the variable is "load", that means that some chosen saved GUI is being loaded up.
       * -boolean loadDataTable
       *    If true, it will load all teh DataTables and what is associated with them, otherwise, only the GUI options (comments) will be loaded
       *
       */
        public void loadGUI(String loadType, boolean loadDataTable, Vector plotsOptions) {
                    try {

                      String filename = "";
                      Object load_obj = new Object();
                      if (loadType=="res") {
                        load_obj = resetGUI; // save GUI contains all the default GUI settings
                      } else if (loadType=="def") {
                        load_obj = getDefaultGUI();
                        // the load_obj is null if there is no default.gui. in that case, the GUI will remain as it is

                      } else if (loadType=="load") {
                        // Open up a dialog for the user to choose the file
                        ExtensionFileFilter filter = new ExtensionFileFilter();

                        filter.addExtension("gui");
                        filter.setDescription(".gui Files");
                        dOpenGUI.setFileFilter(filter);

                        dOpenGUI.setLocation(300,300);
                        dOpenGUI.showOpenDialog(GUI.this);  // display the dialog and block until answered
                        filename = dOpenGUI.getSelectedFile().getName();   // Get the user's response
                        if (filename != null) {
                          if (filename.length() != 0) {    // If user didn't click "Cancel."
                            try {
                            // inform the user that the GUI is being loaded
                            statusTextField.setText("Preparing to load GUI... ");
                            // Create the necessary input streams to load the GUI.
                            File f = new File(dOpenGUI.getCurrentDirectory(), filename);        // Create a File object
                            ObjectInputStream in = new ObjectInputStream(
                                                    new java.util.zip.GZIPInputStream(
                                                      new FileInputStream(f)));  // Read Object.
                            load_obj = in.readObject();      // Assign Object.
                            //in.close();                 // And close the stream.
                            }
                            // Print out exceptions.
                            catch (IOException ioe) { System.out.println("I/O Exception in file loading: "+ioe); }
                          }
                        }
                      }
                      if (load_obj==null) { // this will happen in case the default.gui is being loaded
                        statusTextField.setText("MUSR GUI has been successfully loaded. (Note: file ''/t(e)mp/default.gui'' not found to load) ");
                      } else if ((load_obj instanceof SaveGUI) && (filename != null)) {
                        // inform the user that the GUI is being loaded
                        statusTextField.setText("Please wait, a GUI state is being loaded... Time Left: 6");

                        // Cast Object to a SaveGUI
                        SaveGUI saveGUI = (SaveGUI) load_obj;

                        /* Misc Objects */
                        caseMenuItem.setState(caseMenuItem.getState());

                        nPix = saveGUI.nPix;
                        Nmax = saveGUI.Nmax;

                        dirPath = saveGUI.dirPath;

                            // load RMI import directory paths and file names for musr.triumf.ca
                            openFileTextField1.setText(saveGUI.openFileTextField1.getText());
                            openFileTextField2.setText(saveGUI.openFileTextField2.getText());
                            openFileTextField3.setText(saveGUI.openFileTextField3.getText());
                            openFileTextField4.setText(saveGUI.openFileTextField4.getText());

                            // load RMI import directory paths and file names for cmms.triumf.ca
                            openCFileTextField1.setText(saveGUI.openCFileTextField1.getText());
                            openCFileTextField2.setText(saveGUI.openCFileTextField2.getText());
                            openCFileTextField3.setText(saveGUI.openCFileTextField3.getText());
                            openCFileTextField4.setText(saveGUI.openCFileTextField4.getText());

                        dataTableVariables = (Constants)(ObjectCloner.deepCopy(saveGUI.dataTableVariables));
                        functionsTable = (Functions)(ObjectCloner.deepCopy(saveGUI.functionsTable));

                            /* These are all Plot Option components */
                            for(int j=0; j<saveGUI.plotsOptions.size();j++) {
                            // Note: I copied exact code from saveGUI() so I will just 'swap' the saveOptions and currOptions, in order not to have to re-write all code
                              PlotOptionsFrame currOptions = ((PlotOptionsFrame)saveGUI.plotsOptions.get(j));
                              PlotOptionsFrame saveOptions = ((PlotOptionsFrame)plotsOptions.get(j));
                              // Load Axis Types
                              saveOptions.axisTypeX =  currOptions.axisTypeX;
                              for (int i=0; i<=2;i++) saveOptions.xTypes[i].setState(currOptions.xTypes[i].getState());
                              saveOptions.axisTypeY =  currOptions.axisTypeY;
                              for (int i=0; i<=2;i++) saveOptions.yTypes[i].setState(currOptions.yTypes[i].getState());
                              // Load Point Types and Sizes
                              // Note: groupNumberChoice and graphNumberChoice get automatically reloaded when the GUI is reloaded (no need to load them)
                              if (loadDataTable) { // These are the plot options that relate to DataTables, all the other Plot Options don't
                                saveOptions.gTypeLine.setState(currOptions.gTypeLine.getState()); // Checkbox
                                saveOptions.gTypeOtherChoice.removeAll();
                                for(int i=0; i<currOptions.gTypeOtherChoice.getItemCount(); i++) // Choice
                                  saveOptions.gTypeOtherChoice.add(currOptions.gTypeOtherChoice.getItem(i));
                                saveOptions.gTypeOtherChoice.select(currOptions.gTypeOtherChoice.getSelectedIndex());
                                saveOptions.graphSymbolSizeChoice.removeAll();
                                for(int i=0; i<currOptions.graphSymbolSizeChoice.getItemCount(); i++) // Choice
                                  saveOptions.graphSymbolSizeChoice.add(currOptions.graphSymbolSizeChoice.getItem(i));
                                saveOptions.graphSymbolSizeChoice.select(currOptions.graphSymbolSizeChoice.getSelectedIndex());
                                saveOptions.graphColorChoice.removeAll();
                                for(int i=0; i<currOptions.graphColorChoice.getItemCount(); i++) // Choice
                                  saveOptions.graphColorChoice.add(currOptions.graphColorChoice.getItem(i));
                                saveOptions.filledSymbol.setState(currOptions.filledSymbol.getState()); // Checkbox
                                saveOptions.graphColorChoice.select(currOptions.graphColorChoice.getSelectedIndex());
                                saveOptions.graphTypeLine = (Vector)(ObjectCloner.deepCopy(currOptions.graphTypeLine)); //
                                saveOptions.graphTypeOther = (Vector)(ObjectCloner.deepCopy(currOptions.graphTypeOther)); //
                                saveOptions.graphSymbolSize = (Vector)(ObjectCloner.deepCopy(currOptions.graphSymbolSize)); //
                                saveOptions.graphColor = (Vector)(ObjectCloner.deepCopy(currOptions.graphColor)); //
                                saveOptions.graphSymbolFilled = (Vector)(ObjectCloner.deepCopy(currOptions.graphSymbolFilled)); //
                              }
                              // Title and Labels
                              saveOptions.titleText.setText(currOptions.titleText.getText());
                              saveOptions.xLabelText.setText(currOptions.xLabelText.getText());
                              saveOptions.yLabelText.setText(currOptions.yLabelText.getText());
                              // Limits (Min & Max)
                              saveOptions.xMinText.setText(currOptions.xMinText.getText());
                              saveOptions.yMinText.setText(currOptions.yMinText.getText());
                              saveOptions.xMaxText.setText(currOptions.xMaxText.getText());
                              saveOptions.yMaxText.setText(currOptions.yMaxText.getText());
                              // Strict Limits
                              saveOptions.strictLimitsMenuItem.setState(currOptions.strictLimitsMenuItem.getState());
                              saveOptions.monotonicMenuItem.setState(currOptions.monotonicMenuItem.getState());
                              saveOptions.allPointsLineMenuItem.setState(currOptions.allPointsLineMenuItem.getState());
                              saveOptions.detailedPowerTenMenuItem.setState(currOptions.detailedPowerTenMenuItem.getState());
                              saveOptions.symbolsFilledMenuItem.setState(currOptions.symbolsFilledMenuItem.getState());
                              // Caption TextField
                              saveOptions.captionsTextArea = currOptions.captionsTextArea;
                              // Color Options
                              saveOptions.backgroundMenu.setLabel(currOptions.backgroundMenu.getLabel()); // Menu Label
                              saveOptions.borderlineMenu.setLabel(currOptions.borderlineMenu.getLabel()); // Menu Label
                              saveOptions.backgroundColor = currOptions.backgroundColor;
                              saveOptions.borderlineColor = currOptions.borderlineColor;
                              // Label Font
                              saveOptions.labelTypeMenu.setLabel(currOptions.labelTypeMenu.getLabel()); // Menu Label
                              saveOptions.labelStyleMenu.setLabel(currOptions.labelStyleMenu.getLabel()); // Menu Label
                              saveOptions.labelSizeMenu.setLabel(currOptions.labelSizeMenu.getLabel()); // Menu Label
                              saveOptions.labelFontType = currOptions.labelFontType;
                              saveOptions.labelFontStyle = currOptions.labelFontStyle;
                              saveOptions.labelFontSize = currOptions.labelFontSize;
                              // Title Font
                              saveOptions.titleTypeMenu.setLabel(currOptions.titleTypeMenu.getLabel()); // Menu Label
                              saveOptions.titleStyleMenu.setLabel(currOptions.titleStyleMenu.getLabel()); // Menu Label
                              saveOptions.titleSizeMenu.setLabel(currOptions.titleSizeMenu.getLabel()); // Menu Label
                              saveOptions.titleFontType = currOptions.titleFontType;
                              saveOptions.titleFontStyle = currOptions.titleFontSize;
                              saveOptions.titleFontSize = currOptions.titleFontSize;
                              saveOptions.adjustToFitMenuItem.setState(currOptions.adjustToFitMenuItem.getState());
                              // Numbers Font
                              saveOptions.numbersTypeMenu.setLabel(currOptions.numbersTypeMenu.getLabel()); // Menu Label
                              saveOptions.numbersStyleMenu.setLabel(currOptions.numbersStyleMenu.getLabel()); // Menu Label
                              saveOptions.numbersSizeMenu.setLabel(currOptions.numbersSizeMenu.getLabel()); // Menu Label
                              saveOptions.numbersFontType = currOptions.numbersFontType;
                              saveOptions.numbersFontStyle = currOptions.numbersFontSize;
                              saveOptions.numbersFontSize = currOptions.numbersFontSize;
                              // Plot Setup (pix)
                              saveOptions.upDistance = currOptions.upDistance;
                              saveOptions.downDistance = currOptions.downDistance;
                              saveOptions.leftDistance = currOptions.leftDistance;
                              saveOptions.rightDistance = currOptions.rightDistance;
                              saveOptions.setupMenuP.remove(0);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(0).getLabel(), 0);
                              saveOptions.setupMenuP.remove(1);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(1).getLabel(), 1);
                              saveOptions.setupMenuP.remove(2);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(2).getLabel(), 2);
                              saveOptions.setupMenuP.remove(3);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(3).getLabel(), 3);
                              // Graph Setup
                              saveOptions.errorBarsMenuItem.setState(currOptions.errorBarsMenuItem.getState());
                              saveOptions.errorBarTicsMenuItem.setState(currOptions.errorBarTicsMenuItem.getState());
                              saveOptions.dotFiducialsMenuItem.setState(currOptions.dotFiducialsMenuItem.getState());
                              // Zero Axes Setup
                              saveOptions.xZeroAxisMenuItem.setState(currOptions.xZeroAxisMenuItem.getState());
                              saveOptions.yZeroAxisMenuItem.setState(currOptions.yZeroAxisMenuItem.getState());
                              // Status Info
                              saveOptions.backgroundColorLabel.setText(currOptions.backgroundColorLabel.getText());
                              saveOptions.borderlineColorLabel.setText(currOptions.borderlineColorLabel.getText());
                              saveOptions.labelTypeLabel.setText(currOptions.labelTypeLabel.getText());
                              saveOptions.labelStyleLabel.setText(currOptions.labelStyleLabel.getText());
                              saveOptions.labelSizeLabel.setText(currOptions.labelSizeLabel.getText());
                              saveOptions.titleTypeLabel.setText(currOptions.titleTypeLabel.getText());
                              saveOptions.titleStyleLabel.setText(currOptions.titleStyleLabel.getText());
                              saveOptions.titleSizeLabel.setText(currOptions.titleSizeLabel.getText());
                              saveOptions.numbersTypeLabel.setText(currOptions.numbersTypeLabel.getText());
                              saveOptions.numbersStyleLabel.setText(currOptions.numbersStyleLabel.getText());
                              saveOptions.numbersSizeLabel.setText(currOptions.numbersSizeLabel.getText());

                            }
                        /* The Frames Window Components*/
                        // load the Checkboxes
                        if (loadDataTable) { // These are: Frames, DataTables and Columns
                          gridAutoCheckbox.setState(saveGUI.gridAutoCheckbox.getState());
                          gridManualCheckbox.setState(saveGUI.gridManualCheckbox.getState());
                          visibleCheckbox.setState(saveGUI.visibleCheckbox.getState());
                          invertedCheckbox.setState(saveGUI.invertedCheckbox.getState());
                          // load the Choices
                          gridXChoice.removeAll();
                          for(int i=0; i<saveGUI.gridXChoice.getItemCount(); i++)
                            gridXChoice.add(saveGUI.gridXChoice.getItem(i));
                          gridXChoice.select(1);
                          gridYChoice.removeAll();
                          for(int i=0; i<saveGUI.gridYChoice.getItemCount(); i++)
                            gridYChoice.add(saveGUI.gridYChoice.getItem(i));
                          gridYChoice.select(1);
                          layoutChoice.removeAll();
                          for(int i=0; i<saveGUI.layoutChoice.getItemCount(); i++)
                            layoutChoice.add(saveGUI.layoutChoice.getItem(i));
                          layoutChoice.select(0);

                          // inform the user that the GUI is being loaded
                          statusTextField.setText("Please wait, a GUI state is being loaded... Time Left: 5, (loading inner Objects)...");

                          /* Data tables */
                          dictionary = (Dictionary)(ObjectCloner.deepCopy(saveGUI.dictionary));
                          // Loading the DataTable Names and their columns into the GUI lists
                          dataTableList.removeAll();
                          for(int i=0; i<saveGUI.dataTableList.getItemCount(); i++)
                                dataTableList.add(saveGUI.dataTableList.getItem(i));
                          dataTableList.select(dataTableList.getItemCount()-1);                // Select it
                          reloadColumnList();

                          // inform the user that the GUI is being loaded
                          statusTextField.setText("Please wait, a GUI state is being loaded... Time Left: 4, (loading User Data Tables)...");

                          /* User DataTable Plotting Columns */
                          xAxisDataTable = (Vector)(ObjectCloner.deepCopy(saveGUI.xAxisDataTable));
                          titlexAxis = (Vector)(ObjectCloner.deepCopy(saveGUI.titlexAxis));
                          namexAxis = (Vector)(ObjectCloner.deepCopy(saveGUI.namexAxis));
                          plotsRecord = (Vector)(ObjectCloner.deepCopy(saveGUI.plotsRecord));
                          yAxisDataTable = (Vector)(ObjectCloner.deepCopy(saveGUI.yAxisDataTable));
                          titleyAxis = (Vector)(ObjectCloner.deepCopy(saveGUI.titleyAxis));
                          reloadxAxisList();
                          reloadyAxisList();

                        }


                            //** MUD Variables */
                            for(int i=0; i<saveGUI.runsList.getItemCount(); i++)
                              mudSpectFrame.runsList.add(saveGUI.runsList.getItem(i));
                            mudSpectFrame.histNumberTextField.setText(saveGUI.histNumberTextField.getText());
                            mudSpectFrame.firstBinTextField.setText(saveGUI.firstBinTextField.getText());
                            mudSpectFrame.lastBinTextField.setText(saveGUI.lastBinTextField.getText());
                            mudSpectFrame.packingTextField.setText(saveGUI.packingTextField.getText());
                            mudSpectFrame.firstLastChoice.select(saveGUI.firstLastChoiceIndex);
                            mudSpectFrame.packingChoice.select(saveGUI.packingChoiceIndex);
                            //for (mudSpectFrame.
                            for (int j=0; j<saveGUI.bkgd1Text.size();j++)
                              ((TextField)mudSpectFrame.bkgd1Text.get(j)).setText(((TextField)saveGUI.bkgd1Text.get(j)).getText());
                            for (int j=0; j<saveGUI.bkgd2Text.size();j++)
                              ((TextField)mudSpectFrame.bkgd2Text.get(j)).setText(((TextField)saveGUI.bkgd2Text.get(j)).getText());
                            for (int j=0; j<saveGUI.overrideNT0.size();j++)
                              ((TextField)mudSpectFrame.overrideNT0.get(j)).setText(((TextField)saveGUI.overrideNT0.get(j)).getText());
                            for (int j=0; j<saveGUI.overrideNT1.size();j++)
                              ((TextField)mudSpectFrame.overrideNT1.get(j)).setText(((TextField)saveGUI.overrideNT1.get(j)).getText());
                            for (int j=0; j<saveGUI.overrideNT2.size();j++)
                              ((TextField)mudSpectFrame.overrideNT2.get(j)).setText(((TextField)saveGUI.overrideNT2.get(j)).getText());
                            mudSpectFrame.spectrumChoice.select(saveGUI.spectrumChoiceIndex);
                            // Complex spectrum window
                            for(int i=0; i<saveGUI.useSpectrum.getItemCount(); i++)
                              mudSpectFrame.useSpectrum.add(saveGUI.useSpectrum.getItem(i));
                            mudSpectFrame.useSpectrum.select(saveGUI.useSpectrum.getSelectedIndex());
                            mudSpectFrame.RRFFreq.setText(saveGUI.RRFFreq.getText());
                            mudSpectFrame.RRFPackBin.setText(saveGUI.RRFPackBin.getText());
                            for(int i=0; i<saveGUI.RRFMenu.getItemCount(); i++)
                              mudSpectFrame.RRFMenu.add(saveGUI.RRFMenu.getItem(i));
                            mudSpectFrame.RRFMenu.select(saveGUI.RRFMenu.getSelectedIndex());
                //            for (int j=0; j<saveGUI.phaseRRF.size();j++)
                //              ((TextField)mudSpectFrame.phaseRRF.get(j)).setText(((TextField)saveGUI.phaseRRF.get(j)).getText());

                        // inform the user that the GUI is being loaded
                        statusTextField.setText("Please wait, a GUI state is being loaded... Time Left: 1, (loading Item Listeners)...");
                        //* Extra clean-up
                        // Clear the Vector with all the Plot2D-s in it (if any)
                        plots.clear();
                        // Also clear the PlotFrame from all the plots in it
                        plotFrame.clearPlotFrame();

                        // Refresh:
                        // create new Display Options and display them
                        refreshDisplayOptions(xAxisList.getSelectedIndex());
                        // Send these values to obtain a symbol size for this column
                        // Note; this method will reset axis values in PlotOptions, these have already been saved in the PlotoptionsFrame Object
                     //   refreshNumberOfGraphs(nPix, columnList.getSelectedIndex());  // refresh the PlotOptionsFrame
                        // Reload the plot with the new values if it the plotting frame is open.
                        if (plotFrame.isVisible()) {
                          display();
                        }

                        statusTextField.setText("MUSR GUI has been successfully loaded.");
                      } else {
                        statusTextField.setText("The file \""+filename+"\" is not a standard GUI file or it is from an old version.");
                      }
                      } catch (Exception ce) {
                               System.err.println(ce.getMessage());
                      }
        }

        /**
         * This method is used in the 'Open GUI' menu item to open up a saved GUI state
         */
        public void saveGUI() {
                      String filename = "";
                      try {

                      FileDialog d = new FileDialog(GUI.this, "Save GUI State", FileDialog.SAVE);
                      if (System.getProperty("os.name").indexOf("Windows") != -1)  // it is in windows os
                        d.setDirectory("c:\\temp\\");
                      else // it is in unix or mac os x
                        d.setDirectory("/tmp/");
                      d.setFile("default.gui");
                      d.setLocation(300,300);
                      d.show();  // display the dialog and block until answered
                      filename = d.getFile();   // Get the user's response
                      if (filename != null) {           // If user didn't click "Cancel."
                          try {
                            // inform the user that the GUI is in the process of being saved
                            statusTextField.setText("Please wait, the GUI state is being saved... Time Left: 5");

                            SaveGUI saveGUI = createSaveGUI();


                            // inform the user that the GUI is in the process of being saved
                            statusTextField.setText("Please wait, the GUI state is being saved... Time Left: 3");

                            // inform the user that the GUI is in the process of being saved
                            statusTextField.setText("Please wait, the GUI state is being saved... Time Left: 1");

                            //// saveGUI.plots = (Vector)(ObjectCloner.deepCopy(plots));

                            // Create the necessary output streams to save the Object.
                            File f = new File(d.getDirectory(), filename);        // Create a file object
                            ObjectOutputStream out = new ObjectOutputStream(
                                                      new java.util.zip.GZIPOutputStream(
                                                        new FileOutputStream(f)));  // Save objects
                            out.writeObject((SaveGUI)saveGUI);      // Write the entire Object.
                            out.flush();                 // Always flush the output.
                            out.close();                 // And close the stream.

                            // Also, at the same time save as default (regardless of what the user saved it as)
                            // so that it can be loaded again the next time GUI is opened.
/*
                            ObjectOutputStream outd = new ObjectOutputStream(
                                                      new java.util.zip.GZIPOutputStream(
                                                        new FileOutputStream("default.gui")));  // Save objects
                            outd.writeObject((SaveGUI)saveGUI);      // Write the entire Object.
                            outd.flush();                 // Always flush the output.
                            outd.close();                 // And close the stream.
*/
                          }

                          // Print out exceptions.
                          catch (IOException ioe) { System.out.println("I/O Exception in file writing: "+ioe); }
                        }
                      } catch (Exception ce) {
                               System.err.println(ce.getMessage());
                      }
                      if (filename != null)
                        statusTextField.setText("The GUI state has been saved into the file: "+filename);
                      else statusTextField.setText("");
        }


        /**
         * This method Creates a SaveGUI Object that is identical to the current state of the GUI
         * It returns the SaveGUI Object
         * note: this function is called when user wants to save a GUI, and when the GUI is first opened to save a copy of resetGUI Object
         */
         public SaveGUI createSaveGUI() {
                            SaveGUI saveGUI = new SaveGUI();
             try {

                            /* Misc Objects */
                            saveGUI.nPix = nPix;
                            saveGUI.Nmax = Nmax;
                            saveGUI.dirPath = dirPath;
                            // save the Constants table
                            saveGUI.dataTableVariables = (Constants)(ObjectCloner.deepCopy(dataTableVariables));
                            // save the Functions table
                            saveGUI.functionsTable = (Functions)(ObjectCloner.deepCopy(functionsTable));

                            // save RMI import directory paths and file names for musr.triumf.ca
                            saveGUI.openFileTextField1.setText(openFileTextField1.getText());
                            saveGUI.openFileTextField2.setText(openFileTextField2.getText());
                            saveGUI.openFileTextField3.setText(openFileTextField3.getText());
                            saveGUI.openFileTextField4.setText(openFileTextField4.getText());

                            // save RMI import directory paths and file names for cmms.triumf.ca
                            saveGUI.openCFileTextField1.setText(openCFileTextField1.getText());
                            saveGUI.openCFileTextField2.setText(openCFileTextField2.getText());
                            saveGUI.openCFileTextField3.setText(openCFileTextField3.getText());
                            saveGUI.openCFileTextField4.setText(openCFileTextField4.getText());

                            /* These are all Plot Option components */
                            // First create a PlotOptionsFrame for every Frame
                            for(int i=0; i<plotsOptions.size();i++)
                              saveGUI.plotsOptions.addElement(new PlotOptionsFrame());
                            // Traverse through all the Frames (every frame has it's own PlotOptionsFrame)
                            for(int j=0; j<plotsOptions.size();j++) {
                              PlotOptionsFrame saveOptions = ((PlotOptionsFrame)saveGUI.plotsOptions.get(j));
                              PlotOptionsFrame currOptions = ((PlotOptionsFrame)plotsOptions.get(j));
                              // Save Axis Types
                              saveOptions.axisTypeX =  currOptions.axisTypeX;
                              for (int i=0; i<=2;i++) saveOptions.xTypes[i].setState(currOptions.xTypes[i].getState());
                              saveOptions.axisTypeY =  currOptions.axisTypeY;
                              for (int i=0; i<=2;i++) saveOptions.yTypes[i].setState(currOptions.yTypes[i].getState());
                              // Save Point Types and Sizes
                              // Note: groupNumberChoice and graphNumberChoice get automatically reloaded when the GUI is reloaded (no need to save them)
                              saveOptions.gTypeLine.setState(currOptions.gTypeLine.getState()); // Checkbox
                              for(int i=0; i<currOptions.gTypeOtherChoice.getItemCount(); i++) // Choice
                                saveOptions.gTypeOtherChoice.add(currOptions.gTypeOtherChoice.getItem(i));
                              saveOptions.gTypeOtherChoice.select(currOptions.gTypeOtherChoice.getSelectedIndex());
                              for(int i=0; i<currOptions.graphSymbolSizeChoice.getItemCount(); i++) // Choice
                                saveOptions.graphSymbolSizeChoice.add(currOptions.graphSymbolSizeChoice.getItem(i));
                              saveOptions.graphSymbolSizeChoice.select(currOptions.graphSymbolSizeChoice.getSelectedIndex());
                              for(int i=0; i<currOptions.graphColorChoice.getItemCount(); i++) // Choice
                                saveOptions.graphColorChoice.add(currOptions.graphColorChoice.getItem(i));
                              saveOptions.filledSymbol.setState(currOptions.filledSymbol.getState()); // Checkbox
                              saveOptions.graphColorChoice.select(currOptions.graphColorChoice.getSelectedIndex());
                              saveOptions.graphTypeLine = (Vector)(ObjectCloner.deepCopy(currOptions.graphTypeLine)); //
                              saveOptions.graphTypeOther = (Vector)(ObjectCloner.deepCopy(currOptions.graphTypeOther)); //
                              saveOptions.graphSymbolSize = (Vector)(ObjectCloner.deepCopy(currOptions.graphSymbolSize)); //
                              saveOptions.graphColor = (Vector)(ObjectCloner.deepCopy(currOptions.graphColor)); //
                              saveOptions.graphSymbolFilled = (Vector)(ObjectCloner.deepCopy(currOptions.graphSymbolFilled)); //
                              // Title and Labels
                              saveOptions.titleText.setText(currOptions.titleText.getText());
                              saveOptions.xLabelText.setText(currOptions.xLabelText.getText());
                              saveOptions.yLabelText.setText(currOptions.yLabelText.getText());
                              // Limits (Min & Max)
                              saveOptions.xMinText.setText(currOptions.xMinText.getText());
                              saveOptions.yMinText.setText(currOptions.yMinText.getText());
                              saveOptions.xMaxText.setText(currOptions.xMaxText.getText());
                              saveOptions.yMaxText.setText(currOptions.yMaxText.getText());
                              // Strict Limits
                              saveOptions.strictLimitsMenuItem.setState(currOptions.strictLimitsMenuItem.getState());
                              saveOptions.monotonicMenuItem.setState(currOptions.monotonicMenuItem.getState());
                              saveOptions.allPointsLineMenuItem.setState(currOptions.allPointsLineMenuItem.getState());
                              saveOptions.detailedPowerTenMenuItem.setState(currOptions.detailedPowerTenMenuItem.getState());
                              saveOptions.symbolsFilledMenuItem.setState(currOptions.symbolsFilledMenuItem.getState());
                              // Caption TextField
                              saveOptions.captionsTextArea = currOptions.captionsTextArea;
                              // Color Options
                              saveOptions.backgroundMenu.setLabel(currOptions.backgroundMenu.getLabel()); // Menu Label
                              saveOptions.borderlineMenu.setLabel(currOptions.borderlineMenu.getLabel()); // Menu Label
                              saveOptions.backgroundColor = currOptions.backgroundColor;
                              saveOptions.borderlineColor = currOptions.borderlineColor;
                              // Label Font
                              saveOptions.labelTypeMenu.setLabel(currOptions.labelTypeMenu.getLabel()); // Menu Label
                              saveOptions.labelStyleMenu.setLabel(currOptions.labelStyleMenu.getLabel()); // Menu Label
                              saveOptions.labelSizeMenu.setLabel(currOptions.labelSizeMenu.getLabel()); // Menu Label
                              saveOptions.labelFontType = currOptions.labelFontType;
                              saveOptions.labelFontStyle = currOptions.labelFontStyle;
                              saveOptions.labelFontSize = currOptions.labelFontSize;
                              // Title Font
                              saveOptions.titleTypeMenu.setLabel(currOptions.titleTypeMenu.getLabel()); // Menu Label
                              saveOptions.titleStyleMenu.setLabel(currOptions.titleStyleMenu.getLabel()); // Menu Label
                              saveOptions.titleSizeMenu.setLabel(currOptions.titleSizeMenu.getLabel()); // Menu Label
                              saveOptions.titleFontType = currOptions.titleFontType;
                              saveOptions.titleFontStyle = currOptions.titleFontSize;
                              saveOptions.titleFontSize = currOptions.titleFontSize;
                              saveOptions.adjustToFitMenuItem.setState(currOptions.adjustToFitMenuItem.getState());
                              // Numbers Font
                              saveOptions.numbersTypeMenu.setLabel(currOptions.numbersTypeMenu.getLabel()); // Menu Label
                              saveOptions.numbersStyleMenu.setLabel(currOptions.numbersStyleMenu.getLabel()); // Menu Label
                              saveOptions.numbersSizeMenu.setLabel(currOptions.numbersSizeMenu.getLabel()); // Menu Label
                              saveOptions.numbersFontType = currOptions.numbersFontType;
                              saveOptions.numbersFontStyle = currOptions.numbersFontSize;
                              saveOptions.numbersFontSize = currOptions.numbersFontSize;
                              // Plot Setup (pix)
                              saveOptions.upDistance = currOptions.upDistance;
                              saveOptions.downDistance = currOptions.downDistance;
                              saveOptions.leftDistance = currOptions.leftDistance;
                              saveOptions.rightDistance = currOptions.rightDistance;
                              saveOptions.setupMenuP.remove(0);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(0).getLabel(), 0);
                              saveOptions.setupMenuP.remove(1);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(1).getLabel(), 1);
                              saveOptions.setupMenuP.remove(2);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(2).getLabel(), 2);
                              saveOptions.setupMenuP.remove(3);
                              saveOptions.setupMenuP.insert(currOptions.setupMenuP.getItem(3).getLabel(), 3);
                              // Graph Setup
                              saveOptions.errorBarsMenuItem.setState(currOptions.errorBarsMenuItem.getState());
                              saveOptions.errorBarTicsMenuItem.setState(currOptions.errorBarTicsMenuItem.getState());
                              saveOptions.dotFiducialsMenuItem.setState(currOptions.dotFiducialsMenuItem.getState());
                              // Zero Axes Setup
                              saveOptions.xZeroAxisMenuItem.setState(currOptions.xZeroAxisMenuItem.getState());
                              saveOptions.yZeroAxisMenuItem.setState(currOptions.yZeroAxisMenuItem.getState());
                              // Status Info
                              saveOptions.backgroundColorLabel.setText(currOptions.backgroundColorLabel.getText());
                              saveOptions.borderlineColorLabel.setText(currOptions.borderlineColorLabel.getText());
                              saveOptions.labelTypeLabel.setText(currOptions.labelTypeLabel.getText());
                              saveOptions.labelStyleLabel.setText(currOptions.labelStyleLabel.getText());
                              saveOptions.labelSizeLabel.setText(currOptions.labelSizeLabel.getText());
                              saveOptions.titleTypeLabel.setText(currOptions.titleTypeLabel.getText());
                              saveOptions.titleStyleLabel.setText(currOptions.titleStyleLabel.getText());
                              saveOptions.titleSizeLabel.setText(currOptions.titleSizeLabel.getText());
                              saveOptions.numbersTypeLabel.setText(currOptions.numbersTypeLabel.getText());
                              saveOptions.numbersStyleLabel.setText(currOptions.numbersStyleLabel.getText());
                              saveOptions.numbersSizeLabel.setText(currOptions.numbersSizeLabel.getText());

                            }

                            /* The Frames Window Components*/
                            // save the Checkboxes
                            saveGUI.gridAutoCheckbox.setState(gridAutoCheckbox.getState());
                            saveGUI.gridManualCheckbox.setState(gridManualCheckbox.getState());
                            saveGUI.visibleCheckbox.setState(visibleCheckbox.getState());
                            saveGUI.invertedCheckbox.setState(invertedCheckbox.getState());
                            // save the Choices
                            for(int i=0; i<gridXChoice.getItemCount(); i++)
                              saveGUI.gridXChoice.add(gridXChoice.getItem(i));
                            for(int i=0; i<gridYChoice.getItemCount(); i++)
                              saveGUI.gridYChoice.add(gridYChoice.getItem(i));
                            for(int i=0; i<layoutChoice.getItemCount(); i++)
                              saveGUI.layoutChoice.add(layoutChoice.getItem(i));

                            /* Data tables */
                            // duplicate the dictionary (which contains all the Data Tables)
                            saveGUI.dictionary = (Dictionary)(ObjectCloner.deepCopy(dictionary));
                            // save the Lists
                            for(int i=0; i<dataTableList.getItemCount(); i++)
                              saveGUI.dataTableList.add(dataTableList.getItem(i));
                            ////saveGUI.columnList = (List)(ObjectCloner.deepCopy(columnList)); // this be restored in the GUI with the method reloadColumnList();

                            // inform the user that the GUI is in the process of being saved
                            statusTextField.setText("Please wait, the GUI state is being saved... Time Left: 4");

                            /* User DataTable Plotting Columns */
                            saveGUI.xAxisDataTable = (Vector)(ObjectCloner.deepCopy(xAxisDataTable)); // x Axis Data
                            saveGUI.plotsRecord = (Vector)(ObjectCloner.deepCopy(plotsRecord));
                            saveGUI.titlexAxis = (Vector)(ObjectCloner.deepCopy(titlexAxis));
                            saveGUI.namexAxis = (Vector)(ObjectCloner.deepCopy(namexAxis));
                            saveGUI.yAxisDataTable = (Vector)(ObjectCloner.deepCopy(yAxisDataTable)); // y Axis Data
                            saveGUI.titleyAxis = (Vector)(ObjectCloner.deepCopy(titleyAxis));
                            ////saveGUI.xAxisList = (List)(ObjectCloner.deepCopy(xAxisList)); // this be restored in the GUI with the method reloadxAxisList();
                            ////saveGUI.yAxisList = (List)(ObjectCloner.deepCopy(yAxisList)); // this be restored in the GUI with the method reloadyAxisList();

                            //** MUD Variables */
                            for(int i=0; i<mudSpectFrame.runsList.getItemCount(); i++)
                              saveGUI.runsList.add(mudSpectFrame.runsList.getItem(i));
                            saveGUI.histNumberTextField.setText(mudSpectFrame.histNumberTextField.getText());
                            saveGUI.firstBinTextField.setText(mudSpectFrame.firstBinTextField.getText());
                            saveGUI.lastBinTextField.setText(mudSpectFrame.lastBinTextField.getText());
                            saveGUI.packingTextField.setText(mudSpectFrame.packingTextField.getText());
                            saveGUI.firstLastChoiceIndex = mudSpectFrame.firstLastChoice.getSelectedIndex();
                            saveGUI.packingChoiceIndex = mudSpectFrame.packingChoice.getSelectedIndex();
                            for (int j=0; j<mudSpectFrame.bkgd1Text.size();j++)
                              ((TextField)saveGUI.bkgd1Text.get(j)).setText(((TextField)mudSpectFrame.bkgd1Text.get(j)).getText());
                            for (int j=0; j<mudSpectFrame.bkgd2Text.size();j++)
                              ((TextField)saveGUI.bkgd2Text.get(j)).setText(((TextField)mudSpectFrame.bkgd2Text.get(j)).getText());
                            for (int j=0; j<mudSpectFrame.overrideNT0.size();j++)
                              ((TextField)saveGUI.overrideNT0.get(j)).setText(((TextField)mudSpectFrame.overrideNT0.get(j)).getText());
                            for (int j=0; j<mudSpectFrame.overrideNT1.size();j++)
                              ((TextField)saveGUI.overrideNT1.get(j)).setText(((TextField)mudSpectFrame.overrideNT1.get(j)).getText());
                            for (int j=0; j<mudSpectFrame.overrideNT2.size();j++)
                              ((TextField)saveGUI.overrideNT2.get(j)).setText(((TextField)mudSpectFrame.overrideNT2.get(j)).getText());
                            saveGUI.spectrumChoiceIndex = mudSpectFrame.spectrumChoice.getSelectedIndex();
                            // Complex spectrum window
                            for(int i=0; i<mudSpectFrame.useSpectrum.getItemCount(); i++)
                              saveGUI.useSpectrum.add(mudSpectFrame.useSpectrum.getItem(i));
                            saveGUI.useSpectrum.select(mudSpectFrame.useSpectrum.getSelectedIndex());
                            saveGUI.RRFFreq.setText(mudSpectFrame.RRFFreq.getText());
                            saveGUI.RRFPackBin.setText(mudSpectFrame.RRFPackBin.getText());
                            for(int i=0; i<mudSpectFrame.RRFMenu.getItemCount(); i++)
                              saveGUI.RRFMenu.add(mudSpectFrame.RRFMenu.getItem(i));
                            saveGUI.RRFMenu.select(mudSpectFrame.RRFMenu.getSelectedIndex());
             //               for (int j=0; j<mudSpectFrame.phaseRRF.size();j++)
             //                 ((TextField)saveGUI.phaseRRF.get(j)).setText(((TextField)mudSpectFrame.phaseRRF.get(j)).getText());

             }
               catch (Exception e) { System.out.println("Exception when creating SaveGUI Object: "+e); }

            return saveGUI;
         }

         /**
          * This method gets the index of a DataTable in the Dictionary
          * the reason there is a separate method for this is because, sometimes, the DataTable is not selected so this method takes care of that
          */
          public int getDataTableIndex() {
            int dataTableKey = dictionary.getKey(dataTableList.getSelectedIndex());
            // Note, this will not happen from user input, sometimes when a new datatable is created, it doesn't get selected even when it is requested in the code
            if (dataTableKey == -1 && dataTableList.getItemCount()>0) dataTableKey = dictionary.getKey(dataTableList.getItemCount()-1);
            return dataTableKey;
          }

        /**
         * This method is used to get the default.gui object that should be stored (by default) at /tmp/ (unix) or /temp (windows)
         */
        public SaveGUI getDefaultGUI() {
                     SaveGUI load_obj = null;
                     try {
                        // Support multiple operating systems
                        // Save path as a File() to preserve the folder separators for Window and Unix
                        // Check for operating system (from Properties)
                        ObjectInputStream in =null;
                        if (System.getProperty("os.name").indexOf("Windows") != -1) { // it is in windows os
                          // test if the default file exists
                          File openFile = new File("/temp/default.gui");
                          if (openFile.exists())
                            in = new ObjectInputStream(
                                                  new java.util.zip.GZIPInputStream(
                                                    new FileInputStream(openFile)));  // Read Object (Windows).
                          //else System.out.println("Note: " + openFile.getPath() + " doesn't exist.");
                        } else { // it is in unix
                          // test if the default file exists
                          File openFile = new File("/tmp/default.gui");
                          if (openFile.exists())
                            in = new ObjectInputStream(
                                                  new java.util.zip.GZIPInputStream(
                                                    new FileInputStream(openFile)));  // Read Object (unix).
                          //else System.out.println("Note: " + openFile.getPath() + " doesn't exist.");
                        }
                        // if the file doesn't exist, make an attempt to vreate a new default GUI file and save in the tmp directory
                        if (in != null)
                          // Assign Object.
                          load_obj = (SaveGUI)in.readObject();
                      }  catch (Exception ioe) { System.out.println("I/O Exception in tmp/default.gui file writing: "+ioe); }
                    return load_obj;
        }

        /**
         * This method is used to save the default.gui object at /tmp/ (unix) or /temp (windows)
         */
        public void saveDefaultGUI(SaveGUI load_obj) {
                     try {
                        ObjectOutputStream out = null;
                        if (System.getProperty("os.name").indexOf("Windows") != -1) { // it is in windows os
                            // Create the necessary output streams to save the Object.
                            out = new ObjectOutputStream(
                                                      new java.util.zip.GZIPOutputStream(
                                                        new FileOutputStream(new File("/temp/default.gui"))));  // Save objects
                        } else { // it is in unix
                            // Create the necessary output streams to save the Object.
                             out = new ObjectOutputStream(
                                                      new java.util.zip.GZIPOutputStream(
                                                        new FileOutputStream(new File("/tmp/default.gui"))));  // Save objects
                       }
                            out.writeObject((SaveGUI)load_obj);      // Write the entire Object.
                            out.flush();                 // Always flush the output.
                            out.close();                 // And close the stream.
                      }  catch (Exception ioe) { System.out.println("I/O Exception in tmp/default.gui file writing: "+ioe); }
        }

        /**
         * This method is used to make a new datatable that has all it's columns sorted refering to a reference column, in given datatable
         * it is called from the DBConsole using "sort <datatablenumber> <columnname
         */
        public void sortDataTable(DataTable table, int columnNumber) {
          try {
            // This is the datatable that will be sorted
            DataTable newSortedDataTable = null; // create a new DataTable
            // Now, sort the DataTable
            Sort sort = new Sort(table, columnNumber); // this initiates the sorting (by creating a reference Vector with 'counter' values)
            newSortedDataTable = sort.sortAll();

            // Add the dataTable into the GUI
            addDataTable(newSortedDataTable, true, newSortedDataTable.getColumn(columnNumber).getName());
          } catch (Exception e) {
            System.out.println("Exception in adding sorted DataTable into the GUI: "+e);
            // pop alert
            popAlertDialog(" ","   Error in adding sorted DataTable into the GUI ", e.getMessage() );
          }
        }

        /**
         * This method creates temproary vectors filled with errors for use in expression evaluations
         *  The references from expressions for the errors are:   ABC;POSERR, ABC;POSERR,  ABC;ERR
         */
        public void fillTempErrors(DataTable table) {
         if (table!=null) {
          tempPosErr.removeAllElements();
          tempPosErr.clear();
          tempNegErr.removeAllElements();
          tempNegErr.clear();
          tempErr.removeAllElements();
          tempErr.clear();
          // fill the Error Vectors with all the errors
          for (int i=0; i<table.columnCount(); i++) {
           tempPosErr.add(new ConcreteDataColumn());
           tempNegErr.add(new ConcreteDataColumn());
           tempErr.add(new ConcreteDataColumn());
           for ( int j=0; j<((DataColumn)table.getColumn(i)).getSize(); j++ ) {
             Double posErr =(Double) ((DataColumn)table.getColumn(i)).getErrorPlus(j);
             Double negErr = (Double) ((DataColumn)table.getColumn(i)).getErrorMinus(j);
             ((ConcreteDataColumn) tempPosErr.elementAt(i)).addElement( (Object) posErr );
             ((ConcreteDataColumn) tempNegErr.elementAt(i)).addElement( (Object) negErr );
             ((ConcreteDataColumn) tempErr.elementAt(i)).addElement( (Object) Tools.doubToObj( Math.sqrt( Math.pow(Tools.objToDoub(posErr),2) + Math.pow(Tools.objToDoub(negErr),2) ) ) );
           }
          }
         }
        }


        /**
         *  THis method displays the plot of the selected x and y axis columns
         */
        public void display() {

          statusTextField.setText("> Displaying... ");
          // create plots to have as many plots as there are layout frames

          boolean plottingError = false;                       // Variable in case anything goes wrong.

          plots.clear();
          for(int k = 0; k < layoutChoice.getItemCount(); k++) {                   // create plots for each layout frame
            if (xAxisDataTable.size() > 0) {    // if frame is not empty
               plots.add (k, new Plot2D(GUI.this, (PlotOptionsFrame)plotsOptions.elementAt(k)));
               statusTextField.setText("...");

              int xAxisColumnsSize = ((DataTable)xAxisDataTable.elementAt(k)).columnCount(); // get number of x-axis Columns

              for(int i = 0; i < xAxisColumnsSize; i++) {    // for each x-axis Column, if it is visible
                if( ((PlotRecord)((Vector)plotsRecord.elementAt(k)).elementAt(i)).visible ) {
                  int yAxisColumnsSize=((DataTable)((Vector)yAxisDataTable.elementAt(k)).elementAt(i)).columnCount();

                  // get x-axis column and errors
                  DataColumn xCol = (DataColumn) ((DataTable)xAxisDataTable.elementAt(k)).getColumn(i);

                  // get the layout Frame of this x-axis
                  //int tempLayoutPosition = ((PlotRecord) ((Vector)plotsRecord.elementAt(layoutChoice.getSelectedIndex())).elementAt(i)).layoutPosition;
                  // use k instead of this

                  // set Plots title depending on whether it maches titles from other xAxes
                  String framesTitle = ((Plot2D)plots.elementAt(k)).plotApplet.getPlotTitle();  // what's already in there
                  String thisPlotsTitle = (String) ((Vector)titlexAxis.elementAt(k)).elementAt(i);
                  if (framesTitle.equals("")) ((Plot2D)plots.elementAt(k)).plotApplet.setPlotTitle(thisPlotsTitle);
                  else
                    if (!framesTitle.equals(thisPlotsTitle)) ((Plot2D)plots.elementAt(k)).plotApplet.setPlotTitle("");

                  // Go through all the yAxis in this xAxis and load the Plots
                  for(int j = 0; j < yAxisColumnsSize; j++) {
                    // get y-axis Columns and errors
                    DataColumn yCol = (DataColumn) ((DataTable)((Vector)yAxisDataTable.elementAt(k)).elementAt(i)).getColumn(j);
                    DataColumn yPlusErr = null;
                    DataColumn yMinusErr = null;
                    if (yCol instanceof MultiColumn) { // it always is so this is unnecessary
                      yPlusErr = ((MultiColumn) yCol).getErrorPlusColumn();
                      yMinusErr = ((MultiColumn) yCol).getErrorMinusColumn();
                    }

                    // load this plot to it's assigned Layout Frame
                    if( !((PlotRecord)((Vector)plotsRecord.elementAt(k)).elementAt(i)).inverted )  // put xColumn onto xAxis
                      ((Plot2D)plots.elementAt(k)).plotApplet.loadXY(xCol, yCol);
                    else                                                    // put xColumn onto yAxis
                      ((Plot2D)plots.elementAt(k)).plotApplet.loadXY(yCol, xCol);
                  }

                  if (yAxisColumnsSize == 0 ) {
                    if (!plottingError) statusTextField.setText(")");
                    plottingError = true;
                    statusTextField.setText(""); //clear
                    statusTextField.setText(" # You must add columns to y-axis in " + xCol.getName() + " x-axis ");
                  }
                }
                if (!plottingError) statusTextField.setText(".");
              }
              if (xAxisColumnsSize == 0) {
                if (!plottingError) statusTextField.setText(")");
                plottingError = true;
                statusTextField.setText(" # You must add columns to x-axis from Columns List. ");
              }
            }
          }

          // put the Plots into Ploting Frame if there was no error
          if (!plottingError) {
            plotFrame.clearPlotFrame();
            // for every made plot, set its corresponding plotOptions and add it to plotFrame
            for(int i=0; i<plots.size(); i++) {
              if (xAxisDataTable.size() > 0) { // if it is not empty
                Plot2D currentPlot = ((Plot2D)plots.elementAt(i));
                if (!currentPlot.plotApplet.isEmpty()) {    // If all the components were added to this plot
                  plotFrame.add(currentPlot);    // add it
                  statusTextField.setText(".");
                }
              }
            }
            if(gridGroup.getSelectedCheckbox()==gridManualCheckbox) {
              plotFrame.setAutoGrid(false);
              plotFrame.setLayout(new GridLayout(gridXChoice.getSelectedIndex(), gridYChoice.getSelectedIndex(), 10, 10));
            }
            else plotFrame.setAutoGrid(true);

            // Show the plot
            //plotFrame.setBackground(Color.white);
            plotFrame.setLocation(150,150);
            plotFrame.show();
            statusTextField.setText("Displaying done");
          }
          else
            statusTextField.setText(" ## Plotting Error occured, cannot Plot. (check your X and Y entries)");
        }

        /**
         * This function is called when print is requested from the GUI
         */
        public void printPlot() {
                        printJob = PrinterJob.getPrinterJob();
                        printJob.setPrintable(GUI.this, pageForm);
                        // Ask the user to choose the page format (i.e Portrait, landscape or inverse landscape)
                        //pageForm =  printJob.pageDialog(pageForm);

                        plotsPrint = plots;
/*                        try {
                          plotsPrint = (Vector)(ObjectCloner.deepCopy((Vector)plots));
                        } catch (Exception ex) {System.out.println("Deep copy exception: "+ex);}
  */

                       // THis part of code will create a jpg image of the plot

                       //Dimension size = .getSize();
/*
                      try
                        // The code below is for saving a jpeg file of the plot image on the computer
                       {
                       BufferedImage bufferedImage = new BufferedImage(1000, 1000, BufferedImage.TYPE_INT_RGB);
                       Graphics bufferedGraphics = bufferedImage.createGraphics();
                       // Prepare the buffered image for getting the image (set antialias to true)
                       // Draw the plot into the BufferedGraphics Graphics instance
                        for(int i=0; i<plotsPrint.size(); i++) {
                          ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).paint(bufferedGraphics);
                           // translate the graphing down by the height of each plot
                           bufferedGraphics.translate(0,((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight());//0,plotFrame.getHeight()/plotsPrint.size());
                        }
                       ByteArrayOutputStream bos = new ByteArrayOutputStream();
                       JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(bos);

                       JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bufferedImage);
                       param.setQuality((float)0.75, false);
                       param.setHorizontalSubsampling(0, 1);
                       param.setHorizontalSubsampling(1, 1);
                       param.setHorizontalSubsampling(2, 1);
                       param.setVerticalSubsampling(0, 1);
                       param.setVerticalSubsampling(1, 1);
                       param.setVerticalSubsampling(2, 1);
                       encoder.setJPEGEncodeParam(param);

                       encoder.encode(bufferedImage);

                       byte[] jpgData = bos.toByteArray();
                        FileOutputStream fos = new FileOutputStream("/home/itucakov/public_html/Aloha1.jpg");
                        //fos.write(jpgData);
                         fos.close();
                       } catch (IOException x) {System.out.println("jpg save didn't work:"+x);}
*/
            /*            printService[] services = PrinterJob.lookupPrinterServices();
                        if (services.length == 0) {
                          return; //no printer available
                        }
                        if (printerName != null) {
                          for (int i = 0; i < services.length; i++) {
                            if (printerName.equals(services.getName())) {
                              printJob.setPrintService(services);
                            }
                          }
                        }
            */
                        // Open the print dialog for the user to make settings
                        if (printJob.printDialog()) { // display the options dialog, true if user pressed OK

                        // ***** Note: this print command will call GUI's print() method as well! (see at the botom of the file..) ******
                          try {
                          //  pageForm = printJob.validatePage()
                            printJob.print();
                         //   printerName = printJob.getPrintService().getName();
//              System.out.println("printer: "+printJob.getPrinterJob()+"name of file: "+printJob.getJobName());
                          } catch (Exception ex) {
                            ex.printStackTrace();
                            // pop alert
                            popAlertDialog(" ","   Error in printing ", ex.getMessage());
                          }
                        }
                      }

        /**
         * This method prints the graphic contents of the GUI plot
         * GUI extends interface Printable, so it has this print function to print the plot.
         */
        public int print(Graphics g, PageFormat pf, int pi) throws PrinterException {
          if (pi >= 1) {
            return Printable.NO_SUCH_PAGE;
          }
          // now assign this page format value
          pageForm.setOrientation(pf.getOrientation());
          // Change the size of the plot frame (on the screen) to the size of the chosen size printout + additional 75
          if (pageForm.getOrientation()==pageForm.PORTRAIT)
            plotFrame.setSize((int)plotWidthPort*28 + 150,(int)plotHeightPort*28 + 150 + 75);
          else
            plotFrame.setSize((int)plotHeightLand*28 + 150,(int)plotWidthLand*28 + 150 + 75);

                        // Now check for frame width and height to make sure the graph is not too big for the page
                        // Approximately, the ratio of the plot box would be around 4x5
                        // Note: I have estimated Max. width to be: 570 and Max. height to be 710 for PORTRAIT
                        //        and vice versa for LANDSCAPE and REVERSE LANDSCAPE page formats
                        //   I also estimated that the ratio between centimeters and pixels is 28. So the Widthts and heights
                        //     below will be calculated from the settings that were done in "plot size setup". There, default max.
                        //     values will be pre-set so that they match the maximums below.

                        int maxWidth = 570;  // 15(cm)*28 + 150(edges) = 570 (width should not be bigger than 570)
                        int maxHeight = 720;  // 20(cm)*28 + 150(edges) = 720
                        int niceHeight = 490;  // 12(cm)*28 + 150(edges) = 486  (520 is for 13 cm..)
                        int tempw = 570;//some random value
                        int temph = 520;//some random value
                        if (plotsPrint!=null) {
                          if (plotsPrint.size()!=0) {//just in case the vector is not empty
                          /* Note, this code is in case we want to print the plot as it appears on the screen
                          instead, calculate out the print setup values (below..)
                            tempw = ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth();
                            for(int i=0; i<plotsPrint.size(); i++)
                              temph += ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getHeight();
                          */
                          }
                          // If PageFormat is Portrait
                          if (pageForm.getOrientation()==PageFormat.PORTRAIT) { //Portrait = 1
                            tempw = plotWidthPort*28 + 150;
                            temph = plotHeightPort*28 + 150;
                            // just fit the one graph in the upper area of the Frame
//System.out.println("tempw:"+tempw+"temph:"+temph+"vector size:"+plotsPrint.size());
                            if (plotsPrint.size()==1) {
                              // resize it only if the plot is bigger than the page
                              if (tempw>maxWidth) {
                                ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(maxWidth, niceHeight);
                                }
                              // resize to the value set by the user
                              // note: this code would be unnecessary in case where the print would be obtained from the
                              //       plot size off the screen (the disabled code above)
                              else  ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(tempw, temph);
                            // if there are more than one graphs, then fit them in the whole page
                            } else
                                for(int i=0; i<plotsPrint.size(); i++)
                                  ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).setSize(maxWidth, maxHeight/plotsPrint.size());
                          } else { // it is landscape mode
                            tempw = plotWidthLand*28 + 150;
                            temph = plotHeightLand*28 + 150;
                            // just fit the one graph into the whole area of the Frame
                            if (plotsPrint.size()==1) {
                              // resize it only if the plot is bigger than the page
                              if (temph>=maxHeight)
                                ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(maxHeight, maxWidth);
                              // resize to the value set by the user
                              // note: this code would be unnecessary in case where the print would be obtained from the
                              //       plot size off the screen (the disabled code above)
                              else ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).setSize(temph, tempw);

                            // if there are more than one graphs, then fit them in the whole page
                            } else
                                for(int i=0; i<plotsPrint.size(); i++)
                                  ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).setSize(maxHeight, maxWidth/plotsPrint.size());
                          }
                        }

                        // After all this, resize the Frame to fit the new Graph sizes
                  //this won't work, create a new Applet class!!
                        //  plotFrame.setSize(maxHeight, maxWidth);

          if(plotsPrint.size()!=0) {// if there are plots displayed in the GUI
            // Since the plot will be done one above the other, the width will stay the same
            // The height depends on the number of plots.
            temph = 0;
            for(int i=0; i<plotsPrint.size(); i++) {
              temph += ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight();
            }
//System.out.println("height: "+temph+" width: "+((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth()+"\n");
            // Set the 'range' of Graphics
            if (plotsPrint.size()!=0) // just in case the Vector is empty
              g.setClip(0,0,((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getWidth(),temph);
            for(int i=0; i<plotsPrint.size(); i++) {
              ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).paint(g);
              // translate the graphing down by the height of each plot
              g.translate(0,((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight());//0,plotFrame.getHeight()/plotsPrint.size());
            }
          }


          return Printable.PAGE_EXISTS;
        }
        
        /**
         *Writes the current plot to a PostScript file.
         *
         *@return True if the print job is completed successfully, and false otherwise.
         */
        public boolean printToPostScript()
        {
            //System.out.println("printToPostScript() called");
            try 
            {   
                // Set up destination attribute
                PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
                //get output file location
                JFileChooser chooser = new JFileChooser();

                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    public boolean accept(File f)
                    {
                        String name = f.getName();
                        return name.substring(name.length()-3).equalsIgnoreCase("ps");
                    }

                    public String getDescription()
                    {
                        return "PostScript files";
                    }
                };

                chooser.setFileFilter(filter);
    
                //if the user hit OK, write the to the selected location
                if(chooser.showSaveDialog(GUI.this) == JFileChooser.APPROVE_OPTION) 
                {
                    aset.add(new Destination(new java.net.URI("file:" + chooser.getSelectedFile().getAbsolutePath())));  
                    //initialize print job
                    PrintService[] printServices = PrintServiceLookup.lookupPrintServices(DocFlavor.INPUT_STREAM.POSTSCRIPT, aset);
                    if(printServices.length==0)     //no suitable print services found
                    {
                        JOptionPane.showMessageDialog(null, "This computer has informed View that it is unable to produce PostScript documents.");
                    }
                    else
                    {
                        //get the document object to be printed
                        DocAttributeSet docAttributeSet = new HashDocAttributeSet();
                        //set the attributes which control the shape and size of the output document
                        //NA letter is standard 8.5x11" paper
                        docAttributeSet.add(MediaSizeName.NA_LETTER);
                        //if an orientation has been specified, let the printer know
                        System.out.println("Page orientation:" + pageForm.getOrientation());
                        if(pageForm.getOrientation()==PageFormat.PORTRAIT)
                            docAttributeSet.add(OrientationRequested.PORTRAIT);
                        else if(pageForm.getOrientation()==PageFormat.LANDSCAPE)
                            docAttributeSet.add(OrientationRequested.LANDSCAPE);
                        SimpleDoc docToPrint = new SimpleDoc((GUI)this, DocFlavor.SERVICE_FORMATTED.PRINTABLE, docAttributeSet);
                        //create a print job
                        DocPrintJob job = printServices[0].createPrintJob();
                        job.print(docToPrint, aset);
                    }
                }
            } 
            catch (PrintException e) 
            {
                JOptionPane.showMessageDialog(null, "An error has occurred and the document was not printed.");
                System.out.println(e.getMessage());
            } 
            catch (java.net.URISyntaxException e) 
            {
                JOptionPane.showMessageDialog(null, "An error has occurred and the document was not printed." +
                        "\nThis likely occurred because the specified .ps output location was not valid.");
                System.out.println(e.getMessage());
            }   

            return true;
        }

        /**
         * This Method clears all the textfields in the MinMaxPanel of the PlotOptionsFrame Object
         */
        public void clearMinMaxLimits() {
          // clear X min TextField (xminText)
          ((TextField)
            ((PlotOptionsFrame)plotsOptions.
              elementAt(layoutChoice.getSelectedIndex())).minMaxPanel.getComponent(3)).setText("");
          // clear X max TextField (xmaxText)
          ((TextField)
            ((PlotOptionsFrame)plotsOptions.
              elementAt(layoutChoice.getSelectedIndex())).minMaxPanel.getComponent(5)).setText("");
          // clear Y min TextField (yminText)
          ((TextField)
            ((PlotOptionsFrame)plotsOptions.
              elementAt(layoutChoice.getSelectedIndex())).minMaxPanel.getComponent(8)).setText("");
          // clear Y max TextField (ymaxText)
          ((TextField)
            ((PlotOptionsFrame)plotsOptions.
              elementAt(layoutChoice.getSelectedIndex())).minMaxPanel.getComponent(10)).setText("");
        }

 	/**
         * dispose all the frames and exit
         * called from the quit menu
         */
 	public void disposeAll() {
		GUI.this.dispose();
		plotFrame.dispose();
		//System.exit(-1);
	}

        public static void main(String[] args) {
          JFrame f = new GUI(new GUI_RMIApplet());
          f.setLocation(50,50);
          f.show();
          // Once loaded, hide the GUI
          f.setVisible(false);
        }

  /**
   * This method pops up a dialog box informing the user with the two lines of messages
   */

  public void popAlertDialog(String alert, String message, String message2) {
                                     infoDialog.removeAll();
                                     infoDialog.setLayout(new GridLayout(4,1));
                                     infoDialog.add(new JLabel(alert));
                                     infoDialog.add(new JLabel(message));
                                     infoDialog.add(new JLabel(message2));
                                     infoDialog.add(OKButton);
                                     infoDialog.setSize(new Dimension(450, 150));
                                     infoDialog.setLocation(100,200);
                                     infoDialog.show();
  }
    
  /*
   *Brings up the menu for graph options relating to the title and labels.
   */
    private void showTitleLabelOptions()
    {
        JFrame titleLabelOption = new TitleLabelOptionsDialog();
        titleLabelOption.setLocationRelativeTo(this);
        titleLabelOption.setVisible(true);
    }
    
    /*
     *Brings up the menu for graph options relating to the graph axes.
     */
    private void showAxisOptions()
    {
        JFrame axisOptions = new AxisOptionsDialog();
        axisOptions.setLocationRelativeTo(this);
        axisOptions.setVisible(true);
    }
    
    /*
     *Brings up the menu for graph options relating to the plot area.
     */
    private void showPlotOptions()
    {
        JFrame plotOptions = new PlotOptionsDialog();
        plotOptions.setLocationRelativeTo(this);
        plotOptions.setVisible(true);
    }
    
    /*
     *Factory method for 'OK' and 'Cancel' buttons arranged horizontally
     */
    private JPanel makeOkCancelButtons(ActionListener a)
    {
        JPanel result = new JPanel();
        result.setLayout(new GridLayout(1,2));
        
        JButton okButton = new JButton("OK");
        okButton.addActionListener(a);
        result.add(okButton);
        
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(a);
        result.add(cancelButton);
                
        return result;
    }
        
    private class TitleLabelOptionsDialog extends JFrame implements ActionListener
    {
        JTextField titleText;
        JTextField xLabelText;
        JTextField yLabelText;
        
        JSpinner titleFont;
        JSpinner titleStyle;
        JSpinner titleSize;
        JSpinner labelFont;
        JSpinner labelStyle;
        JSpinner labelSize;
        
        static final int MAX_TEXT_LENGTH = 50;
        
        public TitleLabelOptionsDialog()
        {
            super("Titles and labels");
            
            this.setSize(560, 220);
            Container content = this.getContentPane();
            content.setLayout(new BorderLayout());

            //these options set the text of the title and axes labels
            JPanel textPanel = new JPanel();
            textPanel.setLayout(new GridLayout(4,2));
            textPanel.add(new JLabel("Title:"));
            titleText = new JTextField(MAX_TEXT_LENGTH);
            textPanel.add(titleText);
            textPanel.add(new JLabel("X-axis label:"));
            xLabelText = new JTextField(MAX_TEXT_LENGTH);
            textPanel.add(xLabelText);
            textPanel.add(new JLabel("Y-axis label:"));
            yLabelText = new JTextField(MAX_TEXT_LENGTH);
            textPanel.add(yLabelText);
            textPanel.add(new JLabel("(Leave labels blank to use the default label.)"));

            //these options set the font of the title and axes labels
            JPanel fontPanel = new JPanel();
            fontPanel.setLayout(new GridLayout(2,4));
            
            fontPanel.add(new JLabel("Title font:"));
            titleFont = new JSpinner(new SpinnerListModel(PlotOptionsFrame.FONT_TYPES));
            fontPanel.add(titleFont);
            titleStyle = new JSpinner(new SpinnerListModel(PlotOptionsFrame.FONT_STYLES));
            fontPanel.add(titleStyle);
            titleSize = new JSpinner(new SpinnerNumberModel(new Integer(24), new Integer(6), new Integer(35), new Integer(1)));
            fontPanel.add(titleSize);
            
            fontPanel.add(new JLabel("Label font:"));
            labelFont = new JSpinner(new SpinnerListModel(PlotOptionsFrame.FONT_TYPES));
            fontPanel.add(labelFont);
            labelStyle = new JSpinner(new SpinnerListModel(PlotOptionsFrame.FONT_STYLES));
            fontPanel.add(labelStyle);
            labelSize =  new JSpinner(new SpinnerNumberModel(new Integer(18), new Integer(6), new Integer(35), new Integer(1)));
            fontPanel.add(labelSize);

            //put everything into the contentPane, which is what actually shows up on the screen
            //the mainPane is used to space out the components other than the OK and Cancel buttons suitably
            JPanel mainPane = new JPanel();
            mainPane.setLayout(new BorderLayout());
            mainPane.add(textPanel,BorderLayout.CENTER);
            mainPane.add(fontPanel,BorderLayout.SOUTH);
            content.add(mainPane, BorderLayout.CENTER);
            content.add(makeOkCancelButtons(this),BorderLayout.SOUTH);
        }        
        
        public void actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            
            if(command.equals("OK"))
            {
                
            }
            else if(command.equals("Cancel"))
            {
                //get rid of the window - don't accept the changes
                this.dispose();
            }
        }
    }
    
    private class AxisOptionsDialog extends JFrame implements ActionListener
    {
        //used in axisTypePanel
        CheckboxGroup xAxisCheckboxes;
        Checkbox xAxisLinear;
        Checkbox xAxisLogarithmic;
        Checkbox xAxisInverse;
        CheckboxGroup yAxisCheckboxes;
        Checkbox yAxisLinear;
        Checkbox yAxisLogarithmic;
        Checkbox yAxisInverse;
        
        //used in plotLimitsPanel
        JTextField xAxisMin;
        JTextField xAxisMax;
        JTextField yAxisMin;
        JTextField yAxisMax;
        
        //used in numbersFontPanel
        JSpinner numbersFont;
        JSpinner numbersStyle;
        JSpinner numbersSize;
        
        //used in zeroAxesSetupPanel
        Checkbox xZeroAxis;
        Checkbox yZeroAxis;
        
        public AxisOptionsDialog()
        {
            super("Axis options");
            
            this.setSize(560, 420);
            Container content = this.getContentPane();
            content.setLayout(new GridLayout(5,1));
            
            //set the nature of the x and y axes - linear, logarithmic, or inverse
            JPanel axisTypePanel = new JPanel();
            axisTypePanel.setLayout(new GridLayout(2,4));
            xAxisCheckboxes = new CheckboxGroup();
            axisTypePanel.add(new JLabel("X axis:"));
            xAxisLinear = new Checkbox("Linear", xAxisCheckboxes, true);
            axisTypePanel.add(xAxisLinear);
            xAxisLogarithmic = new Checkbox("Logarithmic", xAxisCheckboxes, false);
            axisTypePanel.add(xAxisLogarithmic);
            xAxisInverse = new Checkbox("Inverse", xAxisCheckboxes, false);
            axisTypePanel.add(xAxisInverse);
            
            yAxisCheckboxes = new CheckboxGroup();
            axisTypePanel.add(new JLabel("Y axis:"));
            yAxisLinear = new Checkbox("Linear", yAxisCheckboxes, true);
            axisTypePanel.add(yAxisLinear);
            yAxisLogarithmic = new Checkbox("Logarithmic", yAxisCheckboxes, false);
            axisTypePanel.add(yAxisLogarithmic);
            yAxisInverse = new Checkbox("Inverse", yAxisCheckboxes, false);
            axisTypePanel.add(yAxisInverse);
            
            //set the limits on values for the x and y axes
            JPanel plotLimitsPanel = new JPanel();
            
            
            JPanel numbersFontPanel = new JPanel();
            
            JPanel zeroAxesSetupPanel = new JPanel();
            
            //add all the panels to the contentPane, which will actually be visible
            content.add(axisTypePanel);
            content.add(plotLimitsPanel);
            content.add(numbersFontPanel);
            content.add(zeroAxesSetupPanel);
            content.add(makeOkCancelButtons(this));            
        }
        
        public void actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            
            if(command.equals("OK"))
            {
                
            }
            else if(command.equals("Cancel"))
            {
                //get rid of the window - don't accept the changes
                this.dispose();
            }
        }   
    }
    
    private class PlotOptionsDialog extends JFrame implements ActionListener
    {
        //used in linePointPanel
        JSpinner symbolSize;
        JSpinner symbolType;
        JSpinner symbolColour;
        Checkbox symbolFilled;
        Checkbox pointsConnected;
        
        //used in areaColourPanel
        JSpinner borderColour;
        JSpinner backgroundColour;
        
        //used in plotLocationPanel
        JSpinner topDist;
        JSpinner bottomDist;
        JSpinner leftDist;
        JSpinner rightDist;
        
        //used in otherOptionsPanel
        Checkbox strictLimits;
        Checkbox ascendingOrder;
        Checkbox caseSensitive;
        Checkbox errorBars;
        Checkbox errorEdgeTics;
        Checkbox dotFiducials;
        
        public PlotOptionsDialog()
        {
            super("Plot options");
            
            this.setSize(350, 590);
            Container content = this.getContentPane();
            content.setLayout(new GridLayout(9,1));
            
            //addresses settings related to graph lines and points
            JPanel linePointPanel = new JPanel();
            linePointPanel.setLayout(new GridLayout(4,2));
            linePointPanel.add(new JLabel("Size:"));
            symbolSize = new JSpinner(new SpinnerNumberModel(new Integer(1), new Integer(1), new Integer(30), new Integer(1)));
            linePointPanel.add(symbolSize);
            linePointPanel.add(new JLabel("Type:"));
            symbolType = new JSpinner(new SpinnerListModel(PlotOptionsFrame.POINT_TYPE));
            linePointPanel.add(symbolType);
            linePointPanel.add(new JLabel("Colour:"));
            symbolColour = new JSpinner(new SpinnerListModel(PlotOptionsFrame.COLORS));
            linePointPanel.add(symbolColour);
            symbolFilled = new Checkbox("Filled");
            linePointPanel.add(symbolFilled);
            pointsConnected = new Checkbox("Points connected");
            linePointPanel.add(pointsConnected);
            
            
            //addresses settings related to plot area colour
            JPanel areaColourPanel = new JPanel();
            areaColourPanel.setLayout(new FlowLayout());
            areaColourPanel.add(new JLabel("Border colour:"));
            borderColour = new JSpinner(new SpinnerListModel(PlotOptionsFrame.COLORS));
            areaColourPanel.add(borderColour);
            areaColourPanel.add(new JLabel("Background colour:"));
            backgroundColour = new JSpinner(new SpinnerListModel(PlotOptionsFrame.COLORS));
            areaColourPanel.add(backgroundColour);
            
            //addresses location of plot in graphic window
            JPanel plotLocationPanel = new JPanel();
            plotLocationPanel.setLayout(new GridLayout(2,4));
            plotLocationPanel.add(new JLabel("Top"));
            plotLocationPanel.add(new JLabel("Bottom"));
            plotLocationPanel.add(new JLabel("Left"));
            plotLocationPanel.add(new JLabel("Right"));
            topDist = new JSpinner(new SpinnerNumberModel(new Integer(90), new Integer(0), new Integer(500), new Integer(5)));
            plotLocationPanel.add(topDist);
            bottomDist = new JSpinner(new SpinnerNumberModel(new Integer(80), new Integer(0), new Integer(500), new Integer(5)));
            plotLocationPanel.add(bottomDist);
            leftDist = new JSpinner(new SpinnerNumberModel(new Integer(130), new Integer(0), new Integer(500), new Integer(5)));
            plotLocationPanel.add(leftDist);
            rightDist = new JSpinner(new SpinnerNumberModel(new Integer(40), new Integer(0), new Integer(500), new Integer(5)));
            plotLocationPanel.add(rightDist);
            
            //addresses other options, like error bars and sorting
            JPanel otherOptionsPanel = new JPanel();
            otherOptionsPanel.setLayout(new GridLayout(2,3));
            strictLimits = new Checkbox("Strict limits");
            otherOptionsPanel.add(strictLimits);
            ascendingOrder = new Checkbox("Ascending order");
            otherOptionsPanel.add(ascendingOrder);
            caseSensitive = new Checkbox("Case sensitive");
            otherOptionsPanel.add(caseSensitive);
            errorBars = new Checkbox("Show error bars");
            otherOptionsPanel.add(errorBars);
            errorEdgeTics = new Checkbox("Error bar edge tics:");
            otherOptionsPanel.add(errorEdgeTics);
            dotFiducials = new Checkbox("Dot grid");
            otherOptionsPanel.add(dotFiducials);
            
            //put everything into the window's content pane, so that it actually shows up
            content.add(new JLabel("Point settings:"));
            content.add(linePointPanel);
            content.add(new JLabel("Colour options:"));
            content.add(areaColourPanel);
            content.add(new JLabel("Plot location:"));
            content.add(plotLocationPanel);
            content.add(new JLabel("Other settings:"));
            content.add(otherOptionsPanel);
            content.add(makeOkCancelButtons(this));
        }
        
        public void actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();
            
            if(command.equals("OK"))
            {
                
            }
            else if(command.equals("Cancel"))
            {
                //get rid of the window - don't accept the changes
                this.dispose();
            }
        }
    }
    
    /**
     * Displays the values contained within the indicated DataTable
     */
    private void displayTable(DataTable d)
    {
        int dataColumnCount = d.getColumns().size();
        int columnCount = 0;
        //count up how many columns the table should display
        //this is not necessarily the same thing as the number of DataColumns
        //since if there are error-bars then more columns will be required
        DataColumn currColumn;
        for(int i=0; i<dataColumnCount; i++)
        {
            currColumn = d.getColumn(i);
            if(Double.parseDouble(currColumn.getErrorPlus(0).toString())==0) //assume there are no error bars if error is exactly zero
            {
                columnCount++;
            }
            else    //assume there are error bars
            {
                columnCount += 3;
            }
        }
        int rowCount = d.getColumn(0).getSize();
        final String[] columnNames = new String[columnCount];
        final Object[][] data = new Object[rowCount][columnCount];
        //go through the columns - if a coulumn has error bars then there are + and - 
        //error bars as well
        int currIndex = 0;
        for(int i=0; i<dataColumnCount; i++)
        {
            currColumn = d.getColumn(i);
            if(Double.parseDouble(currColumn.getErrorPlus(0).toString())==0) //assume there are no error bars if error is exactly zero
            {
                columnNames[currIndex] = currColumn.getName();
                for(int j=0; j<rowCount; j++)
                    data[j][currIndex] = currColumn.getValue(j);
                currIndex++;
            }
            else    //assume there are error bars
            {
                columnNames[currIndex]   = currColumn.getName();
                columnNames[currIndex+1] = currColumn.getName() + "(+ errorbar)";
                columnNames[currIndex+2] = currColumn.getName() + "(- errorbar)";
                for(int j=0; j<rowCount; j++)
                {
                    data[j][currIndex]   = currColumn.getValue(j);
                    data[j][currIndex+1] = currColumn.getErrorPlus(j);
                    data[j][currIndex+2] = currColumn.getErrorMinus(j);
                }
                currIndex += 3;
            }
        }
        
        AbstractTableModel displayTableModel = new AbstractTableModel()
        {
            public String getColumnName(int col)   
            {
                return columnNames[col].toString();
            }
            
            public int getRowCount() 
            { 
                return data.length; 
            }
    
            public int getColumnCount() 
            {
                return columnNames.length; 
            }
    
            public Object getValueAt(int row, int col) 
            {
                return data[row][col];
            }
    
            public boolean isCellEditable(int row, int col)
            {
                return false; 
            }
    
            public void setValueAt(Object value, int row, int col) 
            {
                data[row][col] = value;
                fireTableCellUpdated(row, col);
            }
        };
                
        JTable dataDisplayTable = new JTable(displayTableModel);
                
        //set width of displayed columns
        TableColumn column = null;
        for (int i = 0; i < dataDisplayTable.getColumnCount(); i++) 
        {
            column = dataDisplayTable.getColumnModel().getColumn(i);
            column.setMinWidth(60);
        }

        dataDisplayTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
       
        //put it in a scroll pane to display
        JScrollPane scrollPane = new JScrollPane(dataDisplayTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        //put the scroll pane in a visible window and display it
        JFrame displayFrame = new JFrame(d.getTitle());
        displayFrame.getContentPane().add(scrollPane);
        displayFrame.setSize(600,450);
        displayFrame.setLocationRelativeTo(this);
        displayFrame.setVisible(true);
    }
    
    /**
     *Displays all the currently defined constants in a table separated by category.
     *Triggered when the user selects View --> Constants Table from the drop-down menu.
     */
    private void showConstantTable(JFrame parent)
    {
        final Object[][] data = new Object[dataTableVariables.getSize()][4];
        for(int i=0; i<data.length; i++)
        {
            if(dataTableVariables.getConstantName(i).toString().indexOf("!---")==-1)    //this is an actual constant
            {
                data[i][0] = " ";   //the type entry is empty since it is only used for divider rows
                data[i][1] = dataTableVariables.CONSTANT.get(i);
                data[i][2] = dataTableVariables.CONSTANT_VALUE.get(i);
                data[i][3] = ((String)(dataTableVariables.CONSTANT_DESCRIPTION.get(i))).substring(
                        ((String)(dataTableVariables.CONSTANT_DESCRIPTION.get(i))).indexOf("!")+1); //get rid of initial "!"
            }
            else                                                                        //this is a type of constant
            {
                data[i][0] = ((String)(dataTableVariables.CONSTANT_DESCRIPTION.get(i))).substring(
                        ((String)(dataTableVariables.CONSTANT_DESCRIPTION.get(i))).indexOf("!---")+3);
                data[i][1] = "===";   //This row is mostly just a divider,
                data[i][2] = "===";   //so columns after the first one don't
                data[i][3] = "===";   //carry any information.
            }
        }

        AbstractTableModel constantTableModel = new AbstractTableModel()
        {
            public String getColumnName(int col)   
            {
                switch(col)
                {
                    case 0:
                        return "Type";
                    case 1:
                        return "Name";
                    case 2:
                        return "Value";
                    case 3:
                        return "Comment";
                    default:
                        System.out.println("There has been an error in the constant display table.");
                        System.exit(0);
                        break;
                }
                //this will clearly never be reached, but a return statement is needed in order to make things compile
                return "some horrible error has occurred";
            }
            
            public int getRowCount() 
            { 
                return data.length; 
            }
    
            public int getColumnCount() 
            {
                return 4; 
            }
    
            public Object getValueAt(int row, int col) 
            {
                return data[row][col];
            }
    
            public boolean isCellEditable(int row, int col)
            {
                return false; 
            }
    
            public void setValueAt(Object value, int row, int col) 
            {
                data[row][col] = value;
                fireTableCellUpdated(row, col);
            }
        };
                
        JTable constantDisplayTable = new JTable(constantTableModel);
                
        //set width of displayed columns
        //1. constant type
        constantDisplayTable.getColumnModel().getColumn(0).setMinWidth(120);
        //2. constant name
        constantDisplayTable.getColumnModel().getColumn(1).setMinWidth(60);
        //3. constant value
        constantDisplayTable.getColumnModel().getColumn(2).setMinWidth(75);
        //4. constant comment
        constantDisplayTable.getColumnModel().getColumn(3).setMinWidth(200);

        constantDisplayTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
       
        //put it in a scroll pane to display
        JScrollPane scrollPane = new JScrollPane(constantDisplayTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        //put the scroll pane in a visible window and display it
        JFrame displayFrame = new JFrame("Constants");
        displayFrame.getContentPane().add(scrollPane);
        displayFrame.setSize(450,450);
        displayFrame.setLocationRelativeTo(this);
        displayFrame.setVisible(true);
    }
    
     /**
     *Displays all the currently defined functions in a table separated by category.
     *Triggered when the user selects View --> Functions Table from the drop-down menu.
     */
    private void showFunctionTable(JFrame parent)
    {
        /*
         *InfoDialog d = new InfoDialog(GUI.this, "Functions Table", -1,-1);
                        for(int i=0; i<functionsTable.getSize(); i++)
                          d.addLine(functionsTable.getFunctionName(i).toString()+" = "+
                                     functionsTable.getFunctionValue(i).toString());
                        d.setLocation(200,200);
                        d.show();
         */
        //stuff goes here to initialize data
        final Object[][] data = new Object[functionsTable.getSize()][2];
        for(int i=0; i<data.length; i++)
        {
            data[i][0] = functionsTable.getFunctionName(i).toString();
            data[i][1] = functionsTable.getFunctionValue(i).toString();
        }
        AbstractTableModel functionTableModel = new AbstractTableModel()
        {
            public String getColumnName(int col)   
            {
                switch(col)
                {
                    case 0:
                        return "Name";
                    case 1:
                        return "Expression";
                    default:
                        System.out.println("There has been an error in the constant display table.");
                        System.exit(0);
                        break;
                }
                //this will clearly never be reached, but a return statement is needed in order to make things compile
                return "some horrible error has occurred";
            }
            
            public int getRowCount() 
            { 
                return data.length; 
            }
    
            public int getColumnCount() 
            {
                return 2; 
            }
    
            public Object getValueAt(int row, int col) 
            {
                return data[row][col];
            }
    
            public boolean isCellEditable(int row, int col)
            {
                return false; 
            }
    
            public void setValueAt(Object value, int row, int col) 
            {
                data[row][col] = value;
                fireTableCellUpdated(row, col);
            }
        };
                
        JTable functionDisplayTable = new JTable(functionTableModel);
                
        //set width of displayed columns
        functionDisplayTable.getColumnModel().getColumn(0).setWidth(80);    //labels
        functionDisplayTable.getColumnModel().getColumn(1).setWidth(120);   //expressions

        functionDisplayTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
       
        //put it in a scroll pane to display
        JScrollPane scrollPane = new JScrollPane(functionDisplayTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        //put the scroll pane in a visible window and display it
        JFrame displayFrame = new JFrame("Functions");
        displayFrame.getContentPane().add(scrollPane);
        displayFrame.setSize(180,250);
        displayFrame.setLocationRelativeTo(this);
        displayFrame.setVisible(true);
    }
    
    /*
     *This function writes the plot to file as a .png
     */
    private void savePlotPNG()
    {
        plotsPrint = plots;     //for some reason, plots is copied out into plotsPrint wherever it is used
                                //it would see someone didn't understand passing by reference, but here
                                //I'm keeping it anyway for consistency
        
        if(plotsPrint == null || plotsPrint.size() == 0)
        {
            JOptionPane.showMessageDialog(null, "There is no active plot to save.\nPlease generate a plot, and try again.");
        }
        else
        {
             /*BufferedImage bi;   //the image which will eventually be saved    
            //get the image of the plot, and put it in a buffered image
            Graphics plotGraphics  = ((PlotApplet)((Plot2D)plotsPrint.elementAt(0)).plotApplet).getGraphics();
            //gets configuration details necessary to save image
            GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            //set the appropriate size for the buffered image - TYPE_INT_RGB means to transparency,
            //which is suitable for saving a PNG
            bi = gc.createCompatibleImage(plotGraphics.getClipBounds().width, plotGraphics.getClipBounds().height, BufferedImage.TYPE_INT_RGB);
            //set the graphic for the buffered image to be the plot graphic here
            Graphics bufferedImageGraphics = bi.createGraphics();
            bufferedImageGraphics= plotGraphics;*/
            int plotWidth = 720;    //these values chosen to give reasonably-sized plots
            int plotHeight = 570;
            BufferedImage bufferedImage = new BufferedImage(plotWidth, plotHeight, BufferedImage.TYPE_INT_ARGB);
            Graphics bufferedGraphics = bufferedImage.createGraphics();
            //Ensure that the whole image has a white background - otherwise, the system default will be used
            bufferedGraphics.setColor(Color.WHITE);
            bufferedGraphics.fillRect(0,0,bufferedImage.getWidth(),bufferedImage.getHeight());
            // Draw the plot into the BufferedGraphics Graphics instance
            for(int i=0; i<plotsPrint.size(); i++) 
            {
                ((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).paint(bufferedGraphics);
                // translate the graphing down by the height of each plot
                bufferedGraphics.translate(0,((PlotApplet)((Plot2D)plotsPrint.elementAt(i)).plotApplet).getHeight());//0,plotFrame.getHeight()/plotsPrint.size());
            }

            try 
            {
                //let the user select a location to which to save the file
                 JFileChooser chooser = new JFileChooser();

                javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter()
                {
                    /**
                     *Checks if a given file is valid - here, 'valid' means that one of the following two conditions is met:
                     *  the file is a directory
                     *  the file has the extension '.png' (case-insensitive)
                     */
                    public boolean accept(File f)
                    {
                        if(f.isDirectory())
                            return true;
                        return f.getName().substring(f.getName().length()-3).equalsIgnoreCase("png");
                    }

                    public String getDescription()
                    {
                        return "PNG files";
                    }
                };

                chooser.setFileFilter(filter);

                //if the user hit OK, write the to the selected location
                if(chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
                {
                    ImageIO.write(bufferedImage, "png", chooser.getSelectedFile());
                }
            } 
            catch (IOException e)
            {
                String output = "An error occurred in writing the image to file.\n";
                output = output + e.getMessage();
                output = output + "\nThe image was not successfully written to file.";
                JOptionPane.showMessageDialog(null, output);
            }
        }
    }
    
    
    /*Reload the currently selected data table from file, and reload the corresponding x and y axes*/
    private void refreshDataTable()
    {
        //find the currently selected data table and the corresponding filepath from which it was loaded
        int selectedDataTableIndex = dataTableList.getSelectedIndex(); 
        if (selectedDataTableIndex!=-1)  // if there are data tables in the List
        {
            DataTable selectedDataTable = (DataTable)dictionary.getElement(selectedDataTableIndex);
            String filepath = selectedDataTable.filepath;
        
            //save a list of all the x columns and corresponding y columns
            int[] xColumn = getXColumnList(selectedDataTable);
            int[][] yColumn = new int[xColumn.length][];
            for(int i=0; i<xColumn.length; i++)
            {
                if(xColumn[i] != -1)
                {
                    xAxisList.select(i);
                    reloadyAxisList();
                    yColumn[i] = getYColumnList(selectedDataTable, i);
                }
            }
            
            /*System.out.println("The selected columns are as follows:");
            
            for(int i=0; i<xColumn.length; i++)
            {
                if(xColumn[i] != -1)        //current column is associated with the appropriate data table
                {
                    System.out.println(selectedDataTable.getColumn(xColumn[i]).getName());
                    for(int j=0; j<yColumn[i].length; j++)
                        System.out.println("\t" + selectedDataTable.getColumn(yColumn[i][j]).getName());
                    System.out.println();
                }
            }
            System.out.println("The filepath for the currently selected table is " + filepath);*/
            
            File inputFile = new File(filepath);            
            //note that currently this is only going to be implemented for .db files since other file types do not currently have load methods
            if(hasDBExtension(inputFile))
            {
                //remove the currently selected data table
                removeDataTable();
                //reload the lists of columns now that a new data table has been selected
                //reloadColumnList();
                // load back the currently selected data table from the file where it was saved
                loadDBFile(inputFile.getParent()+inputFile.separator, inputFile.getName());
                //add back the lists of x and y columns
                int selectedLayoutIndex = layoutChoice.getSelectedIndex();      //adding columns requires a parameter that represents the layout choice
                for(int i=0; i<xColumn.length; i++)
                {
                    if(xColumn[i] != -1)        //current column is associated with the appropriate data table
                    {
                        addX(xColumn[i], selectedLayoutIndex);
                        xAxisList.select(xAxisList.getItemCount()-1);
                        for(int j=0; j<yColumn[i].length; j++)  //for each x column add back the corresponding y columns
                        {
                            addY(yColumn[i][j], xAxisList.getItemCount()-1, selectedLayoutIndex);
                        }
                    }
                }
            }
            else        //not loaded from a DB file
                JOptionPane.showMessageDialog(null, "The selected data table cannot be refreshed.\n" +
                        "Refreshing the data table is only possible if the data table was loaded from a DB file.");
        }
        else    //no data table currently selected
            JOptionPane.showMessageDialog(null, "No data table is currently selected.\n" +
                    "Please select the data table to be refreshed, and try again.");
    }

    /**
     *Checks whether f has a valid extension for a DB file.
     *The valid extensions are "db", "dbc", and "cmd"
     */
    private boolean hasDBExtension(File f)
    {
        int dotIndex = f.getName().lastIndexOf(".");
        if(dotIndex<0)      //not a file - likely a directory
            return false;
        else
        {
            String extension = f.getName().substring(dotIndex+1);
            return extension.equalsIgnoreCase("db")||extension.equalsIgnoreCase("dbc")||extension.equalsIgnoreCase("cmd");
        }
    }
    
    /**
     *Returns the indices of the x columns corresponding to the currently selected data table
     *The parameter here gives the index of the currently selected data table
     */
    private int[] getXColumnList(DataTable selectedDataTable)
    {
        int[] result = new int[xAxisList.getItemCount()];
        String currName;
        String filename = selectedDataTable.filepath.substring(selectedDataTable.filepath.lastIndexOf(File.separator)+1);
        //System.out.println("filename:\t" + filename);
        for(int i=0; i<xAxisList.getItemCount(); i++)
        {
            //select the next item and get its name
            xAxisList.select(i);
            currName = xAxisList.getSelectedItem();
            //check if the current axis belongs to the correct file by checking for the filename
            //list entries have the form [columnname] ([filename])
            if(currName.indexOf(filename)>=0)   //the current column belongs to this file
            {
                currName = currName.substring(0,currName.indexOf("(")).trim();      //need to remove the filename from the column
                //find the index of the column wiith the corresponding name in the selected data table, and put that index in the result array
                result[i] = selectedDataTable.getIndex(currName);
            }
            else        //not in the current file - set to -1
            {
                result[i] = -1;
            }
        }
        return result;
    }
    
    /**
     *Returns the indices of the y columns for each of the x columns corresponding to the currently selected data table
     *The first index tracks the corresponding x column and the second index tracks the y column itself
     *The parameters here gives the index of the currently selected data table and the index of the corresponding x column
     */
    private int[] getYColumnList(DataTable selectedDataTable, int xColumnIndex)
    {
        String currName;
        xAxisList.select(xColumnIndex);
        int[] result = new int[yAxisList.getItemCount()];       
        
        for(int j=0; j<result.length; j++)
        {
            //select the next item on the y axis and get its name
            yAxisList.select(j);
            currName = yAxisList.getSelectedItem();
            //find the index of the corresponding column and add it to the result
            result[j] = selectedDataTable.getIndex(currName);
        }
        
        return result;  
    }
}